/* config.c generated by valac 0.38.8, the Vala compiler
 * generated from config.vala, do not modify */

/*
 Copyright (C) 2014 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include "toolbar-editor.h"
#include <stdlib.h>
#include <string.h>
#include <glib/gstdio.h>

enum  {
	ADBLOCK_CONFIG_0_PROPERTY,
	ADBLOCK_CONFIG_PATH_PROPERTY,
	ADBLOCK_CONFIG_ENABLED_PROPERTY,
	ADBLOCK_CONFIG_SIZE_PROPERTY,
	ADBLOCK_CONFIG_NUM_PROPERTIES
};
static GParamSpec* adblock_config_properties[ADBLOCK_CONFIG_NUM_PROPERTIES];
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_key_file_unref0(var) ((var == NULL) ? NULL : (var = (g_key_file_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))

struct _AdblockConfigPrivate {
	GList* subscriptions;
	gchar* _path;
	GKeyFile* keyfile;
	gboolean should_save;
	gboolean _enabled;
	guint _size;
};


static gpointer adblock_config_parent_class = NULL;

#define ADBLOCK_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ADBLOCK_TYPE_CONFIG, AdblockConfigPrivate))
static void _g_object_unref0_ (gpointer var);
static inline void _g_list_free__g_object_unref0_ (GList* self);
static void adblock_config_set_path (AdblockConfig* self, const gchar* value);
static void adblock_config_set_size (AdblockConfig* self, guint value);
static void adblock_config_load_file (AdblockConfig* self, const gchar* filename);
static void adblock_config_enabled_changed (AdblockConfig* self, GParamSpec* pspec);
static void _adblock_config_enabled_changed_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void adblock_config_active_changed (AdblockConfig* self, GObject* subscription, GParamSpec* pspec);
static void adblock_config_update_filters (AdblockConfig* self);
static gboolean adblock_config_contains (AdblockConfig* self, AdblockSubscription* subscription);
static void _adblock_config_active_changed_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void adblock_config_finalize (GObject * obj);
static void _vala_adblock_config_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_adblock_config_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


static void _g_object_unref0_ (gpointer var) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 73 "config.c"
}


static inline void _g_list_free__g_object_unref0_ (GList* self) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_list_free_full (self, (GDestroyNotify) _g_object_unref0_);
#line 80 "config.c"
}


AdblockConfig* adblock_config_construct (GType object_type, const gchar* path, const gchar* presets) {
	AdblockConfig * self = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self = (AdblockConfig*) g_object_new (object_type, NULL);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->should_save = FALSE;
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	__g_list_free__g_object_unref0_0 (self->priv->subscriptions);
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->subscriptions = NULL;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_set_enabled (self, TRUE);
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = path;
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_set_path (self, _tmp0_);
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_set_size (self, (guint) 0);
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = path;
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_load_file (self, _tmp1_);
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp2_ = presets;
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_load_file (self, _tmp2_);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->should_save = TRUE;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return self;
#line 117 "config.c"
}


AdblockConfig* adblock_config_new (const gchar* path, const gchar* presets) {
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return adblock_config_construct (ADBLOCK_TYPE_CONFIG, path, presets);
#line 124 "config.c"
}


static glong string_strnlen (gchar* str, glong maxlen) {
	glong result = 0L;
	gchar* end = NULL;
	gchar* _tmp0_;
	glong _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = str;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = maxlen;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = memchr (_tmp0_, 0, (gsize) _tmp1_);
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	end = _tmp2_;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = end;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp3_ == NULL) {
#line 147 "config.c"
		glong _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = maxlen;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 155 "config.c"
	} else {
		gchar* _tmp5_;
		gchar* _tmp6_;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = end;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = str;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = (glong) (_tmp5_ - _tmp6_);
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 167 "config.c"
	}
}


static gchar* string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	glong _tmp1_;
	glong _tmp8_;
	glong _tmp14_;
	glong _tmp17_;
	glong _tmp18_;
	glong _tmp19_;
	glong _tmp20_;
	glong _tmp21_;
	gchar* _tmp22_;
#line 1341 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = offset;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_ >= ((glong) 0)) {
#line 191 "config.c"
		glong _tmp2_;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = len;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = _tmp2_ >= ((glong) 0);
#line 197 "config.c"
	} else {
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = FALSE;
#line 201 "config.c"
	}
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 205 "config.c"
		glong _tmp3_;
		glong _tmp4_;
		glong _tmp5_;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = offset;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = len;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = string_strnlen ((gchar*) self, _tmp3_ + _tmp4_);
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = _tmp5_;
#line 217 "config.c"
	} else {
		gint _tmp6_;
		gint _tmp7_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = strlen (self);
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = _tmp6_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = (glong) _tmp7_;
#line 227 "config.c"
	}
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp8_ = offset;
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp8_ < ((glong) 0)) {
#line 233 "config.c"
		glong _tmp9_;
		glong _tmp10_;
		glong _tmp11_;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = string_length;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = offset;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		offset = _tmp9_ + _tmp10_;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = offset;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp11_ >= ((glong) 0), NULL);
#line 247 "config.c"
	} else {
		glong _tmp12_;
		glong _tmp13_;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = offset;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = string_length;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp12_ <= _tmp13_, NULL);
#line 257 "config.c"
	}
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp14_ = len;
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp14_ < ((glong) 0)) {
#line 263 "config.c"
		glong _tmp15_;
		glong _tmp16_;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = string_length;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp16_ = offset;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		len = _tmp15_ - _tmp16_;
#line 272 "config.c"
	}
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp17_ = offset;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp18_ = len;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp19_ = string_length;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail ((_tmp17_ + _tmp18_) <= _tmp19_, NULL);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp20_ = offset;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp21_ = len;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp22_ = g_strndup (((gchar*) self) + _tmp20_, (gsize) _tmp21_);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp22_;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 292 "config.c"
}


static void _adblock_config_enabled_changed_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_enabled_changed ((AdblockConfig*) self, pspec);
#line 299 "config.c"
}


static void adblock_config_load_file (AdblockConfig* self, const gchar* filename) {
	const gchar* _tmp0_;
	GKeyFile* _tmp1_;
	GError * _inner_error_ = NULL;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = filename;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (_tmp0_ == NULL) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		return;
#line 315 "config.c"
	}
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = g_key_file_new ();
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_g_key_file_unref0 (self->priv->keyfile);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->keyfile = _tmp1_;
#line 323 "config.c"
	{
		GKeyFile* _tmp2_;
		const gchar* _tmp3_;
		gchar** filters = NULL;
		GKeyFile* _tmp4_;
		gsize _tmp5_;
		gchar** _tmp6_;
		gint filters_length1;
		gint _filters_size_;
		gchar** _tmp7_;
		gint _tmp7__length1;
		gboolean _tmp36_ = FALSE;
		GKeyFile* _tmp37_;
		gboolean _tmp38_;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp2_ = self->priv->keyfile;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp3_ = filename;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_key_file_load_from_file (_tmp2_, _tmp3_, G_KEY_FILE_NONE, &_inner_error_);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
#line 348 "config.c"
				goto __catch0_g_key_file_error_key_not_found;
			}
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
#line 353 "config.c"
				goto __catch0_g_key_file_error_group_not_found;
			}
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
#line 358 "config.c"
				goto __catch0_g_file_error_noent;
			}
			goto __catch0_g_error;
		}
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp4_ = self->priv->keyfile;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp6_ = g_key_file_get_string_list (_tmp4_, "settings", "filters", &_tmp5_, &_inner_error_);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		filters = _tmp6_;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		filters_length1 = _tmp5_;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_filters_size_ = filters_length1;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
#line 377 "config.c"
				goto __catch0_g_key_file_error_key_not_found;
			}
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
#line 382 "config.c"
				goto __catch0_g_key_file_error_group_not_found;
			}
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
#line 387 "config.c"
				goto __catch0_g_file_error_noent;
			}
			goto __catch0_g_error;
		}
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp7_ = filters;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp7__length1 = filters_length1;
#line 396 "config.c"
		{
			gchar** filter_collection = NULL;
			gint filter_collection_length1 = 0;
			gint _filter_collection_size_ = 0;
			gint filter_it = 0;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			filter_collection = _tmp7_;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			filter_collection_length1 = _tmp7__length1;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			for (filter_it = 0; filter_it < _tmp7__length1; filter_it = filter_it + 1) {
#line 408 "config.c"
				const gchar* filter = NULL;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				filter = filter_collection[filter_it];
#line 412 "config.c"
				{
					gboolean active = FALSE;
					gchar* uri = NULL;
					const gchar* _tmp8_;
					gchar* _tmp9_;
					const gchar* _tmp10_;
					gboolean _tmp11_;
					AdblockSubscription* sub = NULL;
					const gchar* _tmp28_;
					AdblockSubscription* _tmp29_;
					AdblockSubscription* _tmp30_;
					gboolean _tmp31_;
					AdblockSubscription* _tmp32_;
					AdblockUpdater* _tmp33_;
					AdblockUpdater* _tmp34_;
					AdblockSubscription* _tmp35_;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					active = FALSE;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp8_ = filter;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp9_ = g_strdup (_tmp8_);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					uri = _tmp9_;
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp10_ = filter;
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp11_ = g_str_has_prefix (_tmp10_, "http-/");
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					if (_tmp11_) {
#line 443 "config.c"
						const gchar* _tmp12_;
						gchar* _tmp13_;
						gchar* _tmp14_;
						gchar* _tmp15_;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp12_ = filter;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp13_ = string_substring (_tmp12_, (glong) 5, (glong) -1);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp14_ = _tmp13_;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp15_ = g_strconcat ("http:", _tmp14_, NULL);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_g_free0 (uri);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						uri = _tmp15_;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_g_free0 (_tmp14_);
#line 462 "config.c"
					} else {
						const gchar* _tmp16_;
						gboolean _tmp17_;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp16_ = filter;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp17_ = g_str_has_prefix (_tmp16_, "file-/");
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						if (_tmp17_) {
#line 472 "config.c"
							const gchar* _tmp18_;
							gchar* _tmp19_;
							gchar* _tmp20_;
							gchar* _tmp21_;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp18_ = filter;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp19_ = string_substring (_tmp18_, (glong) 5, (glong) -1);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp20_ = _tmp19_;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp21_ = g_strconcat ("file:", _tmp20_, NULL);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_g_free0 (uri);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							uri = _tmp21_;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_g_free0 (_tmp20_);
#line 491 "config.c"
						} else {
							const gchar* _tmp22_;
							gboolean _tmp23_;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp22_ = filter;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp23_ = g_str_has_prefix (_tmp22_, "http-:");
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							if (_tmp23_) {
#line 501 "config.c"
								const gchar* _tmp24_;
								gchar* _tmp25_;
								gchar* _tmp26_;
								gchar* _tmp27_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_tmp24_ = filter;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_tmp25_ = string_substring (_tmp24_, (glong) 5, (glong) -1);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_tmp26_ = _tmp25_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_tmp27_ = g_strconcat ("https", _tmp26_, NULL);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_g_free0 (uri);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								uri = _tmp27_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								_g_free0 (_tmp26_);
#line 520 "config.c"
							} else {
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
								active = TRUE;
#line 524 "config.c"
							}
						}
					}
#line 50 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp28_ = uri;
#line 50 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp29_ = adblock_subscription_new (_tmp28_);
#line 50 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					sub = _tmp29_;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp30_ = sub;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp31_ = active;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					adblock_subscription_set_active (_tmp30_, _tmp31_);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp32_ = sub;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp33_ = adblock_updater_new ();
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp34_ = _tmp33_;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					adblock_subscription_add_feature (_tmp32_, (AdblockFeature*) _tmp34_);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_g_object_unref0 (_tmp34_);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp35_ = sub;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					adblock_config_add (self, _tmp35_);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_g_object_unref0 (sub);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_g_free0 (uri);
#line 558 "config.c"
				}
			}
		}
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp37_ = self->priv->keyfile;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp38_ = g_key_file_get_boolean (_tmp37_, "settings", "disabled", &_inner_error_);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp36_ = _tmp38_;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			filters = (_vala_array_free (filters, filters_length1, (GDestroyNotify) g_free), NULL);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
#line 574 "config.c"
				goto __catch0_g_key_file_error_key_not_found;
			}
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)) {
#line 579 "config.c"
				goto __catch0_g_key_file_error_group_not_found;
			}
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			if (g_error_matches (_inner_error_, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
#line 584 "config.c"
				goto __catch0_g_file_error_noent;
			}
			goto __catch0_g_error;
		}
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		adblock_config_set_enabled (self, !_tmp36_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		filters = (_vala_array_free (filters, filters_length1, (GDestroyNotify) g_free), NULL);
#line 593 "config.c"
	}
	goto __finally0;
	__catch0_g_key_file_error_key_not_found:
	{
		GError* key_error = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		key_error = _inner_error_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_inner_error_ = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_error_free0 (key_error);
#line 605 "config.c"
	}
	goto __finally0;
	__catch0_g_key_file_error_group_not_found:
	{
		GError* group_error = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		group_error = _inner_error_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_inner_error_ = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_error_free0 (group_error);
#line 617 "config.c"
	}
	goto __finally0;
	__catch0_g_file_error_noent:
	{
		GError* exist_error = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		exist_error = _inner_error_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_inner_error_ = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_error_free0 (exist_error);
#line 629 "config.c"
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* settings_error = NULL;
		const gchar* _tmp39_;
		GError* _tmp40_;
		const gchar* _tmp41_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		settings_error = _inner_error_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_inner_error_ = NULL;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp39_ = filename;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp40_ = settings_error;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp41_ = _tmp40_->message;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_warning ("config.vala:63: Error reading settings from %s: %s\n", _tmp39_, _tmp41_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_error_free0 (settings_error);
#line 652 "config.c"
	}
	__finally0:
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_clear_error (&_inner_error_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		return;
#line 663 "config.c"
	}
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_signal_connect_object ((GObject*) self, "notify::enabled", (GCallback) _adblock_config_enabled_changed_g_object_notify, self, 0);
#line 667 "config.c"
}


static void adblock_config_enabled_changed (AdblockConfig* self, GParamSpec* pspec) {
	GKeyFile* _tmp0_;
	gboolean _tmp1_;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (pspec != NULL);
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->keyfile;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = self->priv->_enabled;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_key_file_set_boolean (_tmp0_, "settings", "disabled", !_tmp1_);
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_save (self);
#line 686 "config.c"
}


static void adblock_config_active_changed (AdblockConfig* self, GObject* subscription, GParamSpec* pspec) {
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (subscription != NULL);
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (pspec != NULL);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_update_filters (self);
#line 699 "config.c"
}


static void adblock_config_update_filters (AdblockConfig* self) {
	GString* filters = NULL;
	GString* _tmp0_;
	GList* _tmp1_;
	GString* _tmp58_;
	const gchar* _tmp59_;
	gboolean _tmp60_;
	gchar** list = NULL;
	GString* _tmp64_;
	const gchar* _tmp65_;
	gchar** _tmp66_;
	gchar** _tmp67_;
	gint list_length1;
	gint _list_size_;
	GKeyFile* _tmp68_;
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = g_string_new ("");
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	filters = _tmp0_;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = self->priv->subscriptions;
#line 726 "config.c"
	{
		GList* sub_collection = NULL;
		GList* sub_it = NULL;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		sub_collection = _tmp1_;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		for (sub_it = sub_collection; sub_it != NULL; sub_it = sub_it->next) {
#line 734 "config.c"
			AdblockSubscription* sub = NULL;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			sub = (AdblockSubscription*) sub_it->data;
#line 738 "config.c"
			{
				AdblockSubscription* _tmp2_;
				gboolean _tmp3_;
				gboolean _tmp4_;
				gboolean _tmp5_ = FALSE;
				AdblockSubscription* _tmp6_;
				const gchar* _tmp7_;
				const gchar* _tmp8_;
				gboolean _tmp9_;
				GString* _tmp57_;
#line 81 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp2_ = sub;
#line 81 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp3_ = adblock_subscription_get_mutable (_tmp2_);
#line 81 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp4_ = _tmp3_;
#line 81 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				if (!_tmp4_) {
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					continue;
#line 759 "config.c"
				}
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp6_ = sub;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp7_ = adblock_subscription_get_uri (_tmp6_);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp8_ = _tmp7_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp9_ = g_str_has_prefix (_tmp8_, "http:");
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				if (_tmp9_) {
#line 771 "config.c"
					AdblockSubscription* _tmp10_;
					gboolean _tmp11_;
					gboolean _tmp12_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp10_ = sub;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp11_ = adblock_subscription_get_active (_tmp10_);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp12_ = _tmp11_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp5_ = !_tmp12_;
#line 783 "config.c"
				} else {
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp5_ = FALSE;
#line 787 "config.c"
				}
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				if (_tmp5_) {
#line 791 "config.c"
					GString* _tmp13_;
					AdblockSubscription* _tmp14_;
					const gchar* _tmp15_;
					const gchar* _tmp16_;
					gchar* _tmp17_;
					gchar* _tmp18_;
					gchar* _tmp19_;
					gchar* _tmp20_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp13_ = filters;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp14_ = sub;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp15_ = adblock_subscription_get_uri (_tmp14_);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp16_ = _tmp15_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp17_ = string_substring (_tmp16_, (glong) 4, (glong) -1);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp18_ = _tmp17_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp19_ = g_strconcat ("http-", _tmp18_, NULL);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp20_ = _tmp19_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					g_string_append (_tmp13_, _tmp20_);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_g_free0 (_tmp20_);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_g_free0 (_tmp18_);
#line 822 "config.c"
				} else {
					gboolean _tmp21_ = FALSE;
					AdblockSubscription* _tmp22_;
					const gchar* _tmp23_;
					const gchar* _tmp24_;
					gboolean _tmp25_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp22_ = sub;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp23_ = adblock_subscription_get_uri (_tmp22_);
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp24_ = _tmp23_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					_tmp25_ = g_str_has_prefix (_tmp24_, "file:");
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					if (_tmp25_) {
#line 839 "config.c"
						AdblockSubscription* _tmp26_;
						gboolean _tmp27_;
						gboolean _tmp28_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp26_ = sub;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp27_ = adblock_subscription_get_active (_tmp26_);
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp28_ = _tmp27_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp21_ = !_tmp28_;
#line 851 "config.c"
					} else {
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp21_ = FALSE;
#line 855 "config.c"
					}
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					if (_tmp21_) {
#line 859 "config.c"
						GString* _tmp29_;
						AdblockSubscription* _tmp30_;
						const gchar* _tmp31_;
						const gchar* _tmp32_;
						gchar* _tmp33_;
						gchar* _tmp34_;
						gchar* _tmp35_;
						gchar* _tmp36_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp29_ = filters;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp30_ = sub;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp31_ = adblock_subscription_get_uri (_tmp30_);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp32_ = _tmp31_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp33_ = string_substring (_tmp32_, (glong) 5, (glong) -1);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp34_ = _tmp33_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp35_ = g_strconcat ("file-", _tmp34_, NULL);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp36_ = _tmp35_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						g_string_append (_tmp29_, _tmp36_);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_g_free0 (_tmp36_);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_g_free0 (_tmp34_);
#line 890 "config.c"
					} else {
						gboolean _tmp37_ = FALSE;
						AdblockSubscription* _tmp38_;
						const gchar* _tmp39_;
						const gchar* _tmp40_;
						gboolean _tmp41_;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp38_ = sub;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp39_ = adblock_subscription_get_uri (_tmp38_);
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp40_ = _tmp39_;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						_tmp41_ = g_str_has_prefix (_tmp40_, "https:");
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						if (_tmp41_) {
#line 907 "config.c"
							AdblockSubscription* _tmp42_;
							gboolean _tmp43_;
							gboolean _tmp44_;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp42_ = sub;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp43_ = adblock_subscription_get_active (_tmp42_);
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp44_ = _tmp43_;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp37_ = !_tmp44_;
#line 919 "config.c"
						} else {
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp37_ = FALSE;
#line 923 "config.c"
						}
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
						if (_tmp37_) {
#line 927 "config.c"
							GString* _tmp45_;
							AdblockSubscription* _tmp46_;
							const gchar* _tmp47_;
							const gchar* _tmp48_;
							gchar* _tmp49_;
							gchar* _tmp50_;
							gchar* _tmp51_;
							gchar* _tmp52_;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp45_ = filters;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp46_ = sub;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp47_ = adblock_subscription_get_uri (_tmp46_);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp48_ = _tmp47_;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp49_ = string_substring (_tmp48_, (glong) 5, (glong) -1);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp50_ = _tmp49_;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp51_ = g_strconcat ("http-", _tmp50_, NULL);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp52_ = _tmp51_;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							g_string_append (_tmp45_, _tmp52_);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_g_free0 (_tmp52_);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_g_free0 (_tmp50_);
#line 958 "config.c"
						} else {
							GString* _tmp53_;
							AdblockSubscription* _tmp54_;
							const gchar* _tmp55_;
							const gchar* _tmp56_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp53_ = filters;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp54_ = sub;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp55_ = adblock_subscription_get_uri (_tmp54_);
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							_tmp56_ = _tmp55_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
							g_string_append (_tmp53_, _tmp56_);
#line 974 "config.c"
						}
					}
				}
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp57_ = filters;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				g_string_append_c (_tmp57_, ';');
#line 982 "config.c"
			}
		}
	}
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp58_ = filters;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp59_ = _tmp58_->str;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp60_ = g_str_has_suffix (_tmp59_, ";");
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (_tmp60_) {
#line 994 "config.c"
		GString* _tmp61_;
		GString* _tmp62_;
		gssize _tmp63_;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp61_ = filters;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp62_ = filters;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp63_ = _tmp62_->len;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_string_truncate (_tmp61_, (gsize) (_tmp63_ - 1));
#line 1006 "config.c"
	}
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp64_ = filters;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp65_ = _tmp64_->str;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp67_ = _tmp66_ = g_strsplit (_tmp65_, ";", 0);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	list = _tmp67_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	list_length1 = _vala_array_length (_tmp66_);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_list_size_ = list_length1;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp68_ = self->priv->keyfile;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_key_file_set_string_list (_tmp68_, "settings", "filters", (const gchar* const*) list, list_length1);
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_save (self);
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	list = (_vala_array_free (list, list_length1, (GDestroyNotify) g_free), NULL);
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_g_string_free0 (filters);
#line 1030 "config.c"
}


void adblock_config_save (AdblockConfig* self) {
	GError * _inner_error_ = NULL;
#line 103 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 1038 "config.c"
	{
		const gchar* _tmp0_;
		GKeyFile* _tmp1_;
		gchar* _tmp2_;
		gchar* _tmp3_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp0_ = self->priv->_path;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp1_ = self->priv->keyfile;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp2_ = g_key_file_to_data (_tmp1_, NULL, NULL);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp3_ = _tmp2_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_file_set_contents (_tmp0_, _tmp3_, (gssize) -1, &_inner_error_);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_free0 (_tmp3_);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1058 "config.c"
			goto __catch1_g_error;
		}
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp4_;
		const gchar* _tmp5_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_error_ = _inner_error_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_inner_error_ = NULL;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp4_ = _error_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp5_ = _tmp4_->message;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_warning ("config.vala:107: Failed to save settings: %s", _tmp5_);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_error_free0 (_error_);
#line 1080 "config.c"
	}
	__finally1:
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_clear_error (&_inner_error_);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		return;
#line 1091 "config.c"
	}
}


AdblockSubscription* adblock_config_get (AdblockConfig* self, guint index) {
	AdblockSubscription* result = NULL;
	GList* _tmp0_;
	guint _tmp1_;
	gconstpointer _tmp2_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->subscriptions;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = index;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp2_ = g_list_nth_data (_tmp0_, _tmp1_);
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = (AdblockSubscription*) _tmp2_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1113 "config.c"
}


static gboolean adblock_config_contains (AdblockConfig* self, AdblockSubscription* subscription) {
	gboolean result = FALSE;
	GList* _tmp0_;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (subscription != NULL, FALSE);
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->subscriptions;
#line 1126 "config.c"
	{
		GList* sub_collection = NULL;
		GList* sub_it = NULL;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		sub_collection = _tmp0_;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		for (sub_it = sub_collection; sub_it != NULL; sub_it = sub_it->next) {
#line 1134 "config.c"
			AdblockSubscription* sub = NULL;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
			sub = (AdblockSubscription*) sub_it->data;
#line 1138 "config.c"
			{
				AdblockSubscription* _tmp1_;
				const gchar* _tmp2_;
				const gchar* _tmp3_;
				AdblockSubscription* _tmp4_;
				const gchar* _tmp5_;
				const gchar* _tmp6_;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp1_ = sub;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp2_ = adblock_subscription_get_uri (_tmp1_);
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp3_ = _tmp2_;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp4_ = subscription;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp5_ = adblock_subscription_get_uri (_tmp4_);
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				_tmp6_ = _tmp5_;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
				if (g_strcmp0 (_tmp3_, _tmp6_) == 0) {
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					result = TRUE;
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
					return result;
#line 1164 "config.c"
				}
			}
		}
	}
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = FALSE;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1173 "config.c"
}


static void _adblock_config_active_changed_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_active_changed ((AdblockConfig*) self, _sender, pspec);
#line 1180 "config.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return self ? g_object_ref (self) : NULL;
#line 1187 "config.c"
}


gboolean adblock_config_add (AdblockConfig* self, AdblockSubscription* sub) {
	gboolean result = FALSE;
	AdblockSubscription* _tmp0_;
	gboolean _tmp1_;
	AdblockSubscription* _tmp2_;
	AdblockSubscription* _tmp3_;
	AdblockSubscription* _tmp4_;
	guint _tmp5_;
	gboolean _tmp6_;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (sub != NULL, FALSE);
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = sub;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = adblock_config_contains (self, _tmp0_);
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (_tmp1_) {
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		result = FALSE;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		return result;
#line 1214 "config.c"
	}
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp2_ = sub;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_signal_connect_object ((GObject*) _tmp2_, "notify::active", (GCallback) _adblock_config_active_changed_g_object_notify, self, 0);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp3_ = sub;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp4_ = _g_object_ref0 (_tmp3_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->subscriptions = g_list_append (self->priv->subscriptions, _tmp4_);
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp5_ = self->priv->_size;
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_set_size (self, _tmp5_ + 1);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp6_ = self->priv->should_save;
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (_tmp6_) {
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		adblock_config_update_filters (self);
#line 1236 "config.c"
	}
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = TRUE;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1242 "config.c"
}


void adblock_config_remove (AdblockConfig* self, AdblockSubscription* sub) {
	AdblockSubscription* _tmp0_;
	gboolean _tmp1_;
	AdblockSubscription* _tmp2_;
	AdblockSubscription* _tmp3_;
	guint _tmp4_;
	GQuark _tmp5_;
	guint _tmp6_;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (sub != NULL);
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = sub;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp1_ = adblock_config_contains (self, _tmp0_);
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (!_tmp1_) {
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		return;
#line 1266 "config.c"
	}
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp2_ = sub;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv->subscriptions = g_list_remove (self->priv->subscriptions, _tmp2_);
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp3_ = sub;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_signal_parse_name ("notify::active", G_TYPE_OBJECT, &_tmp4_, &_tmp5_, TRUE);
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_signal_handlers_disconnect_matched ((GObject*) _tmp3_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp4_, _tmp5_, NULL, (GCallback) _adblock_config_active_changed_g_object_notify, self);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_update_filters (self);
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp6_ = self->priv->_size;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_set_size (self, _tmp6_ - 1);
#line 1284 "config.c"
}


const gchar* adblock_config_get_path (AdblockConfig* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->_path;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = _tmp0_;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1299 "config.c"
}


static void adblock_config_set_path (AdblockConfig* self, const gchar* value) {
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (g_strcmp0 (value, adblock_config_get_path (self)) != 0) {
#line 1308 "config.c"
		const gchar* _tmp0_;
		gchar* _tmp1_;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp0_ = value;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp1_ = g_strdup (_tmp0_);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_g_free0 (self->priv->_path);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		self->priv->_path = _tmp1_;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_config_properties[ADBLOCK_CONFIG_PATH_PROPERTY]);
#line 1321 "config.c"
	}
}


gboolean adblock_config_get_enabled (AdblockConfig* self) {
	gboolean result;
	gboolean _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->_enabled;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1337 "config.c"
}


void adblock_config_set_enabled (AdblockConfig* self, gboolean value) {
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (adblock_config_get_enabled (self) != value) {
#line 1346 "config.c"
		gboolean _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp0_ = value;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		self->priv->_enabled = _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_config_properties[ADBLOCK_CONFIG_ENABLED_PROPERTY]);
#line 1354 "config.c"
	}
}


guint adblock_config_get_size (AdblockConfig* self) {
	guint result;
	guint _tmp0_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_tmp0_ = self->priv->_size;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	result = _tmp0_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	return result;
#line 1370 "config.c"
}


static void adblock_config_set_size (AdblockConfig* self, guint value) {
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_return_if_fail (self != NULL);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	if (adblock_config_get_size (self) != value) {
#line 1379 "config.c"
		guint _tmp0_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		_tmp0_ = value;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		self->priv->_size = _tmp0_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_config_properties[ADBLOCK_CONFIG_SIZE_PROPERTY]);
#line 1387 "config.c"
	}
}


static void adblock_config_class_init (AdblockConfigClass * klass) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	adblock_config_parent_class = g_type_class_peek_parent (klass);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_type_class_add_private (klass, sizeof (AdblockConfigPrivate));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_adblock_config_get_property;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_adblock_config_set_property;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	G_OBJECT_CLASS (klass)->finalize = adblock_config_finalize;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_CONFIG_PATH_PROPERTY, adblock_config_properties[ADBLOCK_CONFIG_PATH_PROPERTY] = g_param_spec_string ("path", "path", "path", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_CONFIG_ENABLED_PROPERTY, adblock_config_properties[ADBLOCK_CONFIG_ENABLED_PROPERTY] = g_param_spec_boolean ("enabled", "enabled", "enabled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_CONFIG_SIZE_PROPERTY, adblock_config_properties[ADBLOCK_CONFIG_SIZE_PROPERTY] = g_param_spec_uint ("size", "size", "size", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 1409 "config.c"
}


static void adblock_config_instance_init (AdblockConfig * self) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self->priv = ADBLOCK_CONFIG_GET_PRIVATE (self);
#line 1416 "config.c"
}


static void adblock_config_finalize (GObject * obj) {
	AdblockConfig * self;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, ADBLOCK_TYPE_CONFIG, AdblockConfig);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	__g_list_free__g_object_unref0_0 (self->priv->subscriptions);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_g_free0 (self->priv->_path);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	_g_key_file_unref0 (self->priv->keyfile);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	G_OBJECT_CLASS (adblock_config_parent_class)->finalize (obj);
#line 1432 "config.c"
}


GType adblock_config_get_type (void) {
	static volatile gsize adblock_config_type_id__volatile = 0;
	if (g_once_init_enter (&adblock_config_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AdblockConfigClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adblock_config_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdblockConfig), 0, (GInstanceInitFunc) adblock_config_instance_init, NULL };
		GType adblock_config_type_id;
		adblock_config_type_id = g_type_register_static (G_TYPE_OBJECT, "AdblockConfig", &g_define_type_info, 0);
		g_once_init_leave (&adblock_config_type_id__volatile, adblock_config_type_id);
	}
	return adblock_config_type_id__volatile;
}


static void _vala_adblock_config_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AdblockConfig * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_CONFIG, AdblockConfig);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	switch (property_id) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_PATH_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_value_set_string (value, adblock_config_get_path (self));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_ENABLED_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_value_set_boolean (value, adblock_config_get_enabled (self));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_SIZE_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		g_value_set_uint (value, adblock_config_get_size (self));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 1471 "config.c"
		default:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 1477 "config.c"
	}
}


static void _vala_adblock_config_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AdblockConfig * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_CONFIG, AdblockConfig);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
	switch (property_id) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_PATH_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		adblock_config_set_path (self, g_value_get_string (value));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_ENABLED_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		adblock_config_set_enabled (self, g_value_get_boolean (value));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		case ADBLOCK_CONFIG_SIZE_PROPERTY:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		adblock_config_set_size (self, g_value_get_uint (value));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 1505 "config.c"
		default:
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/config.vala"
		break;
#line 1511 "config.c"
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



