/* midori-contextaction.c generated by valac 0.38.8, the Vala compiler
 * generated from midori-contextaction.vala, do not modify */

/*
 Copyright (C) 2013 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>


#define MIDORI_TYPE_CONTEXT_ACTION (midori_context_action_get_type ())
#define MIDORI_CONTEXT_ACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_CONTEXT_ACTION, MidoriContextAction))
#define MIDORI_CONTEXT_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_TYPE_CONTEXT_ACTION, MidoriContextActionClass))
#define MIDORI_IS_CONTEXT_ACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_CONTEXT_ACTION))
#define MIDORI_IS_CONTEXT_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_TYPE_CONTEXT_ACTION))
#define MIDORI_CONTEXT_ACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_TYPE_CONTEXT_ACTION, MidoriContextActionClass))

typedef struct _MidoriContextAction MidoriContextAction;
typedef struct _MidoriContextActionClass MidoriContextActionClass;
typedef struct _MidoriContextActionPrivate MidoriContextActionPrivate;
enum  {
	MIDORI_CONTEXT_ACTION_0_PROPERTY,
	MIDORI_CONTEXT_ACTION_NUM_PROPERTIES
};
static GParamSpec* midori_context_action_properties[MIDORI_CONTEXT_ACTION_NUM_PROPERTIES];
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _Block2Data Block2Data;

#define MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION (midori_separator_context_action_get_type ())
#define MIDORI_SEPARATOR_CONTEXT_ACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION, MidoriSeparatorContextAction))
#define MIDORI_SEPARATOR_CONTEXT_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION, MidoriSeparatorContextActionClass))
#define MIDORI_IS_SEPARATOR_CONTEXT_ACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION))
#define MIDORI_IS_SEPARATOR_CONTEXT_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION))
#define MIDORI_SEPARATOR_CONTEXT_ACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION, MidoriSeparatorContextActionClass))

typedef struct _MidoriSeparatorContextAction MidoriSeparatorContextAction;
typedef struct _MidoriSeparatorContextActionClass MidoriSeparatorContextActionClass;
typedef struct _MidoriSeparatorContextActionPrivate MidoriSeparatorContextActionPrivate;
enum  {
	MIDORI_SEPARATOR_CONTEXT_ACTION_0_PROPERTY,
	MIDORI_SEPARATOR_CONTEXT_ACTION_NUM_PROPERTIES
};
static GParamSpec* midori_separator_context_action_properties[MIDORI_SEPARATOR_CONTEXT_ACTION_NUM_PROPERTIES];

struct _MidoriContextAction {
	GtkAction parent_instance;
	MidoriContextActionPrivate * priv;
};

struct _MidoriContextActionClass {
	GtkActionClass parent_class;
};

struct _MidoriContextActionPrivate {
	GList* action_groups;
	GList* children;
	GtkToolButton* toolitem;
};

typedef void (*MidoriContextActionActionActivateCallback) (GtkAction* action, void* user_data);
struct _Block2Data {
	int _ref_count_;
	MidoriContextAction* self;
	MidoriContextAction* action;
	MidoriContextActionActionActivateCallback callback;
	gpointer callback_target;
};

struct _MidoriSeparatorContextAction {
	MidoriContextAction parent_instance;
	MidoriSeparatorContextActionPrivate * priv;
};

struct _MidoriSeparatorContextActionClass {
	MidoriContextActionClass parent_class;
};


static gpointer midori_context_action_parent_class = NULL;
static gpointer midori_separator_context_action_parent_class = NULL;

GType midori_context_action_get_type (void) G_GNUC_CONST;
#define MIDORI_CONTEXT_ACTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MIDORI_TYPE_CONTEXT_ACTION, MidoriContextActionPrivate))
static void _g_object_unref0_ (gpointer var);
static inline void _g_list_free__g_object_unref0_ (GList* self);
MidoriContextAction* midori_context_action_new (const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id);
MidoriContextAction* midori_context_action_construct (GType object_type, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id);
MidoriContextAction* midori_context_action_new_escaped (const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id);
MidoriContextAction* midori_context_action_construct_escaped (GType object_type, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id);
void midori_context_action_add_simple (MidoriContextAction* self, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id, MidoriContextActionActionActivateCallback callback, void* callback_target);
static Block2Data* block2_data_ref (Block2Data* _data2_);
static void block2_data_unref (void * _userdata_);
static void __lambda8_ (Block2Data* _data2_);
static void ___lambda8__gtk_action_activate (GtkAction* _sender, gpointer self);
void midori_context_action_add (MidoriContextAction* self, GtkAction* action);
MidoriSeparatorContextAction* midori_separator_context_action_new (void);
MidoriSeparatorContextAction* midori_separator_context_action_construct (GType object_type);
GType midori_separator_context_action_get_type (void) G_GNUC_CONST;
void midori_context_action_add_action_group (MidoriContextAction* self, GtkActionGroup* action_group);
void midori_context_action_add_by_name (MidoriContextAction* self, const gchar* name);
static GtkWidget* midori_context_action_real_create_tool_item (GtkAction* base);
static void __lambda9_ (MidoriContextAction* self);
GtkMenu* midori_context_action_create_menu (MidoriContextAction* self, GtkMenu* default_menu, gboolean accels);
static void ___lambda9__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
GtkAction* midori_context_action_get_by_name (MidoriContextAction* self, const gchar* name);
static void midori_context_action_finalize (GObject * obj);


static void _g_object_unref0_ (gpointer var) {
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 128 "midori-contextaction.c"
}


static inline void _g_list_free__g_object_unref0_ (GList* self) {
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_list_free_full (self, (GDestroyNotify) _g_object_unref0_);
#line 135 "midori-contextaction.c"
}


MidoriContextAction* midori_context_action_construct (GType object_type, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id) {
	MidoriContextAction * self = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = name;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = label;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = tooltip;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp3_ = stock_id;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = (MidoriContextAction*) g_object_new (object_type, "name", _tmp0_, "label", _tmp1_, "tooltip", _tmp2_, "stock-id", _tmp3_, NULL);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->action_groups);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->action_groups = NULL;
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->children);
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->children = NULL;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return self;
#line 167 "midori-contextaction.c"
}


MidoriContextAction* midori_context_action_new (const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id) {
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return midori_context_action_construct (MIDORI_TYPE_CONTEXT_ACTION, name, label, tooltip, stock_id);
#line 174 "midori-contextaction.c"
}


static gchar* string_replace (const gchar* self, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	GError * _inner_error_ = NULL;
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if ((*((gchar*) self)) == '\0') {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = TRUE;
#line 193 "midori-contextaction.c"
	} else {
		const gchar* _tmp2_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = (*((gchar*) _tmp2_)) == '\0';
#line 200 "midori-contextaction.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_) {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = TRUE;
#line 206 "midori-contextaction.c"
	} else {
		const gchar* _tmp3_;
		const gchar* _tmp4_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = replacement;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = g_strcmp0 (_tmp3_, _tmp4_) == 0;
#line 216 "midori-contextaction.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 220 "midori-contextaction.c"
		gchar* _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = g_strdup (self);
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 228 "midori-contextaction.c"
	}
	{
		GRegex* regex = NULL;
		const gchar* _tmp6_;
		gchar* _tmp7_;
		gchar* _tmp8_;
		GRegex* _tmp9_;
		GRegex* _tmp10_;
		gchar* _tmp11_ = NULL;
		GRegex* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
		gchar* _tmp15_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = old;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = g_regex_escape_string (_tmp6_, -1);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp8_ = _tmp7_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = g_regex_new (_tmp8_, 0, 0, &_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = _tmp9_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp8_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		regex = _tmp10_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 260 "midori-contextaction.c"
				goto __catch12_g_regex_error;
			}
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 269 "midori-contextaction.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = regex;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = replacement;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp14_ = g_regex_replace_literal (_tmp12_, self, (gssize) -1, 0, _tmp13_, 0, &_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = _tmp14_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 285 "midori-contextaction.c"
				goto __catch12_g_regex_error;
			}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 296 "midori-contextaction.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = _tmp11_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = NULL;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp15_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp11_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 310 "midori-contextaction.c"
	}
	goto __finally12;
	__catch12_g_regex_error:
	{
		GError* e = NULL;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		e = _inner_error_;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_inner_error_ = NULL;
#line 1423 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_assert_not_reached ();
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_error_free0 (e);
#line 324 "midori-contextaction.c"
	}
	__finally12:
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_clear_error (&_inner_error_);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return NULL;
#line 335 "midori-contextaction.c"
	}
}


MidoriContextAction* midori_context_action_construct_escaped (GType object_type, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id) {
	MidoriContextAction * self = NULL;
	gchar* escaped_label = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (label != NULL, NULL);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = label;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = string_replace (_tmp0_, "_", "__");
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	escaped_label = _tmp1_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = name;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp3_ = tooltip;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp4_ = stock_id;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = (MidoriContextAction*) g_object_new (object_type, "name", _tmp2_, "label", escaped_label, "tooltip", _tmp3_, "stock-id", _tmp4_, NULL);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->action_groups);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->action_groups = NULL;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->children);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->children = NULL;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_g_free0 (escaped_label);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return self;
#line 378 "midori-contextaction.c"
}


MidoriContextAction* midori_context_action_new_escaped (const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return midori_context_action_construct_escaped (MIDORI_TYPE_CONTEXT_ACTION, name, label, tooltip, stock_id);
#line 385 "midori-contextaction.c"
}


static Block2Data* block2_data_ref (Block2Data* _data2_) {
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_atomic_int_inc (&_data2_->_ref_count_);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return _data2_;
#line 394 "midori-contextaction.c"
}


static void block2_data_unref (void * _userdata_) {
	Block2Data* _data2_;
	_data2_ = (Block2Data*) _userdata_;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	if (g_atomic_int_dec_and_test (&_data2_->_ref_count_)) {
#line 403 "midori-contextaction.c"
		MidoriContextAction* self;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		self = _data2_->self;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_g_object_unref0 (_data2_->action);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_g_object_unref0 (self);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		g_slice_free (Block2Data, _data2_);
#line 413 "midori-contextaction.c"
	}
}


static void __lambda8_ (Block2Data* _data2_) {
	MidoriContextAction* self;
	MidoriContextActionActionActivateCallback _tmp0_;
	void* _tmp0__target;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = _data2_->self;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = _data2_->callback;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0__target = _data2_->callback_target;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ ((GtkAction*) _data2_->action, _tmp0__target);
#line 430 "midori-contextaction.c"
}


static void ___lambda8__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__lambda8_ (self);
#line 437 "midori-contextaction.c"
}


void midori_context_action_add_simple (MidoriContextAction* self, const gchar* name, const gchar* label, const gchar* tooltip, const gchar* stock_id, MidoriContextActionActionActivateCallback callback, void* callback_target) {
	Block2Data* _data2_;
	MidoriContextActionActionActivateCallback _tmp0_;
	void* _tmp0__target;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	MidoriContextAction* _tmp5_;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (self != NULL);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (name != NULL);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_ = g_slice_new0 (Block2Data);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_->_ref_count_ = 1;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_->self = g_object_ref (self);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = callback;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0__target = callback_target;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_->callback = _tmp0_;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_->callback_target = _tmp0__target;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = name;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = label;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp3_ = tooltip;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp4_ = stock_id;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp5_ = midori_context_action_new (_tmp1_, _tmp2_, _tmp3_, _tmp4_);
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_->action = _tmp5_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_signal_connect_data ((GtkAction*) _data2_->action, "activate", (GCallback) ___lambda8__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	midori_context_action_add (self, (GtkAction*) _data2_->action);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	block2_data_unref (_data2_);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_data2_ = NULL;
#line 488 "midori-contextaction.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return self ? g_object_ref (self) : NULL;
#line 495 "midori-contextaction.c"
}


void midori_context_action_add (MidoriContextAction* self, GtkAction* action) {
	GtkAction* _tmp0_;
	GtkAction* _tmp3_;
	GtkAction* _tmp4_;
	GtkAction* _tmp5_;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (self != NULL);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = action;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	if (_tmp0_ == NULL) {
#line 510 "midori-contextaction.c"
		MidoriSeparatorContextAction* _tmp1_;
		MidoriSeparatorContextAction* _tmp2_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_tmp1_ = midori_separator_context_action_new ();
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_tmp2_ = _tmp1_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		midori_context_action_add (self, (GtkAction*) _tmp2_);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_g_object_unref0 (_tmp2_);
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		return;
#line 523 "midori-contextaction.c"
	}
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp3_ = action;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp4_ = _g_object_ref0 (_tmp3_);
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->children = g_list_append (self->priv->children, _tmp4_);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp5_ = action;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, MIDORI_TYPE_CONTEXT_ACTION)) {
#line 535 "midori-contextaction.c"
		GList* _tmp6_;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_tmp6_ = self->priv->action_groups;
#line 539 "midori-contextaction.c"
		{
			GList* action_group_collection = NULL;
			GList* action_group_it = NULL;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			action_group_collection = _tmp6_;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			for (action_group_it = action_group_collection; action_group_it != NULL; action_group_it = action_group_it->next) {
#line 547 "midori-contextaction.c"
				GtkActionGroup* _tmp7_;
				GtkActionGroup* action_group = NULL;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp7_ = _g_object_ref0 ((GtkActionGroup*) action_group_it->data);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				action_group = _tmp7_;
#line 554 "midori-contextaction.c"
				{
					GtkAction* _tmp8_;
					GtkActionGroup* _tmp9_;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp8_ = action;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp9_ = action_group;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					midori_context_action_add_action_group (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, MIDORI_TYPE_CONTEXT_ACTION) ? ((MidoriContextAction*) _tmp8_) : NULL, _tmp9_);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_g_object_unref0 (action_group);
#line 566 "midori-contextaction.c"
				}
			}
		}
	}
}


void midori_context_action_add_action_group (MidoriContextAction* self, GtkActionGroup* action_group) {
	GtkActionGroup* _tmp0_;
	GtkActionGroup* _tmp1_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (self != NULL);
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (action_group != NULL);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = action_group;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->action_groups = g_list_append (self->priv->action_groups, _tmp1_);
#line 587 "midori-contextaction.c"
}


void midori_context_action_add_by_name (MidoriContextAction* self, const gchar* name) {
	GList* _tmp0_;
	const gchar* _tmp8_;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (self != NULL);
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_if_fail (name != NULL);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = self->priv->action_groups;
#line 600 "midori-contextaction.c"
	{
		GList* action_group_collection = NULL;
		GList* action_group_it = NULL;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		action_group_collection = _tmp0_;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		for (action_group_it = action_group_collection; action_group_it != NULL; action_group_it = action_group_it->next) {
#line 608 "midori-contextaction.c"
			GtkActionGroup* _tmp1_;
			GtkActionGroup* action_group = NULL;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			_tmp1_ = _g_object_ref0 ((GtkActionGroup*) action_group_it->data);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			action_group = _tmp1_;
#line 615 "midori-contextaction.c"
			{
				GtkAction* action = NULL;
				GtkActionGroup* _tmp2_;
				const gchar* _tmp3_;
				GtkAction* _tmp4_;
				GtkAction* _tmp5_;
				GtkAction* _tmp6_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp2_ = action_group;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp3_ = name;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp4_ = gtk_action_group_get_action (_tmp2_, _tmp3_);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp5_ = _g_object_ref0 (_tmp4_);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				action = _tmp5_;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp6_ = action;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				if (_tmp6_ != NULL) {
#line 637 "midori-contextaction.c"
					GtkAction* _tmp7_;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp7_ = action;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					midori_context_action_add (self, _tmp7_);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_g_object_unref0 (action);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_g_object_unref0 (action_group);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					return;
#line 649 "midori-contextaction.c"
				}
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_g_object_unref0 (action);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_g_object_unref0 (action_group);
#line 655 "midori-contextaction.c"
			}
		}
	}
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp8_ = name;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_warning ("midori-contextaction.vala:70: Action %s not known to ContextAction", _tmp8_);
#line 663 "midori-contextaction.c"
}


static void __lambda9_ (MidoriContextAction* self) {
	GtkMenu* popup = NULL;
	GtkMenu* _tmp0_;
	GtkToolButton* _tmp1_;
	guint32 _tmp2_;
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = midori_context_action_create_menu (self, NULL, FALSE);
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	popup = _tmp0_;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	gtk_widget_show ((GtkWidget*) popup);
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = self->priv->toolitem;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	gtk_menu_attach_to_widget (popup, (GtkWidget*) _tmp1_, NULL);
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = gtk_get_current_event_time ();
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	gtk_menu_popup (popup, NULL, NULL, NULL, NULL, (guint) 1, _tmp2_);
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_g_object_unref0 (popup);
#line 688 "midori-contextaction.c"
}


static void ___lambda9__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__lambda9_ ((MidoriContextAction*) self);
#line 695 "midori-contextaction.c"
}


static GtkWidget* midori_context_action_real_create_tool_item (GtkAction* base) {
	MidoriContextAction * self;
	GtkWidget* result = NULL;
	GtkWidget* _tmp0_;
	GtkToolButton* _tmp1_;
	GtkToolButton* _tmp2_;
	GtkToolButton* _tmp3_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = (MidoriContextAction*) base;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = GTK_ACTION_CLASS (midori_context_action_parent_class)->create_tool_item (G_TYPE_CHECK_INSTANCE_CAST (self, GTK_TYPE_ACTION, GtkAction));
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GTK_TYPE_TOOL_BUTTON) ? ((GtkToolButton*) _tmp0_) : NULL);
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_g_object_unref0 (self->priv->toolitem);
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv->toolitem = _tmp1_;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = self->priv->toolitem;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_signal_connect_object (_tmp2_, "clicked", (GCallback) ___lambda9__gtk_tool_button_clicked, self, 0);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp3_ = self->priv->toolitem;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	result = (GtkWidget*) _tmp3_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return result;
#line 726 "midori-contextaction.c"
}


GtkMenu* midori_context_action_create_menu (MidoriContextAction* self, GtkMenu* default_menu, gboolean accels) {
	GtkMenu* result = NULL;
	GtkMenu* _tmp0_ = NULL;
	GtkMenu* _tmp1_;
	GtkMenu* _tmp2_;
	GtkMenu* menu = NULL;
	GtkMenu* _tmp4_;
	GList* _tmp5_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp1_ = default_menu;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = _tmp2_;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	if (_tmp0_ == NULL) {
#line 748 "midori-contextaction.c"
		GtkMenu* _tmp3_;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_tmp3_ = (GtkMenu*) gtk_menu_new ();
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		g_object_ref_sink (_tmp3_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_g_object_unref0 (_tmp0_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		_tmp0_ = _tmp3_;
#line 758 "midori-contextaction.c"
	}
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp4_ = _g_object_ref0 (_tmp0_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	menu = _tmp4_;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp5_ = self->priv->children;
#line 766 "midori-contextaction.c"
	{
		GList* action_collection = NULL;
		GList* action_it = NULL;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		action_collection = _tmp5_;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		for (action_it = action_collection; action_it != NULL; action_it = action_it->next) {
#line 774 "midori-contextaction.c"
			GtkAction* _tmp6_;
			GtkAction* action = NULL;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			_tmp6_ = _g_object_ref0 ((GtkAction*) action_it->data);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			action = _tmp6_;
#line 781 "midori-contextaction.c"
			{
				GtkMenuItem* menuitem = NULL;
				GtkAction* _tmp7_;
				gboolean _tmp26_;
				GtkMenu* _tmp32_;
				GtkMenuItem* _tmp33_;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp7_ = action;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION)) {
#line 792 "midori-contextaction.c"
					GtkSeparatorMenuItem* _tmp8_;
					GtkMenuItem* _tmp9_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp8_ = (GtkSeparatorMenuItem*) gtk_separator_menu_item_new ();
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					g_object_ref_sink (_tmp8_);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_g_object_unref0 (menuitem);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					menuitem = (GtkMenuItem*) _tmp8_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp9_ = menuitem;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					gtk_widget_show ((GtkWidget*) _tmp9_);
#line 807 "midori-contextaction.c"
				} else {
					gboolean _tmp10_ = FALSE;
					GtkAction* _tmp11_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp11_ = action;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp11_, MIDORI_TYPE_CONTEXT_ACTION)) {
#line 815 "midori-contextaction.c"
						GtkAction* _tmp12_;
						GList* _tmp13_;
						gconstpointer _tmp14_;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp12_ = action;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp13_ = (G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, MIDORI_TYPE_CONTEXT_ACTION) ? ((MidoriContextAction*) _tmp12_) : NULL)->priv->children;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp14_ = g_list_nth_data (_tmp13_, (guint) 0);
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp10_ = ((GtkAction*) _tmp14_) != NULL;
#line 827 "midori-contextaction.c"
					} else {
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp10_ = FALSE;
#line 831 "midori-contextaction.c"
					}
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					if (_tmp10_) {
#line 835 "midori-contextaction.c"
						GtkAction* _tmp15_;
						GtkWidget* _tmp16_;
						GtkMenuItem* _tmp17_;
						GtkMenuItem* _tmp18_;
						GtkAction* _tmp19_;
						gboolean _tmp20_;
						GtkMenu* _tmp21_;
						GtkMenu* _tmp22_;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp15_ = action;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp16_ = gtk_action_create_menu_item (_tmp15_);
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp17_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp16_, GTK_TYPE_MENU_ITEM) ? ((GtkMenuItem*) _tmp16_) : NULL);
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_g_object_unref0 (menuitem);
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						menuitem = _tmp17_;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp18_ = menuitem;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp19_ = action;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp20_ = accels;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp21_ = midori_context_action_create_menu (G_TYPE_CHECK_INSTANCE_TYPE (_tmp19_, MIDORI_TYPE_CONTEXT_ACTION) ? ((MidoriContextAction*) _tmp19_) : NULL, NULL, _tmp20_);
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp22_ = _tmp21_;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						gtk_menu_item_set_submenu (_tmp18_, _tmp22_);
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_g_object_unref0 (_tmp22_);
#line 868 "midori-contextaction.c"
					} else {
						GtkAction* _tmp23_;
						GtkWidget* _tmp24_;
						GtkMenuItem* _tmp25_;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp23_ = action;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp24_ = gtk_action_create_menu_item (_tmp23_);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp25_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp24_, GTK_TYPE_MENU_ITEM) ? ((GtkMenuItem*) _tmp24_) : NULL);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_g_object_unref0 (menuitem);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						menuitem = _tmp25_;
#line 883 "midori-contextaction.c"
					}
				}
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp26_ = accels;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				if (!_tmp26_) {
#line 890 "midori-contextaction.c"
					GtkAccelLabel* accel_label = NULL;
					GtkMenuItem* _tmp27_;
					GtkWidget* _tmp28_;
					GtkAccelLabel* _tmp29_;
					GtkAccelLabel* _tmp30_;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp27_ = menuitem;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp28_ = gtk_bin_get_child ((GtkBin*) _tmp27_);
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp29_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp28_, GTK_TYPE_ACCEL_LABEL) ? ((GtkAccelLabel*) _tmp28_) : NULL);
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					accel_label = _tmp29_;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_tmp30_ = accel_label;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					if (_tmp30_ != NULL) {
#line 908 "midori-contextaction.c"
						GtkAccelLabel* _tmp31_;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						_tmp31_ = accel_label;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
						g_object_set (_tmp31_, "accel-closure", NULL, NULL);
#line 914 "midori-contextaction.c"
					}
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					_g_object_unref0 (accel_label);
#line 918 "midori-contextaction.c"
				}
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp32_ = menu;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp33_ = menuitem;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				gtk_menu_shell_append ((GtkMenuShell*) _tmp32_, (GtkWidget*) _tmp33_);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_g_object_unref0 (menuitem);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_g_object_unref0 (action);
#line 930 "midori-contextaction.c"
			}
		}
	}
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	result = menu;
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_g_object_unref0 (_tmp0_);
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return result;
#line 940 "midori-contextaction.c"
}


GtkAction* midori_context_action_get_by_name (MidoriContextAction* self, const gchar* name) {
	GtkAction* result = NULL;
	GList* _tmp0_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_tmp0_ = self->priv->children;
#line 953 "midori-contextaction.c"
	{
		GList* action_collection = NULL;
		GList* action_it = NULL;
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		action_collection = _tmp0_;
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
		for (action_it = action_collection; action_it != NULL; action_it = action_it->next) {
#line 961 "midori-contextaction.c"
			GtkAction* _tmp1_;
			GtkAction* action = NULL;
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			_tmp1_ = _g_object_ref0 ((GtkAction*) action_it->data);
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
			action = _tmp1_;
#line 968 "midori-contextaction.c"
			{
				GtkAction* _tmp2_;
				const gchar* _tmp3_;
				const gchar* _tmp4_;
				const gchar* _tmp5_;
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp2_ = action;
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp3_ = gtk_action_get_name (_tmp2_);
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp4_ = _tmp3_;
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_tmp5_ = name;
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				if (g_strcmp0 (_tmp4_, _tmp5_) == 0) {
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					result = action;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
					return result;
#line 988 "midori-contextaction.c"
				}
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
				_g_object_unref0 (action);
#line 992 "midori-contextaction.c"
			}
		}
	}
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	result = NULL;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return result;
#line 1000 "midori-contextaction.c"
}


static void midori_context_action_class_init (MidoriContextActionClass * klass) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	midori_context_action_parent_class = g_type_class_peek_parent (klass);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	g_type_class_add_private (klass, sizeof (MidoriContextActionPrivate));
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	((GtkActionClass *) klass)->create_tool_item = (GtkWidget* (*) (GtkAction *)) midori_context_action_real_create_tool_item;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	G_OBJECT_CLASS (klass)->finalize = midori_context_action_finalize;
#line 1013 "midori-contextaction.c"
}


static void midori_context_action_instance_init (MidoriContextAction * self) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self->priv = MIDORI_CONTEXT_ACTION_GET_PRIVATE (self);
#line 1020 "midori-contextaction.c"
}


static void midori_context_action_finalize (GObject * obj) {
	MidoriContextAction * self;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MIDORI_TYPE_CONTEXT_ACTION, MidoriContextAction);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->action_groups);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	__g_list_free__g_object_unref0_0 (self->priv->children);
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	_g_object_unref0 (self->priv->toolitem);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	G_OBJECT_CLASS (midori_context_action_parent_class)->finalize (obj);
#line 1036 "midori-contextaction.c"
}


GType midori_context_action_get_type (void) {
	static volatile gsize midori_context_action_type_id__volatile = 0;
	if (g_once_init_enter (&midori_context_action_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriContextActionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_context_action_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriContextAction), 0, (GInstanceInitFunc) midori_context_action_instance_init, NULL };
		GType midori_context_action_type_id;
		midori_context_action_type_id = g_type_register_static (GTK_TYPE_ACTION, "MidoriContextAction", &g_define_type_info, 0);
		g_once_init_leave (&midori_context_action_type_id__volatile, midori_context_action_type_id);
	}
	return midori_context_action_type_id__volatile;
}


MidoriSeparatorContextAction* midori_separator_context_action_construct (GType object_type) {
	MidoriSeparatorContextAction * self = NULL;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	self = (MidoriSeparatorContextAction*) g_object_new (object_type, "name", "SeparatorContextAction", "label", NULL, "tooltip", NULL, "stock-id", NULL, NULL);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return self;
#line 1058 "midori-contextaction.c"
}


MidoriSeparatorContextAction* midori_separator_context_action_new (void) {
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	return midori_separator_context_action_construct (MIDORI_TYPE_SEPARATOR_CONTEXT_ACTION);
#line 1065 "midori-contextaction.c"
}


static void midori_separator_context_action_class_init (MidoriSeparatorContextActionClass * klass) {
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-contextaction.vala"
	midori_separator_context_action_parent_class = g_type_class_peek_parent (klass);
#line 1072 "midori-contextaction.c"
}


static void midori_separator_context_action_instance_init (MidoriSeparatorContextAction * self) {
}


GType midori_separator_context_action_get_type (void) {
	static volatile gsize midori_separator_context_action_type_id__volatile = 0;
	if (g_once_init_enter (&midori_separator_context_action_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriSeparatorContextActionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_separator_context_action_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriSeparatorContextAction), 0, (GInstanceInitFunc) midori_separator_context_action_instance_init, NULL };
		GType midori_separator_context_action_type_id;
		midori_separator_context_action_type_id = g_type_register_static (MIDORI_TYPE_CONTEXT_ACTION, "MidoriSeparatorContextAction", &g_define_type_info, 0);
		g_once_init_leave (&midori_separator_context_action_type_id__volatile, midori_separator_context_action_type_id);
	}
	return midori_separator_context_action_type_id__volatile;
}



