/* midori-panedaction.c generated by valac 0.38.8, the Vala compiler
 * generated from midori-panedaction.vala, do not modify */

/*
 Copyright (C) 2011 Peter Hatina <phatina@redhat.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>


#define MIDORI_TYPE_PANED_ACTION (midori_paned_action_get_type ())
#define MIDORI_PANED_ACTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_PANED_ACTION, MidoriPanedAction))
#define MIDORI_PANED_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_TYPE_PANED_ACTION, MidoriPanedActionClass))
#define MIDORI_IS_PANED_ACTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_PANED_ACTION))
#define MIDORI_IS_PANED_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_TYPE_PANED_ACTION))
#define MIDORI_PANED_ACTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_TYPE_PANED_ACTION, MidoriPanedActionClass))

typedef struct _MidoriPanedAction MidoriPanedAction;
typedef struct _MidoriPanedActionClass MidoriPanedActionClass;
typedef struct _MidoriPanedActionPrivate MidoriPanedActionPrivate;

#define MIDORI_PANED_ACTION_TYPE_CHILD (midori_paned_action_child_get_type ())
typedef struct _MidoriPanedActionChild MidoriPanedActionChild;
enum  {
	MIDORI_PANED_ACTION_0_PROPERTY,
	MIDORI_PANED_ACTION_NUM_PROPERTIES
};
static GParamSpec* midori_paned_action_properties[MIDORI_PANED_ACTION_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _MidoriPanedAction {
	GtkAction parent_instance;
	MidoriPanedActionPrivate * priv;
};

struct _MidoriPanedActionClass {
	GtkActionClass parent_class;
};

struct _MidoriPanedActionChild {
	GtkWidget* widget;
	gchar* name;
	gboolean resize;
	gboolean shrink;
};

struct _MidoriPanedActionPrivate {
	GtkHPaned* hpaned;
	GtkToolItem* toolitem;
	MidoriPanedActionChild child1;
	MidoriPanedActionChild child2;
};


static gpointer midori_paned_action_parent_class = NULL;

GType midori_paned_action_get_type (void) G_GNUC_CONST;
static GType midori_paned_action_child_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static MidoriPanedActionChild* midori_paned_action_child_dup (const MidoriPanedActionChild* self);
static void midori_paned_action_child_free (MidoriPanedActionChild* self);
static void midori_paned_action_child_copy (const MidoriPanedActionChild* self, MidoriPanedActionChild* dest);
static void midori_paned_action_child_destroy (MidoriPanedActionChild* self);
#define MIDORI_PANED_ACTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MIDORI_TYPE_PANED_ACTION, MidoriPanedActionPrivate))
MidoriPanedAction* midori_paned_action_new (void);
MidoriPanedAction* midori_paned_action_construct (GType object_type);
static GtkWidget* midori_paned_action_real_create_tool_item (GtkAction* base);
void midori_paned_action_set_child1 (MidoriPanedAction* self, GtkWidget* widget, const gchar* name, gboolean resize, gboolean shrink);
void midori_paned_action_set_child2 (MidoriPanedAction* self, GtkWidget* widget, const gchar* name, gboolean resize, gboolean shrink);
GtkWidget* midori_paned_action_get_child1 (MidoriPanedAction* self);
GtkWidget* midori_paned_action_get_child2 (MidoriPanedAction* self);
GtkWidget* midori_paned_action_get_child_by_name (MidoriPanedAction* self, const gchar* name);
gchar* midori_paned_action_get_child1_name (MidoriPanedAction* self);
gchar* midori_paned_action_get_child2_name (MidoriPanedAction* self);
static void midori_paned_action_finalize (GObject * obj);


MidoriPanedAction* midori_paned_action_construct (GType object_type) {
	MidoriPanedAction * self = NULL;
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self = (MidoriPanedAction*) g_object_new (object_type, "name", "LocationSearch", NULL);
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return self;
#line 96 "midori-panedaction.c"
}


MidoriPanedAction* midori_paned_action_new (void) {
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return midori_paned_action_construct (MIDORI_TYPE_PANED_ACTION);
#line 103 "midori-panedaction.c"
}


static GtkWidget* midori_paned_action_real_create_tool_item (GtkAction* base) {
	MidoriPanedAction * self;
	GtkWidget* result = NULL;
	GtkAlignment* alignment = NULL;
	GtkAlignment* _tmp0_;
	GtkHPaned* _tmp1_;
	GtkToolItem* _tmp2_;
	GtkToolItem* _tmp3_;
	GtkToolItem* _tmp4_;
	GtkHPaned* _tmp5_;
	GtkHPaned* _tmp6_;
	MidoriPanedActionChild _tmp7_;
	GtkWidget* _tmp8_;
	MidoriPanedActionChild _tmp9_;
	gboolean _tmp10_;
	MidoriPanedActionChild _tmp11_;
	gboolean _tmp12_;
	GtkHPaned* _tmp13_;
	MidoriPanedActionChild _tmp14_;
	GtkWidget* _tmp15_;
	MidoriPanedActionChild _tmp16_;
	gboolean _tmp17_;
	MidoriPanedActionChild _tmp18_;
	gboolean _tmp19_;
	GtkToolItem* _tmp20_;
	GtkToolItem* _tmp21_;
#line 30 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self = (MidoriPanedAction*) base;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = (GtkAlignment*) gtk_alignment_new (0.0f, 0.5f, 1.0f, 0.1f);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_object_ref_sink (_tmp0_);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	alignment = _tmp0_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = (GtkHPaned*) gtk_hpaned_new ();
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_object_ref_sink (_tmp1_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->hpaned);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->hpaned = _tmp1_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = gtk_tool_item_new ();
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_object_ref_sink (_tmp2_);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->toolitem);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->toolitem = _tmp2_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp3_ = self->priv->toolitem;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_tool_item_set_expand (_tmp3_, TRUE);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp4_ = self->priv->toolitem;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_container_add ((GtkContainer*) _tmp4_, (GtkWidget*) alignment);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp5_ = self->priv->hpaned;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_container_add ((GtkContainer*) alignment, (GtkWidget*) _tmp5_);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp6_ = self->priv->hpaned;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp7_ = self->priv->child1;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp8_ = _tmp7_.widget;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp9_ = self->priv->child1;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp10_ = _tmp9_.resize;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp11_ = self->priv->child1;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp12_ = _tmp11_.shrink;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_paned_pack1 ((GtkPaned*) _tmp6_, _tmp8_, _tmp10_, _tmp12_);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp13_ = self->priv->hpaned;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp14_ = self->priv->child2;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp15_ = _tmp14_.widget;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp16_ = self->priv->child2;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp17_ = _tmp16_.resize;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp18_ = self->priv->child2;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp19_ = _tmp18_.shrink;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_paned_pack2 ((GtkPaned*) _tmp13_, _tmp15_, _tmp17_, _tmp19_);
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp20_ = self->priv->toolitem;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp20_);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp21_ = self->priv->toolitem;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = (GtkWidget*) _tmp21_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (alignment);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 213 "midori-panedaction.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return self ? g_object_ref (self) : NULL;
#line 220 "midori-panedaction.c"
}


void midori_paned_action_set_child1 (MidoriPanedAction* self, GtkWidget* widget, const gchar* name, gboolean resize, gboolean shrink) {
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (self != NULL);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (widget != NULL);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (name != NULL);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = widget;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->child1.widget);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child1.widget = _tmp1_;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = name;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_free0 (self->priv->child1.name);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child1.name = _tmp3_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp4_ = resize;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child1.resize = _tmp4_;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp5_ = shrink;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child1.shrink = _tmp5_;
#line 261 "midori-panedaction.c"
}


void midori_paned_action_set_child2 (MidoriPanedAction* self, GtkWidget* widget, const gchar* name, gboolean resize, gboolean shrink) {
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (self != NULL);
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (widget != NULL);
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_if_fail (name != NULL);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = widget;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->child2.widget);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child2.widget = _tmp1_;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = name;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_free0 (self->priv->child2.name);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child2.name = _tmp3_;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp4_ = resize;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child2.resize = _tmp4_;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp5_ = shrink;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->child2.shrink = _tmp5_;
#line 302 "midori-panedaction.c"
}


GtkWidget* midori_paned_action_get_child1 (MidoriPanedAction* self) {
	GtkWidget* result = NULL;
	MidoriPanedActionChild _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = self->priv->child1;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _tmp0_.widget;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = _tmp2_;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 323 "midori-panedaction.c"
}


GtkWidget* midori_paned_action_get_child2 (MidoriPanedAction* self) {
	GtkWidget* result = NULL;
	MidoriPanedActionChild _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = self->priv->child2;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _tmp0_.widget;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = _tmp2_;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 344 "midori-panedaction.c"
}


GtkWidget* midori_paned_action_get_child_by_name (MidoriPanedAction* self, const gchar* name) {
	GtkWidget* result = NULL;
	const gchar* _tmp0_;
	MidoriPanedActionChild _tmp1_;
	const gchar* _tmp2_;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = name;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = self->priv->child1;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = _tmp1_.name;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	if (g_strcmp0 (_tmp0_, _tmp2_) == 0) {
#line 365 "midori-panedaction.c"
		MidoriPanedActionChild _tmp3_;
		GtkWidget* _tmp4_;
		GtkWidget* _tmp5_;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp3_ = self->priv->child1;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp4_ = _tmp3_.widget;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp5_ = _g_object_ref0 (_tmp4_);
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		result = _tmp5_;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		return result;
#line 379 "midori-panedaction.c"
	} else {
		const gchar* _tmp6_;
		MidoriPanedActionChild _tmp7_;
		const gchar* _tmp8_;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp6_ = name;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp7_ = self->priv->child2;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		_tmp8_ = _tmp7_.name;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
		if (g_strcmp0 (_tmp6_, _tmp8_) == 0) {
#line 392 "midori-panedaction.c"
			MidoriPanedActionChild _tmp9_;
			GtkWidget* _tmp10_;
			GtkWidget* _tmp11_;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
			_tmp9_ = self->priv->child2;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
			_tmp10_ = _tmp9_.widget;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
			_tmp11_ = _g_object_ref0 (_tmp10_);
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
			result = _tmp11_;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
			return result;
#line 406 "midori-panedaction.c"
		}
	}
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = NULL;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 413 "midori-panedaction.c"
}


gchar* midori_paned_action_get_child1_name (MidoriPanedAction* self) {
	gchar* result = NULL;
	MidoriPanedActionChild _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = self->priv->child1;
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _tmp0_.name;
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = _tmp2_;
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 434 "midori-panedaction.c"
}


gchar* midori_paned_action_get_child2_name (MidoriPanedAction* self) {
	gchar* result = NULL;
	MidoriPanedActionChild _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = self->priv->child2;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _tmp0_.name;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = g_strdup (_tmp1_);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	result = _tmp2_;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return result;
#line 455 "midori-panedaction.c"
}


static void midori_paned_action_child_copy (const MidoriPanedActionChild* self, MidoriPanedActionChild* dest) {
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gboolean _tmp4_;
	gboolean _tmp5_;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp0_ = (*self).widget;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 ((*dest).widget);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	(*dest).widget = _tmp1_;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp2_ = (*self).name;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_free0 ((*dest).name);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	(*dest).name = _tmp3_;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp4_ = (*self).resize;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	(*dest).resize = _tmp4_;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_tmp5_ = (*self).shrink;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	(*dest).shrink = _tmp5_;
#line 490 "midori-panedaction.c"
}


static void midori_paned_action_child_destroy (MidoriPanedActionChild* self) {
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 ((*self).widget);
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_free0 ((*self).name);
#line 499 "midori-panedaction.c"
}


static MidoriPanedActionChild* midori_paned_action_child_dup (const MidoriPanedActionChild* self) {
	MidoriPanedActionChild* dup;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	dup = g_new0 (MidoriPanedActionChild, 1);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	midori_paned_action_child_copy (self, dup);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	return dup;
#line 511 "midori-panedaction.c"
}


static void midori_paned_action_child_free (MidoriPanedActionChild* self) {
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	midori_paned_action_child_destroy (self);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_free (self);
#line 520 "midori-panedaction.c"
}


static GType midori_paned_action_child_get_type (void) {
	static volatile gsize midori_paned_action_child_type_id__volatile = 0;
	if (g_once_init_enter (&midori_paned_action_child_type_id__volatile)) {
		GType midori_paned_action_child_type_id;
		midori_paned_action_child_type_id = g_boxed_type_register_static ("MidoriPanedActionChild", (GBoxedCopyFunc) midori_paned_action_child_dup, (GBoxedFreeFunc) midori_paned_action_child_free);
		g_once_init_leave (&midori_paned_action_child_type_id__volatile, midori_paned_action_child_type_id);
	}
	return midori_paned_action_child_type_id__volatile;
}


static void midori_paned_action_class_init (MidoriPanedActionClass * klass) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	midori_paned_action_parent_class = g_type_class_peek_parent (klass);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	g_type_class_add_private (klass, sizeof (MidoriPanedActionPrivate));
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	((GtkActionClass *) klass)->create_tool_item = (GtkWidget* (*) (GtkAction *)) midori_paned_action_real_create_tool_item;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	G_OBJECT_CLASS (klass)->finalize = midori_paned_action_finalize;
#line 544 "midori-panedaction.c"
}


static void midori_paned_action_instance_init (MidoriPanedAction * self) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv = MIDORI_PANED_ACTION_GET_PRIVATE (self);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->hpaned = NULL;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self->priv->toolitem = NULL;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	memset (&self->priv->child1, 0, sizeof (MidoriPanedActionChild));
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	memset (&self->priv->child2, 0, sizeof (MidoriPanedActionChild));
#line 559 "midori-panedaction.c"
}


static void midori_paned_action_finalize (GObject * obj) {
	MidoriPanedAction * self;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MIDORI_TYPE_PANED_ACTION, MidoriPanedAction);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->hpaned);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	_g_object_unref0 (self->priv->toolitem);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	midori_paned_action_child_destroy (&self->priv->child1);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	midori_paned_action_child_destroy (&self->priv->child2);
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-panedaction.vala"
	G_OBJECT_CLASS (midori_paned_action_parent_class)->finalize (obj);
#line 577 "midori-panedaction.c"
}


GType midori_paned_action_get_type (void) {
	static volatile gsize midori_paned_action_type_id__volatile = 0;
	if (g_once_init_enter (&midori_paned_action_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriPanedActionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_paned_action_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriPanedAction), 0, (GInstanceInitFunc) midori_paned_action_instance_init, NULL };
		GType midori_paned_action_type_id;
		midori_paned_action_type_id = g_type_register_static (GTK_TYPE_ACTION, "MidoriPanedAction", &g_define_type_info, 0);
		g_once_init_leave (&midori_paned_action_type_id__volatile, midori_paned_action_type_id);
	}
	return midori_paned_action_type_id__volatile;
}



