/* midori-extensions-column.c generated by valac 0.38.8, the Vala compiler
 * generated from midori-extensions-column.vala, do not modify */

/*
   Copyright (C) 2012 André Stösel <andre@stoesel.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>


#define MIDORI_TYPE_EXTENSIONS_COLUMN (midori_extensions_column_get_type ())
#define MIDORI_EXTENSIONS_COLUMN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_EXTENSIONS_COLUMN, MidoriExtensionsColumn))
#define MIDORI_EXTENSIONS_COLUMN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_TYPE_EXTENSIONS_COLUMN, MidoriExtensionsColumnClass))
#define MIDORI_IS_EXTENSIONS_COLUMN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_EXTENSIONS_COLUMN))
#define MIDORI_IS_EXTENSIONS_COLUMN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_TYPE_EXTENSIONS_COLUMN))
#define MIDORI_EXTENSIONS_COLUMN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_TYPE_EXTENSIONS_COLUMN, MidoriExtensionsColumnClass))

typedef struct _MidoriExtensionsColumn MidoriExtensionsColumn;
typedef struct _MidoriExtensionsColumnClass MidoriExtensionsColumnClass;
typedef struct _MidoriExtensionsColumnPrivate MidoriExtensionsColumnPrivate;
enum  {
	MIDORI_EXTENSIONS_COLUMN_0_PROPERTY,
	MIDORI_EXTENSIONS_COLUMN_NUM_PROPERTIES
};
static GParamSpec* midori_extensions_column_properties[MIDORI_EXTENSIONS_COLUMN_NUM_PROPERTIES];
enum  {
	MIDORI_EXTENSIONS_COLUMN_ROW_CLICKED_SIGNAL,
	MIDORI_EXTENSIONS_COLUMN_NUM_SIGNALS
};
static guint midori_extensions_column_signals[MIDORI_EXTENSIONS_COLUMN_NUM_SIGNALS] = {0};

struct _MidoriExtensionsColumn {
	GtkTreeViewColumn parent_instance;
	MidoriExtensionsColumnPrivate * priv;
};

struct _MidoriExtensionsColumnClass {
	GtkTreeViewColumnClass parent_class;
};


static gpointer midori_extensions_column_parent_class = NULL;

GType midori_extensions_column_get_type (void) G_GNUC_CONST;
MidoriExtensionsColumn* midori_extensions_column_new (void);
MidoriExtensionsColumn* midori_extensions_column_construct (GType object_type);
static void g_cclosure_user_marshal_VOID__OBJECT_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);


MidoriExtensionsColumn* midori_extensions_column_construct (GType object_type) {
	MidoriExtensionsColumn * self = NULL;
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	self = (MidoriExtensionsColumn*) g_object_new (object_type, NULL);
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	return self;
#line 66 "midori-extensions-column.c"
}


MidoriExtensionsColumn* midori_extensions_column_new (void) {
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	return midori_extensions_column_construct (MIDORI_TYPE_EXTENSIONS_COLUMN);
#line 73 "midori-extensions-column.c"
}


static void g_cclosure_user_marshal_VOID__OBJECT_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__OBJECT_BOXED) (gpointer data1, gpointer arg_1, gpointer arg_2, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_BOXED callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	g_return_if_fail (n_param_values == 3);
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
		data1 = closure->data;
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
		data2 = param_values->data[0].v_pointer;
#line 92 "midori-extensions-column.c"
	} else {
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
		data1 = param_values->data[0].v_pointer;
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
		data2 = closure->data;
#line 98 "midori-extensions-column.c"
	}
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	callback = (GMarshalFunc_VOID__OBJECT_BOXED) (marshal_data ? marshal_data : cc->callback);
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	callback (data1, g_value_get_object (param_values + 1), g_value_get_boxed (param_values + 2), data2);
#line 104 "midori-extensions-column.c"
}


static void midori_extensions_column_class_init (MidoriExtensionsColumnClass * klass) {
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	midori_extensions_column_parent_class = g_type_class_peek_parent (klass);
#line 12 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-extensions-column.vala"
	midori_extensions_column_signals[MIDORI_EXTENSIONS_COLUMN_ROW_CLICKED_SIGNAL] = g_signal_new ("row-clicked", MIDORI_TYPE_EXTENSIONS_COLUMN, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_BOXED, G_TYPE_NONE, 2, GTK_TYPE_TREE_VIEW, gtk_tree_path_get_type ());
#line 113 "midori-extensions-column.c"
}


static void midori_extensions_column_instance_init (MidoriExtensionsColumn * self) {
}


GType midori_extensions_column_get_type (void) {
	static volatile gsize midori_extensions_column_type_id__volatile = 0;
	if (g_once_init_enter (&midori_extensions_column_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriExtensionsColumnClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_extensions_column_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriExtensionsColumn), 0, (GInstanceInitFunc) midori_extensions_column_instance_init, NULL };
		GType midori_extensions_column_type_id;
		midori_extensions_column_type_id = g_type_register_static (GTK_TYPE_TREE_VIEW_COLUMN, "MidoriExtensionsColumn", &g_define_type_info, 0);
		g_once_init_leave (&midori_extensions_column_type_id__volatile, midori_extensions_column_type_id);
	}
	return midori_extensions_column_type_id__volatile;
}



