/* midori-viewable.c generated by valac 0.38.8, the Vala compiler
 * generated from midori-viewable.vala, do not modify */

/*
 Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#define MIDORI_TYPE_VIEWABLE (midori_viewable_get_type ())
#define MIDORI_VIEWABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_VIEWABLE, MidoriViewable))
#define MIDORI_IS_VIEWABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_VIEWABLE))
#define MIDORI_VIEWABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), MIDORI_TYPE_VIEWABLE, MidoriViewableIface))

typedef struct _MidoriViewable MidoriViewable;
typedef struct _MidoriViewableIface MidoriViewableIface;
enum  {
	MIDORI_VIEWABLE_POPULATE_OPTION_MENU_SIGNAL,
	MIDORI_VIEWABLE_NUM_SIGNALS
};
static guint midori_viewable_signals[MIDORI_VIEWABLE_NUM_SIGNALS] = {0};

struct _MidoriViewableIface {
	GTypeInterface parent_iface;
	const gchar* (*get_stock_id) (MidoriViewable* self);
	const gchar* (*get_label) (MidoriViewable* self);
	GtkWidget* (*get_toolbar) (MidoriViewable* self);
};



GType midori_viewable_get_type (void) G_GNUC_CONST;
const gchar* midori_viewable_get_stock_id (MidoriViewable* self);
const gchar* midori_viewable_get_label (MidoriViewable* self);
GtkWidget* midori_viewable_get_toolbar (MidoriViewable* self);


const gchar* midori_viewable_get_stock_id (MidoriViewable* self) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	return MIDORI_VIEWABLE_GET_INTERFACE (self)->get_stock_id (self);
#line 56 "midori-viewable.c"
}


const gchar* midori_viewable_get_label (MidoriViewable* self) {
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	return MIDORI_VIEWABLE_GET_INTERFACE (self)->get_label (self);
#line 65 "midori-viewable.c"
}


GtkWidget* midori_viewable_get_toolbar (MidoriViewable* self) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	return MIDORI_VIEWABLE_GET_INTERFACE (self)->get_toolbar (self);
#line 74 "midori-viewable.c"
}


static void midori_viewable_base_init (MidoriViewableIface * iface) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	static gboolean initialized = FALSE;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
	if (!initialized) {
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
		initialized = TRUE;
#line 13 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-viewable.vala"
		midori_viewable_signals[MIDORI_VIEWABLE_POPULATE_OPTION_MENU_SIGNAL] = g_signal_new ("populate-option-menu", MIDORI_TYPE_VIEWABLE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GTK_TYPE_MENU);
#line 87 "midori-viewable.c"
	}
}


GType midori_viewable_get_type (void) {
	static volatile gsize midori_viewable_type_id__volatile = 0;
	if (g_once_init_enter (&midori_viewable_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriViewableIface), (GBaseInitFunc) midori_viewable_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType midori_viewable_type_id;
		midori_viewable_type_id = g_type_register_static (G_TYPE_INTERFACE, "MidoriViewable", &g_define_type_info, 0);
		g_once_init_leave (&midori_viewable_type_id__volatile, midori_viewable_type_id);
	}
	return midori_viewable_type_id__volatile;
}



