/* updater.c generated by valac 0.38.8, the Vala compiler
 * generated from updater.vala, do not modify */

/*
 Copyright (C) 2014 Paweł Forysiuk <tuxator@o2.pl>
 Copyright (C) 2014 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include "toolbar-editor.h"
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>

enum  {
	ADBLOCK_UPDATER_0_PROPERTY,
	ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY,
	ADBLOCK_UPDATER_EXPIRES_PROPERTY,
	ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY,
	ADBLOCK_UPDATER_NUM_PROPERTIES
};
static GParamSpec* adblock_updater_properties[ADBLOCK_UPDATER_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_date_time_unref0(var) ((var == NULL) ? NULL : (var = (g_date_time_unref (var), NULL)))

struct _AdblockUpdaterPrivate {
	gchar* expires_meta;
	gchar* last_mod_meta;
	GDateTime* _last_updated;
	GDateTime* _expires;
	gboolean _needs_update;
};


static gpointer adblock_updater_parent_class = NULL;

#define ADBLOCK_UPDATER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ADBLOCK_TYPE_UPDATER, AdblockUpdaterPrivate))
static void adblock_updater_real_clear (AdblockFeature* base);
static gboolean adblock_updater_real_header (AdblockFeature* base, const gchar* key, const gchar* value);
static gboolean adblock_updater_real_parsed (AdblockFeature* base, GFile* file);
static void adblock_updater_process_dates (AdblockUpdater* self, GFile* file);
static gint adblock_updater_get_month_from_string (AdblockUpdater* self, const gchar* month);
static void adblock_updater_finalize (GObject * obj);
static void _vala_adblock_updater_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_adblock_updater_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


AdblockUpdater* adblock_updater_construct (GType object_type) {
	AdblockUpdater * self = NULL;
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self = (AdblockUpdater*) adblock_feature_construct (object_type);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return self;
#line 66 "updater.c"
}


AdblockUpdater* adblock_updater_new (void) {
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return adblock_updater_construct (ADBLOCK_TYPE_UPDATER);
#line 73 "updater.c"
}


static void adblock_updater_real_clear (AdblockFeature* base) {
	AdblockUpdater * self;
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self = (AdblockUpdater*) base;
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_free0 (self->priv->expires_meta);
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self->priv->expires_meta = NULL;
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_free0 (self->priv->last_mod_meta);
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self->priv->last_mod_meta = NULL;
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_last_updated (self, NULL);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_expires (self, NULL);
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_needs_update (self, FALSE);
#line 95 "updater.c"
}


static glong string_strnlen (gchar* str, glong maxlen) {
	glong result = 0L;
	gchar* end = NULL;
	gchar* _tmp0_;
	glong _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = str;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = maxlen;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = memchr (_tmp0_, 0, (gsize) _tmp1_);
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	end = _tmp2_;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = end;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp3_ == NULL) {
#line 118 "updater.c"
		glong _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = maxlen;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 126 "updater.c"
	} else {
		gchar* _tmp5_;
		gchar* _tmp6_;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = end;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = str;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = (glong) (_tmp5_ - _tmp6_);
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 138 "updater.c"
	}
}


static gchar* string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	glong _tmp1_;
	glong _tmp8_;
	glong _tmp14_;
	glong _tmp17_;
	glong _tmp18_;
	glong _tmp19_;
	glong _tmp20_;
	glong _tmp21_;
	gchar* _tmp22_;
#line 1341 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = offset;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_ >= ((glong) 0)) {
#line 162 "updater.c"
		glong _tmp2_;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = len;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = _tmp2_ >= ((glong) 0);
#line 168 "updater.c"
	} else {
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = FALSE;
#line 172 "updater.c"
	}
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 176 "updater.c"
		glong _tmp3_;
		glong _tmp4_;
		glong _tmp5_;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = offset;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = len;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = string_strnlen ((gchar*) self, _tmp3_ + _tmp4_);
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = _tmp5_;
#line 188 "updater.c"
	} else {
		gint _tmp6_;
		gint _tmp7_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = strlen (self);
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = _tmp6_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = (glong) _tmp7_;
#line 198 "updater.c"
	}
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp8_ = offset;
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp8_ < ((glong) 0)) {
#line 204 "updater.c"
		glong _tmp9_;
		glong _tmp10_;
		glong _tmp11_;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = string_length;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = offset;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		offset = _tmp9_ + _tmp10_;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = offset;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp11_ >= ((glong) 0), NULL);
#line 218 "updater.c"
	} else {
		glong _tmp12_;
		glong _tmp13_;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = offset;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = string_length;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp12_ <= _tmp13_, NULL);
#line 228 "updater.c"
	}
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp14_ = len;
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp14_ < ((glong) 0)) {
#line 234 "updater.c"
		glong _tmp15_;
		glong _tmp16_;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = string_length;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp16_ = offset;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		len = _tmp15_ - _tmp16_;
#line 243 "updater.c"
	}
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp17_ = offset;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp18_ = len;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp19_ = string_length;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail ((_tmp17_ + _tmp18_) <= _tmp19_, NULL);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp20_ = offset;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp21_ = len;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp22_ = g_strndup (((gchar*) self) + _tmp20_, (gsize) _tmp21_);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp22_;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 263 "updater.c"
}


static gboolean adblock_updater_real_header (AdblockFeature* base, const gchar* key, const gchar* value) {
	AdblockUpdater * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	const gchar* _tmp1_;
	gboolean _tmp2_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self = (AdblockUpdater*) base;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (key != NULL, FALSE);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (value != NULL, FALSE);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp1_ = key;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp2_ = g_str_has_prefix (_tmp1_, "Last mod");
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp2_) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp0_ = TRUE;
#line 287 "updater.c"
	} else {
		const gchar* _tmp3_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp3_ = key;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp0_ = g_strcmp0 (_tmp3_, "Updated") == 0;
#line 294 "updater.c"
	}
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp0_) {
#line 298 "updater.c"
		const gchar* _tmp4_;
		gchar* _tmp5_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp4_ = value;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp5_ = g_strdup (_tmp4_);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_g_free0 (self->priv->last_mod_meta);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		self->priv->last_mod_meta = _tmp5_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		result = TRUE;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		return result;
#line 313 "updater.c"
	} else {
		const gchar* _tmp6_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp6_ = key;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		if (g_strcmp0 (_tmp6_, "Expires") == 0) {
#line 320 "updater.c"
			const gchar* _tmp7_;
			gchar* _tmp8_;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp7_ = value;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp8_ = g_strdup (_tmp7_);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_g_free0 (self->priv->expires_meta);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			self->priv->expires_meta = _tmp8_;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			result = TRUE;
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			return result;
#line 335 "updater.c"
		} else {
			const gchar* _tmp9_;
			gboolean _tmp10_;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp9_ = key;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp10_ = g_str_has_prefix (_tmp9_, "! This list expires after");
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp10_) {
#line 345 "updater.c"
				const gchar* _tmp11_;
				gchar* _tmp12_;
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp11_ = key;
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp12_ = string_substring (_tmp11_, (glong) 26, (glong) -1);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_g_free0 (self->priv->expires_meta);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				self->priv->expires_meta = _tmp12_;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				result = TRUE;
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				return result;
#line 360 "updater.c"
			}
		}
	}
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = FALSE;
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 368 "updater.c"
}


static gboolean adblock_updater_real_parsed (AdblockFeature* base, GFile* file) {
	AdblockUpdater * self;
	gboolean result = FALSE;
	GFile* _tmp0_;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self = (AdblockUpdater*) base;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (file != NULL, FALSE);
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = file;
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_process_dates (self, _tmp0_);
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = TRUE;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 388 "updater.c"
}


static gint adblock_updater_get_month_from_string (AdblockUpdater* self, const gchar* month) {
	gint result = 0;
	const gchar* _tmp0_;
	gchar** months = NULL;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar** _tmp13_;
	gint months_length1;
	gint _months_size_;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = month;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp0_ == NULL) {
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		result = 0;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		return result;
#line 421 "updater.c"
	}
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp1_ = g_strdup ("Jan");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp2_ = g_strdup ("Feb");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp3_ = g_strdup ("Mar");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp4_ = g_strdup ("Apr");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp5_ = g_strdup ("May");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp6_ = g_strdup ("Jun");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp7_ = g_strdup ("Jul");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp8_ = g_strdup ("Aug");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp9_ = g_strdup ("Sep");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp10_ = g_strdup ("Oct");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp11_ = g_strdup ("Nov");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp12_ = g_strdup ("Dec");
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_ = g_new0 (gchar*, 12 + 1);
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[0] = _tmp1_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[1] = _tmp2_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[2] = _tmp3_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[3] = _tmp4_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[4] = _tmp5_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[5] = _tmp6_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[6] = _tmp7_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[7] = _tmp8_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[8] = _tmp9_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[9] = _tmp10_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[10] = _tmp11_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp13_[11] = _tmp12_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	months = _tmp13_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	months_length1 = 12;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_months_size_ = months_length1;
#line 479 "updater.c"
	{
		gint i = 0;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		i = 0;
#line 484 "updater.c"
		{
			gboolean _tmp14_ = FALSE;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp14_ = TRUE;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			while (TRUE) {
#line 491 "updater.c"
				gint _tmp16_;
				gchar** _tmp17_;
				gint _tmp17__length1;
				const gchar* _tmp18_;
				gchar** _tmp19_;
				gint _tmp19__length1;
				gint _tmp20_;
				const gchar* _tmp21_;
				gboolean _tmp22_;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				if (!_tmp14_) {
#line 503 "updater.c"
					gint _tmp15_;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					_tmp15_ = i;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					i = _tmp15_ + 1;
#line 509 "updater.c"
				}
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp14_ = FALSE;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp16_ = i;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp17_ = months;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp17__length1 = months_length1;
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				if (!(_tmp16_ <= _tmp17__length1)) {
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					break;
#line 523 "updater.c"
				}
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp18_ = month;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp19_ = months;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp19__length1 = months_length1;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp20_ = i;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp21_ = _tmp19_[_tmp20_];
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp22_ = g_str_has_prefix (_tmp18_, _tmp21_);
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				if (_tmp22_) {
#line 539 "updater.c"
					gint _tmp23_;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					_tmp23_ = i;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					result = _tmp23_ + 1;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					months = (_vala_array_free (months, months_length1, (GDestroyNotify) g_free), NULL);
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					return result;
#line 549 "updater.c"
				}
			}
		}
	}
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = 0;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	months = (_vala_array_free (months, months_length1, (GDestroyNotify) g_free), NULL);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 560 "updater.c"
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	gchar* _tmp1_;
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 580 "updater.c"
}


static gchar string_get (const gchar* self, glong index) {
	gchar result = '\0';
	glong _tmp0_;
	gchar _tmp1_;
#line 1118 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, '\0');
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = index;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = ((gchar*) self)[_tmp0_];
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 598 "updater.c"
}


static void adblock_updater_process_dates (AdblockUpdater* self, GFile* file) {
	GDateTime* now = NULL;
	GDateTime* _tmp0_;
	gboolean _tmp1_ = FALSE;
	const gchar* _tmp2_;
	const gchar* _tmp112_;
	GDateTime* _tmp139_;
	GDateTime* _tmp140_;
	gint _tmp141_;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_if_fail (self != NULL);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_if_fail (file != NULL);
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = g_date_time_new_now_local ();
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	now = _tmp0_;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_last_updated (self, NULL);
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_expires (self, NULL);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp2_ = self->priv->last_mod_meta;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp2_ != NULL) {
#line 627 "updater.c"
		gboolean _tmp3_ = FALSE;
		const gchar* _tmp4_;
		gboolean _tmp5_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp4_ = self->priv->last_mod_meta;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp5_ = string_contains (_tmp4_, " ");
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		if (_tmp5_) {
#line 637 "updater.c"
			const gchar* _tmp6_;
			gchar _tmp7_;
			gboolean _tmp8_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp6_ = self->priv->last_mod_meta;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp7_ = string_get (_tmp6_, (glong) 0);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp8_ = g_ascii_isdigit (_tmp7_);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp3_ = _tmp8_ == TRUE;
#line 649 "updater.c"
		} else {
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp3_ = FALSE;
#line 653 "updater.c"
		}
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp1_ = _tmp3_;
#line 657 "updater.c"
	} else {
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp1_ = FALSE;
#line 661 "updater.c"
	}
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp1_) {
#line 665 "updater.c"
		gint h = 0;
		gint min = 0;
		gint d = 0;
		gint m = 0;
		gint y = 0;
		gboolean _tmp9_ = FALSE;
		const gchar* _tmp10_;
		gboolean _tmp11_;
		gint _tmp104_;
		gint _tmp105_;
		gint _tmp106_;
		gint _tmp107_;
		gint _tmp108_;
		GDateTime* _tmp109_;
		GDateTime* _tmp110_;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		h = 0;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		min = 0;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp10_ = self->priv->last_mod_meta;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp11_ = string_contains (_tmp10_, ".");
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		if (_tmp11_) {
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp9_ = TRUE;
#line 693 "updater.c"
		} else {
			const gchar* _tmp12_;
			gboolean _tmp13_;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp12_ = self->priv->last_mod_meta;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp13_ = string_contains (_tmp12_, "-");
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp9_ = _tmp13_;
#line 703 "updater.c"
		}
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		if (_tmp9_) {
#line 707 "updater.c"
			gchar** parts = NULL;
			const gchar* _tmp14_;
			gchar** _tmp15_;
			gchar** _tmp16_;
			gint parts_length1;
			gint _parts_size_;
			gchar** date_parts = NULL;
			gint date_parts_length1 = 0;
			gint _date_parts_size_ = 0;
			gchar* split_char = NULL;
			gchar* _tmp17_;
			gboolean _tmp18_ = FALSE;
			gchar** _tmp19_;
			gint _tmp19__length1;
			const gchar* _tmp20_;
			gchar** _tmp34_;
			gint _tmp34__length1;
			const gchar* _tmp35_;
			gboolean _tmp36_;
			gchar** _tmp42_;
			gint _tmp42__length1;
			const gchar* _tmp43_;
			const gchar* _tmp44_;
			gchar** _tmp45_;
			gchar** _tmp46_;
			gchar** _tmp47_;
			gint _tmp47__length1;
			const gchar* _tmp48_;
			gint _tmp49_;
			gchar** _tmp50_;
			gint _tmp50__length1;
			const gchar* _tmp51_;
			gint _tmp52_;
			gint _tmp53_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp14_ = self->priv->last_mod_meta;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp16_ = _tmp15_ = g_strsplit (_tmp14_, " ", 2);
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = _tmp16_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts_length1 = _vala_array_length (_tmp15_);
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_parts_size_ = parts_length1;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp17_ = g_strdup (" ");
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			split_char = _tmp17_;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp19_ = parts;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp19__length1 = parts_length1;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp20_ = _tmp19_[1];
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (g_strcmp0 (_tmp20_, "") != 0) {
#line 764 "updater.c"
				gchar** _tmp21_;
				gint _tmp21__length1;
				const gchar* _tmp22_;
				gboolean _tmp23_;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp21_ = parts;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp21__length1 = parts_length1;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp22_ = _tmp21_[1];
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp23_ = string_contains (_tmp22_, ":");
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp18_ = _tmp23_;
#line 779 "updater.c"
			} else {
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp18_ = FALSE;
#line 783 "updater.c"
			}
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp18_) {
#line 787 "updater.c"
				gchar** time_parts = NULL;
				gchar** _tmp24_;
				gint _tmp24__length1;
				const gchar* _tmp25_;
				gchar** _tmp26_;
				gchar** _tmp27_;
				gint time_parts_length1;
				gint _time_parts_size_;
				gchar** _tmp28_;
				gint _tmp28__length1;
				const gchar* _tmp29_;
				gint _tmp30_;
				gchar** _tmp31_;
				gint _tmp31__length1;
				const gchar* _tmp32_;
				gint _tmp33_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp24_ = parts;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp24__length1 = parts_length1;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp25_ = _tmp24_[1];
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp27_ = _tmp26_ = g_strsplit (_tmp25_, ":", 2);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts = _tmp27_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts_length1 = _vala_array_length (_tmp26_);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_time_parts_size_ = time_parts_length1;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp28_ = time_parts;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp28__length1 = time_parts_length1;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp29_ = _tmp28_[0];
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp30_ = atoi (_tmp29_);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				h = _tmp30_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp31_ = time_parts;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp31__length1 = time_parts_length1;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp32_ = _tmp31_[1];
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp33_ = atoi (_tmp32_);
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				min = _tmp33_;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts = (_vala_array_free (time_parts, time_parts_length1, (GDestroyNotify) g_free), NULL);
#line 840 "updater.c"
			}
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp34_ = parts;
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp34__length1 = parts_length1;
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp35_ = _tmp34_[0];
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp36_ = string_contains (_tmp35_, ".");
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp36_) {
#line 852 "updater.c"
				gchar* _tmp37_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp37_ = g_strdup (".");
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_g_free0 (split_char);
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				split_char = _tmp37_;
#line 860 "updater.c"
			} else {
				gchar** _tmp38_;
				gint _tmp38__length1;
				const gchar* _tmp39_;
				gboolean _tmp40_;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp38_ = parts;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp38__length1 = parts_length1;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp39_ = _tmp38_[0];
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp40_ = string_contains (_tmp39_, "-");
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				if (_tmp40_) {
#line 876 "updater.c"
					gchar* _tmp41_;
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					_tmp41_ = g_strdup ("-");
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					_g_free0 (split_char);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
					split_char = _tmp41_;
#line 884 "updater.c"
				}
			}
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp42_ = parts;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp42__length1 = parts_length1;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp43_ = _tmp42_[0];
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp44_ = split_char;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp46_ = _tmp45_ = g_strsplit (_tmp43_, _tmp44_, 3);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			date_parts = (_vala_array_free (date_parts, date_parts_length1, (GDestroyNotify) g_free), NULL);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			date_parts = _tmp46_;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			date_parts_length1 = _vala_array_length (_tmp45_);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_date_parts_size_ = date_parts_length1;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp47_ = date_parts;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp47__length1 = date_parts_length1;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp48_ = _tmp47_[1];
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp49_ = atoi (_tmp48_);
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			m = _tmp49_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp50_ = date_parts;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp50__length1 = date_parts_length1;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp51_ = _tmp50_[2];
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp52_ = strlen (_tmp51_);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp53_ = _tmp52_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp53_ == 4) {
#line 927 "updater.c"
				gchar** _tmp54_;
				gint _tmp54__length1;
				const gchar* _tmp55_;
				gint _tmp56_;
				gchar** _tmp57_;
				gint _tmp57__length1;
				const gchar* _tmp58_;
				gint _tmp59_;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp54_ = date_parts;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp54__length1 = date_parts_length1;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp55_ = _tmp54_[2];
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp56_ = atoi (_tmp55_);
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				y = _tmp56_;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp57_ = date_parts;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp57__length1 = date_parts_length1;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp58_ = _tmp57_[0];
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp59_ = atoi (_tmp58_);
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				d = _tmp59_;
#line 956 "updater.c"
			} else {
				gchar** _tmp60_;
				gint _tmp60__length1;
				const gchar* _tmp61_;
				gint _tmp62_;
				gchar** _tmp63_;
				gint _tmp63__length1;
				const gchar* _tmp64_;
				gint _tmp65_;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp60_ = date_parts;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp60__length1 = date_parts_length1;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp61_ = _tmp60_[0];
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp62_ = atoi (_tmp61_);
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				y = _tmp62_;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp63_ = date_parts;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp63__length1 = date_parts_length1;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp64_ = _tmp63_[2];
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp65_ = atoi (_tmp64_);
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				d = _tmp65_;
#line 986 "updater.c"
			}
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_g_free0 (split_char);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			date_parts = (_vala_array_free (date_parts, date_parts_length1, (GDestroyNotify) g_free), NULL);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 994 "updater.c"
		} else {
			gchar** parts = NULL;
			const gchar* _tmp66_;
			gchar** _tmp67_;
			gchar** _tmp68_;
			gint parts_length1;
			gint _parts_size_;
			gboolean _tmp69_ = FALSE;
			gchar** _tmp70_;
			gint _tmp70__length1;
			const gchar* _tmp71_;
			gchar** _tmp85_;
			gint _tmp85__length1;
			const gchar* _tmp86_;
			gint _tmp87_;
			gchar** _tmp88_;
			gint _tmp88__length1;
			const gchar* _tmp89_;
			gint _tmp90_;
			gint _tmp91_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp66_ = self->priv->last_mod_meta;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp68_ = _tmp67_ = g_strsplit (_tmp66_, " ", 4);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = _tmp68_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts_length1 = _vala_array_length (_tmp67_);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_parts_size_ = parts_length1;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp70_ = parts;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp70__length1 = parts_length1;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp71_ = _tmp70_[3];
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp71_ != NULL) {
#line 1033 "updater.c"
				gchar** _tmp72_;
				gint _tmp72__length1;
				const gchar* _tmp73_;
				gboolean _tmp74_;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp72_ = parts;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp72__length1 = parts_length1;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp73_ = _tmp72_[3];
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp74_ = string_contains (_tmp73_, ":");
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp69_ = _tmp74_;
#line 1048 "updater.c"
			} else {
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp69_ = FALSE;
#line 1052 "updater.c"
			}
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp69_) {
#line 1056 "updater.c"
				gchar** time_parts = NULL;
				gchar** _tmp75_;
				gint _tmp75__length1;
				const gchar* _tmp76_;
				gchar** _tmp77_;
				gchar** _tmp78_;
				gint time_parts_length1;
				gint _time_parts_size_;
				gchar** _tmp79_;
				gint _tmp79__length1;
				const gchar* _tmp80_;
				gint _tmp81_;
				gchar** _tmp82_;
				gint _tmp82__length1;
				const gchar* _tmp83_;
				gint _tmp84_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp75_ = parts;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp75__length1 = parts_length1;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp76_ = _tmp75_[3];
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp78_ = _tmp77_ = g_strsplit (_tmp76_, ":", 2);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts = _tmp78_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts_length1 = _vala_array_length (_tmp77_);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_time_parts_size_ = time_parts_length1;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp79_ = time_parts;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp79__length1 = time_parts_length1;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp80_ = _tmp79_[0];
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp81_ = atoi (_tmp80_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				h = _tmp81_;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp82_ = time_parts;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp82__length1 = time_parts_length1;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp83_ = _tmp82_[1];
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp84_ = atoi (_tmp83_);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				min = _tmp84_;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				time_parts = (_vala_array_free (time_parts, time_parts_length1, (GDestroyNotify) g_free), NULL);
#line 1109 "updater.c"
			}
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp85_ = parts;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp85__length1 = parts_length1;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp86_ = _tmp85_[1];
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp87_ = adblock_updater_get_month_from_string (self, _tmp86_);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			m = _tmp87_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp88_ = parts;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp88__length1 = parts_length1;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp89_ = _tmp88_[2];
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp90_ = strlen (_tmp89_);
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp91_ = _tmp90_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp91_ == 4) {
#line 1133 "updater.c"
				gchar** _tmp92_;
				gint _tmp92__length1;
				const gchar* _tmp93_;
				gint _tmp94_;
				gchar** _tmp95_;
				gint _tmp95__length1;
				const gchar* _tmp96_;
				gint _tmp97_;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp92_ = parts;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp92__length1 = parts_length1;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp93_ = _tmp92_[2];
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp94_ = atoi (_tmp93_);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				y = _tmp94_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp95_ = parts;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp95__length1 = parts_length1;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp96_ = _tmp95_[0];
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp97_ = atoi (_tmp96_);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				d = _tmp97_;
#line 1162 "updater.c"
			} else {
				gchar** _tmp98_;
				gint _tmp98__length1;
				const gchar* _tmp99_;
				gint _tmp100_;
				gchar** _tmp101_;
				gint _tmp101__length1;
				const gchar* _tmp102_;
				gint _tmp103_;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp98_ = parts;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp98__length1 = parts_length1;
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp99_ = _tmp98_[0];
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp100_ = atoi (_tmp99_);
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				y = _tmp100_;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp101_ = parts;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp101__length1 = parts_length1;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp102_ = _tmp101_[2];
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp103_ = atoi (_tmp102_);
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				d = _tmp103_;
#line 1192 "updater.c"
			}
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 1196 "updater.c"
		}
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp104_ = y;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp105_ = m;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp106_ = d;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp107_ = h;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp108_ = min;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp109_ = g_date_time_new_local (_tmp104_, _tmp105_, _tmp106_, _tmp107_, _tmp108_, 0.0);
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp110_ = _tmp109_;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_last_updated (self, _tmp110_);
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_g_date_time_unref0 (_tmp110_);
#line 1216 "updater.c"
	} else {
		GDateTime* _tmp111_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp111_ = now;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_last_updated (self, _tmp111_);
#line 1223 "updater.c"
	}
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp112_ = self->priv->expires_meta;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (_tmp112_ != NULL) {
#line 1229 "updater.c"
		const gchar* _tmp113_;
		gboolean _tmp114_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp113_ = self->priv->expires_meta;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp114_ = string_contains (_tmp113_, "days");
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		if (_tmp114_) {
#line 1238 "updater.c"
			gchar** parts = NULL;
			const gchar* _tmp115_;
			gchar** _tmp116_;
			gchar** _tmp117_;
			gint parts_length1;
			gint _parts_size_;
			GDateTime* _tmp118_;
			gchar** _tmp119_;
			gint _tmp119__length1;
			const gchar* _tmp120_;
			gint _tmp121_;
			GDateTime* _tmp122_;
			GDateTime* _tmp123_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp115_ = self->priv->expires_meta;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp117_ = _tmp116_ = g_strsplit (_tmp115_, " ", 0);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = _tmp117_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts_length1 = _vala_array_length (_tmp116_);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_parts_size_ = parts_length1;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp118_ = self->priv->_last_updated;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp119_ = parts;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp119__length1 = parts_length1;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp120_ = _tmp119_[0];
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp121_ = atoi (_tmp120_);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp122_ = g_date_time_add_days (_tmp118_, _tmp121_);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp123_ = _tmp122_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			adblock_updater_set_expires (self, _tmp123_);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_g_date_time_unref0 (_tmp123_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 1282 "updater.c"
		} else {
			const gchar* _tmp124_;
			gboolean _tmp125_;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp124_ = self->priv->expires_meta;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			_tmp125_ = string_contains (_tmp124_, "hours");
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
			if (_tmp125_) {
#line 1292 "updater.c"
				gchar** parts = NULL;
				const gchar* _tmp126_;
				gchar** _tmp127_;
				gchar** _tmp128_;
				gint parts_length1;
				gint _parts_size_;
				GDateTime* _tmp129_;
				gchar** _tmp130_;
				gint _tmp130__length1;
				const gchar* _tmp131_;
				gint _tmp132_;
				GDateTime* _tmp133_;
				GDateTime* _tmp134_;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp126_ = self->priv->expires_meta;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp128_ = _tmp127_ = g_strsplit (_tmp126_, " ", 0);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				parts = _tmp128_;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				parts_length1 = _vala_array_length (_tmp127_);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_parts_size_ = parts_length1;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp129_ = self->priv->_last_updated;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp130_ = parts;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp130__length1 = parts_length1;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp131_ = _tmp130_[0];
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp132_ = atoi (_tmp131_);
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp133_ = g_date_time_add_hours (_tmp129_, _tmp132_);
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_tmp134_ = _tmp133_;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				adblock_updater_set_expires (self, _tmp134_);
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				_g_date_time_unref0 (_tmp134_);
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
				parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 1336 "updater.c"
			}
		}
	} else {
		gint days_to_expire = 0;
		GDateTime* _tmp135_;
		gint _tmp136_;
		GDateTime* _tmp137_;
		GDateTime* _tmp138_;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		days_to_expire = 7;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp135_ = self->priv->_last_updated;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp136_ = days_to_expire;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp137_ = g_date_time_add_days (_tmp135_, _tmp136_);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp138_ = _tmp137_;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_expires (self, _tmp138_);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_g_date_time_unref0 (_tmp138_);
#line 1359 "updater.c"
	}
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp139_ = now;
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp140_ = self->priv->_expires;
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp141_ = g_date_time_compare (_tmp139_, _tmp140_);
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_set_needs_update (self, _tmp141_ == 1);
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_date_time_unref0 (now);
#line 1371 "updater.c"
}


GDateTime* adblock_updater_get_last_updated (AdblockUpdater* self) {
	GDateTime* result;
	GDateTime* _tmp0_;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = self->priv->_last_updated;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = _tmp0_;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 1386 "updater.c"
}


static gpointer _g_date_time_ref0 (gpointer self) {
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return self ? g_date_time_ref (self) : NULL;
#line 1393 "updater.c"
}


void adblock_updater_set_last_updated (AdblockUpdater* self, GDateTime* value) {
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_if_fail (self != NULL);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (adblock_updater_get_last_updated (self) != value) {
#line 1402 "updater.c"
		GDateTime* _tmp0_;
		GDateTime* _tmp1_;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp0_ = value;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp1_ = _g_date_time_ref0 (_tmp0_);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_g_date_time_unref0 (self->priv->_last_updated);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		self->priv->_last_updated = _tmp1_;
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_updater_properties[ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY]);
#line 1415 "updater.c"
	}
}


GDateTime* adblock_updater_get_expires (AdblockUpdater* self) {
	GDateTime* result;
	GDateTime* _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = self->priv->_expires;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = _tmp0_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 1431 "updater.c"
}


void adblock_updater_set_expires (AdblockUpdater* self, GDateTime* value) {
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_if_fail (self != NULL);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (adblock_updater_get_expires (self) != value) {
#line 1440 "updater.c"
		GDateTime* _tmp0_;
		GDateTime* _tmp1_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp0_ = value;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp1_ = _g_date_time_ref0 (_tmp0_);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_g_date_time_unref0 (self->priv->_expires);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		self->priv->_expires = _tmp1_;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_updater_properties[ADBLOCK_UPDATER_EXPIRES_PROPERTY]);
#line 1453 "updater.c"
	}
}


gboolean adblock_updater_get_needs_update (AdblockUpdater* self) {
	gboolean result;
	gboolean _tmp0_;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_tmp0_ = self->priv->_needs_update;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	result = _tmp0_;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	return result;
#line 1469 "updater.c"
}


void adblock_updater_set_needs_update (AdblockUpdater* self, gboolean value) {
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_return_if_fail (self != NULL);
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	if (adblock_updater_get_needs_update (self) != value) {
#line 1478 "updater.c"
		gboolean _tmp0_;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		_tmp0_ = value;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		self->priv->_needs_update = _tmp0_;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_updater_properties[ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY]);
#line 1486 "updater.c"
	}
}


static void adblock_updater_class_init (AdblockUpdaterClass * klass) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	adblock_updater_parent_class = g_type_class_peek_parent (klass);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_type_class_add_private (klass, sizeof (AdblockUpdaterPrivate));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	((AdblockFeatureClass *) klass)->clear = (void (*) (AdblockFeature *)) adblock_updater_real_clear;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	((AdblockFeatureClass *) klass)->header = (gboolean (*) (AdblockFeature *, const gchar*, const gchar*)) adblock_updater_real_header;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	((AdblockFeatureClass *) klass)->parsed = (gboolean (*) (AdblockFeature *, GFile*)) adblock_updater_real_parsed;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_adblock_updater_get_property;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_adblock_updater_set_property;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	G_OBJECT_CLASS (klass)->finalize = adblock_updater_finalize;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY, adblock_updater_properties[ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY] = g_param_spec_boxed ("last-updated", "last-updated", "last-updated", G_TYPE_DATE_TIME, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_UPDATER_EXPIRES_PROPERTY, adblock_updater_properties[ADBLOCK_UPDATER_EXPIRES_PROPERTY] = g_param_spec_boxed ("expires", "expires", "expires", G_TYPE_DATE_TIME, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY, adblock_updater_properties[ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY] = g_param_spec_boolean ("needs-update", "needs-update", "needs-update", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1514 "updater.c"
}


static void adblock_updater_instance_init (AdblockUpdater * self) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self->priv = ADBLOCK_UPDATER_GET_PRIVATE (self);
#line 1521 "updater.c"
}


static void adblock_updater_finalize (GObject * obj) {
	AdblockUpdater * self;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, ADBLOCK_TYPE_UPDATER, AdblockUpdater);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_free0 (self->priv->expires_meta);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_free0 (self->priv->last_mod_meta);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_date_time_unref0 (self->priv->_last_updated);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	_g_date_time_unref0 (self->priv->_expires);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	G_OBJECT_CLASS (adblock_updater_parent_class)->finalize (obj);
#line 1539 "updater.c"
}


GType adblock_updater_get_type (void) {
	static volatile gsize adblock_updater_type_id__volatile = 0;
	if (g_once_init_enter (&adblock_updater_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AdblockUpdaterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adblock_updater_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdblockUpdater), 0, (GInstanceInitFunc) adblock_updater_instance_init, NULL };
		GType adblock_updater_type_id;
		adblock_updater_type_id = g_type_register_static (ADBLOCK_TYPE_FEATURE, "AdblockUpdater", &g_define_type_info, 0);
		g_once_init_leave (&adblock_updater_type_id__volatile, adblock_updater_type_id);
	}
	return adblock_updater_type_id__volatile;
}


static void _vala_adblock_updater_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AdblockUpdater * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_UPDATER, AdblockUpdater);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	switch (property_id) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_value_set_boxed (value, adblock_updater_get_last_updated (self));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_EXPIRES_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_value_set_boxed (value, adblock_updater_get_expires (self));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		g_value_set_boolean (value, adblock_updater_get_needs_update (self));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 1578 "updater.c"
		default:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 1584 "updater.c"
	}
}


static void _vala_adblock_updater_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AdblockUpdater * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_UPDATER, AdblockUpdater);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
	switch (property_id) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_LAST_UPDATED_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_last_updated (self, g_value_get_boxed (value));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_EXPIRES_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_expires (self, g_value_get_boxed (value));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		case ADBLOCK_UPDATER_NEEDS_UPDATE_PROPERTY:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		adblock_updater_set_needs_update (self, g_value_get_boolean (value));
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 1612 "updater.c"
		default:
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/updater.vala"
		break;
#line 1618 "updater.c"
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



