/* apps.c generated by valac 0.38.8, the Vala compiler
 * generated from apps.vala, do not modify */

/*
   Copyright (C) 2013 Christian Dywan <christian@twotoasts.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include "midori/midori-stock.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#include "midori-core.h"
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include "midori/midori-browser.h"
#include "katze/katze.h"
#include "midori/midori-view.h"
#include <gdk/gdk.h>
#include <pango/pango.h>
#include "midori/midori-extension.h"
#include "midori/midori-panel.h"
#include "midori/midori-app.h"
#include "apps.h"


#define APPS_TYPE_LAUNCHER (apps_launcher_get_type ())
#define APPS_LAUNCHER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPS_TYPE_LAUNCHER, AppsLauncher))
#define APPS_LAUNCHER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPS_TYPE_LAUNCHER, AppsLauncherClass))
#define APPS_IS_LAUNCHER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPS_TYPE_LAUNCHER))
#define APPS_IS_LAUNCHER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPS_TYPE_LAUNCHER))
#define APPS_LAUNCHER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPS_TYPE_LAUNCHER, AppsLauncherClass))

typedef struct _AppsLauncher AppsLauncher;
typedef struct _AppsLauncherClass AppsLauncherClass;
typedef struct _AppsLauncherPrivate AppsLauncherPrivate;
enum  {
	APPS_LAUNCHER_0_PROPERTY,
	APPS_LAUNCHER_NUM_PROPERTIES
};
static GParamSpec* apps_launcher_properties[APPS_LAUNCHER_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_key_file_unref0(var) ((var == NULL) ? NULL : (var = (g_key_file_unref (var), NULL)))
typedef struct _AppsLauncherCreateAppData AppsLauncherCreateAppData;
typedef struct _AppsLauncherCreateProfileData AppsLauncherCreateProfileData;
typedef struct _AppsLauncherCreateData AppsLauncherCreateData;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))

#define APPS_TYPE_SIDEBAR (apps_sidebar_get_type ())
#define APPS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPS_TYPE_SIDEBAR, AppsSidebar))
#define APPS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPS_TYPE_SIDEBAR, AppsSidebarClass))
#define APPS_IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPS_TYPE_SIDEBAR))
#define APPS_IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPS_TYPE_SIDEBAR))
#define APPS_SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPS_TYPE_SIDEBAR, AppsSidebarClass))

typedef struct _AppsSidebar AppsSidebar;
typedef struct _AppsSidebarClass AppsSidebarClass;
typedef struct _AppsSidebarPrivate AppsSidebarPrivate;
enum  {
	APPS_SIDEBAR_0_PROPERTY,
	APPS_SIDEBAR_NUM_PROPERTIES
};
static GParamSpec* apps_sidebar_properties[APPS_SIDEBAR_NUM_PROPERTIES];
#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))
#define _g_list_free0(var) ((var == NULL) ? NULL : (var = (g_list_free (var), NULL)))

#define APPS_TYPE_MANAGER (apps_manager_get_type ())
#define APPS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), APPS_TYPE_MANAGER, AppsManager))
#define APPS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), APPS_TYPE_MANAGER, AppsManagerClass))
#define APPS_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), APPS_TYPE_MANAGER))
#define APPS_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), APPS_TYPE_MANAGER))
#define APPS_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), APPS_TYPE_MANAGER, AppsManagerClass))

typedef struct _AppsManager AppsManager;
typedef struct _AppsManagerClass AppsManagerClass;
typedef struct _AppsManagerPrivate AppsManagerPrivate;
enum  {
	APPS_MANAGER_0_PROPERTY,
	APPS_MANAGER_NUM_PROPERTIES
};
static GParamSpec* apps_manager_properties[APPS_MANAGER_NUM_PROPERTIES];
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
typedef struct _AppsManagerPopulateAppsData AppsManagerPopulateAppsData;
typedef struct _Block1Data Block1Data;

#define TYPE_EXTENSIONS_APPS_DESKTOP (extensions_apps_desktop_get_type ())
#define EXTENSIONS_APPS_DESKTOP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_EXTENSIONS_APPS_DESKTOP, ExtensionsAppsDesktop))
#define EXTENSIONS_APPS_DESKTOP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_EXTENSIONS_APPS_DESKTOP, ExtensionsAppsDesktopClass))
#define IS_EXTENSIONS_APPS_DESKTOP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_EXTENSIONS_APPS_DESKTOP))
#define IS_EXTENSIONS_APPS_DESKTOP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_EXTENSIONS_APPS_DESKTOP))
#define EXTENSIONS_APPS_DESKTOP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_EXTENSIONS_APPS_DESKTOP, ExtensionsAppsDesktopClass))

typedef struct _ExtensionsAppsDesktop ExtensionsAppsDesktop;
typedef struct _ExtensionsAppsDesktopClass ExtensionsAppsDesktopClass;
typedef struct _ExtensionsAppsDesktopPrivate ExtensionsAppsDesktopPrivate;
enum  {
	EXTENSIONS_APPS_DESKTOP_0_PROPERTY,
	EXTENSIONS_APPS_DESKTOP_NUM_PROPERTIES
};
static GParamSpec* extensions_apps_desktop_properties[EXTENSIONS_APPS_DESKTOP_NUM_PROPERTIES];
typedef struct _ExtensionsAppsDesktopRunData ExtensionsAppsDesktopRunData;

struct _AppsLauncher {
	GObject parent_instance;
	AppsLauncherPrivate * priv;
	GFile* file;
	gchar* name;
	gchar* icon_name;
	gchar* exec;
	gchar* uri;
};

struct _AppsLauncherClass {
	GObjectClass parent_class;
};

struct _AppsLauncherCreateAppData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GAsyncReadyCallback _callback_;
	gboolean _task_complete_;
	gchar* uri;
	gchar* title;
	GtkWidget* proxy;
	GFile* result;
	gchar* checksum;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	GFile* folder;
	GFile* _tmp2_;
	GFile* _tmp3_;
	GFile* _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	GtkWidget* _tmp7_;
	GFile* _tmp8_;
};

struct _AppsLauncherCreateProfileData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GAsyncReadyCallback _callback_;
	gboolean _task_complete_;
	GtkWidget* proxy;
	GFile* result;
	gchar* uuid;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* config;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	GFile* folder;
	GFile* _tmp4_;
	GFile* _tmp5_;
	GFile* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	GtkWidget* _tmp9_;
	GFile* _tmp10_;
};

struct _AppsLauncherCreateData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GAsyncReadyCallback _callback_;
	gboolean _task_complete_;
	gchar* prefix;
	GFile* folder;
	gchar* uri;
	gchar* title;
	GtkWidget* proxy;
	gchar* name;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* filename;
	const gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* icon_name;
	gchar* _tmp7_;
	gboolean testing;
	GtkWidget* _tmp8_;
	GFile* file;
	GFile* _tmp9_;
	GFile* _tmp10_;
	GFile* _tmp11_;
	GError* exist_error;
	GError* _error_;
	GFile* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	GError* _tmp15_;
	const gchar* _tmp16_;
	const gchar* _tmp17_;
	GFile* _tmp18_;
	const gchar* _tmp19_;
	gboolean _tmp20_;
	gchar* _tmp21_;
	gchar* desktop_file;
	const gchar* _tmp22_;
	const gchar* _tmp23_;
	const gchar* _tmp24_;
	const gchar* _tmp25_;
	const gchar* _tmp26_;
	gchar* _tmp27_;
	GFileOutputStream* stream;
	GFile* _tmp28_;
	GFileOutputStream* _tmp29_;
	GFileOutputStream* _tmp30_;
	const gchar* _tmp31_;
	guint8* _tmp32_;
	gint _tmp32__length1;
	guint8* _tmp33_;
	gint _tmp33__length1;
	GFile* data_dir;
	const gchar* _tmp34_;
	GFile* _tmp35_;
	GFile* desktop_dir;
	GFile* _tmp36_;
	GFile* _tmp37_;
	GFile* _tmp38_;
	GError* _vala1_exist_error;
	GFile* _tmp39_;
	GFile* _tmp40_;
	const gchar* _tmp41_;
	gchar* _tmp42_;
	gchar* _tmp43_;
	GFile* _tmp44_;
	GFile* _tmp45_;
	GtkWidget* _tmp46_;
	MidoriBrowser* browser;
	GtkWidget* _tmp47_;
	GtkWidget* _tmp48_;
	MidoriBrowser* _tmp49_;
	MidoriBrowser* _tmp50_;
	const gchar* _tmp51_;
	gchar* _tmp52_;
	gchar* _tmp53_;
	GError* _vala1__error_;
	GFile* _tmp54_;
	gchar* _tmp55_;
	gchar* _tmp56_;
	GError* _tmp57_;
	const gchar* _tmp58_;
	GtkWidget* _tmp59_;
	MidoriBrowser* _vala1_browser;
	GtkWidget* _tmp60_;
	GtkWidget* _tmp61_;
	MidoriBrowser* _tmp62_;
	MidoriBrowser* _tmp63_;
	GFile* _tmp64_;
	gchar* _tmp65_;
	gchar* _tmp66_;
	GError* _tmp67_;
	const gchar* _tmp68_;
	gchar* _tmp69_;
	gchar* _tmp70_;
	GError * _inner_error_;
};

struct _AppsSidebar {
	GtkVBox parent_instance;
	AppsSidebarPrivate * priv;
};

struct _AppsSidebarClass {
	GtkVBoxClass parent_class;
};

struct _AppsSidebarPrivate {
	GtkToolbar* toolbar;
	GtkListStore* store;
	GtkTreeView* treeview;
	KatzeArray* array;
	GFile* app_folder;
	GFile* profile_folder;
};

struct _AppsManager {
	MidoriExtension parent_instance;
	AppsManagerPrivate * priv;
	KatzeArray* array;
	GFile* app_folder;
	GFile* profile_folder;
	GList* monitors;
	GList* widgets;
};

struct _AppsManagerClass {
	MidoriExtensionClass parent_class;
};

struct _AppsManagerPopulateAppsData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GAsyncReadyCallback _callback_;
	gboolean _task_complete_;
	AppsManager* self;
	GFile* app_folder;
	GFile* _tmp0_;
	GError* exist_error;
	GFileMonitor* monitor;
	GFile* _tmp1_;
	GFileMonitor* _tmp2_;
	GFileMonitor* _tmp3_;
	GFileMonitor* _tmp4_;
	GFileMonitor* _tmp5_;
	GFileEnumerator* enumerator;
	GFile* _tmp6_;
	GFileEnumerator* _tmp7_;
	GList* files;
	GFileEnumerator* _tmp8_;
	GList* _tmp9_;
	GList* _tmp10_;
	GList* _tmp11_;
	GList* info_collection;
	GList* info_it;
	GFileInfo* _tmp12_;
	GFileInfo* info;
	GFile* file;
	GFile* _tmp13_;
	GFileInfo* _tmp14_;
	const gchar* _tmp15_;
	GFile* _tmp16_;
	AppsLauncher* launcher;
	GFile* _tmp17_;
	AppsLauncher* _tmp18_;
	gboolean _tmp19_;
	AppsLauncher* _tmp20_;
	gboolean _tmp21_;
	KatzeArray* _tmp22_;
	AppsLauncher* _tmp23_;
	GError* _error_;
	GFile* _tmp24_;
	gchar* _tmp25_;
	gchar* _tmp26_;
	GError* _tmp27_;
	const gchar* _tmp28_;
	GError* io_error;
	GFile* _tmp29_;
	gchar* _tmp30_;
	gchar* _tmp31_;
	GError* _tmp32_;
	const gchar* _tmp33_;
	GError * _inner_error_;
};

struct _Block1Data {
	int _ref_count_;
	AppsManager* self;
	MidoriBrowser* browser;
};

struct _ExtensionsAppsDesktop {
	MidoriTestJob parent_instance;
	ExtensionsAppsDesktopPrivate * priv;
};

struct _ExtensionsAppsDesktopClass {
	MidoriTestJobClass parent_class;
};

struct _ExtensionsAppsDesktopRunData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GTask* _async_result;
	GAsyncReadyCallback _callback_;
	gboolean _task_complete_;
	ExtensionsAppsDesktop* self;
	GCancellable* cancellable;
	gchar* uri;
	gchar* _tmp0_;
	gchar* checksum;
	gchar* _tmp1_;
	GFile* apps;
	GFile* _tmp2_;
	GFile* _tmp3_;
	GFile* _tmp4_;
	GFile* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	GFile* data_dir;
	const gchar* _tmp8_;
	GFile* _tmp9_;
	GFile* desktop_dir;
	GFile* _tmp10_;
	GFile* _tmp11_;
	GFile* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	GFile* folder;
	GFile* _tmp15_;
	AppsLauncher* launcher;
	AppsLauncher* _tmp16_;
	gchar* _tmp17_;
	gchar* _tmp18_;
	const gchar* _tmp19_;
	GFile* _tmp20_;
	GFile* _tmp21_;
	GError * _inner_error_;
};


static gpointer apps_launcher_parent_class = NULL;
static GInitableIface * apps_launcher_g_initable_parent_iface = NULL;
static gpointer apps_sidebar_parent_class = NULL;
static MidoriViewableIface * apps_sidebar_midori_viewable_parent_iface = NULL;
static gpointer apps_manager_parent_class = NULL;
static gpointer extensions_apps_desktop_parent_class = NULL;

#define APPS_APP_PREFIX PACKAGE_NAME " -a "
#define APPS_PROFILE_PREFIX PACKAGE_NAME " -c "
GType apps_launcher_get_type (void) G_GNUC_CONST;
gchar* apps_launcher_get_favicon_name_for_uri (const gchar* prefix, GFile* folder, const gchar* uri, gboolean testing);
gchar* apps_launcher_prepare_desktop_file (const gchar* prefix, const gchar* name, const gchar* uri, const gchar* title, const gchar* icon_name);
GFile* apps_launcher_get_app_folder (void);
static void apps_launcher_create_app_data_free (gpointer _data);
static void apps_launcher_create_app_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data);
void apps_launcher_create_app (const gchar* uri, const gchar* title, GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_);
GFile* apps_launcher_create_app_finish (GAsyncResult* _res_);
static gboolean apps_launcher_create_app_co (AppsLauncherCreateAppData* _data_);
void apps_launcher_create (const gchar* prefix, GFile* folder, const gchar* uri, const gchar* title, GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_);
void apps_launcher_create_finish (GAsyncResult* _res_);
static void apps_launcher_create_app_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
GFile* apps_launcher_get_profile_folder (void);
static void apps_launcher_create_profile_data_free (gpointer _data);
static void apps_launcher_create_profile_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data);
void apps_launcher_create_profile (GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_);
GFile* apps_launcher_create_profile_finish (GAsyncResult* _res_);
static gboolean apps_launcher_create_profile_co (AppsLauncherCreateProfileData* _data_);
static void apps_launcher_create_profile_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void apps_launcher_create_data_free (gpointer _data);
static void apps_launcher_create_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data);
static gboolean apps_launcher_create_co (AppsLauncherCreateData* _data_);
static void apps_launcher_create_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
AppsLauncher* apps_launcher_new (GFile* file);
AppsLauncher* apps_launcher_construct (GType object_type, GFile* file);
static gboolean apps_launcher_real_init (GInitable* base, GCancellable* cancellable, GError** error);
static void apps_launcher_finalize (GObject * obj);
GType apps_sidebar_get_type (void) G_GNUC_CONST;
#define APPS_SIDEBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), APPS_TYPE_SIDEBAR, AppsSidebarPrivate))
static const gchar* apps_sidebar_real_get_stock_id (MidoriViewable* base);
static const gchar* apps_sidebar_real_get_label (MidoriViewable* base);
static GtkWidget* apps_sidebar_real_get_toolbar (MidoriViewable* base);
static void ___lambda4_ (AppsSidebar* self);
static void ____lambda4__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void ___lambda5_ (AppsSidebar* self);
static void ____lambda5__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self);
static void apps_sidebar_row_activated (AppsSidebar* self, GtkTreePath* path, GtkTreeViewColumn* column);
static gboolean apps_sidebar_button_released (AppsSidebar* self, GdkEventButton* event);
AppsSidebar* apps_sidebar_new (KatzeArray* array, GFile* app_folder, GFile* profile_folder);
AppsSidebar* apps_sidebar_construct (GType object_type, KatzeArray* array, GFile* app_folder, GFile* profile_folder);
static gint apps_sidebar_tree_sort_func (AppsSidebar* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gint _apps_sidebar_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void apps_sidebar_on_render_icon (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _apps_sidebar_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void apps_sidebar_on_render_text (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _apps_sidebar_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void apps_sidebar_on_render_button (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _apps_sidebar_on_render_button_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void _apps_sidebar_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean _apps_sidebar_button_released_gtk_widget_button_release_event (GtkWidget* _sender, GdkEventButton* event, gpointer self);
static void apps_sidebar_launcher_added (AppsSidebar* self, GObject* item);
static void _apps_sidebar_launcher_added_katze_array_add_item (KatzeArray* _sender, GObject* item, gpointer self);
static void apps_sidebar_launcher_removed (AppsSidebar* self, GObject* item);
static void _apps_sidebar_launcher_removed_katze_array_remove_item (KatzeArray* _sender, GObject* item, gpointer self);
static void apps_sidebar_finalize (GObject * obj);
GType apps_manager_get_type (void) G_GNUC_CONST;
static void _g_object_unref0_ (gpointer var);
static inline void _g_list_free__g_object_unref0_ (GList* self);
static void apps_manager_app_changed (AppsManager* self, GFile* file, GFile* other, GFileMonitorEvent event);
static void apps_manager_populate_apps_data_free (gpointer _data);
static void apps_manager_populate_apps_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data);
static void apps_manager_populate_apps (AppsManager* self, GFile* app_folder, GAsyncReadyCallback _callback_, gpointer _user_data_);
static void apps_manager_populate_apps_finish (AppsManager* self, GAsyncResult* _res_);
static gboolean apps_manager_populate_apps_co (AppsManagerPopulateAppsData* _data_);
static void _apps_manager_app_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self);
static void apps_manager_populate_apps_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void apps_manager_browser_added (AppsManager* self, MidoriBrowser* browser);
static Block1Data* block1_data_ref (Block1Data* _data1_);
static void block1_data_unref (void * _userdata_);
static void __lambda6_ (Block1Data* _data1_);
static void ___lambda6__gtk_action_activate (GtkAction* _sender, gpointer self);
static void apps_manager_activated (AppsManager* self, MidoriApp* app);
static void _apps_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self);
static void apps_manager_deactivated (AppsManager* self);
AppsManager* apps_manager_new (void);
AppsManager* apps_manager_construct (GType object_type);
static void _apps_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self);
static void _apps_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self);
static void apps_manager_finalize (GObject * obj);
GType extensions_apps_desktop_get_type (void) G_GNUC_CONST;
void extensions_apps_desktop_test (void);
ExtensionsAppsDesktop* extensions_apps_desktop_new (void);
ExtensionsAppsDesktop* extensions_apps_desktop_construct (GType object_type);
static void extensions_apps_desktop_real_run_data_free (gpointer _data);
static void extensions_apps_desktop_real_run_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data);
static void extensions_apps_desktop_real_run (MidoriTestJob* base, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean extensions_apps_desktop_real_run_co (ExtensionsAppsDesktopRunData* _data_);
static void extensions_apps_desktop_run_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void _extensions_apps_desktop_test_gtest_func (void);


gchar* apps_launcher_get_favicon_name_for_uri (const gchar* prefix, GFile* folder, const gchar* uri, gboolean testing) {
	gchar* result = NULL;
	gchar* icon_name = NULL;
	gchar* _tmp0_;
	gboolean _tmp1_;
	const gchar* _tmp2_;
	GError * _inner_error_ = NULL;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (prefix != NULL, NULL);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (folder != NULL, NULL);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = g_strdup (MIDORI_STOCK_WEB_BROWSER);
#line 25 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	icon_name = _tmp0_;
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = testing;
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp1_ == TRUE) {
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		result = icon_name;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return result;
#line 552 "apps.c"
	}
#line 30 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = prefix;
#line 30 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (g_strcmp0 (_tmp2_, APPS_PROFILE_PREFIX) != 0) {
#line 558 "apps.c"
		{
			GdkPixbuf* pixbuf = NULL;
			const gchar* _tmp3_;
			GdkPixbuf* _tmp4_;
			GdkPixbuf* _tmp5_;
			gchar* icon_filename = NULL;
			GFile* _tmp7_;
			GFile* _tmp8_;
			GFile* _tmp9_;
			gchar* _tmp10_;
			gchar* _tmp11_;
			GdkPixbuf* _tmp12_;
			const gchar* _tmp13_;
			const gchar* _tmp14_;
			gchar* _tmp15_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp3_ = uri;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp4_ = midori_paths_get_icon (_tmp3_, NULL);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			pixbuf = _tmp4_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp5_ = pixbuf;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (_tmp5_ == NULL) {
#line 584 "apps.c"
				GError* _tmp6_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp6_ = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_EXIST, "No favicon loaded");
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_inner_error_ = _tmp6_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (pixbuf);
#line 592 "apps.c"
				goto __catch0_g_error;
			}
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp7_ = folder;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp8_ = g_file_get_child (_tmp7_, "icon.png");
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp9_ = _tmp8_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp10_ = g_file_get_path (_tmp9_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp11_ = _tmp10_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_tmp9_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			icon_filename = _tmp11_;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp12_ = pixbuf;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp13_ = icon_filename;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			gdk_pixbuf_save (_tmp12_, _tmp13_, "png", &_inner_error_, NULL, "compression", "7", NULL, NULL);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_free0 (icon_filename);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (pixbuf);
#line 621 "apps.c"
				goto __catch0_g_error;
			}
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp14_ = icon_filename;
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp15_ = g_strdup (_tmp14_);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (icon_name);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			icon_name = _tmp15_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (icon_filename);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (pixbuf);
#line 636 "apps.c"
		}
		goto __finally0;
		__catch0_g_error:
		{
			GError* _error_ = NULL;
			GFile* _tmp16_;
			gchar* _tmp17_;
			gchar* _tmp18_;
			GError* _tmp19_;
			const gchar* _tmp20_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_error_ = _inner_error_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_inner_error_ = NULL;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp16_ = folder;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp17_ = g_file_get_path (_tmp16_);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp18_ = _tmp17_;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp19_ = _error_;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp20_ = _tmp19_->message;
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_warning (_ ("Failed to fetch application icon in %s: %s"), _tmp18_, _tmp20_);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (_tmp18_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_error_free0 (_error_);
#line 667 "apps.c"
		}
		__finally0:
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (icon_name);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_clear_error (&_inner_error_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			return NULL;
#line 680 "apps.c"
		}
	}
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = icon_name;
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 687 "apps.c"
}


static gchar* string_delimit (const gchar* self, const gchar* delimiters, gchar new_delimiter) {
	gchar* result = NULL;
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar _tmp3_;
#line 1250 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1250 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (delimiters != NULL, NULL);
#line 1251 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1251 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1252 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _result_;
#line 1252 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = delimiters;
#line 1252 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = new_delimiter;
#line 1252 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_strdelimit (_tmp1_, _tmp2_, _tmp3_);
#line 1253 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _result_;
#line 1253 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 718 "apps.c"
}


gchar* apps_launcher_prepare_desktop_file (const gchar* prefix, const gchar* name, const gchar* uri, const gchar* title, const gchar* icon_name) {
	gchar* result = NULL;
	gchar* exec = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	GKeyFile* keyfile = NULL;
	GKeyFile* _tmp3_;
	gchar* entry = NULL;
	gchar* _tmp4_;
	GKeyFile* _tmp5_;
	const gchar* _tmp6_;
	GKeyFile* _tmp7_;
	const gchar* _tmp8_;
	GKeyFile* _tmp9_;
	const gchar* _tmp10_;
	const gchar* _tmp11_;
	GKeyFile* _tmp12_;
	const gchar* _tmp13_;
	const gchar* _tmp14_;
	GKeyFile* _tmp15_;
	const gchar* _tmp16_;
	GKeyFile* _tmp17_;
	const gchar* _tmp18_;
	const gchar* _tmp19_;
	GKeyFile* _tmp20_;
	const gchar* _tmp21_;
	const gchar* _tmp22_;
	gboolean _tmp23_;
	GKeyFile* _tmp29_;
	gchar* _tmp30_;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (prefix != NULL, NULL);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (name != NULL, NULL);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (title != NULL, NULL);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (icon_name != NULL, NULL);
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = prefix;
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = uri;
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = g_strconcat (_tmp0_, _tmp1_, NULL);
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (exec);
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	exec = _tmp2_;
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = g_key_file_new ();
#line 62 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	keyfile = _tmp3_;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = g_strdup ("Desktop Entry");
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	entry = _tmp4_;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = keyfile;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = entry;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp5_, _tmp6_, "Version", "1.0");
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = keyfile;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = entry;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp7_, _tmp8_, "Type", "Application");
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = keyfile;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp10_ = entry;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp11_ = name;
#line 67 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp9_, _tmp10_, "Name", _tmp11_);
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp12_ = keyfile;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp13_ = entry;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp14_ = exec;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp12_, _tmp13_, "Exec", _tmp14_);
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp15_ = keyfile;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp16_ = entry;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp15_, _tmp16_, "TryExec", PACKAGE_NAME);
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp17_ = keyfile;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp18_ = entry;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp19_ = icon_name;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp17_, _tmp18_, "Icon", _tmp19_);
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp20_ = keyfile;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp21_ = entry;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_key_file_set_string (_tmp20_, _tmp21_, "Categories", "Network;");
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp22_ = exec;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp23_ = g_str_has_prefix (_tmp22_, APPS_APP_PREFIX);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp23_) {
#line 835 "apps.c"
		GKeyFile* _tmp24_;
		const gchar* _tmp25_;
		const gchar* _tmp26_;
		gchar* _tmp27_;
		gchar* _tmp28_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp24_ = keyfile;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp25_ = entry;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp26_ = uri;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp27_ = string_delimit (_tmp26_, ":.\\/", '_');
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp28_ = _tmp27_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_key_file_set_string (_tmp24_, _tmp25_, "StartupWMClass", _tmp28_);
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp28_);
#line 855 "apps.c"
	}
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp29_ = keyfile;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp30_ = g_key_file_to_data (_tmp29_, NULL, NULL);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _tmp30_;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (entry);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_key_file_unref0 (keyfile);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (exec);
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 871 "apps.c"
}


GFile* apps_launcher_get_app_folder (void) {
	GFile* result = NULL;
	GFile* data_dir = NULL;
	const gchar* _tmp0_;
	GFile* _tmp1_;
	GFile* _tmp2_;
	GFile* _tmp3_;
	GFile* _tmp4_;
	GFile* _tmp5_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = midori_paths_get_user_data_dir ();
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = g_file_new_for_path (_tmp0_);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _tmp1_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = g_file_get_child (_tmp2_, PACKAGE_NAME);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = _tmp3_;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_tmp2_);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	data_dir = _tmp4_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = g_file_get_child (data_dir, "apps");
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _tmp5_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (data_dir);
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 906 "apps.c"
}


static void apps_launcher_create_app_data_free (gpointer _data) {
	AppsLauncherCreateAppData* _data_;
	_data_ = _data;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uri);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->title);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->result);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_slice_free (AppsLauncherCreateAppData, _data_);
#line 923 "apps.c"
}


static void apps_launcher_create_app_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data) {
	AppsLauncherCreateAppData* _task_data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_ = g_task_get_task_data (G_TASK (res));
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_task_data_->_callback_ != NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_task_data_->_callback_ (source_object, res, user_data);
#line 935 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_->_task_complete_ = TRUE;
#line 939 "apps.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return self ? g_object_ref (self) : NULL;
#line 946 "apps.c"
}


void apps_launcher_create_app (const gchar* uri, const gchar* title, GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	AppsLauncherCreateAppData* _data_;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	GtkWidget* _tmp4_;
	GtkWidget* _tmp5_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_slice_new0 (AppsLauncherCreateAppData);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_callback_ = _callback_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_async_result = g_task_new (NULL, NULL, apps_launcher_create_app_async_ready_wrapper, _user_data_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_callback_ == NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_task_complete_ = TRUE;
#line 968 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_set_task_data (_data_->_async_result, _data_, apps_launcher_create_app_data_free);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = uri;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uri);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->uri = _tmp1_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = title;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->title);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->title = _tmp3_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = proxy;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = _g_object_ref0 (_tmp4_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->proxy = _tmp5_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_app_co (_data_);
#line 998 "apps.c"
}


GFile* apps_launcher_create_app_finish (GAsyncResult* _res_) {
	GFile* result;
	AppsLauncherCreateAppData* _data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _data_->result;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->result = NULL;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 1013 "apps.c"
}


static void apps_launcher_create_app_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	AppsLauncherCreateAppData* _data_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = _user_data_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_source_object_ = source_object;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_res_ = _res_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_task_complete_ = TRUE;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_app_co (_data_);
#line 1029 "apps.c"
}


static gboolean apps_launcher_create_app_co (AppsLauncherCreateAppData* _data_) {
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	switch (_data_->_state_) {
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 0:
#line 1038 "apps.c"
		goto _state_0;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 1:
#line 1042 "apps.c"
		goto _state_1;
		default:
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_assert_not_reached ();
#line 1047 "apps.c"
	}
	_state_0:
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp0_ = _data_->uri;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp1_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _data_->_tmp0_, (gsize) -1);
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->checksum = _data_->_tmp1_;
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp2_ = apps_launcher_get_app_folder ();
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->folder = _data_->_tmp2_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp3_ = g_file_get_child (_data_->folder, _data_->checksum);
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp4_ = _data_->_tmp3_;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp5_ = _data_->uri;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp6_ = _data_->title;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp7_ = _data_->proxy;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_state_ = 1;
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create (APPS_APP_PREFIX, _data_->_tmp4_, _data_->_tmp5_, _data_->_tmp6_, _data_->_tmp7_, apps_launcher_create_app_ready, _data_);
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 1076 "apps.c"
	_state_1:
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_finish (_data_->_res_);
#line 90 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->_tmp4_);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp8_ = g_file_get_child (_data_->folder, _data_->checksum);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->result = _data_->_tmp8_;
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->folder);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->checksum);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_state_ != 0) {
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (_data_->_task_complete_ != TRUE) {
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
#line 1098 "apps.c"
		}
	}
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_unref (_data_->_async_result);
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 1105 "apps.c"
}


GFile* apps_launcher_get_profile_folder (void) {
	GFile* result = NULL;
	GFile* data_dir = NULL;
	const gchar* _tmp0_;
	GFile* _tmp1_;
	GFile* _tmp2_;
	GFile* _tmp3_;
	GFile* _tmp4_;
	GFile* _tmp5_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = midori_paths_get_user_data_dir ();
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = g_file_new_for_path (_tmp0_);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _tmp1_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = g_file_get_child (_tmp2_, PACKAGE_NAME);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = _tmp3_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_tmp2_);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	data_dir = _tmp4_;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = g_file_get_child (data_dir, "profiles");
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _tmp5_;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (data_dir);
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 1140 "apps.c"
}


static void apps_launcher_create_profile_data_free (gpointer _data) {
	AppsLauncherCreateProfileData* _data_;
	_data_ = _data;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->result);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_slice_free (AppsLauncherCreateProfileData, _data_);
#line 1153 "apps.c"
}


static void apps_launcher_create_profile_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data) {
	AppsLauncherCreateProfileData* _task_data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_ = g_task_get_task_data (G_TASK (res));
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_task_data_->_callback_ != NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_task_data_->_callback_ (source_object, res, user_data);
#line 1165 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_->_task_complete_ = TRUE;
#line 1169 "apps.c"
}


void apps_launcher_create_profile (GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	AppsLauncherCreateProfileData* _data_;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_slice_new0 (AppsLauncherCreateProfileData);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_callback_ = _callback_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_async_result = g_task_new (NULL, NULL, apps_launcher_create_profile_async_ready_wrapper, _user_data_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_callback_ == NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_task_complete_ = TRUE;
#line 1187 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_set_task_data (_data_->_async_result, _data_, apps_launcher_create_profile_data_free);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = proxy;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->proxy = _tmp1_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_profile_co (_data_);
#line 1201 "apps.c"
}


GFile* apps_launcher_create_profile_finish (GAsyncResult* _res_) {
	GFile* result;
	AppsLauncherCreateProfileData* _data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _data_->result;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->result = NULL;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 1216 "apps.c"
}


static void apps_launcher_create_profile_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	AppsLauncherCreateProfileData* _data_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = _user_data_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_source_object_ = source_object;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_res_ = _res_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_task_complete_ = TRUE;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_profile_co (_data_);
#line 1232 "apps.c"
}


static gboolean apps_launcher_create_profile_co (AppsLauncherCreateProfileData* _data_) {
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	switch (_data_->_state_) {
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 0:
#line 1241 "apps.c"
		goto _state_0;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 1:
#line 1245 "apps.c"
		goto _state_1;
		default:
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_assert_not_reached ();
#line 1250 "apps.c"
	}
	_state_0:
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp0_ = g_dbus_generate_guid ();
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp1_ = g_strdup (_data_->_tmp0_);
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->uuid = _data_->_tmp1_;
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp2_ = midori_paths_get_user_data_dir ();
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp3_ = g_build_path (G_DIR_SEPARATOR_S, _data_->_tmp2_, PACKAGE_NAME, "profiles", _data_->uuid, NULL);
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->config = _data_->_tmp3_;
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp4_ = apps_launcher_get_profile_folder ();
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->folder = _data_->_tmp4_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp5_ = g_file_get_child (_data_->folder, _data_->uuid);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp6_ = _data_->_tmp5_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp7_ = g_strdup_printf (_ ("Midori (%s)"), _data_->uuid);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp8_ = _data_->_tmp7_;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp9_ = _data_->proxy;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_state_ = 1;
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create (APPS_PROFILE_PREFIX, _data_->_tmp6_, _data_->config, _data_->_tmp8_, _data_->_tmp9_, apps_launcher_create_profile_ready, _data_);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 1285 "apps.c"
	_state_1:
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_finish (_data_->_res_);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->_tmp8_);
#line 105 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->_tmp6_);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp10_ = g_file_get_child (_data_->folder, _data_->uuid);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->result = _data_->_tmp10_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->folder);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->config);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uuid);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_state_ != 0) {
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (_data_->_task_complete_ != TRUE) {
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
#line 1311 "apps.c"
		}
	}
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_unref (_data_->_async_result);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 1318 "apps.c"
}


static void apps_launcher_create_data_free (gpointer _data) {
	AppsLauncherCreateData* _data_;
	_data_ = _data;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->prefix);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->folder);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uri);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->title);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_slice_free (AppsLauncherCreateData, _data_);
#line 1337 "apps.c"
}


static void apps_launcher_create_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data) {
	AppsLauncherCreateData* _task_data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_ = g_task_get_task_data (G_TASK (res));
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_task_data_->_callback_ != NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_task_data_->_callback_ (source_object, res, user_data);
#line 1349 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_->_task_complete_ = TRUE;
#line 1353 "apps.c"
}


void apps_launcher_create (const gchar* prefix, GFile* folder, const gchar* uri, const gchar* title, GtkWidget* proxy, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	AppsLauncherCreateData* _data_;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	GFile* _tmp2_;
	GFile* _tmp3_;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	GtkWidget* _tmp8_;
	GtkWidget* _tmp9_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_slice_new0 (AppsLauncherCreateData);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_callback_ = _callback_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_async_result = g_task_new (NULL, NULL, apps_launcher_create_async_ready_wrapper, _user_data_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_callback_ == NULL) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_task_complete_ = TRUE;
#line 1379 "apps.c"
	}
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_set_task_data (_data_->_async_result, _data_, apps_launcher_create_data_free);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = prefix;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->prefix);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->prefix = _tmp1_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = folder;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->folder);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->folder = _tmp3_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = uri;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = g_strdup (_tmp4_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uri);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->uri = _tmp5_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = title;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = g_strdup (_tmp6_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->title);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->title = _tmp7_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = proxy;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = _g_object_ref0 (_tmp8_);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->proxy);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->proxy = _tmp9_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_co (_data_);
#line 1425 "apps.c"
}


void apps_launcher_create_finish (GAsyncResult* _res_) {
	AppsLauncherCreateData* _data_;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
#line 1433 "apps.c"
}


static gchar* string_strip (const gchar* self) {
	gchar* result = NULL;
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	const gchar* _tmp1_;
#line 1242 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1243 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1243 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1244 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _result_;
#line 1244 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_strstrip (_tmp1_);
#line 1245 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _result_;
#line 1245 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 1456 "apps.c"
}


static void apps_launcher_create_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	AppsLauncherCreateData* _data_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = _user_data_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_source_object_ = source_object;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_res_ = _res_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_task_complete_ = TRUE;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_co (_data_);
#line 1472 "apps.c"
}


static guint8* string_get_data (const gchar* self, int* result_length1) {
	guint8* result;
	guint8* res = NULL;
	gint res_length1;
	gint _res_size_;
	gint _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	guint8* _tmp3_;
	gint _tmp3__length1;
	guint8* _tmp4_;
	gint _tmp4__length1;
#line 1436 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res = (guint8*) self;
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res_length1 = -1;
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_res_size_ = res_length1;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = strlen (self);
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _tmp0_;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res_length1 = (gint) _tmp1_;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = res_length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = res;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3__length1 = res_length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp4_ = _tmp3_;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp4__length1 = _tmp3__length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (result_length1) {
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		*result_length1 = _tmp4__length1;
#line 1516 "apps.c"
	}
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp4_;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 1522 "apps.c"
}


static gboolean apps_launcher_create_co (AppsLauncherCreateData* _data_) {
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	switch (_data_->_state_) {
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 0:
#line 1531 "apps.c"
		goto _state_0;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 1:
#line 1535 "apps.c"
		goto _state_1;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 2:
#line 1539 "apps.c"
		goto _state_2;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 3:
#line 1543 "apps.c"
		goto _state_3;
		default:
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_assert_not_reached ();
#line 1548 "apps.c"
	}
	_state_0:
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp0_ = _data_->title;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp1_ = string_delimit (_data_->_tmp0_, "‪/", ' ');
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp2_ = _data_->_tmp1_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp3_ = string_strip (_data_->_tmp2_);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp4_ = _data_->_tmp3_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->_tmp2_);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->name = _data_->_tmp4_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp5_ = _data_->name;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp6_ = midori_download_clean_filename (_data_->_tmp5_);
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->filename = _data_->_tmp6_;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp7_ = g_strdup (MIDORI_STOCK_WEB_BROWSER);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->icon_name = _data_->_tmp7_;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->testing = FALSE;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp8_ = _data_->proxy;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_tmp8_ == NULL) {
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->testing = TRUE;
#line 1583 "apps.c"
	}
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp9_ = _data_->folder;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp10_ = g_file_get_child (_data_->_tmp9_, "desc");
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->file = _data_->_tmp10_;
#line 1591 "apps.c"
	{
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp11_ = _data_->folder;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_file_make_directory_with_parents (_data_->_tmp11_, NULL, &_data_->_inner_error_);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (g_error_matches (_data_->_inner_error_, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
#line 1601 "apps.c"
				goto __catch1_g_io_error_exists;
			}
			goto __catch1_g_error;
		}
	}
	goto __finally1;
	__catch1_g_io_error_exists:
	{
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->exist_error = _data_->_inner_error_;
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_inner_error_ = NULL;
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_data_->exist_error);
#line 1616 "apps.c"
	}
	goto __finally1;
	__catch1_g_error:
	{
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_error_ = _data_->_inner_error_;
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_inner_error_ = NULL;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp12_ = _data_->file;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp13_ = g_file_get_path (_data_->_tmp12_);
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp14_ = _data_->_tmp13_;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp15_ = _data_->_error_;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp16_ = _data_->_tmp15_->message;
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_warning (_ ("Failed to create new launcher (%s): %s"), _data_->_tmp14_, _data_->_tmp16_);
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->_tmp14_);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_data_->_error_);
#line 1641 "apps.c"
	}
	__finally1:
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->file);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->icon_name);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->filename);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->name);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_clear_error (&_data_->_inner_error_);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_unref (_data_->_async_result);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 1662 "apps.c"
	}
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp17_ = _data_->prefix;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp18_ = _data_->folder;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp19_ = _data_->uri;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp20_ = _data_->testing;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp21_ = apps_launcher_get_favicon_name_for_uri (_data_->_tmp17_, _data_->_tmp18_, _data_->_tmp19_, _data_->_tmp20_);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->icon_name);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->icon_name = _data_->_tmp21_;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp22_ = _data_->prefix;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp23_ = _data_->name;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp24_ = _data_->uri;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp25_ = _data_->title;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp26_ = _data_->icon_name;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp27_ = apps_launcher_prepare_desktop_file (_data_->_tmp22_, _data_->_tmp23_, _data_->_tmp24_, _data_->_tmp25_, _data_->_tmp26_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->desktop_file = _data_->_tmp27_;
#line 1692 "apps.c"
	{
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp28_ = _data_->file;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_state_ = 1;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_file_replace_async (_data_->_tmp28_, NULL, FALSE, G_FILE_CREATE_NONE, G_PRIORITY_DEFAULT, NULL, apps_launcher_create_ready, _data_);
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 1702 "apps.c"
		_state_1:
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp29_ = g_file_replace_finish (_data_->_tmp28_, _data_->_res_, &_data_->_inner_error_);
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->stream = _data_->_tmp29_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 1710 "apps.c"
			goto __catch2_g_error;
		}
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp30_ = _data_->stream;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp31_ = _data_->desktop_file;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp32_ = string_get_data (_data_->_tmp31_, &_data_->_tmp32__length1);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp33_ = _data_->_tmp32_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp33__length1 = _data_->_tmp32__length1;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_state_ = 2;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_output_stream_write_async ((GOutputStream*) _data_->_tmp30_, _data_->_tmp33_, (gsize) _data_->_tmp33__length1, G_PRIORITY_DEFAULT, NULL, apps_launcher_create_ready, _data_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 1729 "apps.c"
		_state_2:
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_output_stream_write_finish ((GOutputStream*) _data_->_tmp30_, _data_->_res_, &_data_->_inner_error_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->stream);
#line 1737 "apps.c"
			goto __catch2_g_error;
		}
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp34_ = midori_paths_get_user_data_dir ();
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp35_ = g_file_new_for_path (_data_->_tmp34_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->data_dir = _data_->_tmp35_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp36_ = _data_->data_dir;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp37_ = g_file_get_child (_data_->_tmp36_, "applications");
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->desktop_dir = _data_->_tmp37_;
#line 1752 "apps.c"
		{
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp38_ = _data_->desktop_dir;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_file_make_directory_with_parents (_data_->_tmp38_, NULL, &_data_->_inner_error_);
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				if (g_error_matches (_data_->_inner_error_, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
#line 1762 "apps.c"
					goto __catch3_g_io_error_exists;
				}
				goto __finally3;
			}
		}
		goto __finally3;
		__catch3_g_io_error_exists:
		{
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_vala1_exist_error = _data_->_inner_error_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_inner_error_ = NULL;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_error_free0 (_data_->_vala1_exist_error);
#line 1777 "apps.c"
		}
		__finally3:
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->desktop_dir);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->data_dir);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->stream);
#line 1788 "apps.c"
			goto __catch2_g_error;
		}
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp39_ = _data_->file;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp40_ = _data_->desktop_dir;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp41_ = _data_->filename;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp42_ = g_strconcat (_data_->_tmp41_, ".desktop", NULL);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp43_ = _data_->_tmp42_;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp44_ = g_file_get_child (_data_->_tmp40_, _data_->_tmp43_);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp45_ = _data_->_tmp44_;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_state_ = 3;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_file_copy_async (_data_->_tmp39_, _data_->_tmp45_, G_FILE_COPY_NONE, G_PRIORITY_DEFAULT, NULL, NULL, NULL, apps_launcher_create_ready, _data_);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 1811 "apps.c"
		_state_3:
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_file_copy_finish (_data_->_tmp39_, _data_->_res_, &_data_->_inner_error_);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->_tmp45_);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->_tmp43_);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->desktop_dir);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->data_dir);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->stream);
#line 1827 "apps.c"
			goto __catch2_g_error;
		}
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp46_ = _data_->proxy;
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (_data_->_tmp46_ != NULL) {
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp47_ = _data_->proxy;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp48_ = gtk_widget_get_toplevel (_data_->_tmp47_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp49_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_data_->_tmp48_, MIDORI_TYPE_BROWSER) ? ((MidoriBrowser*) _data_->_tmp48_) : NULL);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->browser = _data_->_tmp49_;
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp50_ = _data_->browser;
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp51_ = _data_->name;
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp52_ = g_strdup_printf (_ ("You can now run <b>%s</b> from your launcher or menu"), _data_->_tmp51_);
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp53_ = _data_->_tmp52_;
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_signal_emit_by_name (_data_->_tmp50_, "send-notification", _ ("Launcher created"), _data_->_tmp53_);
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (_data_->_tmp53_);
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->browser);
#line 1856 "apps.c"
		}
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->desktop_dir);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->data_dir);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->stream);
#line 1864 "apps.c"
	}
	goto __finally2;
	__catch2_g_error:
	{
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_vala1__error_ = _data_->_inner_error_;
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_inner_error_ = NULL;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp54_ = _data_->file;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp55_ = g_file_get_path (_data_->_tmp54_);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp56_ = _data_->_tmp55_;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp57_ = _data_->_vala1__error_;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp58_ = _data_->_tmp57_->message;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_warning (_ ("Failed to create new launcher (%s): %s"), _data_->_tmp56_, _data_->_tmp58_);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->_tmp56_);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp59_ = _data_->proxy;
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (_data_->_tmp59_ != NULL) {
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp60_ = _data_->proxy;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp61_ = gtk_widget_get_toplevel (_data_->_tmp60_);
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp62_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_data_->_tmp61_, MIDORI_TYPE_BROWSER) ? ((MidoriBrowser*) _data_->_tmp61_) : NULL);
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_vala1_browser = _data_->_tmp62_;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp63_ = _data_->_vala1_browser;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp64_ = _data_->file;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp65_ = g_file_get_path (_data_->_tmp64_);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp66_ = _data_->_tmp65_;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp67_ = _data_->_vala1__error_;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp68_ = _data_->_tmp67_->message;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp69_ = g_strdup_printf (_ ("Failed to create new launcher (%s): %s"), _data_->_tmp66_, _data_->_tmp68_);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp70_ = _data_->_tmp69_;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_signal_emit_by_name (_data_->_tmp63_, "send-notification", _ ("Error creating launcher"), _data_->_tmp70_);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (_data_->_tmp70_);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_free0 (_data_->_tmp66_);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->_vala1_browser);
#line 1923 "apps.c"
		}
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_data_->_vala1__error_);
#line 1927 "apps.c"
	}
	__finally2:
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->desktop_file);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->file);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->icon_name);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->filename);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->name);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_clear_error (&_data_->_inner_error_);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_unref (_data_->_async_result);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 1950 "apps.c"
	}
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->desktop_file);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->file);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->icon_name);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->filename);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->name);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_state_ != 0) {
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (_data_->_task_complete_ != TRUE) {
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
#line 1970 "apps.c"
		}
	}
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_unref (_data_->_async_result);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 1977 "apps.c"
}


AppsLauncher* apps_launcher_construct (GType object_type, GFile* file) {
	AppsLauncher * self = NULL;
	GFile* _tmp0_;
	GFile* _tmp1_;
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (file != NULL, NULL);
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsLauncher*) g_object_new (object_type, NULL);
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = file;
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->file);
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->file = _tmp1_;
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return self;
#line 1999 "apps.c"
}


AppsLauncher* apps_launcher_new (GFile* file) {
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return apps_launcher_construct (APPS_TYPE_LAUNCHER, file);
#line 2006 "apps.c"
}


static gchar* string_replace (const gchar* self, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	GError * _inner_error_ = NULL;
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if ((*((gchar*) self)) == '\0') {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = TRUE;
#line 2025 "apps.c"
	} else {
		const gchar* _tmp2_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = (*((gchar*) _tmp2_)) == '\0';
#line 2032 "apps.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_) {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = TRUE;
#line 2038 "apps.c"
	} else {
		const gchar* _tmp3_;
		const gchar* _tmp4_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = replacement;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = g_strcmp0 (_tmp3_, _tmp4_) == 0;
#line 2048 "apps.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 2052 "apps.c"
		gchar* _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = g_strdup (self);
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 2060 "apps.c"
	}
	{
		GRegex* regex = NULL;
		const gchar* _tmp6_;
		gchar* _tmp7_;
		gchar* _tmp8_;
		GRegex* _tmp9_;
		GRegex* _tmp10_;
		gchar* _tmp11_ = NULL;
		GRegex* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
		gchar* _tmp15_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = old;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = g_regex_escape_string (_tmp6_, -1);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp8_ = _tmp7_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = g_regex_new (_tmp8_, 0, 0, &_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = _tmp9_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp8_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		regex = _tmp10_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 2092 "apps.c"
				goto __catch5_g_regex_error;
			}
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 2101 "apps.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = regex;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = replacement;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp14_ = g_regex_replace_literal (_tmp12_, self, (gssize) -1, 0, _tmp13_, 0, &_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = _tmp14_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 2117 "apps.c"
				goto __catch5_g_regex_error;
			}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 2128 "apps.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = _tmp11_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = NULL;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp15_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp11_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 2142 "apps.c"
	}
	goto __finally5;
	__catch5_g_regex_error:
	{
		GError* e = NULL;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		e = _inner_error_;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_inner_error_ = NULL;
#line 1423 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_assert_not_reached ();
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_error_free0 (e);
#line 2156 "apps.c"
	}
	__finally5:
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_clear_error (&_inner_error_);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return NULL;
#line 2167 "apps.c"
	}
}


static gboolean apps_launcher_real_init (GInitable* base, GCancellable* cancellable, GError** error) {
	AppsLauncher * self;
	gboolean result = FALSE;
	GKeyFile* keyfile = NULL;
	GKeyFile* _tmp0_;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_;
	gchar* _tmp11_;
	gboolean _tmp12_ = FALSE;
	const gchar* _tmp13_;
	gboolean _tmp14_;
	gchar* _tmp17_ = NULL;
	gchar* _tmp18_;
	gchar* _tmp20_;
	gchar* _tmp21_ = NULL;
	gchar* _tmp22_;
	gchar* _tmp24_;
	const gchar* _tmp25_;
	gchar* _tmp26_;
	gchar* _tmp27_;
	gchar* _tmp28_;
	GError * _inner_error_ = NULL;
#line 169 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsLauncher*) base;
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = g_key_file_new ();
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	keyfile = _tmp0_;
#line 2200 "apps.c"
	{
		GFile* _tmp1_;
		GFile* _tmp2_;
		GFile* _tmp3_;
		gchar* _tmp4_;
		gchar* _tmp5_;
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp1_ = self->file;
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp2_ = g_file_get_child (_tmp1_, "desc");
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp3_ = _tmp2_;
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp4_ = g_file_get_path (_tmp3_);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp5_ = _tmp4_;
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_key_file_load_from_file (keyfile, _tmp5_, G_KEY_FILE_NONE, &_inner_error_);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp5_);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_tmp3_);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2225 "apps.c"
			goto __catch4_g_error;
		}
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError* desc_error = NULL;
		GError* _tmp6_;
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		desc_error = _inner_error_;
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_inner_error_ = NULL;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp6_ = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_EXIST, _ ("No file \"desc\" found"));
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_inner_error_ = _tmp6_;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (desc_error);
#line 2244 "apps.c"
		goto __finally4;
	}
	__finally4:
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2250 "apps.c"
		gboolean _tmp7_ = FALSE;
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_propagate_error (error, _inner_error_);
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_key_file_unref0 (keyfile);
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return _tmp7_;
#line 2258 "apps.c"
	}
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = g_key_file_get_string (keyfile, "Desktop Entry", "Exec", &_inner_error_);
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = _tmp9_;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2266 "apps.c"
		gboolean _tmp10_ = FALSE;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_propagate_error (error, _inner_error_);
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_key_file_unref0 (keyfile);
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return _tmp10_;
#line 2274 "apps.c"
	}
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp11_ = _tmp8_;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = NULL;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->exec);
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->exec = _tmp11_;
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp13_ = self->exec;
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp14_ = g_str_has_prefix (_tmp13_, APPS_APP_PREFIX);
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (!_tmp14_) {
#line 2290 "apps.c"
		const gchar* _tmp15_;
		gboolean _tmp16_;
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp15_ = self->exec;
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp16_ = g_str_has_prefix (_tmp15_, APPS_PROFILE_PREFIX);
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp12_ = !_tmp16_;
#line 2299 "apps.c"
	} else {
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp12_ = FALSE;
#line 2303 "apps.c"
	}
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp12_) {
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		result = FALSE;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp8_);
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_key_file_unref0 (keyfile);
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return result;
#line 2315 "apps.c"
	}
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp18_ = g_key_file_get_string (keyfile, "Desktop Entry", "Name", &_inner_error_);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp17_ = _tmp18_;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2323 "apps.c"
		gboolean _tmp19_ = FALSE;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_propagate_error (error, _inner_error_);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp8_);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_key_file_unref0 (keyfile);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return _tmp19_;
#line 2333 "apps.c"
	}
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp20_ = _tmp17_;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp17_ = NULL;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->name);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->name = _tmp20_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp22_ = g_key_file_get_string (keyfile, "Desktop Entry", "Icon", &_inner_error_);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp21_ = _tmp22_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2349 "apps.c"
		gboolean _tmp23_ = FALSE;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_propagate_error (error, _inner_error_);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp17_);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp8_);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_key_file_unref0 (keyfile);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return _tmp23_;
#line 2361 "apps.c"
	}
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp24_ = _tmp21_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp21_ = NULL;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->icon_name);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->icon_name = _tmp24_;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp25_ = self->exec;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp26_ = string_replace (_tmp25_, APPS_APP_PREFIX, "");
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp27_ = _tmp26_;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp28_ = string_replace (_tmp27_, APPS_PROFILE_PREFIX, "");
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->uri);
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->uri = _tmp28_;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_tmp27_);
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = TRUE;
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_tmp21_);
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_tmp17_);
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_tmp8_);
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_key_file_unref0 (keyfile);
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 2397 "apps.c"
}


static void apps_launcher_class_init (AppsLauncherClass * klass) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_parent_class = g_type_class_peek_parent (klass);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (klass)->finalize = apps_launcher_finalize;
#line 2406 "apps.c"
}


static void apps_launcher_g_initable_interface_init (GInitableIface * iface) {
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_g_initable_parent_iface = g_type_interface_peek_parent (iface);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iface->init = (gboolean (*) (GInitable *, GCancellable*, GError**)) apps_launcher_real_init;
#line 2415 "apps.c"
}


static void apps_launcher_instance_init (AppsLauncher * self) {
}


static void apps_launcher_finalize (GObject * obj) {
	AppsLauncher * self;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, APPS_TYPE_LAUNCHER, AppsLauncher);
#line 17 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->file);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->name);
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->icon_name);
#line 20 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->exec);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (self->uri);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (apps_launcher_parent_class)->finalize (obj);
#line 2439 "apps.c"
}


GType apps_launcher_get_type (void) {
	static volatile gsize apps_launcher_type_id__volatile = 0;
	if (g_once_init_enter (&apps_launcher_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AppsLauncherClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) apps_launcher_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppsLauncher), 0, (GInstanceInitFunc) apps_launcher_instance_init, NULL };
		static const GInterfaceInfo g_initable_info = { (GInterfaceInitFunc) apps_launcher_g_initable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType apps_launcher_type_id;
		apps_launcher_type_id = g_type_register_static (G_TYPE_OBJECT, "AppsLauncher", &g_define_type_info, 0);
		g_type_add_interface_static (apps_launcher_type_id, g_initable_get_type (), &g_initable_info);
		g_once_init_leave (&apps_launcher_type_id__volatile, apps_launcher_type_id);
	}
	return apps_launcher_type_id__volatile;
}


static const gchar* apps_sidebar_real_get_stock_id (MidoriViewable* base) {
	AppsSidebar * self;
	const gchar* result = NULL;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsSidebar*) base;
#line 197 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = MIDORI_STOCK_WEB_BROWSER;
#line 197 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 2466 "apps.c"
}


static const gchar* apps_sidebar_real_get_label (MidoriViewable* base) {
	AppsSidebar * self;
	const gchar* result = NULL;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsSidebar*) base;
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _ ("Applications");
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 2479 "apps.c"
}


static void ___lambda4_ (AppsSidebar* self) {
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_profile ((GtkWidget*) self, NULL, NULL);
#line 2486 "apps.c"
}


static void ____lambda4__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	___lambda4_ ((AppsSidebar*) self);
#line 2493 "apps.c"
}


static void ___lambda5_ (AppsSidebar* self) {
	MidoriView* view = NULL;
	GtkWidget* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
	MidoriView* _tmp3_;
	gchar* checksum = NULL;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	GFile* _tmp6_;
	GFile* _tmp7_;
	GFile* _tmp8_;
	const gchar* _tmp9_;
	const gchar* _tmp10_;
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = gtk_widget_get_toplevel ((GtkWidget*) self);
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = midori_browser_get_tab (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, MIDORI_TYPE_BROWSER) ? ((MidoriBrowser*) _tmp0_) : NULL);
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _tmp1_;
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, MIDORI_TYPE_VIEW) ? ((MidoriView*) _tmp2_) : NULL);
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	view = _tmp3_;
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = midori_view_get_display_uri (view);
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _tmp4_, (gsize) -1);
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	checksum = _tmp5_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = self->priv->app_folder;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = g_file_get_child (_tmp6_, checksum);
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = _tmp7_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = midori_view_get_display_uri (view);
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp10_ = midori_view_get_display_title (view);
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create (APPS_APP_PREFIX, _tmp8_, _tmp9_, _tmp10_, (GtkWidget*) self, NULL, NULL);
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_tmp8_);
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (checksum);
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (view);
#line 2545 "apps.c"
}


static void ____lambda5__gtk_tool_button_clicked (GtkToolButton* _sender, gpointer self) {
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	___lambda5_ ((AppsSidebar*) self);
#line 2552 "apps.c"
}


static GtkWidget* apps_sidebar_real_get_toolbar (MidoriViewable* base) {
	AppsSidebar * self;
	GtkWidget* result = NULL;
	GtkToolbar* _tmp0_;
	GtkToolbar* _tmp20_;
	GtkWidget* _tmp21_;
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsSidebar*) base;
#line 205 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = self->priv->toolbar;
#line 205 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp0_ == NULL) {
#line 2568 "apps.c"
		GtkToolbar* _tmp1_;
		GtkToolButton* profile = NULL;
		GtkToolButton* _tmp2_;
		GtkToolButton* _tmp3_;
		GtkToolButton* _tmp4_;
		GtkToolButton* _tmp5_;
		GtkToolButton* _tmp6_;
		GtkToolButton* _tmp7_;
		GtkToolButton* _tmp8_;
		GtkToolbar* _tmp9_;
		GtkToolButton* _tmp10_;
		GtkToolButton* app = NULL;
		GtkToolButton* _tmp11_;
		GtkToolButton* _tmp12_;
		GtkToolButton* _tmp13_;
		GtkToolButton* _tmp14_;
		GtkToolButton* _tmp15_;
		GtkToolButton* _tmp16_;
		GtkToolButton* _tmp17_;
		GtkToolbar* _tmp18_;
		GtkToolButton* _tmp19_;
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp1_ = (GtkToolbar*) gtk_toolbar_new ();
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_ref_sink (_tmp1_);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (self->priv->toolbar);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		self->priv->toolbar = _tmp1_;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp2_ = (GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_ADD);
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_ref_sink (_tmp2_);
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		profile = _tmp2_;
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp3_ = profile;
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_button_set_label (_tmp3_, _ ("New _Profile"));
#line 212 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp4_ = profile;
#line 212 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) _tmp4_, _ ("Creates a new, independent profile and a launcher"));
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp5_ = profile;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_button_set_use_underline (_tmp5_, TRUE);
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp6_ = profile;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_item_set_is_important ((GtkToolItem*) _tmp6_, TRUE);
#line 215 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp7_ = profile;
#line 215 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_widget_show ((GtkWidget*) _tmp7_);
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp8_ = profile;
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_signal_connect_object (_tmp8_, "clicked", (GCallback) ____lambda4__gtk_tool_button_clicked, self, 0);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp9_ = self->priv->toolbar;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp10_ = profile;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_toolbar_insert (_tmp9_, (GtkToolItem*) _tmp10_, -1);
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp11_ = (GtkToolButton*) gtk_tool_button_new_from_stock (GTK_STOCK_ADD);
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_ref_sink (_tmp11_);
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		app = _tmp11_;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp12_ = app;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_button_set_label (_tmp12_, _ ("New _App"));
#line 224 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp13_ = app;
#line 224 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_widget_set_tooltip_text ((GtkWidget*) _tmp13_, _ ("Creates a new app for a specific site"));
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp14_ = app;
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_button_set_use_underline (_tmp14_, TRUE);
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp15_ = app;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tool_item_set_is_important ((GtkToolItem*) _tmp15_, TRUE);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp16_ = app;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_widget_show ((GtkWidget*) _tmp16_);
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp17_ = app;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_signal_connect_object (_tmp17_, "clicked", (GCallback) ____lambda5__gtk_tool_button_clicked, self, 0);
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp18_ = self->priv->toolbar;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp19_ = app;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_toolbar_insert (_tmp18_, (GtkToolItem*) _tmp19_, -1);
#line 205 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (app);
#line 205 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (profile);
#line 2674 "apps.c"
	}
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp20_ = self->priv->toolbar;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp21_ = _g_object_ref0 ((GtkWidget*) _tmp20_);
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _tmp21_;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 2684 "apps.c"
}


static void apps_sidebar_row_activated (AppsSidebar* self, GtkTreePath* path, GtkTreeViewColumn* column) {
	GtkTreeIter iter = {0};
	GtkListStore* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
	GError * _inner_error_ = NULL;
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (path != NULL);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (column != NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = self->priv->store;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = path;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp0_, &_tmp2_, _tmp1_);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iter = _tmp2_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp3_) {
#line 2711 "apps.c"
		AppsLauncher* launcher = NULL;
		GtkListStore* _tmp4_;
		GtkTreeIter _tmp5_;
#line 243 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp4_ = self->priv->store;
#line 243 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp5_ = iter;
#line 243 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp4_, &_tmp5_, 0, &launcher, -1);
#line 2721 "apps.c"
		{
			AppsLauncher* _tmp6_;
			const gchar* _tmp7_;
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp6_ = launcher;
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp7_ = _tmp6_->exec;
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_spawn_command_line_async (_tmp7_, &_inner_error_);
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2733 "apps.c"
				goto __catch6_g_error;
			}
		}
		goto __finally6;
		__catch6_g_error:
		{
			GError* _error_ = NULL;
			MidoriBrowser* browser = NULL;
			GtkWidget* _tmp8_;
			MidoriBrowser* _tmp9_;
			MidoriBrowser* _tmp10_;
			GError* _tmp11_;
			const gchar* _tmp12_;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_error_ = _inner_error_;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_inner_error_ = NULL;
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp8_ = gtk_widget_get_toplevel ((GtkWidget*) self);
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp9_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, MIDORI_TYPE_BROWSER) ? ((MidoriBrowser*) _tmp8_) : NULL);
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			browser = _tmp9_;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp10_ = browser;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp11_ = _error_;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp12_ = _tmp11_->message;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_signal_emit_by_name (_tmp10_, "send-notification", _ ("Error launching"), _tmp12_);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (browser);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_error_free0 (_error_);
#line 2769 "apps.c"
		}
		__finally6:
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (launcher);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_clear_error (&_inner_error_);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			return;
#line 2782 "apps.c"
		}
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (launcher);
#line 2786 "apps.c"
	}
}


static gboolean apps_sidebar_button_released (AppsSidebar* self, GdkEventButton* event) {
	gboolean result = FALSE;
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GdkEventButton _tmp0_;
	guint _tmp1_;
	GtkTreeView* _tmp2_;
	GdkEventButton _tmp3_;
	gdouble _tmp4_;
	GdkEventButton _tmp5_;
	gdouble _tmp6_;
	GtkTreePath* _tmp7_ = NULL;
	GtkTreeViewColumn* _tmp8_ = NULL;
	gboolean _tmp9_;
	GtkTreeViewColumn* _tmp10_;
	GError * _inner_error_ = NULL;
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = *event;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _tmp0_.button;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp1_ != ((guint) 1)) {
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		result = FALSE;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (column);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_gtk_tree_path_free0 (path);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return result;
#line 2825 "apps.c"
	}
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = self->priv->treeview;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = *event;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = _tmp3_.x;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = *event;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = _tmp5_.y;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = gtk_tree_view_get_path_at_pos (_tmp2_, (gint) _tmp4_, (gint) _tmp6_, &_tmp7_, &_tmp8_, NULL, NULL);
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_gtk_tree_path_free0 (path);
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	path = _tmp7_;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp10_ = _g_object_ref0 (_tmp8_);
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	column = _tmp10_;
#line 259 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_tmp9_) {
#line 2851 "apps.c"
		GtkTreePath* _tmp11_;
#line 260 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp11_ = path;
#line 260 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (_tmp11_ != NULL) {
#line 2857 "apps.c"
			GtkTreeViewColumn* _tmp12_;
			GtkTreeView* _tmp13_;
			GtkTreeViewColumn* _tmp14_;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp12_ = column;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp13_ = self->priv->treeview;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp14_ = gtk_tree_view_get_column (_tmp13_, 2);
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (_tmp12_ == _tmp14_) {
#line 2869 "apps.c"
				GtkTreeIter iter = {0};
				GtkListStore* _tmp15_;
				GtkTreePath* _tmp16_;
				GtkTreeIter _tmp17_ = {0};
				gboolean _tmp18_;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp15_ = self->priv->store;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp16_ = path;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp18_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp15_, &_tmp17_, _tmp16_);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				iter = _tmp17_;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				if (_tmp18_) {
#line 2885 "apps.c"
					AppsLauncher* launcher = NULL;
					GtkListStore* _tmp19_;
					GtkTreeIter _tmp20_;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_tmp19_ = self->priv->store;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_tmp20_ = iter;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					gtk_tree_model_get ((GtkTreeModel*) _tmp19_, &_tmp20_, 0, &launcher, -1);
#line 2895 "apps.c"
					{
						AppsLauncher* _tmp21_;
						GFile* _tmp22_;
						GtkListStore* _tmp23_;
						gchar* filename = NULL;
						AppsLauncher* _tmp24_;
						const gchar* _tmp25_;
						gchar* _tmp26_;
						GFile* data_dir = NULL;
						const gchar* _tmp27_;
						GFile* _tmp28_;
						GFile* _tmp29_;
						GFile* _tmp30_;
						GFile* _tmp31_;
						const gchar* _tmp32_;
						gchar* _tmp33_;
						gchar* _tmp34_;
						GFile* _tmp35_;
						GFile* _tmp36_;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp21_ = launcher;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp22_ = _tmp21_->file;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						g_file_trash (_tmp22_, NULL, &_inner_error_);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2923 "apps.c"
							goto __catch7_g_error;
						}
#line 268 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp23_ = self->priv->store;
#line 268 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						gtk_list_store_remove (_tmp23_, &iter);
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp24_ = launcher;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp25_ = _tmp24_->name;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp26_ = midori_download_clean_filename (_tmp25_);
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						filename = _tmp26_;
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp27_ = midori_paths_get_user_data_dir ();
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp28_ = g_file_new_for_path (_tmp27_);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						data_dir = _tmp28_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp29_ = data_dir;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp30_ = g_file_get_child (_tmp29_, "applications");
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp31_ = _tmp30_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp32_ = filename;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp33_ = g_strconcat (_tmp32_, ".desktop", NULL);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp34_ = _tmp33_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp35_ = g_file_get_child (_tmp31_, _tmp34_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp36_ = _tmp35_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						g_file_trash (_tmp36_, NULL, &_inner_error_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (_tmp36_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_free0 (_tmp34_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (_tmp31_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (data_dir);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_free0 (filename);
#line 2974 "apps.c"
							goto __catch7_g_error;
						}
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (data_dir);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_free0 (filename);
#line 2981 "apps.c"
					}
					goto __finally7;
					__catch7_g_error:
					{
						GError* _error_ = NULL;
						AppsLauncher* _tmp37_;
						GFile* _tmp38_;
						gchar* _tmp39_;
						gchar* _tmp40_;
						GError* _tmp41_;
						const gchar* _tmp42_;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_error_ = _inner_error_;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_inner_error_ = NULL;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp37_ = launcher;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp38_ = _tmp37_->file;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp39_ = g_file_get_path (_tmp38_);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp40_ = _tmp39_;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp41_ = _error_;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_tmp42_ = _tmp41_->message;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						g_critical ("apps.vala:283: Failed to remove launcher (%s): %s", _tmp40_, _tmp42_);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_free0 (_tmp40_);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_error_free0 (_error_);
#line 3015 "apps.c"
					}
					__finally7:
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3020 "apps.c"
						gboolean _tmp43_ = FALSE;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (launcher);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (column);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_gtk_tree_path_free0 (path);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						g_clear_error (&_inner_error_);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						return _tmp43_;
#line 3034 "apps.c"
					}
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					result = TRUE;
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_g_object_unref0 (launcher);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_g_object_unref0 (column);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_gtk_tree_path_free0 (path);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					return result;
#line 3046 "apps.c"
				}
			}
		}
	}
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = FALSE;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_gtk_tree_path_free0 (path);
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 3059 "apps.c"
}


static gint _apps_sidebar_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	gint result;
	result = apps_sidebar_tree_sort_func ((AppsSidebar*) self, model, a, b);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 3068 "apps.c"
}


static void _apps_sidebar_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_on_render_icon ((AppsSidebar*) self, cell_layout, cell, tree_model, iter);
#line 3075 "apps.c"
}


static void _apps_sidebar_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_on_render_text ((AppsSidebar*) self, cell_layout, cell, tree_model, iter);
#line 3082 "apps.c"
}


static void _apps_sidebar_on_render_button_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_on_render_button ((AppsSidebar*) self, cell_layout, cell, tree_model, iter);
#line 3089 "apps.c"
}


static void _apps_sidebar_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_row_activated ((AppsSidebar*) self, path, column);
#line 3096 "apps.c"
}


static gboolean _apps_sidebar_button_released_gtk_widget_button_release_event (GtkWidget* _sender, GdkEventButton* event, gpointer self) {
	gboolean result;
	result = apps_sidebar_button_released ((AppsSidebar*) self, event);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 3105 "apps.c"
}


static void _apps_sidebar_launcher_added_katze_array_add_item (KatzeArray* _sender, GObject* item, gpointer self) {
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_launcher_added ((AppsSidebar*) self, item);
#line 3112 "apps.c"
}


static void _apps_sidebar_launcher_removed_katze_array_remove_item (KatzeArray* _sender, GObject* item, gpointer self) {
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_launcher_removed ((AppsSidebar*) self, item);
#line 3119 "apps.c"
}


AppsSidebar* apps_sidebar_construct (GType object_type, KatzeArray* array, GFile* app_folder, GFile* profile_folder) {
	AppsSidebar * self = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkListStore* _tmp0_;
	GtkTreeView* _tmp1_;
	GtkTreeView* _tmp2_;
	GtkListStore* _tmp3_;
	GtkListStore* _tmp4_;
	GtkTreeViewColumn* _tmp5_;
	GtkCellRendererPixbuf* renderer_icon = NULL;
	GtkCellRendererPixbuf* _tmp6_;
	GtkTreeViewColumn* _tmp7_;
	GtkCellRendererPixbuf* _tmp8_;
	GtkTreeViewColumn* _tmp9_;
	GtkCellRendererPixbuf* _tmp10_;
	GtkTreeView* _tmp11_;
	GtkTreeViewColumn* _tmp12_;
	GtkTreeViewColumn* _tmp13_;
	GtkTreeViewColumn* _tmp14_;
	GtkCellRendererText* renderer_text = NULL;
	GtkCellRendererText* _tmp15_;
	GtkTreeViewColumn* _tmp16_;
	GtkCellRendererText* _tmp17_;
	GtkTreeViewColumn* _tmp18_;
	GtkTreeViewColumn* _tmp19_;
	GtkCellRendererText* _tmp20_;
	GtkTreeView* _tmp21_;
	GtkTreeViewColumn* _tmp22_;
	GtkTreeViewColumn* _tmp23_;
	GtkCellRendererPixbuf* renderer_button = NULL;
	GtkCellRendererPixbuf* _tmp24_;
	GtkTreeViewColumn* _tmp25_;
	GtkCellRendererPixbuf* _tmp26_;
	GtkTreeViewColumn* _tmp27_;
	GtkCellRendererPixbuf* _tmp28_;
	GtkTreeView* _tmp29_;
	GtkTreeViewColumn* _tmp30_;
	GtkTreeView* _tmp31_;
	GtkTreeView* _tmp32_;
	GtkTreeView* _tmp33_;
	GtkTreeView* _tmp34_;
	KatzeArray* _tmp35_;
	KatzeArray* _tmp36_;
	KatzeArray* _tmp37_;
	KatzeArray* _tmp38_;
	KatzeArray* _tmp39_;
	GList* _tmp40_;
	GFile* _tmp43_;
	GFile* _tmp44_;
	GFile* _tmp45_;
	GFile* _tmp46_;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (array != NULL, NULL);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (app_folder != NULL, NULL);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (profile_folder != NULL, NULL);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsSidebar*) g_object_new (object_type, NULL);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = self->priv->store;
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) _tmp0_);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp1_);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->treeview);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->treeview = _tmp1_;
#line 297 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = self->priv->treeview;
#line 297 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_set_headers_visible (_tmp2_, FALSE);
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = self->priv->store;
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp3_, 0, GTK_SORT_ASCENDING);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = self->priv->store;
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) _tmp4_, 0, _apps_sidebar_tree_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = gtk_tree_view_column_new ();
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp5_);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	column = _tmp5_;
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp6_);
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	renderer_icon = _tmp6_;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = column;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = renderer_icon;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp7_, (GtkCellRenderer*) _tmp8_, FALSE);
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = column;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp10_ = renderer_icon;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp9_, (GtkCellRenderer*) _tmp10_, _apps_sidebar_on_render_icon_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp11_ = self->priv->treeview;
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp12_ = column;
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_append_column (_tmp11_, _tmp12_);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp13_ = gtk_tree_view_column_new ();
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp13_);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	column = _tmp13_;
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp14_ = column;
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_column_set_sizing (_tmp14_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp15_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp15_);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	renderer_text = _tmp15_;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp16_ = column;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp17_ = renderer_text;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp16_, (GtkCellRenderer*) _tmp17_, TRUE);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp18_ = column;
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_column_set_expand (_tmp18_, TRUE);
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp19_ = column;
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp20_ = renderer_text;
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp19_, (GtkCellRenderer*) _tmp20_, _apps_sidebar_on_render_text_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp21_ = self->priv->treeview;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp22_ = column;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_append_column (_tmp21_, _tmp22_);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp23_ = gtk_tree_view_column_new ();
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp23_);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	column = _tmp23_;
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp24_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp24_);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	renderer_button = _tmp24_;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp25_ = column;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp26_ = renderer_button;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp25_, (GtkCellRenderer*) _tmp26_, FALSE);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp27_ = column;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp28_ = renderer_button;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp27_, (GtkCellRenderer*) _tmp28_, _apps_sidebar_on_render_button_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp29_ = self->priv->treeview;
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp30_ = column;
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_view_append_column (_tmp29_, _tmp30_);
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp31_ = self->priv->treeview;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object (_tmp31_, "row-activated", (GCallback) _apps_sidebar_row_activated_gtk_tree_view_row_activated, self, 0);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp32_ = self->priv->treeview;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object ((GtkWidget*) _tmp32_, "button-release-event", (GCallback) _apps_sidebar_button_released_gtk_widget_button_release_event, self, 0);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp33_ = self->priv->treeview;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_widget_show ((GtkWidget*) _tmp33_);
#line 325 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp34_ = self->priv->treeview;
#line 325 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _tmp34_, TRUE, TRUE, (guint) 0);
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp35_ = array;
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp36_ = _g_object_ref0 (_tmp35_);
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->array);
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->array = _tmp36_;
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp37_ = array;
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object (_tmp37_, "add-item", (GCallback) _apps_sidebar_launcher_added_katze_array_add_item, self, 0);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp38_ = array;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object (_tmp38_, "remove-item", (GCallback) _apps_sidebar_launcher_removed_katze_array_remove_item, self, 0);
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp39_ = array;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp40_ = katze_array_get_items (_tmp39_);
#line 3344 "apps.c"
	{
		GList* item_collection = NULL;
		GList* item_it = NULL;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		item_collection = _tmp40_;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
#line 3352 "apps.c"
			GObject* _tmp41_;
			GObject* item = NULL;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp41_ = _g_object_ref0 ((GObject*) ((KatzeItem*) item_it->data));
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			item = _tmp41_;
#line 3359 "apps.c"
			{
				GObject* _tmp42_;
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp42_ = item;
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				apps_sidebar_launcher_added (self, _tmp42_);
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (item);
#line 3368 "apps.c"
			}
		}
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_list_free0 (item_collection);
#line 3373 "apps.c"
	}
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp43_ = app_folder;
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp44_ = _g_object_ref0 (_tmp43_);
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->app_folder);
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->app_folder = _tmp44_;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp45_ = profile_folder;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp46_ = _g_object_ref0 (_tmp45_);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->profile_folder);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->profile_folder = _tmp46_;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (renderer_button);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (renderer_text);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (renderer_icon);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (column);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return self;
#line 3401 "apps.c"
}


AppsSidebar* apps_sidebar_new (KatzeArray* array, GFile* app_folder, GFile* profile_folder) {
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return apps_sidebar_construct (APPS_TYPE_SIDEBAR, array, app_folder, profile_folder);
#line 3408 "apps.c"
}


static gint apps_sidebar_tree_sort_func (AppsSidebar* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	gint result = 0;
	AppsLauncher* launcher1 = NULL;
	AppsLauncher* launcher2 = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkTreeModel* _tmp2_;
	GtkTreeIter _tmp3_;
	GCompareFunc _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gint _tmp7_;
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (a != NULL, 0);
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_val_if_fail (b != NULL, 0);
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = model;
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = *a;
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &launcher1, -1);
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = model;
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = *b;
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_model_get (_tmp2_, &_tmp3_, 0, &launcher2, -1);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = g_strcmp0;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = launcher1->name;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = launcher2->name;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = _tmp4_ (_tmp5_, _tmp6_);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = _tmp7_;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (launcher2);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (launcher1);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 3460 "apps.c"
}


static void apps_sidebar_launcher_added (AppsSidebar* self, GObject* item) {
	AppsLauncher* launcher = NULL;
	GObject* _tmp0_;
	AppsLauncher* _tmp1_;
	GtkTreeIter iter = {0};
	GtkListStore* _tmp2_;
	GtkTreeIter _tmp3_ = {0};
	GtkListStore* _tmp4_;
	GtkTreeIter _tmp5_;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (item != NULL);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = item;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, APPS_TYPE_LAUNCHER) ? ((AppsLauncher*) _tmp0_) : NULL);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	launcher = _tmp1_;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = self->priv->store;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_list_store_append (_tmp2_, &_tmp3_);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iter = _tmp3_;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = self->priv->store;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = iter;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_list_store_set (_tmp4_, &_tmp5_, 0, launcher, -1);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (launcher);
#line 3497 "apps.c"
}


static void apps_sidebar_launcher_removed (AppsSidebar* self, GObject* item) {
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (item != NULL);
#line 3506 "apps.c"
}


static void apps_sidebar_on_render_icon (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	AppsLauncher* launcher = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp12_;
	GError * _inner_error_ = NULL;
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (column != NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (renderer != NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (model != NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (iter != NULL);
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = model;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = *iter;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &launcher, -1);
#line 3532 "apps.c"
	{
		gint icon_width = 0;
		gint icon_height = 0;
		GtkSettings* _tmp2_;
		gint _tmp3_ = 0;
		gint _tmp4_ = 0;
		GdkPixbuf* pixbuf = NULL;
		const gchar* _tmp5_;
		gint _tmp6_;
		gint _tmp7_;
		GdkPixbuf* _tmp8_;
		GtkCellRenderer* _tmp9_;
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		icon_width = 48;
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		icon_height = 48;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp2_ = gtk_widget_get_settings ((GtkWidget*) self);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		gtk_icon_size_lookup_for_settings (_tmp2_, GTK_ICON_SIZE_DIALOG, &_tmp3_, &_tmp4_);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		icon_width = _tmp3_;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		icon_height = _tmp4_;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp5_ = launcher->icon_name;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp6_ = icon_width;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp7_ = icon_height;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp8_ = gdk_pixbuf_new_from_file_at_size (_tmp5_, _tmp6_, _tmp7_, &_inner_error_);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		pixbuf = _tmp8_;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3569 "apps.c"
			goto __catch8_g_error;
		}
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp9_ = renderer;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_set ((GObject*) _tmp9_, "pixbuf", pixbuf, NULL);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (pixbuf);
#line 3578 "apps.c"
	}
	goto __finally8;
	__catch8_g_error:
	{
		GError* _error_ = NULL;
		GtkCellRenderer* _tmp10_;
		const gchar* _tmp11_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_error_ = _inner_error_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_inner_error_ = NULL;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp10_ = renderer;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp11_ = launcher->icon_name;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_set ((GObject*) _tmp10_, "icon-name", _tmp11_, NULL);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_error_);
#line 3598 "apps.c"
	}
	__finally8:
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (launcher);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_clear_error (&_inner_error_);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return;
#line 3611 "apps.c"
	}
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp12_ = renderer;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_set ((GObject*) _tmp12_, "stock-size", GTK_ICON_SIZE_DIALOG, "xpad", 4, NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (launcher);
#line 3619 "apps.c"
}


static void apps_sidebar_on_render_text (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	AppsLauncher* launcher = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	const gchar* _tmp3_;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (column != NULL);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (renderer != NULL);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (model != NULL);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (iter != NULL);
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = model;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = *iter;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &launcher, -1);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = renderer;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = launcher->name;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = launcher->uri;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = g_markup_printf_escaped ("<b>%s</b>\n%s", _tmp3_, _tmp4_);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = _tmp5_;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_set ((GObject*) _tmp2_, "markup", _tmp6_, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_tmp6_);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (launcher);
#line 3664 "apps.c"
}


static void apps_sidebar_on_render_button (AppsSidebar* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	GtkCellRenderer* _tmp0_;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (column != NULL);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (renderer != NULL);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (model != NULL);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (iter != NULL);
#line 388 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = renderer;
#line 388 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_set ((GObject*) _tmp0_, "stock-id", GTK_STOCK_DELETE, "stock-size", GTK_ICON_SIZE_MENU, NULL);
#line 3684 "apps.c"
}


static void apps_sidebar_class_init (AppsSidebarClass * klass) {
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_parent_class = g_type_class_peek_parent (klass);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_type_class_add_private (klass, sizeof (AppsSidebarPrivate));
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (klass)->finalize = apps_sidebar_finalize;
#line 3695 "apps.c"
}


static void apps_sidebar_midori_viewable_interface_init (MidoriViewableIface * iface) {
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_sidebar_midori_viewable_parent_iface = g_type_interface_peek_parent (iface);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iface->get_stock_id = (const gchar* (*) (MidoriViewable *)) apps_sidebar_real_get_stock_id;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iface->get_label = (const gchar* (*) (MidoriViewable *)) apps_sidebar_real_get_label;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	iface->get_toolbar = (GtkWidget* (*) (MidoriViewable *)) apps_sidebar_real_get_toolbar;
#line 3708 "apps.c"
}


static void apps_sidebar_instance_init (AppsSidebar * self) {
	GtkListStore* _tmp0_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv = APPS_SIDEBAR_GET_PRIVATE (self);
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->toolbar = NULL;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = gtk_list_store_new (1, APPS_TYPE_LAUNCHER);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->priv->store = _tmp0_;
#line 3722 "apps.c"
}


static void apps_sidebar_finalize (GObject * obj) {
	AppsSidebar * self;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, APPS_TYPE_SIDEBAR, AppsSidebar);
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->toolbar);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->store);
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->treeview);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->array);
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->app_folder);
#line 194 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->priv->profile_folder);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (apps_sidebar_parent_class)->finalize (obj);
#line 3744 "apps.c"
}


GType apps_sidebar_get_type (void) {
	static volatile gsize apps_sidebar_type_id__volatile = 0;
	if (g_once_init_enter (&apps_sidebar_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AppsSidebarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) apps_sidebar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppsSidebar), 0, (GInstanceInitFunc) apps_sidebar_instance_init, NULL };
		static const GInterfaceInfo midori_viewable_info = { (GInterfaceInitFunc) apps_sidebar_midori_viewable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType apps_sidebar_type_id;
		apps_sidebar_type_id = g_type_register_static (GTK_TYPE_VBOX, "AppsSidebar", &g_define_type_info, 0);
		g_type_add_interface_static (apps_sidebar_type_id, MIDORI_TYPE_VIEWABLE, &midori_viewable_info);
		g_once_init_leave (&apps_sidebar_type_id__volatile, apps_sidebar_type_id);
	}
	return apps_sidebar_type_id__volatile;
}


static void _g_object_unref0_ (gpointer var) {
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 3765 "apps.c"
}


static inline void _g_list_free__g_object_unref0_ (GList* self) {
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_list_free_full (self, (GDestroyNotify) _g_object_unref0_);
#line 3772 "apps.c"
}


static void apps_manager_app_changed (AppsManager* self, GFile* file, GFile* other, GFileMonitorEvent event) {
	GError * _inner_error_ = NULL;
#line 400 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 400 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (file != NULL);
#line 3782 "apps.c"
	{
		GFileMonitorEvent _tmp0_;
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp0_ = event;
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		switch (_tmp0_) {
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			case G_FILE_MONITOR_EVENT_DELETED:
#line 3791 "apps.c"
			{
#line 405 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				break;
#line 3795 "apps.c"
			}
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			case G_FILE_MONITOR_EVENT_CREATED:
#line 3799 "apps.c"
			{
				AppsLauncher* launcher = NULL;
				GFile* _tmp1_;
				AppsLauncher* _tmp2_;
				gboolean _tmp3_ = FALSE;
				AppsLauncher* _tmp4_;
				gboolean _tmp5_;
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp1_ = file;
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp2_ = apps_launcher_new (_tmp1_);
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				launcher = _tmp2_;
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp4_ = launcher;
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp5_ = g_initable_init ((GInitable*) _tmp4_, NULL, &_inner_error_);
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp3_ = _tmp5_;
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_g_object_unref0 (launcher);
#line 3823 "apps.c"
					goto __catch9_g_error;
				}
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				if (_tmp3_) {
#line 3828 "apps.c"
					KatzeArray* _tmp6_;
					AppsLauncher* _tmp7_;
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_tmp6_ = self->array;
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_tmp7_ = launcher;
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					g_signal_emit_by_name (_tmp6_, "add-item", (GObject*) _tmp7_);
#line 3837 "apps.c"
				}
#line 410 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (launcher);
#line 410 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				break;
#line 3843 "apps.c"
			}
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			case G_FILE_MONITOR_EVENT_CHANGED:
#line 3847 "apps.c"
			{
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				break;
#line 3851 "apps.c"
			}
			default:
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			break;
#line 3856 "apps.c"
		}
	}
	goto __finally9;
	__catch9_g_error:
	{
		GError* _error_ = NULL;
		GFile* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
		GError* _tmp11_;
		const gchar* _tmp12_;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_error_ = _inner_error_;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_inner_error_ = NULL;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp8_ = file;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp9_ = g_file_get_path (_tmp8_);
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp10_ = _tmp9_;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp11_ = _error_;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_tmp12_ = _tmp11_->message;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_warning ("apps.vala:417: Application changed (%s): %s", _tmp10_, _tmp12_);
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_tmp10_);
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_error_);
#line 3888 "apps.c"
	}
	__finally9:
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_clear_error (&_inner_error_);
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return;
#line 3899 "apps.c"
	}
}


static void apps_manager_populate_apps_data_free (gpointer _data) {
	AppsManagerPopulateAppsData* _data_;
	_data_ = _data;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->app_folder);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->self);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_slice_free (AppsManagerPopulateAppsData, _data_);
#line 3913 "apps.c"
}


static void apps_manager_populate_apps_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data) {
	AppsManagerPopulateAppsData* _task_data_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_ = g_task_get_task_data (G_TASK (res));
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_task_data_->_callback_ != NULL) {
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_task_data_->_callback_ (source_object, res, user_data);
#line 3925 "apps.c"
	}
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_->_task_complete_ = TRUE;
#line 3929 "apps.c"
}


static void apps_manager_populate_apps (AppsManager* self, GFile* app_folder, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	AppsManagerPopulateAppsData* _data_;
	AppsManager* _tmp0_;
	GFile* _tmp1_;
	GFile* _tmp2_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_slice_new0 (AppsManagerPopulateAppsData);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_callback_ = _callback_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_async_result = g_task_new (G_OBJECT (self), NULL, apps_manager_populate_apps_async_ready_wrapper, _user_data_);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_callback_ == NULL) {
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_task_complete_ = TRUE;
#line 3948 "apps.c"
	}
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_set_task_data (_data_->_async_result, _data_, apps_manager_populate_apps_data_free);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->self = _tmp0_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = app_folder;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->app_folder);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->app_folder = _tmp2_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_populate_apps_co (_data_);
#line 3966 "apps.c"
}


static void apps_manager_populate_apps_finish (AppsManager* self, GAsyncResult* _res_) {
	AppsManagerPopulateAppsData* _data_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_task_propagate_pointer (G_TASK (_res_), NULL);
#line 3974 "apps.c"
}


static void _apps_manager_app_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self) {
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_app_changed ((AppsManager*) self, file, other_file, event_type);
#line 3981 "apps.c"
}


static void apps_manager_populate_apps_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	AppsManagerPopulateAppsData* _data_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = _user_data_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_source_object_ = source_object;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_res_ = _res_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_task_complete_ = TRUE;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_populate_apps_co (_data_);
#line 3997 "apps.c"
}


static gboolean apps_manager_populate_apps_co (AppsManagerPopulateAppsData* _data_) {
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	switch (_data_->_state_) {
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 0:
#line 4006 "apps.c"
		goto _state_0;
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 1:
#line 4010 "apps.c"
		goto _state_1;
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 2:
#line 4014 "apps.c"
		goto _state_2;
		default:
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_assert_not_reached ();
#line 4019 "apps.c"
	}
	_state_0:
	{
		{
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp0_ = _data_->app_folder;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_file_make_directory_with_parents (_data_->_tmp0_, NULL, &_data_->_inner_error_);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				if (g_error_matches (_data_->_inner_error_, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
#line 4032 "apps.c"
					goto __catch11_g_io_error_exists;
				}
				goto __finally11;
			}
		}
		goto __finally11;
		__catch11_g_io_error_exists:
		{
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->exist_error = _data_->_inner_error_;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_inner_error_ = NULL;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_error_free0 (_data_->exist_error);
#line 4047 "apps.c"
		}
		__finally11:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 4052 "apps.c"
			goto __catch10_g_error;
		}
#line 429 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp1_ = _data_->app_folder;
#line 429 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp2_ = g_file_monitor_directory (_data_->_tmp1_, 0, NULL, &_data_->_inner_error_);
#line 429 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->monitor = _data_->_tmp2_;
#line 429 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 4063 "apps.c"
			goto __catch10_g_error;
		}
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp3_ = _data_->monitor;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_signal_connect_object (_data_->_tmp3_, "changed", (GCallback) _apps_manager_app_changed_g_file_monitor_changed, _data_->self, 0);
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp4_ = _data_->monitor;
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp5_ = _g_object_ref0 (_data_->_tmp4_);
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->self->monitors = g_list_append (_data_->self->monitors, _data_->_tmp5_);
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp6_ = _data_->app_folder;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_state_ = 1;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_file_enumerate_children_async (_data_->_tmp6_, "standard::name", 0, G_PRIORITY_DEFAULT, NULL, apps_manager_populate_apps_ready, _data_);
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 4084 "apps.c"
		_state_1:
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp7_ = g_file_enumerate_children_finish (_data_->_tmp6_, _data_->_res_, &_data_->_inner_error_);
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->enumerator = _data_->_tmp7_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_g_object_unref0 (_data_->monitor);
#line 4094 "apps.c"
			goto __catch10_g_error;
		}
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (TRUE) {
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp8_ = _data_->enumerator;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_state_ = 2;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_file_enumerator_next_files_async (_data_->_tmp8_, 10, G_PRIORITY_DEFAULT, NULL, apps_manager_populate_apps_ready, _data_);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			return FALSE;
#line 4107 "apps.c"
			_state_2:
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp9_ = g_file_enumerator_next_files_finish (_data_->_tmp8_, _data_->_res_, &_data_->_inner_error_);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->files = _data_->_tmp9_;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (_data_->enumerator);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (_data_->monitor);
#line 4119 "apps.c"
				goto __catch10_g_error;
			}
#line 436 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp10_ = _data_->files;
#line 436 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			if (_data_->_tmp10_ == NULL) {
#line 437 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				__g_list_free__g_object_unref0_0 (_data_->files);
#line 437 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				break;
#line 4130 "apps.c"
			}
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_data_->_tmp11_ = _data_->files;
#line 4134 "apps.c"
			{
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_data_->info_collection = _data_->_tmp11_;
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				for (_data_->info_it = _data_->info_collection; _data_->info_it != NULL; _data_->info_it = _data_->info_it->next) {
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_data_->_tmp12_ = _g_object_ref0 ((GFileInfo*) _data_->info_it->data);
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
					_data_->info = _data_->_tmp12_;
#line 4144 "apps.c"
					{
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_data_->_tmp13_ = _data_->app_folder;
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_data_->_tmp14_ = _data_->info;
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_data_->_tmp15_ = g_file_info_get_name (_data_->_tmp14_);
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_data_->_tmp16_ = g_file_get_child (_data_->_tmp13_, _data_->_tmp15_);
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_data_->file = _data_->_tmp16_;
#line 4156 "apps.c"
						{
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp17_ = _data_->file;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp18_ = apps_launcher_new (_data_->_tmp17_);
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->launcher = _data_->_tmp18_;
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp20_ = _data_->launcher;
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp21_ = g_initable_init ((GInitable*) _data_->_tmp20_, NULL, &_data_->_inner_error_);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp19_ = _data_->_tmp21_;
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
								_g_object_unref0 (_data_->launcher);
#line 4174 "apps.c"
								goto __catch12_g_error;
							}
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							if (_data_->_tmp19_) {
#line 443 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
								_data_->_tmp22_ = _data_->self->array;
#line 443 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
								_data_->_tmp23_ = _data_->launcher;
#line 443 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
								g_signal_emit_by_name (_data_->_tmp22_, "add-item", (GObject*) _data_->_tmp23_);
#line 4185 "apps.c"
							}
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (_data_->launcher);
#line 4189 "apps.c"
						}
						goto __finally12;
						__catch12_g_error:
						{
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_error_ = _data_->_inner_error_;
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_inner_error_ = NULL;
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp24_ = _data_->file;
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp25_ = g_file_get_path (_data_->_tmp24_);
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp26_ = _data_->_tmp25_;
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp27_ = _data_->_error_;
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_data_->_tmp28_ = _data_->_tmp27_->message;
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							g_warning ("apps.vala:446: Failed to parse launcher (%s): %s", _data_->_tmp26_, _data_->_tmp28_);
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_free0 (_data_->_tmp26_);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_error_free0 (_data_->_error_);
#line 4214 "apps.c"
						}
						__finally12:
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (_data_->file);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (_data_->info);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							__g_list_free__g_object_unref0_0 (_data_->files);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (_data_->enumerator);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
							_g_object_unref0 (_data_->monitor);
#line 4229 "apps.c"
							goto __catch10_g_error;
						}
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (_data_->file);
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
						_g_object_unref0 (_data_->info);
#line 4236 "apps.c"
					}
				}
			}
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			__g_list_free__g_object_unref0_0 (_data_->files);
#line 4242 "apps.c"
		}
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->enumerator);
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->monitor);
#line 4248 "apps.c"
	}
	goto __finally10;
	__catch10_g_error:
	{
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->io_error = _data_->_inner_error_;
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_inner_error_ = NULL;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp29_ = _data_->app_folder;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp30_ = g_file_get_path (_data_->_tmp29_);
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp31_ = _data_->_tmp30_;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp32_ = _data_->io_error;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_tmp33_ = _data_->_tmp32_->message;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_warning ("apps.vala:452: Failed to list apps (%s): %s", _data_->_tmp31_, _data_->_tmp33_);
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->_tmp31_);
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_error_free0 (_data_->io_error);
#line 4273 "apps.c"
	}
	__finally10:
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_clear_error (&_data_->_inner_error_);
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_unref (_data_->_async_result);
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 4286 "apps.c"
	}
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_state_ != 0) {
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (_data_->_task_complete_ != TRUE) {
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
#line 4296 "apps.c"
		}
	}
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_unref (_data_->_async_result);
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 4303 "apps.c"
}


static Block1Data* block1_data_ref (Block1Data* _data1_) {
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_atomic_int_inc (&_data1_->_ref_count_);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return _data1_;
#line 4312 "apps.c"
}


static void block1_data_unref (void * _userdata_) {
	Block1Data* _data1_;
	_data1_ = (Block1Data*) _userdata_;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (g_atomic_int_dec_and_test (&_data1_->_ref_count_)) {
#line 4321 "apps.c"
		AppsManager* self;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		self = _data1_->self;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data1_->browser);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (self);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_slice_free (Block1Data, _data1_);
#line 4331 "apps.c"
	}
}


static void __lambda6_ (Block1Data* _data1_) {
	AppsManager* self;
	MidoriView* view = NULL;
	MidoriBrowser* _tmp0_;
	GtkWidget* _tmp1_;
	GtkWidget* _tmp2_;
	MidoriView* _tmp3_;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
#line 464 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = _data1_->self;
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = _data1_->browser;
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = midori_browser_get_tab (_tmp0_);
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _tmp1_;
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, MIDORI_TYPE_VIEW) ? ((MidoriView*) _tmp2_) : NULL);
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	view = _tmp3_;
#line 466 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = midori_view_get_display_uri (view);
#line 466 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = midori_view_get_display_title (view);
#line 466 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_app (_tmp4_, _tmp5_, (GtkWidget*) view, NULL, NULL);
#line 464 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (view);
#line 4365 "apps.c"
}


static void ___lambda6__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 464 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__lambda6_ (self);
#line 4372 "apps.c"
}


static void apps_manager_browser_added (AppsManager* self, MidoriBrowser* browser) {
	Block1Data* _data1_;
	MidoriBrowser* _tmp0_;
	MidoriBrowser* _tmp1_;
	GtkAccelGroup* accels = NULL;
	GtkAccelGroup* _tmp2_;
	MidoriBrowser* _tmp3_;
	GtkActionGroup* action_group = NULL;
	MidoriBrowser* _tmp4_;
	GtkActionGroup* _tmp5_;
	GtkActionGroup* _tmp6_;
	GtkAction* action = NULL;
	GtkAction* _tmp7_;
	AppsSidebar* viewable = NULL;
	KatzeArray* _tmp8_;
	GFile* _tmp9_;
	GFile* _tmp10_;
	AppsSidebar* _tmp11_;
	MidoriBrowser* _tmp12_;
	MidoriPanel* _tmp13_;
	MidoriPanel* _tmp14_;
	MidoriPanel* _tmp15_;
	GtkWidget* _tmp16_;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (browser != NULL);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data1_ = g_slice_new0 (Block1Data);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data1_->_ref_count_ = 1;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data1_->self = g_object_ref (self);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = browser;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data1_->browser);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data1_->browser = _tmp1_;
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = gtk_accel_group_new ();
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	accels = _tmp2_;
#line 459 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = _data1_->browser;
#line 459 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_window_add_accel_group ((GtkWindow*) _tmp3_, accels);
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = _data1_->browser;
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = midori_browser_get_action_group (_tmp4_);
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = _g_object_ref0 (_tmp5_);
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	action_group = _tmp6_;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp7_ = gtk_action_new ("CreateLauncher", _ ("Create _Launcher"), _ ("Creates a new app for a specific site"), NULL);
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	action = _tmp7_;
#line 464 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_data (action, "activate", (GCallback) ___lambda6__gtk_action_activate, block1_data_ref (_data1_), (GClosureNotify) block1_data_unref, 0);
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_action_group_add_action_with_accel (action_group, action, "<Ctrl><Shift>A");
#line 469 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_action_set_accel_group (action, accels);
#line 470 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_action_connect_accelerator (action);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = self->array;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp9_ = self->app_folder;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp10_ = self->profile_folder;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp11_ = apps_sidebar_new (_tmp8_, _tmp9_, _tmp10_);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_ref_sink (_tmp11_);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	viewable = _tmp11_;
#line 473 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	gtk_widget_show ((GtkWidget*) viewable);
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp12_ = _data1_->browser;
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_get (_tmp12_, "panel", &_tmp13_, NULL);
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp14_ = _tmp13_;
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp15_ = _tmp14_;
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	midori_panel_append_page (_tmp15_, (MidoriViewable*) viewable);
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_tmp15_);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp16_ = _g_object_ref0 ((GtkWidget*) viewable);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->widgets = g_list_append (self->widgets, _tmp16_);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (viewable);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (action);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (action_group);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (accels);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	block1_data_unref (_data1_);
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data1_ = NULL;
#line 4487 "apps.c"
}


static void _apps_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self) {
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_browser_added ((AppsManager*) self, browser);
#line 4494 "apps.c"
}


static void apps_manager_activated (AppsManager* self, MidoriApp* app) {
	KatzeArray* _tmp0_;
	GFile* _tmp1_;
	GFile* _tmp2_;
	GFile* _tmp3_;
	GFile* _tmp4_;
	MidoriApp* _tmp5_;
	GList* _tmp6_;
	MidoriApp* _tmp8_;
#line 478 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 478 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (app != NULL);
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = katze_array_new (APPS_TYPE_LAUNCHER);
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->array);
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->array = _tmp0_;
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__g_list_free__g_object_unref0_0 (self->monitors);
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->monitors = NULL;
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = apps_launcher_get_app_folder ();
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->app_folder);
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->app_folder = _tmp1_;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = self->app_folder;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_populate_apps (self, _tmp2_, NULL, NULL);
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp3_ = apps_launcher_get_profile_folder ();
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->profile_folder);
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->profile_folder = _tmp3_;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp4_ = self->profile_folder;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_populate_apps (self, _tmp4_, NULL, NULL);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__g_list_free__g_object_unref0_0 (self->widgets);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->widgets = NULL;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp5_ = app;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = midori_app_get_browsers (_tmp5_);
#line 4549 "apps.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		browser_collection = _tmp6_;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 4557 "apps.c"
			MidoriBrowser* browser = NULL;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 4561 "apps.c"
			{
				MidoriBrowser* _tmp7_;
#line 490 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp7_ = browser;
#line 490 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				apps_manager_browser_added (self, _tmp7_);
#line 4568 "apps.c"
			}
		}
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_list_free0 (browser_collection);
#line 4573 "apps.c"
	}
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = app;
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object (_tmp8_, "add-browser", (GCallback) _apps_manager_browser_added_midori_app_add_browser, self, 0);
#line 4579 "apps.c"
}


static void apps_manager_deactivated (AppsManager* self) {
	MidoriApp* app = NULL;
	MidoriApp* _tmp0_;
	MidoriApp* _tmp1_;
	GList* _tmp2_;
	MidoriApp* _tmp6_;
	guint _tmp7_;
	GList* _tmp8_;
	MidoriApp* _tmp11_;
	GList* _tmp12_;
#line 494 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_return_if_fail (self != NULL);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = midori_extension_get_app ((MidoriExtension*) self);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	app = _tmp1_;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = self->monitors;
#line 4603 "apps.c"
	{
		GList* monitor_collection = NULL;
		GList* monitor_it = NULL;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		monitor_collection = _tmp2_;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		for (monitor_it = monitor_collection; monitor_it != NULL; monitor_it = monitor_it->next) {
#line 4611 "apps.c"
			GFileMonitor* _tmp3_;
			GFileMonitor* monitor = NULL;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp3_ = _g_object_ref0 ((GFileMonitor*) monitor_it->data);
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			monitor = _tmp3_;
#line 4618 "apps.c"
			{
				GFileMonitor* _tmp4_;
				guint _tmp5_;
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp4_ = monitor;
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				g_signal_parse_name ("changed", g_file_monitor_get_type (), &_tmp5_, NULL, FALSE);
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				g_signal_handlers_disconnect_matched (_tmp4_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp5_, 0, NULL, (GCallback) _apps_manager_app_changed_g_file_monitor_changed, self);
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (monitor);
#line 4630 "apps.c"
			}
		}
	}
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__g_list_free__g_object_unref0_0 (self->monitors);
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self->monitors = NULL;
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp6_ = app;
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_parse_name ("add-browser", MIDORI_TYPE_APP, &_tmp7_, NULL, FALSE);
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_handlers_disconnect_matched (_tmp6_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp7_, 0, NULL, (GCallback) _apps_manager_browser_added_midori_app_add_browser, self);
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp8_ = self->widgets;
#line 4646 "apps.c"
	{
		GList* widget_collection = NULL;
		GList* widget_it = NULL;
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		widget_collection = _tmp8_;
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		for (widget_it = widget_collection; widget_it != NULL; widget_it = widget_it->next) {
#line 4654 "apps.c"
			GtkWidget* _tmp9_;
			GtkWidget* widget = NULL;
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			_tmp9_ = _g_object_ref0 ((GtkWidget*) widget_it->data);
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			widget = _tmp9_;
#line 4661 "apps.c"
			{
				GtkWidget* _tmp10_;
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp10_ = widget;
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				gtk_object_destroy ((GtkObject*) _tmp10_);
#line 501 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (widget);
#line 4670 "apps.c"
			}
		}
	}
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp11_ = app;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp12_ = midori_app_get_browsers (_tmp11_);
#line 4678 "apps.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		browser_collection = _tmp12_;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 4686 "apps.c"
			MidoriBrowser* browser = NULL;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 4690 "apps.c"
			{
				GtkActionGroup* action_group = NULL;
				MidoriBrowser* _tmp13_;
				GtkActionGroup* _tmp14_;
				GtkActionGroup* _tmp15_;
				GtkAction* action = NULL;
				GtkActionGroup* _tmp16_;
				GtkAction* _tmp17_;
				GtkAction* _tmp18_;
				GtkActionGroup* _tmp19_;
				GtkAction* _tmp20_;
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp13_ = browser;
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp14_ = midori_browser_get_action_group (_tmp13_);
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp15_ = _g_object_ref0 (_tmp14_);
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				action_group = _tmp15_;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp16_ = action_group;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp17_ = gtk_action_group_get_action (_tmp16_, "CreateLauncher");
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp18_ = _g_object_ref0 (_tmp17_);
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				action = _tmp18_;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp19_ = action_group;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_tmp20_ = action;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				gtk_action_group_remove_action (_tmp19_, _tmp20_);
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (action);
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
				_g_object_unref0 (action_group);
#line 4728 "apps.c"
			}
		}
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_list_free0 (browser_collection);
#line 4733 "apps.c"
	}
#line 494 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (app);
#line 4737 "apps.c"
}


static void _apps_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self) {
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_activated ((AppsManager*) self, app);
#line 4744 "apps.c"
}


static void _apps_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self) {
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_deactivated ((AppsManager*) self);
#line 4751 "apps.c"
}


AppsManager* apps_manager_construct (GType object_type) {
	AppsManager * self = NULL;
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (AppsManager*) g_object_new (object_type, "name", _ ("Web App Manager"), "description", _ ("Manage websites installed as applications"), "version", "0.1" MIDORI_VERSION_SUFFIX, "authors", "Christian Dywan <christian@twotoasts.de>", NULL);
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object ((MidoriExtension*) self, "activate", (GCallback) _apps_manager_activated_midori_extension_activate, self, 0);
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_signal_connect_object ((MidoriExtension*) self, "deactivate", (GCallback) _apps_manager_deactivated_midori_extension_deactivate, self, 0);
#line 511 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return self;
#line 4765 "apps.c"
}


AppsManager* apps_manager_new (void) {
#line 511 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return apps_manager_construct (APPS_TYPE_MANAGER);
#line 4772 "apps.c"
}


static void apps_manager_class_init (AppsManagerClass * klass) {
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_manager_parent_class = g_type_class_peek_parent (klass);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (klass)->finalize = apps_manager_finalize;
#line 4781 "apps.c"
}


static void apps_manager_instance_init (AppsManager * self) {
}


static void apps_manager_finalize (GObject * obj) {
	AppsManager * self;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, APPS_TYPE_MANAGER, AppsManager);
#line 394 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->array);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->app_folder);
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (self->profile_folder);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__g_list_free__g_object_unref0_0 (self->monitors);
#line 398 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	__g_list_free__g_object_unref0_0 (self->widgets);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	G_OBJECT_CLASS (apps_manager_parent_class)->finalize (obj);
#line 4805 "apps.c"
}


GType apps_manager_get_type (void) {
	static volatile gsize apps_manager_type_id__volatile = 0;
	if (g_once_init_enter (&apps_manager_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AppsManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) apps_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AppsManager), 0, (GInstanceInitFunc) apps_manager_instance_init, NULL };
		GType apps_manager_type_id;
		apps_manager_type_id = g_type_register_static (MIDORI_TYPE_EXTENSION, "AppsManager", &g_define_type_info, 0);
		g_once_init_leave (&apps_manager_type_id__volatile, apps_manager_type_id);
	}
	return apps_manager_type_id__volatile;
}


MidoriExtension* extension_init (void) {
	MidoriExtension* result = NULL;
	AppsManager* _tmp0_;
#line 524 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = apps_manager_new ();
#line 524 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	result = (MidoriExtension*) _tmp0_;
#line 524 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return result;
#line 4830 "apps.c"
}


void extensions_apps_desktop_test (void) {
	ExtensionsAppsDesktop* _tmp0_;
	ExtensionsAppsDesktop* _tmp1_;
#line 528 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = extensions_apps_desktop_new ();
#line 528 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = _tmp0_;
#line 528 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	midori_test_job_run_sync ((MidoriTestJob*) _tmp1_);
#line 528 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_tmp1_);
#line 4845 "apps.c"
}


static void extensions_apps_desktop_real_run_data_free (gpointer _data) {
	ExtensionsAppsDesktopRunData* _data_;
	_data_ = _data;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->cancellable);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->self);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_slice_free (ExtensionsAppsDesktopRunData, _data_);
#line 4858 "apps.c"
}


static void extensions_apps_desktop_real_run_async_ready_wrapper (GObject *source_object, GAsyncResult *res, void *user_data) {
	ExtensionsAppsDesktopRunData* _task_data_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_ = g_task_get_task_data (G_TASK (res));
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_task_data_->_callback_ != NULL) {
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_task_data_->_callback_ (source_object, res, user_data);
#line 4870 "apps.c"
	}
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_task_data_->_task_complete_ = TRUE;
#line 4874 "apps.c"
}


static void extensions_apps_desktop_real_run (MidoriTestJob* base, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	ExtensionsAppsDesktop * self;
	ExtensionsAppsDesktopRunData* _data_;
	ExtensionsAppsDesktop* _tmp0_;
	GCancellable* _tmp1_;
	GCancellable* _tmp2_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (ExtensionsAppsDesktop*) base;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_slice_new0 (ExtensionsAppsDesktopRunData);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_callback_ = _callback_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_async_result = g_task_new (G_OBJECT (self), cancellable, extensions_apps_desktop_real_run_async_ready_wrapper, _user_data_);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_callback_ == NULL) {
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_data_->_task_complete_ = TRUE;
#line 4896 "apps.c"
	}
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_set_task_data (_data_->_async_result, _data_, extensions_apps_desktop_real_run_data_free);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->self = _tmp0_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp1_ = cancellable;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->cancellable);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->cancellable = _tmp2_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	extensions_apps_desktop_real_run_co (_data_);
#line 4914 "apps.c"
}


static void extensions_apps_desktop_real_run_finish (MidoriTestJob* base, GAsyncResult* _res_, GError** error) {
	ExtensionsAppsDesktopRunData* _data_;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = g_task_propagate_pointer (G_TASK (_res_), error);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (NULL == _data_) {
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return;
#line 4926 "apps.c"
	}
}


static void extensions_apps_desktop_run_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	ExtensionsAppsDesktopRunData* _data_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_ = _user_data_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_source_object_ = source_object;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_res_ = _res_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_task_complete_ = TRUE;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	extensions_apps_desktop_real_run_co (_data_);
#line 4943 "apps.c"
}


static gboolean extensions_apps_desktop_real_run_co (ExtensionsAppsDesktopRunData* _data_) {
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	switch (_data_->_state_) {
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 0:
#line 4952 "apps.c"
		goto _state_0;
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 1:
#line 4956 "apps.c"
		goto _state_1;
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		case 2:
#line 4960 "apps.c"
		goto _state_2;
		default:
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_assert_not_reached ();
#line 4965 "apps.c"
	}
	_state_0:
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp0_ = g_strdup ("http://example.com");
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->uri = _data_->_tmp0_;
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp1_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _data_->uri, (gsize) -1);
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->checksum = _data_->_tmp1_;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp2_ = apps_launcher_get_app_folder ();
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp3_ = _data_->_tmp2_;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp4_ = g_file_get_child (_data_->_tmp3_, _data_->checksum);
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp5_ = _data_->_tmp4_;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->_tmp3_);
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->apps = _data_->_tmp5_;
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp6_ = g_file_get_path (_data_->apps);
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp7_ = _data_->_tmp6_;
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	midori_paths_remove_path (_data_->_tmp7_);
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->_tmp7_);
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp8_ = midori_paths_get_user_data_dir ();
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp9_ = g_file_new_for_path (_data_->_tmp8_);
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->data_dir = _data_->_tmp9_;
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp10_ = g_file_get_child (_data_->data_dir, "applications");
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->desktop_dir = _data_->_tmp10_;
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp11_ = g_file_get_child (_data_->desktop_dir, "Example.desktop");
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp12_ = _data_->_tmp11_;
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp13_ = g_file_get_path (_data_->_tmp12_);
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp14_ = _data_->_tmp13_;
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	midori_paths_remove_path (_data_->_tmp14_);
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->_tmp14_);
#line 537 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->_tmp12_);
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_state_ = 1;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_app (_data_->uri, "Example", NULL, extensions_apps_desktop_run_ready, _data_);
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 5026 "apps.c"
	_state_1:
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp15_ = apps_launcher_create_app_finish (_data_->_res_);
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->folder = _data_->_tmp15_;
#line 540 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp16_ = apps_launcher_new (_data_->folder);
#line 540 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->launcher = _data_->_tmp16_;
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_initable_init ((GInitable*) _data_->launcher, NULL, &_data_->_inner_error_);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (G_UNLIKELY (_data_->_inner_error_ != NULL)) {
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_task_return_error (_data_->_async_result, _data_->_inner_error_);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->launcher);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->folder);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->desktop_dir);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->data_dir);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_object_unref0 (_data_->apps);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->checksum);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		_g_free0 (_data_->uri);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		g_object_unref (_data_->_async_result);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		return FALSE;
#line 5060 "apps.c"
	}
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp17_ = g_file_get_path (_data_->folder);
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp18_ = _data_->_tmp17_;
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp19_ = _data_->launcher->uri;
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	katze_assert_str_equal (_data_->_tmp18_, _data_->_tmp19_, _data_->uri);
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->_tmp18_);
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_state_ = 2;
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	apps_launcher_create_profile (NULL, extensions_apps_desktop_run_ready, _data_);
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 5078 "apps.c"
	_state_2:
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp20_ = apps_launcher_create_profile_finish (_data_->_res_);
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_data_->_tmp21_ = _data_->_tmp20_;
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->_tmp21_);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->launcher);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->folder);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->desktop_dir);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->data_dir);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_object_unref0 (_data_->apps);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->checksum);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	_g_free0 (_data_->uri);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_task_return_pointer (_data_->_async_result, _data_, NULL);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	if (_data_->_state_ != 0) {
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
		while (_data_->_task_complete_ != TRUE) {
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
			g_main_context_iteration (g_task_get_context (_data_->_async_result), TRUE);
#line 5108 "apps.c"
		}
	}
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_object_unref (_data_->_async_result);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return FALSE;
#line 5115 "apps.c"
}


ExtensionsAppsDesktop* extensions_apps_desktop_construct (GType object_type) {
	ExtensionsAppsDesktop * self = NULL;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	self = (ExtensionsAppsDesktop*) midori_test_job_construct (object_type);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return self;
#line 5125 "apps.c"
}


ExtensionsAppsDesktop* extensions_apps_desktop_new (void) {
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	return extensions_apps_desktop_construct (TYPE_EXTENSIONS_APPS_DESKTOP);
#line 5132 "apps.c"
}


static void extensions_apps_desktop_class_init (ExtensionsAppsDesktopClass * klass) {
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	extensions_apps_desktop_parent_class = g_type_class_peek_parent (klass);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	((MidoriTestJobClass *) klass)->run = (void (*) (MidoriTestJob *, GCancellable*, GAsyncReadyCallback, gpointer)) extensions_apps_desktop_real_run;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	((MidoriTestJobClass *) klass)->run_finish = (void (*) (MidoriTestJob *, GAsyncResult*, GError**)) extensions_apps_desktop_real_run_finish;
#line 5143 "apps.c"
}


static void extensions_apps_desktop_instance_init (ExtensionsAppsDesktop * self) {
}


GType extensions_apps_desktop_get_type (void) {
	static volatile gsize extensions_apps_desktop_type_id__volatile = 0;
	if (g_once_init_enter (&extensions_apps_desktop_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExtensionsAppsDesktopClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) extensions_apps_desktop_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExtensionsAppsDesktop), 0, (GInstanceInitFunc) extensions_apps_desktop_instance_init, NULL };
		GType extensions_apps_desktop_type_id;
		extensions_apps_desktop_type_id = g_type_register_static (MIDORI_TEST_TYPE_JOB, "ExtensionsAppsDesktop", &g_define_type_info, 0);
		g_once_init_leave (&extensions_apps_desktop_type_id__volatile, extensions_apps_desktop_type_id);
	}
	return extensions_apps_desktop_type_id__volatile;
}


static void _extensions_apps_desktop_test_gtest_func (void) {
#line 548 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	extensions_apps_desktop_test ();
#line 5166 "apps.c"
}


void extension_test (void) {
#line 548 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/apps.vala"
	g_test_add_func ("/extensions/apps/desktop", _extensions_apps_desktop_test_gtest_func);
#line 5173 "apps.c"
}



