/* subscriptions.c generated by valac 0.38.8, the Vala compiler
 * generated from subscriptions.vala, do not modify */

/*
 Copyright (C) 2009-2014 Christian Dywan <christian@twotoasts.de>
 Copyright (C) 2009-2012 Alexander Butenko <a.butenka@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include "toolbar-editor.h"
#include <stdlib.h>
#include <string.h>
#include <gio/gio.h>
#include "webkit/webkit.h"
#include <stdio.h>
#include "midori-core.h"

enum  {
	ADBLOCK_FEATURE_0_PROPERTY,
	ADBLOCK_FEATURE_NUM_PROPERTIES
};
static GParamSpec* adblock_feature_properties[ADBLOCK_FEATURE_NUM_PROPERTIES];
enum  {
	ADBLOCK_SUBSCRIPTION_0_PROPERTY,
	ADBLOCK_SUBSCRIPTION_URI_PROPERTY,
	ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY,
	ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY,
	ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY,
	ADBLOCK_SUBSCRIPTION_VALID_PROPERTY,
	ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY,
	ADBLOCK_SUBSCRIPTION_NUM_PROPERTIES
};
static GParamSpec* adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _AdblockSubscriptionPrivate {
	gboolean debug_parse;
	gchar* _uri;
	gchar* _title;
	gboolean _active;
	gboolean _mutable;
	gboolean _valid;
	GHashTable* cache;
	GList* features;
	WebKitDownload* download;
	guint _size;
};


static gpointer adblock_feature_parent_class = NULL;
static gpointer adblock_subscription_parent_class = NULL;

static gboolean adblock_feature_real_header (AdblockFeature* self, const gchar* key, const gchar* value);
static gboolean adblock_feature_real_parsed (AdblockFeature* self, GFile* file);
static AdblockDirective* adblock_feature_real_match (AdblockFeature* self, const gchar* request_uri, const gchar* page_uri, GError** error);
static void adblock_feature_real_clear (AdblockFeature* self);
#define ADBLOCK_SUBSCRIPTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ADBLOCK_TYPE_SUBSCRIPTION, AdblockSubscriptionPrivate))
static void _g_object_unref0_ (gpointer var);
static inline void _g_list_free__g_object_unref0_ (GList* self);
static void adblock_subscription_set_size (AdblockSubscription* self, guint value);
static void _g_free0_ (gpointer var);
void adblock_subscription_parse_line (AdblockSubscription* self, const gchar* line, GError** error);
static void adblock_subscription_add_url_pattern (AdblockSubscription* self, const gchar* prefix, const gchar* type, const gchar* line, GError** error);
static void adblock_subscription_frame_add_private (AdblockSubscription* self, const gchar* line, const gchar* sep);
static void adblock_subscription_update_css_hash (AdblockSubscription* self, const gchar* domain, const gchar* value);
static gboolean adblock_subscription_css_element_seems_valid (AdblockSubscription* self, const gchar* element);
static gboolean _vala_string_array_contains (gchar* * stack, int stack_length, gchar* needle);
gchar* adblock_fixup_regex (const gchar* prefix, const gchar* src);
static gboolean adblock_subscription_compile_regexp (AdblockSubscription* self, const gchar* patt, const gchar* opts, GError** error);
void adblock_debug (const gchar* format, ...);
static void adblock_subscription_download_status (AdblockSubscription* self, GParamSpec* pspec);
static void _adblock_subscription_download_status_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void adblock_subscription_set_valid (AdblockSubscription* self, gboolean value);
static AdblockDirective* _adblock_directive_dup (AdblockDirective* self);
static void adblock_subscription_finalize (GObject * obj);
static void _vala_adblock_subscription_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_adblock_subscription_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


static gboolean adblock_feature_real_header (AdblockFeature* self, const gchar* key, const gchar* value) {
	gboolean result = FALSE;
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (key != NULL, FALSE);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (value != NULL, FALSE);
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = FALSE;
#line 16 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 107 "subscriptions.c"
}


gboolean adblock_feature_header (AdblockFeature* self, const gchar* key, const gchar* value) {
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 15 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return ADBLOCK_FEATURE_GET_CLASS (self)->header (self, key, value);
#line 116 "subscriptions.c"
}


static gboolean adblock_feature_real_parsed (AdblockFeature* self, GFile* file) {
	gboolean result = FALSE;
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (file != NULL, FALSE);
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = TRUE;
#line 19 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 128 "subscriptions.c"
}


gboolean adblock_feature_parsed (AdblockFeature* self, GFile* file) {
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 18 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return ADBLOCK_FEATURE_GET_CLASS (self)->parsed (self, file);
#line 137 "subscriptions.c"
}


static AdblockDirective* adblock_feature_real_match (AdblockFeature* self, const gchar* request_uri, const gchar* page_uri, GError** error) {
	AdblockDirective* result = NULL;
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (request_uri != NULL, NULL);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (page_uri != NULL, NULL);
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = NULL;
#line 22 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 151 "subscriptions.c"
}


AdblockDirective* adblock_feature_match (AdblockFeature* self, const gchar* request_uri, const gchar* page_uri, GError** error) {
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 21 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return ADBLOCK_FEATURE_GET_CLASS (self)->match (self, request_uri, page_uri, error);
#line 160 "subscriptions.c"
}


static void adblock_feature_real_clear (AdblockFeature* self) {
}


void adblock_feature_clear (AdblockFeature* self) {
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	ADBLOCK_FEATURE_GET_CLASS (self)->clear (self);
#line 173 "subscriptions.c"
}


AdblockFeature* adblock_feature_construct (GType object_type) {
	AdblockFeature * self = NULL;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self = (AdblockFeature*) g_object_new (object_type, NULL);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return self;
#line 183 "subscriptions.c"
}


static void adblock_feature_class_init (AdblockFeatureClass * klass) {
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_feature_parent_class = g_type_class_peek_parent (klass);
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	((AdblockFeatureClass *) klass)->header = (gboolean (*) (AdblockFeature *, const gchar*, const gchar*)) adblock_feature_real_header;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	((AdblockFeatureClass *) klass)->parsed = (gboolean (*) (AdblockFeature *, GFile*)) adblock_feature_real_parsed;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	((AdblockFeatureClass *) klass)->match = (AdblockDirective* (*) (AdblockFeature *, const gchar*, const gchar*, GError**)) adblock_feature_real_match;
#line 14 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	((AdblockFeatureClass *) klass)->clear = (void (*) (AdblockFeature *)) adblock_feature_real_clear;
#line 198 "subscriptions.c"
}


static void adblock_feature_instance_init (AdblockFeature * self) {
}


GType adblock_feature_get_type (void) {
	static volatile gsize adblock_feature_type_id__volatile = 0;
	if (g_once_init_enter (&adblock_feature_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AdblockFeatureClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adblock_feature_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdblockFeature), 0, (GInstanceInitFunc) adblock_feature_instance_init, NULL };
		GType adblock_feature_type_id;
		adblock_feature_type_id = g_type_register_static (G_TYPE_OBJECT, "AdblockFeature", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&adblock_feature_type_id__volatile, adblock_feature_type_id);
	}
	return adblock_feature_type_id__volatile;
}


static void _g_object_unref0_ (gpointer var) {
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 221 "subscriptions.c"
}


static inline void _g_list_free__g_object_unref0_ (GList* self) {
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_list_free_full (self, (GDestroyNotify) _g_object_unref0_);
#line 228 "subscriptions.c"
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	gchar* _tmp1_;
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 248 "subscriptions.c"
}


AdblockSubscription* adblock_subscription_construct (GType object_type, const gchar* uri) {
	AdblockSubscription * self = NULL;
	const gchar* _tmp0_ = NULL;
	const gchar* _tmp1_;
	gboolean _tmp2_;
	const gchar* _tmp3_;
	AdblockOptions* _tmp4_;
	AdblockOptions* _tmp5_;
	AdblockWhitelist* _tmp6_;
	AdblockWhitelist* _tmp7_;
	AdblockOptions* _tmp8_;
	AdblockKeys* _tmp9_;
	AdblockKeys* _tmp10_;
	AdblockOptions* _tmp11_;
	AdblockPattern* _tmp12_;
	AdblockPattern* _tmp13_;
	AdblockElement* _tmp14_;
	AdblockElement* _tmp15_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self = (AdblockSubscription*) g_object_new (object_type, NULL);
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = g_getenv ("MIDORI_DEBUG");
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = _tmp1_;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp0_ == NULL) {
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = "";
#line 282 "subscriptions.c"
	}
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = string_contains (_tmp0_, "adblock:parse");
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->debug_parse = _tmp2_;
#line 50 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = uri;
#line 50 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_set_uri (self, _tmp3_);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4_ = adblock_options_new ();
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->optslist);
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->optslist = _tmp4_;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp5_ = self->optslist;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp6_ = adblock_whitelist_new (_tmp5_);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->whitelist);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->whitelist = _tmp6_;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7_ = self->whitelist;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_add_feature (self, (AdblockFeature*) _tmp7_);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp8_ = self->optslist;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp9_ = adblock_keys_new (_tmp8_);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->keys);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->keys = _tmp9_;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp10_ = self->keys;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_add_feature (self, (AdblockFeature*) _tmp10_);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp11_ = self->optslist;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp12_ = adblock_pattern_new (_tmp11_);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->pattern);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->pattern = _tmp12_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp13_ = self->pattern;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_add_feature (self, (AdblockFeature*) _tmp13_);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp14_ = adblock_element_new ();
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->element);
#line 59 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->element = _tmp14_;
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp15_ = self->element;
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_add_feature (self, (AdblockFeature*) _tmp15_);
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_clear (self);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return self;
#line 348 "subscriptions.c"
}


AdblockSubscription* adblock_subscription_new (const gchar* uri) {
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return adblock_subscription_construct (ADBLOCK_TYPE_SUBSCRIPTION, uri);
#line 355 "subscriptions.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return self ? g_object_ref (self) : NULL;
#line 362 "subscriptions.c"
}


void adblock_subscription_add_feature (AdblockSubscription* self, AdblockFeature* feature) {
	AdblockFeature* _tmp0_;
	AdblockFeature* _tmp1_;
	guint _tmp2_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (feature != NULL);
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = feature;
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->features = g_list_append (self->priv->features, _tmp1_);
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = self->priv->_size;
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_set_size (self, _tmp2_ + 1);
#line 384 "subscriptions.c"
}


AdblockFeature* adblock_subscription_get (AdblockSubscription* self, guint index) {
	AdblockFeature* result = NULL;
	GList* _tmp0_;
	guint _tmp1_;
	gconstpointer _tmp2_;
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->features;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = index;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = g_list_nth_data (_tmp0_, _tmp1_);
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = (AdblockFeature*) _tmp2_;
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 405 "subscriptions.c"
}


static void _g_free0_ (gpointer var) {
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	var = (g_free (var), NULL);
#line 412 "subscriptions.c"
}


void adblock_subscription_clear (AdblockSubscription* self) {
	GHashFunc _tmp0_;
	GEqualFunc _tmp1_;
	GHashTable* _tmp2_;
	GList* _tmp3_;
	AdblockOptions* _tmp5_;
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = g_str_hash;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = g_str_equal;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = g_hash_table_new_full (_tmp0_, _tmp1_, _g_free0_, _g_free0_);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_hash_table_unref0 (self->priv->cache);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->cache = _tmp2_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = self->priv->features;
#line 436 "subscriptions.c"
	{
		GList* feature_collection = NULL;
		GList* feature_it = NULL;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		feature_collection = _tmp3_;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		for (feature_it = feature_collection; feature_it != NULL; feature_it = feature_it->next) {
#line 444 "subscriptions.c"
			AdblockFeature* feature = NULL;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			feature = (AdblockFeature*) feature_it->data;
#line 448 "subscriptions.c"
			{
				AdblockFeature* _tmp4_;
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp4_ = feature;
#line 78 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				adblock_feature_clear (_tmp4_);
#line 455 "subscriptions.c"
			}
		}
	}
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp5_ = self->optslist;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_options_clear (_tmp5_);
#line 463 "subscriptions.c"
}


static const gchar* string_offset (const gchar* self, glong offset) {
	const gchar* result = NULL;
	glong _tmp0_;
#line 1157 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1158 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = offset;
#line 1158 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = (const gchar*) (((gchar*) self) + _tmp0_);
#line 1158 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 478 "subscriptions.c"
}


static gchar string_get (const gchar* self, glong index) {
	gchar result = '\0';
	glong _tmp0_;
	gchar _tmp1_;
#line 1118 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, '\0');
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = index;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = ((gchar*) self)[_tmp0_];
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 496 "subscriptions.c"
}


void adblock_subscription_parse_line (AdblockSubscription* self, const gchar* line, GError** error) {
	const gchar* _tmp0_;
	gboolean _tmp1_;
	const gchar* _tmp17_;
	gchar _tmp18_;
	const gchar* _tmp19_;
	gboolean _tmp20_;
	const gchar* _tmp21_;
	gchar _tmp22_;
	const gchar* _tmp23_;
	gboolean _tmp24_;
	const gchar* _tmp25_;
	gboolean _tmp26_;
	const gchar* _tmp28_;
	gboolean _tmp29_;
	const gchar* _tmp31_;
	gboolean _tmp32_;
	const gchar* _tmp41_;
	GError * _inner_error_ = NULL;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = line;
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = g_str_has_prefix (_tmp0_, "@@");
#line 83 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp1_) {
#line 527 "subscriptions.c"
		gboolean _tmp2_ = FALSE;
		const gchar* _tmp3_;
		gboolean _tmp4_;
		const gchar* _tmp7_;
		gboolean _tmp8_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = line;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = string_contains (_tmp3_, "$");
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp4_) {
#line 539 "subscriptions.c"
			const gchar* _tmp5_;
			gboolean _tmp6_;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp5_ = line;
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp6_ = string_contains (_tmp5_, "domain");
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp2_ = _tmp6_;
#line 548 "subscriptions.c"
		} else {
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp2_ = FALSE;
#line 552 "subscriptions.c"
		}
#line 84 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp2_) {
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return;
#line 558 "subscriptions.c"
		}
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp7_ = line;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp8_ = g_str_has_prefix (_tmp7_, "@@||");
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp8_) {
#line 566 "subscriptions.c"
			const gchar* _tmp9_;
			const gchar* _tmp10_;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp9_ = line;
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp10_ = string_offset (_tmp9_, (glong) 4);
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_subscription_add_url_pattern (self, "^", "whitelist", _tmp10_, &_inner_error_);
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				g_propagate_error (error, _inner_error_);
#line 87 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 581 "subscriptions.c"
			}
		} else {
			const gchar* _tmp11_;
			gboolean _tmp12_;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp11_ = line;
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp12_ = g_str_has_prefix (_tmp11_, "@@|");
#line 88 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (_tmp12_) {
#line 592 "subscriptions.c"
				const gchar* _tmp13_;
				const gchar* _tmp14_;
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp13_ = line;
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp14_ = string_offset (_tmp13_, (glong) 3);
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				adblock_subscription_add_url_pattern (self, "^", "whitelist", _tmp14_, &_inner_error_);
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					g_propagate_error (error, _inner_error_);
#line 89 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					return;
#line 607 "subscriptions.c"
				}
			} else {
				const gchar* _tmp15_;
				const gchar* _tmp16_;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp15_ = line;
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp16_ = string_offset (_tmp15_, (glong) 2);
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				adblock_subscription_add_url_pattern (self, "", "whitelist", _tmp16_, &_inner_error_);
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					g_propagate_error (error, _inner_error_);
#line 91 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					return;
#line 624 "subscriptions.c"
				}
			}
		}
#line 92 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 630 "subscriptions.c"
	}
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp17_ = line;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp18_ = string_get (_tmp17_, (glong) 0);
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp18_ == '[') {
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 640 "subscriptions.c"
	}
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp19_ = line;
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp20_ = g_str_has_prefix (_tmp19_, "##");
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp20_) {
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 650 "subscriptions.c"
	}
#line 103 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp21_ = line;
#line 103 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp22_ = string_get (_tmp21_, (glong) 0);
#line 103 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp22_ == '#') {
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 660 "subscriptions.c"
	}
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp23_ = line;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp24_ = string_contains (_tmp23_, "#@#");
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp24_) {
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 670 "subscriptions.c"
	}
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp25_ = line;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp26_ = string_contains (_tmp25_, "##");
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp26_) {
#line 678 "subscriptions.c"
		const gchar* _tmp27_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp27_ = line;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_frame_add_private (self, _tmp27_, "##");
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 686 "subscriptions.c"
	}
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp28_ = line;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp29_ = string_contains (_tmp28_, "#");
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp29_) {
#line 694 "subscriptions.c"
		const gchar* _tmp30_;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp30_ = line;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_frame_add_private (self, _tmp30_, "#");
#line 117 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 702 "subscriptions.c"
	}
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp31_ = line;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp32_ = g_str_has_prefix (_tmp31_, "|");
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp32_) {
#line 710 "subscriptions.c"
		const gchar* _tmp33_;
		gboolean _tmp34_;
		const gchar* _tmp35_;
		gboolean _tmp36_;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp33_ = line;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp34_ = string_contains (_tmp33_, "$");
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp34_) {
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return;
#line 723 "subscriptions.c"
		}
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp35_ = line;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp36_ = g_str_has_prefix (_tmp35_, "||");
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp36_) {
#line 731 "subscriptions.c"
			const gchar* _tmp37_;
			const gchar* _tmp38_;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp37_ = line;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp38_ = string_offset (_tmp37_, (glong) 2);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_subscription_add_url_pattern (self, "", "fulluri", _tmp38_, &_inner_error_);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				g_propagate_error (error, _inner_error_);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 746 "subscriptions.c"
			}
		} else {
			const gchar* _tmp39_;
			const gchar* _tmp40_;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp39_ = line;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp40_ = string_offset (_tmp39_, (glong) 1);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_subscription_add_url_pattern (self, "^", "fulluri", _tmp40_, &_inner_error_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				g_propagate_error (error, _inner_error_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 763 "subscriptions.c"
			}
		}
#line 130 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 768 "subscriptions.c"
	}
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp41_ = line;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_add_url_pattern (self, "", "uri", _tmp41_, &_inner_error_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_propagate_error (error, _inner_error_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 780 "subscriptions.c"
	}
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return;
#line 784 "subscriptions.c"
}


static gchar* string_strip (const gchar* self) {
	gchar* result = NULL;
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	const gchar* _tmp1_;
#line 1242 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1243 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1243 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1244 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _result_;
#line 1244 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_strstrip (_tmp1_);
#line 1245 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _result_;
#line 1245 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 807 "subscriptions.c"
}


static glong string_strnlen (gchar* str, glong maxlen) {
	glong result = 0L;
	gchar* end = NULL;
	gchar* _tmp0_;
	glong _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = str;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = maxlen;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = memchr (_tmp0_, 0, (gsize) _tmp1_);
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	end = _tmp2_;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = end;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp3_ == NULL) {
#line 830 "subscriptions.c"
		glong _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = maxlen;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 838 "subscriptions.c"
	} else {
		gchar* _tmp5_;
		gchar* _tmp6_;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = end;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = str;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = (glong) (_tmp5_ - _tmp6_);
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 850 "subscriptions.c"
	}
}


static gchar* string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	glong _tmp1_;
	glong _tmp8_;
	glong _tmp14_;
	glong _tmp17_;
	glong _tmp18_;
	glong _tmp19_;
	glong _tmp20_;
	glong _tmp21_;
	gchar* _tmp22_;
#line 1341 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = offset;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_ >= ((glong) 0)) {
#line 874 "subscriptions.c"
		glong _tmp2_;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = len;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = _tmp2_ >= ((glong) 0);
#line 880 "subscriptions.c"
	} else {
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = FALSE;
#line 884 "subscriptions.c"
	}
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 888 "subscriptions.c"
		glong _tmp3_;
		glong _tmp4_;
		glong _tmp5_;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = offset;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = len;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = string_strnlen ((gchar*) self, _tmp3_ + _tmp4_);
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = _tmp5_;
#line 900 "subscriptions.c"
	} else {
		gint _tmp6_;
		gint _tmp7_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = strlen (self);
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = _tmp6_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = (glong) _tmp7_;
#line 910 "subscriptions.c"
	}
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp8_ = offset;
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp8_ < ((glong) 0)) {
#line 916 "subscriptions.c"
		glong _tmp9_;
		glong _tmp10_;
		glong _tmp11_;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = string_length;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = offset;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		offset = _tmp9_ + _tmp10_;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = offset;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp11_ >= ((glong) 0), NULL);
#line 930 "subscriptions.c"
	} else {
		glong _tmp12_;
		glong _tmp13_;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = offset;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = string_length;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp12_ <= _tmp13_, NULL);
#line 940 "subscriptions.c"
	}
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp14_ = len;
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp14_ < ((glong) 0)) {
#line 946 "subscriptions.c"
		glong _tmp15_;
		glong _tmp16_;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = string_length;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp16_ = offset;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		len = _tmp15_ - _tmp16_;
#line 955 "subscriptions.c"
	}
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp17_ = offset;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp18_ = len;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp19_ = string_length;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail ((_tmp17_ + _tmp18_) <= _tmp19_, NULL);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp20_ = offset;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp21_ = len;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp22_ = g_strndup (((gchar*) self) + _tmp20_, (gsize) _tmp21_);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp22_;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 975 "subscriptions.c"
}


static void adblock_subscription_frame_add_private (AdblockSubscription* self, const gchar* line, const gchar* sep) {
	gchar** data = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar** _tmp2_;
	gchar** _tmp3_;
	gint data_length1;
	gint _data_size_;
	gboolean _tmp4_ = FALSE;
	gboolean _tmp5_ = FALSE;
	gboolean _tmp6_ = FALSE;
	gchar** _tmp7_;
	gint _tmp7__length1;
	const gchar* _tmp8_;
	gchar** _tmp21_;
	gint _tmp21__length1;
	const gchar* _tmp22_;
	const gchar* _tmp23_;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (line != NULL);
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (sep != NULL);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = line;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = sep;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = _tmp2_ = g_strsplit (_tmp0_, _tmp1_, 2);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data = _tmp3_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data_length1 = _vala_array_length (_tmp2_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_data_size_ = data_length1;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7_ = data;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7__length1 = data_length1;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp8_ = _tmp7_[1];
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp8_ != NULL) {
#line 1023 "subscriptions.c"
		gchar** _tmp9_;
		gint _tmp9__length1;
		const gchar* _tmp10_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp9_ = data;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp9__length1 = data_length1;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp10_ = _tmp9_[1];
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp6_ = g_strcmp0 (_tmp10_, "") != 0;
#line 1035 "subscriptions.c"
	} else {
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp6_ = FALSE;
#line 1039 "subscriptions.c"
	}
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (!_tmp6_) {
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = TRUE;
#line 1045 "subscriptions.c"
	} else {
		gchar** _tmp11_;
		gint _tmp11__length1;
		const gchar* _tmp12_;
		const gchar* _tmp13_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11_ = data;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11__length1 = data_length1;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp12_ = _tmp11_[1];
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp13_ = g_utf8_strchr (_tmp12_, (gssize) -1, (gunichar) '\'');
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = _tmp13_ != NULL;
#line 1061 "subscriptions.c"
	}
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp5_) {
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = TRUE;
#line 1067 "subscriptions.c"
	} else {
		gboolean _tmp14_ = FALSE;
		gchar** _tmp15_;
		gint _tmp15__length1;
		const gchar* _tmp16_;
		const gchar* _tmp17_;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp15_ = data;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp15__length1 = data_length1;
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp16_ = _tmp15_[1];
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp17_ = g_utf8_strchr (_tmp16_, (gssize) -1, (gunichar) ':');
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp17_ != NULL) {
#line 1084 "subscriptions.c"
			gchar** _tmp18_;
			gint _tmp18__length1;
			const gchar* _tmp19_;
			gboolean _tmp20_;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp18_ = data;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp18__length1 = data_length1;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp19_ = _tmp18_[1];
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp20_ = g_regex_match_simple (".*\\[.*:.*\\].*", _tmp19_, G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp14_ = !_tmp20_;
#line 1099 "subscriptions.c"
		} else {
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp14_ = FALSE;
#line 1103 "subscriptions.c"
		}
#line 141 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = _tmp14_;
#line 1107 "subscriptions.c"
	}
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp4_) {
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 1115 "subscriptions.c"
	}
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp21_ = data;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp21__length1 = data_length1;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp22_ = _tmp21_[0];
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp23_ = g_utf8_strchr (_tmp22_, (gssize) -1, (gunichar) ',');
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp23_ != NULL) {
#line 1127 "subscriptions.c"
		gchar** domains = NULL;
		gchar** _tmp24_;
		gint _tmp24__length1;
		const gchar* _tmp25_;
		gchar** _tmp26_;
		gchar** _tmp27_;
		gint domains_length1;
		gint _domains_size_;
		gchar** _tmp28_;
		gint _tmp28__length1;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp24_ = data;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp24__length1 = data_length1;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp25_ = _tmp24_[0];
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp27_ = _tmp26_ = g_strsplit (_tmp25_, ",", -1);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		domains = _tmp27_;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		domains_length1 = _vala_array_length (_tmp26_);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_domains_size_ = domains_length1;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp28_ = domains;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp28__length1 = domains_length1;
#line 1156 "subscriptions.c"
		{
			gchar** domain_collection = NULL;
			gint domain_collection_length1 = 0;
			gint _domain_collection_size_ = 0;
			gint domain_it = 0;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			domain_collection = _tmp28_;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			domain_collection_length1 = _tmp28__length1;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			for (domain_it = 0; domain_it < _tmp28__length1; domain_it = domain_it + 1) {
#line 1168 "subscriptions.c"
				const gchar* domain = NULL;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				domain = domain_collection[domain_it];
#line 1172 "subscriptions.c"
				{
					const gchar* _tmp29_;
					gchar* stripped = NULL;
					const gchar* _tmp30_;
					gchar* _tmp31_;
					const gchar* _tmp32_;
					gchar _tmp33_;
					const gchar* _tmp36_;
					gchar** _tmp37_;
					gint _tmp37__length1;
					const gchar* _tmp38_;
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp29_ = domain;
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (g_strcmp0 (_tmp29_, "~pregecko2") == 0) {
#line 153 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						continue;
#line 1190 "subscriptions.c"
					}
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp30_ = domain;
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp31_ = string_strip (_tmp30_);
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					stripped = _tmp31_;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp32_ = stripped;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp33_ = string_get (_tmp32_, (glong) 0);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (_tmp33_ == '~') {
#line 1204 "subscriptions.c"
						const gchar* _tmp34_;
						gchar* _tmp35_;
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp34_ = stripped;
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp35_ = string_substring (_tmp34_, (glong) 1, (glong) -1);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_g_free0 (stripped);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						stripped = _tmp35_;
#line 1215 "subscriptions.c"
					}
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp36_ = stripped;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp37_ = data;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp37__length1 = data_length1;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp38_ = _tmp37_[1];
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					adblock_subscription_update_css_hash (self, _tmp36_, _tmp38_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_g_free0 (stripped);
#line 1229 "subscriptions.c"
				}
			}
		}
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		domains = (_vala_array_free (domains, domains_length1, (GDestroyNotify) g_free), NULL);
#line 1235 "subscriptions.c"
	} else {
		gchar** _tmp39_;
		gint _tmp39__length1;
		const gchar* _tmp40_;
		gchar** _tmp41_;
		gint _tmp41__length1;
		const gchar* _tmp42_;
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp39_ = data;
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp39__length1 = data_length1;
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp40_ = _tmp39_[0];
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp41_ = data;
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp41__length1 = data_length1;
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp42_ = _tmp41_[1];
#line 162 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_update_css_hash (self, _tmp40_, _tmp42_);
#line 1257 "subscriptions.c"
	}
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 1261 "subscriptions.c"
}


static gboolean _vala_string_array_contains (gchar* * stack, int stack_length, gchar* needle) {
	int i;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	for (i = 0; i < stack_length; i++) {
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (g_strcmp0 (stack[i], needle) == 0) {
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return TRUE;
#line 1273 "subscriptions.c"
		}
	}
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return FALSE;
#line 1278 "subscriptions.c"
}


static gboolean adblock_subscription_css_element_seems_valid (AdblockSubscription* self, const gchar* element) {
	gboolean result = FALSE;
	gboolean is_valid = FALSE;
	gchar** valid_elements = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	gchar* _tmp15_;
	gchar* _tmp16_;
	gchar* _tmp17_;
	gchar* _tmp18_;
	gchar* _tmp19_;
	gchar* _tmp20_;
	gchar* _tmp21_;
	gchar* _tmp22_;
	gchar* _tmp23_;
	gchar* _tmp24_;
	gchar* _tmp25_;
	gchar* _tmp26_;
	gchar* _tmp27_;
	gchar* _tmp28_;
	gchar* _tmp29_;
	gchar* _tmp30_;
	gchar* _tmp31_;
	gchar* _tmp32_;
	gchar* _tmp33_;
	gchar* _tmp34_;
	gchar* _tmp35_;
	gchar* _tmp36_;
	gchar* _tmp37_;
	gchar* _tmp38_;
	gchar* _tmp39_;
	gchar* _tmp40_;
	gchar* _tmp41_;
	gchar* _tmp42_;
	gchar* _tmp43_;
	gchar* _tmp44_;
	gchar* _tmp45_;
	gchar* _tmp46_;
	gchar* _tmp47_;
	gchar* _tmp48_;
	gchar* _tmp49_;
	gchar* _tmp50_;
	gchar* _tmp51_;
	gchar* _tmp52_;
	gchar* _tmp53_;
	gchar* _tmp54_;
	gchar* _tmp55_;
	gchar* _tmp56_;
	gchar* _tmp57_;
	gchar* _tmp58_;
	gchar* _tmp59_;
	gchar* _tmp60_;
	gchar* _tmp61_;
	gchar* _tmp62_;
	gchar* _tmp63_;
	gchar* _tmp64_;
	gchar** _tmp65_;
	gint valid_elements_length1;
	gint _valid_elements_size_;
	gboolean _tmp66_ = FALSE;
	gboolean _tmp67_ = FALSE;
	const gchar* _tmp68_;
	gboolean _tmp69_;
	const gchar* _tmp78_ = NULL;
	const gchar* _tmp79_;
	gboolean debug_selectors = FALSE;
	gboolean _tmp80_;
	gboolean _tmp81_;
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (element != NULL, FALSE);
#line 167 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	is_valid = TRUE;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = g_strdup ("::after");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = g_strdup ("::before");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = g_strdup ("a");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = g_strdup ("abbr");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4_ = g_strdup ("address");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp5_ = g_strdup ("article");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp6_ = g_strdup ("aside");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7_ = g_strdup ("b");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp8_ = g_strdup ("blockquote");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp9_ = g_strdup ("caption");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp10_ = g_strdup ("center");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp11_ = g_strdup ("cite");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp12_ = g_strdup ("code");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp13_ = g_strdup ("div");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp14_ = g_strdup ("dl");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp15_ = g_strdup ("dt");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp16_ = g_strdup ("dd");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp17_ = g_strdup ("em");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp18_ = g_strdup ("feed");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp19_ = g_strdup ("fieldset");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp20_ = g_strdup ("figcaption");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp21_ = g_strdup ("figure");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp22_ = g_strdup ("font");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp23_ = g_strdup ("footer");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp24_ = g_strdup ("form");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp25_ = g_strdup ("h1");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp26_ = g_strdup ("h2");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp27_ = g_strdup ("h3");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp28_ = g_strdup ("h4");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp29_ = g_strdup ("h5");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp30_ = g_strdup ("h6");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp31_ = g_strdup ("header");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp32_ = g_strdup ("hgroup");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp33_ = g_strdup ("i");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp34_ = g_strdup ("iframe");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp35_ = g_strdup ("iframe html *");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp36_ = g_strdup ("img");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp37_ = g_strdup ("kbd");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp38_ = g_strdup ("label");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp39_ = g_strdup ("legend");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp40_ = g_strdup ("li");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp41_ = g_strdup ("m");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp42_ = g_strdup ("main");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp43_ = g_strdup ("marquee");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp44_ = g_strdup ("menu");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp45_ = g_strdup ("nav");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp46_ = g_strdup ("ol");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp47_ = g_strdup ("option");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp48_ = g_strdup ("p");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp49_ = g_strdup ("pre");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp50_ = g_strdup ("q");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp51_ = g_strdup ("samp");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp52_ = g_strdup ("section");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp53_ = g_strdup ("small");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp54_ = g_strdup ("span");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp55_ = g_strdup ("strong");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp56_ = g_strdup ("summary");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp57_ = g_strdup ("table");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp58_ = g_strdup ("tr");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp59_ = g_strdup ("tbody");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp60_ = g_strdup ("td");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp61_ = g_strdup ("th");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp62_ = g_strdup ("thead");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp63_ = g_strdup ("tt");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp64_ = g_strdup ("ul");
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_ = g_new0 (gchar*, 65 + 1);
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[0] = _tmp0_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[1] = _tmp1_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[2] = _tmp2_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[3] = _tmp3_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[4] = _tmp4_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[5] = _tmp5_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[6] = _tmp6_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[7] = _tmp7_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[8] = _tmp8_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[9] = _tmp9_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[10] = _tmp10_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[11] = _tmp11_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[12] = _tmp12_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[13] = _tmp13_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[14] = _tmp14_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[15] = _tmp15_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[16] = _tmp16_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[17] = _tmp17_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[18] = _tmp18_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[19] = _tmp19_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[20] = _tmp20_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[21] = _tmp21_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[22] = _tmp22_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[23] = _tmp23_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[24] = _tmp24_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[25] = _tmp25_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[26] = _tmp26_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[27] = _tmp27_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[28] = _tmp28_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[29] = _tmp29_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[30] = _tmp30_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[31] = _tmp31_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[32] = _tmp32_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[33] = _tmp33_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[34] = _tmp34_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[35] = _tmp35_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[36] = _tmp36_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[37] = _tmp37_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[38] = _tmp38_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[39] = _tmp39_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[40] = _tmp40_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[41] = _tmp41_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[42] = _tmp42_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[43] = _tmp43_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[44] = _tmp44_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[45] = _tmp45_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[46] = _tmp46_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[47] = _tmp47_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[48] = _tmp48_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[49] = _tmp49_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[50] = _tmp50_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[51] = _tmp51_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[52] = _tmp52_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[53] = _tmp53_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[54] = _tmp54_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[55] = _tmp55_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[56] = _tmp56_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[57] = _tmp57_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[58] = _tmp58_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[59] = _tmp59_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[60] = _tmp60_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[61] = _tmp61_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[62] = _tmp62_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[63] = _tmp63_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp65_[64] = _tmp64_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	valid_elements = _tmp65_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	valid_elements_length1 = 65;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_valid_elements_size_ = valid_elements_length1;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp68_ = element;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp69_ = g_str_has_prefix (_tmp68_, ".");
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (!_tmp69_) {
#line 1643 "subscriptions.c"
		const gchar* _tmp70_;
		gboolean _tmp71_;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp70_ = element;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp71_ = g_str_has_prefix (_tmp70_, "#");
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp67_ = !_tmp71_;
#line 1652 "subscriptions.c"
	} else {
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp67_ = FALSE;
#line 1656 "subscriptions.c"
	}
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp67_) {
#line 1660 "subscriptions.c"
		const gchar* _tmp72_;
		gchar** _tmp73_;
		gchar** _tmp74_;
		gchar** _tmp75_;
		gint _tmp75__length1;
		const gchar* _tmp76_;
		gchar** _tmp77_;
		gint _tmp77__length1;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp72_ = element;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp74_ = _tmp73_ = g_strsplit (_tmp72_, "[", 0);
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp75_ = _tmp74_;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp75__length1 = _vala_array_length (_tmp73_);
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp76_ = _tmp75_[0];
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp77_ = valid_elements;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp77__length1 = valid_elements_length1;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp66_ = !_vala_string_array_contains (_tmp77_, _tmp77__length1, _tmp76_);
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp75_ = (_vala_array_free (_tmp75_, _tmp75__length1, (GDestroyNotify) g_free), NULL);
#line 1687 "subscriptions.c"
	} else {
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp66_ = FALSE;
#line 1691 "subscriptions.c"
	}
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp66_) {
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		is_valid = FALSE;
#line 1697 "subscriptions.c"
	}
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp79_ = g_getenv ("MIDORI_DEBUG");
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp78_ = _tmp79_;
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp78_ == NULL) {
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp78_ = "";
#line 1707 "subscriptions.c"
	}
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp80_ = string_contains (_tmp78_, "adblock:css");
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	debug_selectors = _tmp80_;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp81_ = debug_selectors;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp81_) {
#line 1717 "subscriptions.c"
		const gchar* _tmp82_ = NULL;
		gboolean _tmp83_;
		FILE* _tmp84_;
		const gchar* _tmp85_;
		const gchar* _tmp86_;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp83_ = is_valid;
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp83_) {
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp82_ = "selector";
#line 1729 "subscriptions.c"
		} else {
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp82_ = "INVALID?";
#line 1733 "subscriptions.c"
		}
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp84_ = stdout;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp85_ = self->priv->_title;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp86_ = element;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		fprintf (_tmp84_, "Adblock '%s' %s: %s\n", _tmp85_, _tmp82_, _tmp86_);
#line 1743 "subscriptions.c"
	}
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = is_valid;
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	valid_elements = (_vala_array_free (valid_elements, valid_elements_length1, (GDestroyNotify) g_free), NULL);
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 1751 "subscriptions.c"
}


static void adblock_subscription_update_css_hash (AdblockSubscription* self, const gchar* domain, const gchar* value) {
	const gchar* _tmp0_;
	gboolean _tmp1_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (domain != NULL);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (value != NULL);
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = value;
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = adblock_subscription_css_element_seems_valid (self, _tmp0_);
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp1_) {
#line 1770 "subscriptions.c"
		gchar* olddata = NULL;
		AdblockElement* _tmp2_;
		const gchar* _tmp3_;
		gchar* _tmp4_;
		const gchar* _tmp5_;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp2_ = self->element;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = domain;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = adblock_element_lookup (_tmp2_, _tmp3_);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		olddata = _tmp4_;
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = olddata;
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp5_ != NULL) {
#line 1788 "subscriptions.c"
			gchar* newdata = NULL;
			const gchar* _tmp6_;
			gchar* _tmp7_;
			gchar* _tmp8_;
			const gchar* _tmp9_;
			gchar* _tmp10_;
			gchar* _tmp11_;
			AdblockElement* _tmp12_;
			const gchar* _tmp13_;
			const gchar* _tmp14_;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp6_ = olddata;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp7_ = g_strconcat (_tmp6_, " , ", NULL);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp8_ = _tmp7_;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp9_ = value;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp10_ = g_strconcat (_tmp8_, _tmp9_, NULL);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp11_ = _tmp10_;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (_tmp8_);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			newdata = _tmp11_;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp12_ = self->element;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp13_ = domain;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp14_ = newdata;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_element_insert (_tmp12_, _tmp13_, _tmp14_);
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (newdata);
#line 1825 "subscriptions.c"
		} else {
			AdblockElement* _tmp15_;
			const gchar* _tmp16_;
			const gchar* _tmp17_;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp15_ = self->element;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp16_ = domain;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp17_ = value;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_element_insert (_tmp15_, _tmp16_, _tmp17_);
#line 1838 "subscriptions.c"
		}
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (olddata);
#line 1842 "subscriptions.c"
	}
}


static void adblock_subscription_add_url_pattern (AdblockSubscription* self, const gchar* prefix, const gchar* type, const gchar* line, GError** error) {
	gchar** data = NULL;
	const gchar* _tmp0_;
	gchar** _tmp1_;
	gchar** _tmp2_;
	gint data_length1;
	gint _data_size_;
	gboolean _tmp3_ = FALSE;
	gchar** _tmp4_;
	gint _tmp4__length1;
	gchar* patt = NULL;
	gchar* opts = NULL;
	gchar** _tmp7_;
	gint _tmp7__length1;
	const gchar* _tmp8_;
	gchar* _tmp9_;
	const gchar* _tmp10_;
	gchar* _tmp11_;
	gchar** _tmp12_;
	gint _tmp12__length1;
	const gchar* _tmp13_;
	const gchar* _tmp20_;
	gboolean _tmp21_;
	gchar* format_patt = NULL;
	const gchar* _tmp22_;
	const gchar* _tmp23_;
	gchar* _tmp24_;
	gboolean _tmp25_;
	const gchar* _tmp29_;
	const gchar* _tmp30_;
	GError * _inner_error_ = NULL;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (prefix != NULL);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (type != NULL);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (line != NULL);
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = line;
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = _tmp1_ = g_strsplit (_tmp0_, "$", 2);
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data = _tmp2_;
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data_length1 = _vala_array_length (_tmp1_);
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_data_size_ = data_length1;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4_ = data;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4__length1 = data_length1;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp4_ == NULL) {
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = TRUE;
#line 1904 "subscriptions.c"
	} else {
		gchar** _tmp5_;
		gint _tmp5__length1;
		const gchar* _tmp6_;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = data;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5__length1 = data_length1;
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp6_ = _tmp5_[0];
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = _tmp6_ == NULL;
#line 1917 "subscriptions.c"
	}
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp3_) {
#line 203 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 203 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 1925 "subscriptions.c"
	}
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7_ = data;
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp7__length1 = data_length1;
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp8_ = _tmp7_[0];
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp9_ = g_strdup (_tmp8_);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (patt);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	patt = _tmp9_;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp10_ = type;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp11_ = g_strdup (_tmp10_);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (opts);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	opts = _tmp11_;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp12_ = data;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp12__length1 = data_length1;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp13_ = _tmp12_[1];
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp13_ != NULL) {
#line 1955 "subscriptions.c"
		const gchar* _tmp14_;
		gchar* _tmp15_;
		gchar* _tmp16_;
		gchar** _tmp17_;
		gint _tmp17__length1;
		const gchar* _tmp18_;
		gchar* _tmp19_;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp14_ = type;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp15_ = g_strconcat (_tmp14_, ",", NULL);
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp16_ = _tmp15_;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp17_ = data;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp17__length1 = data_length1;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp18_ = _tmp17_[1];
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp19_ = g_strconcat (_tmp16_, _tmp18_, NULL);
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (opts);
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		opts = _tmp19_;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (_tmp16_);
#line 1983 "subscriptions.c"
	}
#line 212 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp20_ = opts;
#line 212 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp21_ = g_regex_match_simple ("subdocument", _tmp20_, G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY);
#line 212 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp21_) {
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (opts);
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (patt);
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 1999 "subscriptions.c"
	}
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp22_ = prefix;
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp23_ = patt;
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp24_ = adblock_fixup_regex (_tmp22_, _tmp23_);
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	format_patt = _tmp24_;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp25_ = self->priv->debug_parse;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp25_) {
#line 2013 "subscriptions.c"
		FILE* _tmp26_;
		const gchar* _tmp27_;
		const gchar* _tmp28_;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp26_ = stdout;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp27_ = format_patt;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp28_ = opts;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		fprintf (_tmp26_, "got: %s opts %s\n", _tmp27_, _tmp28_);
#line 2025 "subscriptions.c"
	}
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp29_ = format_patt;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp30_ = opts;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_compile_regexp (self, _tmp29_, _tmp30_, &_inner_error_);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_propagate_error (error, _inner_error_);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (format_patt);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (opts);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (patt);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 2047 "subscriptions.c"
	}
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (format_patt);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (opts);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (patt);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	data = (_vala_array_free (data, data_length1, (GDestroyNotify) g_free), NULL);
#line 2057 "subscriptions.c"
}


static gboolean adblock_subscription_compile_regexp (AdblockSubscription* self, const gchar* patt, const gchar* opts, GError** error) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	gboolean _tmp68_ = FALSE;
	GError * _inner_error_ = NULL;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (opts != NULL, FALSE);
#line 224 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = patt;
#line 224 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp0_ == NULL) {
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		result = FALSE;
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return result;
#line 2078 "subscriptions.c"
	}
	{
		GRegex* regex = NULL;
		const gchar* _tmp1_;
		GRegex* _tmp2_;
		gboolean _tmp3_ = FALSE;
		const gchar* _tmp4_;
		gboolean _tmp5_;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp1_ = patt;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp2_ = g_regex_new (_tmp1_, G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, &_inner_error_);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		regex = _tmp2_;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2095 "subscriptions.c"
			goto __catch14_g_error;
		}
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = patt;
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = g_regex_match_simple ("^/.*[\\^\\$\\*].*/$", _tmp4_, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY);
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp5_) {
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp3_ = TRUE;
#line 2106 "subscriptions.c"
		} else {
			gboolean _tmp6_ = FALSE;
			const gchar* _tmp7_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp7_ = opts;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (_tmp7_ != NULL) {
#line 2114 "subscriptions.c"
				const gchar* _tmp8_;
				gboolean _tmp9_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp8_ = opts;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp9_ = string_contains (_tmp8_, "whitelist");
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp6_ = _tmp9_;
#line 2123 "subscriptions.c"
			} else {
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp6_ = FALSE;
#line 2127 "subscriptions.c"
			}
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp3_ = _tmp6_;
#line 2131 "subscriptions.c"
		}
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp3_) {
#line 2135 "subscriptions.c"
			gboolean _tmp10_;
			const gchar* _tmp13_;
			gboolean _tmp14_;
			AdblockOptions* _tmp21_;
			const gchar* _tmp22_;
			const gchar* _tmp23_;
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp10_ = self->priv->debug_parse;
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (_tmp10_) {
#line 2146 "subscriptions.c"
				FILE* _tmp11_;
				const gchar* _tmp12_;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp11_ = stdout;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp12_ = patt;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				fprintf (_tmp11_, "patt: %s\n", _tmp12_);
#line 2155 "subscriptions.c"
			}
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp13_ = opts;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp14_ = string_contains (_tmp13_, "whitelist");
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (_tmp14_) {
#line 2163 "subscriptions.c"
				AdblockWhitelist* _tmp15_;
				const gchar* _tmp16_;
				GRegex* _tmp17_;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp15_ = self->whitelist;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp16_ = patt;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp17_ = regex;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				adblock_filter_insert ((AdblockFilter*) _tmp15_, _tmp16_, _tmp17_);
#line 2175 "subscriptions.c"
			} else {
				AdblockPattern* _tmp18_;
				const gchar* _tmp19_;
				GRegex* _tmp20_;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp18_ = self->pattern;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp19_ = patt;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp20_ = regex;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				adblock_filter_insert ((AdblockFilter*) _tmp18_, _tmp19_, _tmp20_);
#line 2188 "subscriptions.c"
			}
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp21_ = self->optslist;
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp22_ = patt;
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp23_ = opts;
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_options_insert (_tmp21_, _tmp22_, _tmp23_);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			result = FALSE;
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_regex_unref0 (regex);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return result;
#line 2204 "subscriptions.c"
		} else {
			gint pos = 0;
			gint len = 0;
			gint signature_size = 0;
			gchar* sig = NULL;
			const gchar* _tmp24_;
			gint _tmp25_;
			gint _tmp26_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			pos = 0;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			signature_size = 8;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp24_ = patt;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp25_ = strlen (_tmp24_);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp26_ = _tmp25_;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			len = _tmp26_;
#line 2225 "subscriptions.c"
			{
				gint _tmp27_;
				gint _tmp28_;
				gboolean _tmp29_ = FALSE;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp27_ = len;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp28_ = signature_size;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				pos = _tmp27_ - _tmp28_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp29_ = TRUE;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				while (TRUE) {
#line 2240 "subscriptions.c"
					gint _tmp31_;
					const gchar* _tmp32_;
					gint _tmp33_;
					const gchar* _tmp34_;
					gint _tmp35_;
					gchar* _tmp36_;
					gboolean _tmp37_ = FALSE;
					const gchar* _tmp38_;
					gboolean _tmp39_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (!_tmp29_) {
#line 2252 "subscriptions.c"
						gint _tmp30_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp30_ = pos;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						pos = _tmp30_ - 1;
#line 2258 "subscriptions.c"
					}
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp29_ = FALSE;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp31_ = pos;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (!(_tmp31_ >= 0)) {
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						break;
#line 2268 "subscriptions.c"
					}
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp32_ = patt;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp33_ = pos;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp34_ = string_offset (_tmp32_, (glong) _tmp33_);
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp35_ = signature_size;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp36_ = g_strndup (_tmp34_, (gsize) _tmp35_);
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_g_free0 (sig);
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					sig = _tmp36_;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp38_ = sig;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp39_ = g_regex_match_simple ("[\\*]", _tmp38_, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY);
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (!_tmp39_) {
#line 2290 "subscriptions.c"
						AdblockKeys* _tmp40_;
						const gchar* _tmp41_;
						GRegex* _tmp42_;
						GRegex* _tmp43_;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp40_ = self->keys;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp41_ = sig;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp42_ = adblock_filter_lookup ((AdblockFilter*) _tmp40_, _tmp41_);
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp43_ = _tmp42_;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp37_ = _tmp43_ == NULL;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_g_regex_unref0 (_tmp43_);
#line 2307 "subscriptions.c"
					} else {
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp37_ = FALSE;
#line 2311 "subscriptions.c"
					}
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (_tmp37_) {
#line 2315 "subscriptions.c"
						AdblockKeys* _tmp44_;
						const gchar* _tmp45_;
						GRegex* _tmp46_;
						AdblockOptions* _tmp47_;
						const gchar* _tmp48_;
						const gchar* _tmp49_;
#line 252 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp44_ = self->keys;
#line 252 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp45_ = sig;
#line 252 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp46_ = regex;
#line 252 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						adblock_filter_insert ((AdblockFilter*) _tmp44_, _tmp45_, _tmp46_);
#line 253 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp47_ = self->optslist;
#line 253 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp48_ = sig;
#line 253 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp49_ = opts;
#line 253 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						adblock_options_insert (_tmp47_, _tmp48_, _tmp49_);
#line 2338 "subscriptions.c"
					} else {
						gboolean _tmp50_ = FALSE;
						gboolean _tmp51_ = FALSE;
						const gchar* _tmp52_;
						gboolean _tmp53_;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp52_ = sig;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp53_ = g_str_has_prefix (_tmp52_, "*");
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						if (_tmp53_) {
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp51_ = TRUE;
#line 2352 "subscriptions.c"
						} else {
							const gchar* _tmp54_;
							gboolean _tmp55_;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp54_ = sig;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp55_ = g_str_has_prefix (_tmp54_, "\\");
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp51_ = _tmp55_;
#line 2362 "subscriptions.c"
						}
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						if (_tmp51_) {
#line 2366 "subscriptions.c"
							AdblockPattern* _tmp56_;
							const gchar* _tmp57_;
							GRegex* _tmp58_;
							GRegex* _tmp59_;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp56_ = self->pattern;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp57_ = sig;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp58_ = adblock_filter_lookup ((AdblockFilter*) _tmp56_, _tmp57_);
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp59_ = _tmp58_;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp50_ = _tmp59_ == NULL;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_g_regex_unref0 (_tmp59_);
#line 2383 "subscriptions.c"
						} else {
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp50_ = FALSE;
#line 2387 "subscriptions.c"
						}
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						if (_tmp50_) {
#line 2391 "subscriptions.c"
							AdblockPattern* _tmp60_;
							const gchar* _tmp61_;
							GRegex* _tmp62_;
							AdblockOptions* _tmp63_;
							const gchar* _tmp64_;
							const gchar* _tmp65_;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp60_ = self->pattern;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp61_ = sig;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp62_ = regex;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							adblock_filter_insert ((AdblockFilter*) _tmp60_, _tmp61_, _tmp62_);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp63_ = self->optslist;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp64_ = sig;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							_tmp65_ = opts;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
							adblock_options_insert (_tmp63_, _tmp64_, _tmp65_);
#line 2414 "subscriptions.c"
						}
					}
				}
			}
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (sig);
#line 2421 "subscriptions.c"
		}
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		result = FALSE;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_regex_unref0 (regex);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return result;
#line 2429 "subscriptions.c"
	}
	goto __finally14;
	__catch14_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp66_;
		const gchar* _tmp67_;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_error_ = _inner_error_;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_inner_error_ = NULL;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp66_ = _error_;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp67_ = _tmp66_->message;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_warning ("subscriptions.vala:266: Adblock compile regexp: %s", _tmp67_);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		result = TRUE;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_error_free0 (_error_);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return result;
#line 2453 "subscriptions.c"
	}
	__finally14:
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_propagate_error (error, _inner_error_);
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return _tmp68_;
#line 2460 "subscriptions.c"
}


void adblock_subscription_parse_header (AdblockSubscription* self, const gchar* header, GError** error) {
	gchar* key = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* value = NULL;
	gchar* _tmp2_;
	const gchar* _tmp3_;
	gboolean _tmp4_;
	const gchar* _tmp25_;
	const gchar* _tmp26_;
	const gchar* _tmp27_;
	GList* _tmp29_;
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (header != NULL);
#line 276 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = header;
#line 276 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 276 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	key = _tmp1_;
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = g_strdup ("");
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	value = _tmp2_;
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = header;
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4_ = string_contains (_tmp3_, ":");
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp4_) {
#line 2496 "subscriptions.c"
		gchar** parts = NULL;
		const gchar* _tmp5_;
		gchar** _tmp6_;
		gchar** _tmp7_;
		gint parts_length1;
		gint _parts_size_;
		gboolean _tmp8_ = FALSE;
		gboolean _tmp9_ = FALSE;
		gboolean _tmp10_ = FALSE;
		gchar** _tmp11_;
		gint _tmp11__length1;
		const gchar* _tmp12_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = header;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp7_ = _tmp6_ = g_strsplit (_tmp5_, ":", 2);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		parts = _tmp7_;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		parts_length1 = _vala_array_length (_tmp6_);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_parts_size_ = parts_length1;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11_ = parts;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11__length1 = parts_length1;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp12_ = _tmp11_[0];
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp12_ != NULL) {
#line 2527 "subscriptions.c"
			gchar** _tmp13_;
			gint _tmp13__length1;
			const gchar* _tmp14_;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp13_ = parts;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp13__length1 = parts_length1;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp14_ = _tmp13_[0];
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp10_ = g_strcmp0 (_tmp14_, "") != 0;
#line 2539 "subscriptions.c"
		} else {
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp10_ = FALSE;
#line 2543 "subscriptions.c"
		}
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp10_) {
#line 2547 "subscriptions.c"
			gchar** _tmp15_;
			gint _tmp15__length1;
			const gchar* _tmp16_;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp15_ = parts;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp15__length1 = parts_length1;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp16_ = _tmp15_[1];
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp9_ = _tmp16_ != NULL;
#line 2559 "subscriptions.c"
		} else {
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp9_ = FALSE;
#line 2563 "subscriptions.c"
		}
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp9_) {
#line 2567 "subscriptions.c"
			gchar** _tmp17_;
			gint _tmp17__length1;
			const gchar* _tmp18_;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp17_ = parts;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp17__length1 = parts_length1;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp18_ = _tmp17_[1];
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp8_ = g_strcmp0 (_tmp18_, "") != 0;
#line 2579 "subscriptions.c"
		} else {
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp8_ = FALSE;
#line 2583 "subscriptions.c"
		}
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp8_) {
#line 2587 "subscriptions.c"
			gchar** _tmp19_;
			gint _tmp19__length1;
			const gchar* _tmp20_;
			gchar* _tmp21_;
			gchar** _tmp22_;
			gint _tmp22__length1;
			const gchar* _tmp23_;
			gchar* _tmp24_;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp19_ = parts;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp19__length1 = parts_length1;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp20_ = _tmp19_[0];
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp21_ = string_substring (_tmp20_, (glong) 2, (glong) -1);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (key);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			key = _tmp21_;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp22_ = parts;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp22__length1 = parts_length1;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp23_ = _tmp22_[1];
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp24_ = string_substring (_tmp23_, (glong) 1, (glong) -1);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (value);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			value = _tmp24_;
#line 2620 "subscriptions.c"
		}
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 2624 "subscriptions.c"
	}
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp25_ = key;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp26_ = value;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_debug ("Header '%s' says '%s'", _tmp25_, _tmp26_, NULL);
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp27_ = key;
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (g_strcmp0 (_tmp27_, "Title") == 0) {
#line 2636 "subscriptions.c"
		const gchar* _tmp28_;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp28_ = value;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_title (self, _tmp28_);
#line 2642 "subscriptions.c"
	}
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp29_ = self->priv->features;
#line 2646 "subscriptions.c"
	{
		GList* feature_collection = NULL;
		GList* feature_it = NULL;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		feature_collection = _tmp29_;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		for (feature_it = feature_collection; feature_it != NULL; feature_it = feature_it->next) {
#line 2654 "subscriptions.c"
			AdblockFeature* feature = NULL;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			feature = (AdblockFeature*) feature_it->data;
#line 2658 "subscriptions.c"
			{
				AdblockFeature* _tmp30_;
				const gchar* _tmp31_;
				const gchar* _tmp32_;
				gboolean _tmp33_;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp30_ = feature;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp31_ = key;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp32_ = value;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp33_ = adblock_feature_header (_tmp30_, _tmp31_, _tmp32_);
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				if (_tmp33_) {
#line 291 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					break;
#line 2676 "subscriptions.c"
				}
			}
		}
	}
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (value);
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (key);
#line 2685 "subscriptions.c"
}


static void adblock_subscription_download_status (AdblockSubscription* self, GParamSpec* pspec) {
	WebKitDownload* _tmp0_;
	WebKitDownloadStatus _tmp1_;
	GError * _inner_error_ = NULL;
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (pspec != NULL);
#line 297 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->download;
#line 297 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = webkit_download_get_status (_tmp0_);
#line 297 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp1_ != WEBKIT_DOWNLOAD_STATUS_FINISHED) {
#line 298 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 2705 "subscriptions.c"
	}
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->priv->download);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->download = NULL;
#line 2711 "subscriptions.c"
	{
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_parse (self, &_inner_error_);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 2717 "subscriptions.c"
			goto __catch15_g_error;
		}
	}
	goto __finally15;
	__catch15_g_error:
	{
		GError* _error_ = NULL;
		const gchar* _tmp2_;
		GError* _tmp3_;
		const gchar* _tmp4_;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_error_ = _inner_error_;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_inner_error_ = NULL;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp2_ = self->priv->_uri;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = _error_;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = _tmp3_->message;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_warning ("subscriptions.vala:304: Error parsing %s: %s", _tmp2_, _tmp4_);
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_error_free0 (_error_);
#line 2742 "subscriptions.c"
	}
	__finally15:
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_clear_error (&_inner_error_);
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 2753 "subscriptions.c"
	}
}


static void _adblock_subscription_download_status_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_download_status ((AdblockSubscription*) self, pspec);
#line 2761 "subscriptions.c"
}


static gchar* string_chomp (const gchar* self) {
	gchar* result = NULL;
	gchar* _result_ = NULL;
	gchar* _tmp0_;
	const gchar* _tmp1_;
#line 1226 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1227 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = g_strdup (self);
#line 1227 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_result_ = _tmp0_;
#line 1228 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _result_;
#line 1228 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_strchomp (_tmp1_);
#line 1229 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _result_;
#line 1229 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 2784 "subscriptions.c"
}


void adblock_subscription_parse (AdblockSubscription* self, GError** error) {
	gboolean _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	gboolean _tmp4_;
	GFile* filter_file = NULL;
	const gchar* _tmp17_;
	GFile* _tmp18_;
	GDataInputStream* stream = NULL;
	gchar* line = NULL;
	GList* _tmp56_;
	GError * _inner_error_ = NULL;
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_active;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (!_tmp0_) {
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 2809 "subscriptions.c"
	}
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp1_ = self->priv->_uri;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp2_ = self->path;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_debug ("Parsing %s (%s)", _tmp1_, _tmp2_, NULL);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_clear (self);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp3_ = self->priv->_uri;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp4_ = g_str_has_prefix (_tmp3_, "file://");
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (_tmp4_) {
#line 2825 "subscriptions.c"
		gchar* _tmp5_ = NULL;
		const gchar* _tmp6_;
		gchar* _tmp7_;
		gchar* _tmp8_;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp6_ = self->priv->_uri;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp7_ = g_filename_from_uri (_tmp6_, NULL, &_inner_error_);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = _tmp7_;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			g_propagate_error (error, _inner_error_);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return;
#line 2842 "subscriptions.c"
		}
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp8_ = _tmp5_;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = NULL;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (self->path);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->path = _tmp8_;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (_tmp5_);
#line 2854 "subscriptions.c"
	} else {
		gchar* cache_dir = NULL;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		const gchar* _tmp11_;
		gchar* filename = NULL;
		const gchar* _tmp12_;
		gchar* _tmp13_;
		const gchar* _tmp14_;
		const gchar* _tmp15_;
		gchar* _tmp16_;
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp9_ = g_get_user_cache_dir ();
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp10_ = g_build_filename (_tmp9_, PACKAGE_NAME, "adblock", NULL);
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		cache_dir = _tmp10_;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11_ = cache_dir;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		midori_paths_mkdir_with_parents (_tmp11_, 0700);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp12_ = self->priv->_uri;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp13_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _tmp12_, (gsize) -1);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		filename = _tmp13_;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp14_ = cache_dir;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp15_ = filename;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp16_ = g_build_filename (_tmp14_, _tmp15_, NULL);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (self->path);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->path = _tmp16_;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (filename);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (cache_dir);
#line 2896 "subscriptions.c"
	}
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp17_ = self->path;
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp18_ = g_file_new_for_path (_tmp17_);
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	filter_file = _tmp18_;
#line 2904 "subscriptions.c"
	{
		GFileInputStream* _tmp19_ = NULL;
		GFile* _tmp20_;
		GFileInputStream* _tmp21_;
		GDataInputStream* _tmp22_;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp20_ = filter_file;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp21_ = g_file_read (_tmp20_, NULL, &_inner_error_);
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp19_ = _tmp21_;
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (g_error_matches (_inner_error_, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
#line 2920 "subscriptions.c"
				goto __catch16_g_io_error_not_found;
			}
			goto __finally16;
		}
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp22_ = g_data_input_stream_new ((GInputStream*) _tmp19_);
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (stream);
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		stream = _tmp22_;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (_tmp19_);
#line 2933 "subscriptions.c"
	}
	goto __finally16;
	__catch16_g_io_error_not_found:
	{
		GError* exist_error = NULL;
		const gchar* _tmp23_;
		gboolean _tmp24_;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		exist_error = _inner_error_;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_inner_error_ = NULL;
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp23_ = self->priv->_uri;
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp24_ = g_str_has_prefix (_tmp23_, "file://");
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (!_tmp24_) {
#line 2951 "subscriptions.c"
			WebKitDownload* _tmp25_;
			gchar* destination_uri = NULL;
			const gchar* _tmp26_;
			gchar* _tmp27_;
			const gchar* _tmp28_;
			const gchar* _tmp29_;
			const gchar* _tmp30_;
			WebKitNetworkRequest* _tmp31_;
			WebKitNetworkRequest* _tmp32_;
			WebKitDownload* _tmp33_;
			WebKitDownload* _tmp34_;
			const gchar* _tmp35_;
			gboolean _tmp36_;
			WebKitDownload* _tmp39_;
			const gchar* _tmp40_;
			WebKitDownload* _tmp41_;
			WebKitDownload* _tmp42_;
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp25_ = self->priv->download;
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (_tmp25_ != NULL) {
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_error_free0 (exist_error);
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (stream);
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (filter_file);
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 2981 "subscriptions.c"
			}
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp26_ = self->path;
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp27_ = g_filename_to_uri (_tmp26_, NULL, &_inner_error_);
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			destination_uri = _tmp27_;
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_error_free0 (exist_error);
#line 2993 "subscriptions.c"
				goto __finally16;
			}
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp28_ = self->priv->_uri;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp29_ = destination_uri;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_debug ("Fetching %s to %s now", _tmp28_, _tmp29_, NULL);
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp30_ = self->priv->_uri;
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp31_ = webkit_network_request_new (_tmp30_);
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp32_ = _tmp31_;
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp33_ = webkit_download_new (_tmp32_);
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (self->priv->download);
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			self->priv->download = _tmp33_;
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (_tmp32_);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp34_ = self->priv->download;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp35_ = destination_uri;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp36_ = midori_download_has_enough_space (_tmp34_, _tmp35_, TRUE);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (!_tmp36_) {
#line 3024 "subscriptions.c"
				const gchar* _tmp37_;
				GError* _tmp38_;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp37_ = self->path;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp38_ = g_error_new (G_FILE_ERROR, G_FILE_ERROR_EXIST, "Can't download to \"%s\"", _tmp37_);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_inner_error_ = _tmp38_;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (destination_uri);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_error_free0 (exist_error);
#line 3037 "subscriptions.c"
				goto __finally16;
			}
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp39_ = self->priv->download;
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp40_ = destination_uri;
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			webkit_download_set_destination_uri (_tmp39_, _tmp40_);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp41_ = self->priv->download;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			g_signal_connect_object ((GObject*) _tmp41_, "notify::status", (GCallback) _adblock_subscription_download_status_g_object_notify, self, 0);
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp42_ = self->priv->download;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			webkit_download_start (_tmp42_);
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (destination_uri);
#line 3056 "subscriptions.c"
		}
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_error_free0 (exist_error);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (stream);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (filter_file);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 3066 "subscriptions.c"
	}
	__finally16:
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_propagate_error (error, _inner_error_);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (stream);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (filter_file);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 3079 "subscriptions.c"
	}
#line 353 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_set_valid (self, FALSE);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	while (TRUE) {
#line 3085 "subscriptions.c"
		gchar* _tmp43_ = NULL;
		GDataInputStream* _tmp44_;
		gchar* _tmp45_;
		gchar* _tmp46_;
		const gchar* _tmp47_;
		const gchar* _tmp48_;
		gchar* chomped = NULL;
		const gchar* _tmp49_;
		gchar* _tmp50_;
		const gchar* _tmp51_;
		const gchar* _tmp52_;
		gchar _tmp53_;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp44_ = stream;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp45_ = g_data_input_stream_read_line (_tmp44_, NULL, NULL, &_inner_error_);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp43_ = _tmp45_;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			g_propagate_error (error, _inner_error_);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (line);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (stream);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (filter_file);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return;
#line 3116 "subscriptions.c"
		}
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp46_ = _tmp43_;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp43_ = NULL;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (line);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		line = _tmp46_;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp47_ = line;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (!(_tmp47_ != NULL)) {
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (_tmp43_);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			break;
#line 3134 "subscriptions.c"
		}
#line 356 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp48_ = line;
#line 356 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp48_ == NULL) {
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (_tmp43_);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			continue;
#line 3144 "subscriptions.c"
		}
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp49_ = line;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp50_ = string_chomp (_tmp49_);
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		chomped = _tmp50_;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp51_ = chomped;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (g_strcmp0 (_tmp51_, "") == 0) {
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (chomped);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_free0 (_tmp43_);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			continue;
#line 3162 "subscriptions.c"
		}
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp52_ = line;
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp53_ = string_get (_tmp52_, (glong) 0);
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp53_ == '!') {
#line 3170 "subscriptions.c"
			const gchar* _tmp54_;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp54_ = chomped;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_subscription_parse_header (self, _tmp54_, &_inner_error_);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				g_propagate_error (error, _inner_error_);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (chomped);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (_tmp43_);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (line);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (stream);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (filter_file);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 3192 "subscriptions.c"
			}
		} else {
			const gchar* _tmp55_;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_tmp55_ = chomped;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			adblock_subscription_parse_line (self, _tmp55_, &_inner_error_);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				g_propagate_error (error, _inner_error_);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (chomped);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (_tmp43_);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_free0 (line);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (stream);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_g_object_unref0 (filter_file);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				return;
#line 3216 "subscriptions.c"
			}
		}
#line 366 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_valid (self, TRUE);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (chomped);
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (_tmp43_);
#line 3225 "subscriptions.c"
	}
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp56_ = self->priv->features;
#line 3229 "subscriptions.c"
	{
		GList* feature_collection = NULL;
		GList* feature_it = NULL;
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		feature_collection = _tmp56_;
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		for (feature_it = feature_collection; feature_it != NULL; feature_it = feature_it->next) {
#line 3237 "subscriptions.c"
			AdblockFeature* feature = NULL;
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			feature = (AdblockFeature*) feature_it->data;
#line 3241 "subscriptions.c"
			{
				AdblockFeature* _tmp57_;
				GFile* _tmp58_;
				gboolean _tmp59_;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp57_ = feature;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp58_ = filter_file;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				_tmp59_ = adblock_feature_parsed (_tmp57_, _tmp58_);
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				if (!_tmp59_) {
#line 371 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					adblock_subscription_set_valid (self, FALSE);
#line 3256 "subscriptions.c"
				}
			}
		}
	}
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (line);
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (stream);
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (filter_file);
#line 3267 "subscriptions.c"
}


static AdblockDirective* _adblock_directive_dup (AdblockDirective* self) {
	AdblockDirective* dup;
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	dup = g_new0 (AdblockDirective, 1);
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	memcpy (dup, self, sizeof (AdblockDirective));
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return dup;
#line 3279 "subscriptions.c"
}


static gpointer __adblock_directive_dup0 (gpointer self) {
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return self ? _adblock_directive_dup (self) : NULL;
#line 3286 "subscriptions.c"
}


AdblockDirective* adblock_subscription_get_directive (AdblockSubscription* self, const gchar* request_uri, const gchar* page_uri) {
	AdblockDirective* result = NULL;
	GError * _inner_error_ = NULL;
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (request_uri != NULL, NULL);
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (page_uri != NULL, NULL);
#line 3299 "subscriptions.c"
	{
		AdblockDirective* directive = NULL;
		GHashTable* _tmp0_;
		const gchar* _tmp1_;
		gconstpointer _tmp2_;
		AdblockDirective* _tmp3_;
		AdblockDirective* _tmp4_;
		GList* _tmp5_;
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = self->priv->cache;
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp1_ = request_uri;
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp2_ = g_hash_table_lookup (_tmp0_, _tmp1_);
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = __adblock_directive_dup0 ((AdblockDirective*) _tmp2_);
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		directive = _tmp3_;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = directive;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (_tmp4_ != NULL) {
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			result = directive;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			return result;
#line 3326 "subscriptions.c"
		}
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = self->priv->features;
#line 3330 "subscriptions.c"
		{
			GList* feature_collection = NULL;
			GList* feature_it = NULL;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			feature_collection = _tmp5_;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			for (feature_it = feature_collection; feature_it != NULL; feature_it = feature_it->next) {
#line 3338 "subscriptions.c"
				AdblockFeature* feature = NULL;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
				feature = (AdblockFeature*) feature_it->data;
#line 3342 "subscriptions.c"
				{
					AdblockDirective* _tmp6_ = NULL;
					AdblockFeature* _tmp7_;
					const gchar* _tmp8_;
					const gchar* _tmp9_;
					AdblockDirective* _tmp10_;
					AdblockDirective* _tmp11_;
					AdblockDirective* _tmp12_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp7_ = feature;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp8_ = request_uri;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp9_ = page_uri;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp10_ = adblock_feature_match (_tmp7_, _tmp8_, _tmp9_, &_inner_error_);
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp6_ = _tmp10_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_g_free0 (directive);
#line 3365 "subscriptions.c"
						goto __catch17_g_error;
					}
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp11_ = _tmp6_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp6_ = NULL;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_g_free0 (directive);
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					directive = _tmp11_;
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_tmp12_ = directive;
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					if (_tmp12_ != NULL) {
#line 3380 "subscriptions.c"
						AdblockFeature* _tmp13_;
						GType _tmp14_;
						const gchar* _tmp15_;
						AdblockDirective* _tmp16_;
						GEnumValue* _tmp17_;
						const gchar* _tmp18_;
						const gchar* _tmp19_;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp13_ = feature;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp14_ = G_TYPE_FROM_INSTANCE ((GObject*) _tmp13_);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp15_ = g_type_name (_tmp14_);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp16_ = directive;
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp17_ = g_enum_get_value (g_type_class_ref (ADBLOCK_TYPE_DIRECTIVE), *_tmp16_);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp18_ = request_uri;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_tmp19_ = page_uri;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						adblock_debug ("%s gave %s for %s (%s)\n", _tmp15_, (_tmp17_ != NULL) ? _tmp17_->value_name : NULL, _tmp18_, _tmp19_, NULL);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						result = directive;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						_g_free0 (_tmp6_);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
						return result;
#line 3410 "subscriptions.c"
					}
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
					_g_free0 (_tmp6_);
#line 3414 "subscriptions.c"
				}
			}
		}
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (directive);
#line 3420 "subscriptions.c"
	}
	goto __finally17;
	__catch17_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp20_;
		const gchar* _tmp21_;
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_error_ = _inner_error_;
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_inner_error_ = NULL;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp20_ = _error_;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp21_ = _tmp20_->message;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_warning ("subscriptions.vala:389: Adblock match error: %s\n", _tmp21_);
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_error_free0 (_error_);
#line 3440 "subscriptions.c"
	}
	__finally17:
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_clear_error (&_inner_error_);
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return NULL;
#line 3451 "subscriptions.c"
	}
#line 391 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = NULL;
#line 391 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3457 "subscriptions.c"
}


static guint8* string_get_data (const gchar* self, int* result_length1) {
	guint8* result;
	guint8* res = NULL;
	gint res_length1;
	gint _res_size_;
	gint _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	guint8* _tmp3_;
	gint _tmp3__length1;
	guint8* _tmp4_;
	gint _tmp4__length1;
#line 1436 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res = (guint8*) self;
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res_length1 = -1;
#line 1437 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_res_size_ = res_length1;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = strlen (self);
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = _tmp0_;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	res_length1 = (gint) _tmp1_;
#line 1438 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = res_length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = res;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3__length1 = res_length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp4_ = _tmp3_;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp4__length1 = _tmp3__length1;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (result_length1) {
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		*result_length1 = _tmp4__length1;
#line 3501 "subscriptions.c"
	}
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp4_;
#line 1439 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 3507 "subscriptions.c"
}


void adblock_subscription_add_rule (AdblockSubscription* self, const gchar* rule) {
	GError * _inner_error_ = NULL;
#line 394 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 394 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (rule != NULL);
#line 3517 "subscriptions.c"
	{
		GFile* file = NULL;
		const gchar* _tmp0_;
		GFile* _tmp1_;
		GFileOutputStream* _tmp2_ = NULL;
		GFile* _tmp3_;
		GFileOutputStream* _tmp4_;
		const gchar* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
		guint8* _tmp8_;
		gint _tmp8__length1;
		guint8* _tmp9_;
		gint _tmp9__length1;
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = self->priv->_uri;
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp1_ = g_file_new_for_uri (_tmp0_);
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		file = _tmp1_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp3_ = file;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp4_ = g_file_append_to (_tmp3_, G_FILE_CREATE_NONE, NULL, &_inner_error_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp2_ = _tmp4_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (file);
#line 3548 "subscriptions.c"
			goto __catch18_g_error;
		}
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp5_ = rule;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp6_ = g_strdup_printf ("%s\n", _tmp5_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp7_ = _tmp6_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp8_ = string_get_data (_tmp7_, &_tmp8__length1);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp9_ = _tmp8_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp9__length1 = _tmp8__length1;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_output_stream_write ((GOutputStream*) _tmp2_, _tmp9_, (gsize) _tmp9__length1, NULL, &_inner_error_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (_tmp7_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (_tmp2_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (file);
#line 3573 "subscriptions.c"
			goto __catch18_g_error;
		}
#line 398 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_parse (self, &_inner_error_);
#line 398 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 398 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (_tmp2_);
#line 398 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
			_g_object_unref0 (file);
#line 3584 "subscriptions.c"
			goto __catch18_g_error;
		}
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (_tmp2_);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_object_unref0 (file);
#line 3591 "subscriptions.c"
	}
	goto __finally18;
	__catch18_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp10_;
		const gchar* _tmp11_;
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_error_ = _inner_error_;
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_inner_error_ = NULL;
#line 400 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp10_ = _error_;
#line 400 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp11_ = _tmp10_->message;
#line 400 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_warning ("subscriptions.vala:400: Failed to add custom rule: %s", _tmp11_);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_error_free0 (_error_);
#line 3611 "subscriptions.c"
	}
	__finally18:
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_clear_error (&_inner_error_);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		return;
#line 3622 "subscriptions.c"
	}
}


const gchar* adblock_subscription_get_uri (AdblockSubscription* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_uri;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3638 "subscriptions.c"
}


void adblock_subscription_set_uri (AdblockSubscription* self, const gchar* value) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (g_strcmp0 (value, adblock_subscription_get_uri (self)) != 0) {
#line 3647 "subscriptions.c"
		const gchar* _tmp0_;
		gchar* _tmp1_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp1_ = g_strdup (_tmp0_);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (self->priv->_uri);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_uri = _tmp1_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_URI_PROPERTY]);
#line 3660 "subscriptions.c"
	}
}


const gchar* adblock_subscription_get_title (AdblockSubscription* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_title;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3676 "subscriptions.c"
}


void adblock_subscription_set_title (AdblockSubscription* self, const gchar* value) {
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (g_strcmp0 (value, adblock_subscription_get_title (self)) != 0) {
#line 3685 "subscriptions.c"
		const gchar* _tmp0_;
		gchar* _tmp1_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp1_ = g_strdup (_tmp0_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_g_free0 (self->priv->_title);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_title = _tmp1_;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY]);
#line 3698 "subscriptions.c"
	}
}


gboolean adblock_subscription_get_active (AdblockSubscription* self) {
	gboolean result;
	gboolean _tmp0_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_active;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3714 "subscriptions.c"
}


void adblock_subscription_set_active (AdblockSubscription* self, gboolean value) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (adblock_subscription_get_active (self) != value) {
#line 3723 "subscriptions.c"
		gboolean _tmp0_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_active = _tmp0_;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY]);
#line 3731 "subscriptions.c"
	}
}


gboolean adblock_subscription_get_mutable (AdblockSubscription* self) {
	gboolean result;
	gboolean _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_mutable;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3747 "subscriptions.c"
}


void adblock_subscription_set_mutable (AdblockSubscription* self, gboolean value) {
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (adblock_subscription_get_mutable (self) != value) {
#line 3756 "subscriptions.c"
		gboolean _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_mutable = _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY]);
#line 3764 "subscriptions.c"
	}
}


gboolean adblock_subscription_get_valid (AdblockSubscription* self) {
	gboolean result;
	gboolean _tmp0_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_valid;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3780 "subscriptions.c"
}


static void adblock_subscription_set_valid (AdblockSubscription* self, gboolean value) {
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (adblock_subscription_get_valid (self) != value) {
#line 3789 "subscriptions.c"
		gboolean _tmp0_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_valid = _tmp0_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_VALID_PROPERTY]);
#line 3797 "subscriptions.c"
	}
}


guint adblock_subscription_get_size (AdblockSubscription* self) {
	guint result;
	guint _tmp0_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_tmp0_ = self->priv->_size;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	result = _tmp0_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	return result;
#line 3813 "subscriptions.c"
}


static void adblock_subscription_set_size (AdblockSubscription* self, guint value) {
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_return_if_fail (self != NULL);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	if (adblock_subscription_get_size (self) != value) {
#line 3822 "subscriptions.c"
		guint _tmp0_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		_tmp0_ = value;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		self->priv->_size = _tmp0_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_object_notify_by_pspec ((GObject *) self, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY]);
#line 3830 "subscriptions.c"
	}
}


static void adblock_subscription_class_init (AdblockSubscriptionClass * klass) {
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	adblock_subscription_parent_class = g_type_class_peek_parent (klass);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_type_class_add_private (klass, sizeof (AdblockSubscriptionPrivate));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_adblock_subscription_get_property;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_adblock_subscription_set_property;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	G_OBJECT_CLASS (klass)->finalize = adblock_subscription_finalize;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_URI_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_URI_PROPERTY] = g_param_spec_string ("uri", "uri", "uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY] = g_param_spec_string ("title", "title", "title", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY] = g_param_spec_boolean ("active", "active", "active", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY] = g_param_spec_boolean ("mutable", "mutable", "mutable", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_VALID_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_VALID_PROPERTY] = g_param_spec_boolean ("valid", "valid", "valid", TRUE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY, adblock_subscription_properties[ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY] = g_param_spec_uint ("size", "size", "size", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 3858 "subscriptions.c"
}


static void adblock_subscription_instance_init (AdblockSubscription * self) {
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv = ADBLOCK_SUBSCRIPTION_GET_PRIVATE (self);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->_uri = NULL;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->_title = NULL;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->_active = TRUE;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->_mutable = TRUE;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self->priv->_valid = TRUE;
#line 3875 "subscriptions.c"
}


static void adblock_subscription_finalize (GObject * obj) {
	AdblockSubscription * self;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, ADBLOCK_TYPE_SUBSCRIPTION, AdblockSubscription);
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (self->path);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (self->priv->_uri);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_free0 (self->priv->_title);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_hash_table_unref0 (self->priv->cache);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	__g_list_free__g_object_unref0_0 (self->priv->features);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->pattern);
#line 39 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->keys);
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->optslist);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->whitelist);
#line 42 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->element);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	_g_object_unref0 (self->priv->download);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	G_OBJECT_CLASS (adblock_subscription_parent_class)->finalize (obj);
#line 3907 "subscriptions.c"
}


GType adblock_subscription_get_type (void) {
	static volatile gsize adblock_subscription_type_id__volatile = 0;
	if (g_once_init_enter (&adblock_subscription_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (AdblockSubscriptionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) adblock_subscription_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AdblockSubscription), 0, (GInstanceInitFunc) adblock_subscription_instance_init, NULL };
		GType adblock_subscription_type_id;
		adblock_subscription_type_id = g_type_register_static (G_TYPE_OBJECT, "AdblockSubscription", &g_define_type_info, 0);
		g_once_init_leave (&adblock_subscription_type_id__volatile, adblock_subscription_type_id);
	}
	return adblock_subscription_type_id__volatile;
}


static void _vala_adblock_subscription_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AdblockSubscription * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_SUBSCRIPTION, AdblockSubscription);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	switch (property_id) {
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_URI_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_string (value, adblock_subscription_get_uri (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_string (value, adblock_subscription_get_title (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_boolean (value, adblock_subscription_get_active (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_boolean (value, adblock_subscription_get_mutable (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_VALID_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_boolean (value, adblock_subscription_get_valid (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		g_value_set_uint (value, adblock_subscription_get_size (self));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 3964 "subscriptions.c"
		default:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 3970 "subscriptions.c"
	}
}


static void _vala_adblock_subscription_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AdblockSubscription * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, ADBLOCK_TYPE_SUBSCRIPTION, AdblockSubscription);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
	switch (property_id) {
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_URI_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_uri (self, g_value_get_string (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_TITLE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_title (self, g_value_get_string (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_ACTIVE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_active (self, g_value_get_boolean (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_MUTABLE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_mutable (self, g_value_get_boolean (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_VALID_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_valid (self, g_value_get_boolean (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		case ADBLOCK_SUBSCRIPTION_SIZE_PROPERTY:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		adblock_subscription_set_size (self, g_value_get_uint (value));
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 4016 "subscriptions.c"
		default:
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/adblock/subscriptions.vala"
		break;
#line 4022 "subscriptions.c"
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



