/* open-with.c generated by valac 0.38.8, the Vala compiler
 * generated from open-with.vala, do not modify */

/*
   Copyright (C) 2014 Christian Dywan <christian@twotoasts.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include "midori-core.h"
#include <gtk/gtk.h>
#include "midori/midori-browser.h"
#include <glib/gi18n-lib.h>
#include <pango/pango.h>
#include <gdk/gdk.h>
#include "katze/katze.h"
#include "midori/midori-extension.h"
#include "webkit/webkit.h"
#include "midori/midori-websettings.h"
#include "midori/midori-app.h"
#include "midori/midori-view.h"
#include "open-with.h"

#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS (external_applications_associations_get_type ())
#define EXTERNAL_APPLICATIONS_ASSOCIATIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS, ExternalApplicationsAssociations))
#define EXTERNAL_APPLICATIONS_ASSOCIATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS, ExternalApplicationsAssociationsClass))
#define EXTERNAL_APPLICATIONS_IS_ASSOCIATIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS))
#define EXTERNAL_APPLICATIONS_IS_ASSOCIATIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS))
#define EXTERNAL_APPLICATIONS_ASSOCIATIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS, ExternalApplicationsAssociationsClass))

typedef struct _ExternalApplicationsAssociations ExternalApplicationsAssociations;
typedef struct _ExternalApplicationsAssociationsClass ExternalApplicationsAssociationsClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _ExternalApplicationsAssociationsPrivate ExternalApplicationsAssociationsPrivate;
enum  {
	EXTERNAL_APPLICATIONS_ASSOCIATIONS_0_PROPERTY,
	EXTERNAL_APPLICATIONS_ASSOCIATIONS_NUM_PROPERTIES
};
static GParamSpec* external_applications_associations_properties[EXTERNAL_APPLICATIONS_ASSOCIATIONS_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))

#define EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG (external_applications_customizer_dialog_get_type ())
#define EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG, ExternalApplicationsCustomizerDialog))
#define EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG, ExternalApplicationsCustomizerDialogClass))
#define EXTERNAL_APPLICATIONS_IS_CUSTOMIZER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG))
#define EXTERNAL_APPLICATIONS_IS_CUSTOMIZER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG))
#define EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG, ExternalApplicationsCustomizerDialogClass))

typedef struct _ExternalApplicationsCustomizerDialog ExternalApplicationsCustomizerDialog;
typedef struct _ExternalApplicationsCustomizerDialogClass ExternalApplicationsCustomizerDialogClass;
typedef struct _ExternalApplicationsCustomizerDialogPrivate ExternalApplicationsCustomizerDialogPrivate;
enum  {
	EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG_0_PROPERTY,
	EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG_NUM_PROPERTIES
};
static GParamSpec* external_applications_customizer_dialog_properties[EXTERNAL_APPLICATIONS_CUSTOMIZER_DIALOG_NUM_PROPERTIES];

#define EXTERNAL_APPLICATIONS_TYPE_CHOOSER (external_applications_chooser_get_type ())
#define EXTERNAL_APPLICATIONS_CHOOSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER, ExternalApplicationsChooser))
#define EXTERNAL_APPLICATIONS_CHOOSER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER, ExternalApplicationsChooserClass))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER))
#define EXTERNAL_APPLICATIONS_CHOOSER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER, ExternalApplicationsChooserClass))

typedef struct _ExternalApplicationsChooser ExternalApplicationsChooser;
typedef struct _ExternalApplicationsChooserClass ExternalApplicationsChooserClass;
typedef struct _ExternalApplicationsChooserPrivate ExternalApplicationsChooserPrivate;
enum  {
	EXTERNAL_APPLICATIONS_CHOOSER_0_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_NUM_PROPERTIES
};
static GParamSpec* external_applications_chooser_properties[EXTERNAL_APPLICATIONS_CHOOSER_NUM_PROPERTIES];
typedef struct _Block1Data Block1Data;
enum  {
	EXTERNAL_APPLICATIONS_CHOOSER_SELECTED_SIGNAL,
	EXTERNAL_APPLICATIONS_CHOOSER_CUSTOMIZED_SIGNAL,
	EXTERNAL_APPLICATIONS_CHOOSER_NUM_SIGNALS
};
static guint external_applications_chooser_signals[EXTERNAL_APPLICATIONS_CHOOSER_NUM_SIGNALS] = {0};

#define EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG (external_applications_chooser_dialog_get_type ())
#define EXTERNAL_APPLICATIONS_CHOOSER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialog))
#define EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialogClass))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG))
#define EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialogClass))

typedef struct _ExternalApplicationsChooserDialog ExternalApplicationsChooserDialog;
typedef struct _ExternalApplicationsChooserDialogClass ExternalApplicationsChooserDialogClass;
typedef struct _ExternalApplicationsChooserDialogPrivate ExternalApplicationsChooserDialogPrivate;
enum  {
	EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_0_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_NUM_PROPERTIES
};
static GParamSpec* external_applications_chooser_dialog_properties[EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_NUM_PROPERTIES];

#define EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON (external_applications_chooser_button_get_type ())
#define EXTERNAL_APPLICATIONS_CHOOSER_BUTTON(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButton))
#define EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButtonClass))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER_BUTTON(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON))
#define EXTERNAL_APPLICATIONS_IS_CHOOSER_BUTTON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON))
#define EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButtonClass))

typedef struct _ExternalApplicationsChooserButton ExternalApplicationsChooserButton;
typedef struct _ExternalApplicationsChooserButtonClass ExternalApplicationsChooserButtonClass;
typedef struct _ExternalApplicationsChooserButtonPrivate ExternalApplicationsChooserButtonPrivate;
enum  {
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_0_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY,
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_NUM_PROPERTIES
};
static GParamSpec* external_applications_chooser_button_properties[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_NUM_PROPERTIES];
typedef struct _Block2Data Block2Data;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
enum  {
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_SELECTED_SIGNAL,
	EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_NUM_SIGNALS
};
static guint external_applications_chooser_button_signals[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_NUM_SIGNALS] = {0};

#define EXTERNAL_APPLICATIONS_TYPE_TYPES (external_applications_types_get_type ())
#define EXTERNAL_APPLICATIONS_TYPES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_TYPES, ExternalApplicationsTypes))
#define EXTERNAL_APPLICATIONS_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_TYPES, ExternalApplicationsTypesClass))
#define EXTERNAL_APPLICATIONS_IS_TYPES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_TYPES))
#define EXTERNAL_APPLICATIONS_IS_TYPES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_TYPES))
#define EXTERNAL_APPLICATIONS_TYPES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_TYPES, ExternalApplicationsTypesClass))

typedef struct _ExternalApplicationsTypes ExternalApplicationsTypes;
typedef struct _ExternalApplicationsTypesClass ExternalApplicationsTypesClass;
typedef struct _ExternalApplicationsTypesPrivate ExternalApplicationsTypesPrivate;
enum  {
	EXTERNAL_APPLICATIONS_TYPES_0_PROPERTY,
	EXTERNAL_APPLICATIONS_TYPES_NUM_PROPERTIES
};
static GParamSpec* external_applications_types_properties[EXTERNAL_APPLICATIONS_TYPES_NUM_PROPERTIES];
#define __g_list_free__g_free0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_free0_ (var), NULL)))
enum  {
	EXTERNAL_APPLICATIONS_TYPES_SELECTED_SIGNAL,
	EXTERNAL_APPLICATIONS_TYPES_NUM_SIGNALS
};
static guint external_applications_types_signals[EXTERNAL_APPLICATIONS_TYPES_NUM_SIGNALS] = {0};

#define EXTERNAL_APPLICATIONS_TYPE_MANAGER (external_applications_manager_get_type ())
#define EXTERNAL_APPLICATIONS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), EXTERNAL_APPLICATIONS_TYPE_MANAGER, ExternalApplicationsManager))
#define EXTERNAL_APPLICATIONS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), EXTERNAL_APPLICATIONS_TYPE_MANAGER, ExternalApplicationsManagerClass))
#define EXTERNAL_APPLICATIONS_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EXTERNAL_APPLICATIONS_TYPE_MANAGER))
#define EXTERNAL_APPLICATIONS_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), EXTERNAL_APPLICATIONS_TYPE_MANAGER))
#define EXTERNAL_APPLICATIONS_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), EXTERNAL_APPLICATIONS_TYPE_MANAGER, ExternalApplicationsManagerClass))

typedef struct _ExternalApplicationsManager ExternalApplicationsManager;
typedef struct _ExternalApplicationsManagerClass ExternalApplicationsManagerClass;
typedef struct _ExternalApplicationsManagerPrivate ExternalApplicationsManagerPrivate;
enum  {
	EXTERNAL_APPLICATIONS_MANAGER_0_PROPERTY,
	EXTERNAL_APPLICATIONS_MANAGER_NUM_PROPERTIES
};
static GParamSpec* external_applications_manager_properties[EXTERNAL_APPLICATIONS_MANAGER_NUM_PROPERTIES];

#define EXTERNAL_APPLICATIONS_MANAGER_TYPE_NEXT_STEP (external_applications_manager_next_step_get_type ())
typedef struct _Block3Data Block3Data;
typedef struct _Block4Data Block4Data;
typedef struct _Block5Data Block5Data;
typedef struct _Block6Data Block6Data;
typedef struct _Block7Data Block7Data;
#define _g_list_free0(var) ((var == NULL) ? NULL : (var = (g_list_free (var), NULL)))

struct _ExternalApplicationsAssociations {
	GObject parent_instance;
	ExternalApplicationsAssociationsPrivate * priv;
};

struct _ExternalApplicationsAssociationsClass {
	GObjectClass parent_class;
};

struct _ExternalApplicationsCustomizerDialog {
	GtkDialog parent_instance;
	ExternalApplicationsCustomizerDialogPrivate * priv;
	GtkEntry* name_entry;
	GtkEntry* commandline_entry;
};

struct _ExternalApplicationsCustomizerDialogClass {
	GtkDialogClass parent_class;
};

struct _ExternalApplicationsChooser {
	GtkVBox parent_instance;
	ExternalApplicationsChooserPrivate * priv;
};

struct _ExternalApplicationsChooserClass {
	GtkVBoxClass parent_class;
};

struct _ExternalApplicationsChooserPrivate {
	GtkListStore* store;
	GtkTreeView* treeview;
	GList* available;
	gchar* content_type;
	gchar* uri;
};

struct _Block1Data {
	int _ref_count_;
	ExternalApplicationsChooser* self;
	GAppInfo* app_info;
};

struct _ExternalApplicationsChooserDialog {
	GtkDialog parent_instance;
	ExternalApplicationsChooserDialogPrivate * priv;
};

struct _ExternalApplicationsChooserDialogClass {
	GtkDialogClass parent_class;
};

struct _ExternalApplicationsChooserDialogPrivate {
	ExternalApplicationsChooser* _chooser;
};

struct _ExternalApplicationsChooserButton {
	GtkButton parent_instance;
	ExternalApplicationsChooserButtonPrivate * priv;
};

struct _ExternalApplicationsChooserButtonClass {
	GtkButtonClass parent_class;
};

struct _ExternalApplicationsChooserButtonPrivate {
	GAppInfo* _app_info;
	gchar* _commandline;
	ExternalApplicationsChooserDialog* dialog;
	GtkLabel* app_name;
	GtkImage* icon;
};

struct _Block2Data {
	int _ref_count_;
	ExternalApplicationsChooserButton* self;
	gchar* commandline;
};

struct _ExternalApplicationsTypes {
	GtkVBox parent_instance;
	ExternalApplicationsTypesPrivate * priv;
	GtkListStore* store;
};

struct _ExternalApplicationsTypesClass {
	GtkVBoxClass parent_class;
};

struct _ExternalApplicationsTypesPrivate {
	GtkTreeView* treeview;
};

struct _ExternalApplicationsManager {
	MidoriExtension parent_instance;
	ExternalApplicationsManagerPrivate * priv;
};

struct _ExternalApplicationsManagerClass {
	MidoriExtensionClass parent_class;
};

typedef enum  {
	EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN,
	EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_OPEN_WITH
} ExternalApplicationsManagerNextStep;

struct _Block3Data {
	int _ref_count_;
	ExternalApplicationsManager* self;
	WebKitDownload* download;
	gchar* content_type;
	GtkWidget* widget;
	ExternalApplicationsManagerNextStep next_step;
};

struct _Block4Data {
	int _ref_count_;
	ExternalApplicationsManager* self;
	MidoriTab* tab;
};

struct _Block5Data {
	int _ref_count_;
	Block4Data * _data4_;
	gchar* uri;
};

struct _Block6Data {
	int _ref_count_;
	Block4Data * _data4_;
	gchar* uri;
};

struct _Block7Data {
	int _ref_count_;
	ExternalApplicationsManager* self;
	MidoriWebSettings* settings;
	ExternalApplicationsTypes* types;
	KatzePreferences* preferences;
};


static gpointer external_applications_associations_parent_class = NULL;
static gpointer external_applications_customizer_dialog_parent_class = NULL;
static gpointer external_applications_chooser_parent_class = NULL;
static gpointer external_applications_chooser_dialog_parent_class = NULL;
static gpointer external_applications_chooser_button_parent_class = NULL;
static gpointer external_applications_types_parent_class = NULL;
static gpointer external_applications_manager_parent_class = NULL;

gboolean external_applications_open_app_info (GAppInfo* app_info, const gchar* uri, const gchar* content_type);
static void _g_object_unref0_ (gpointer var);
static inline void _g_list_free__g_object_unref0_ (GList* self);
ExternalApplicationsAssociations* external_applications_associations_new (void);
ExternalApplicationsAssociations* external_applications_associations_construct (GType object_type);
GType external_applications_associations_get_type (void) G_GNUC_CONST;
void external_applications_associations_remember (ExternalApplicationsAssociations* self, const gchar* content_type, GAppInfo* app_info, GError** error);
gboolean external_applications_associations_open (ExternalApplicationsAssociations* self, const gchar* content_type, const gchar* uri);
void external_applications_associations_remember_custom_commandline (ExternalApplicationsAssociations* self, const gchar* content_type, const gchar* commandline, const gchar* name, const gchar* uri);
gchar* external_applications_get_commandline (GAppInfo* app_info);
gchar* external_applications_describe_app_info (GAppInfo* app_info);
GIcon* external_applications_app_info_get_icon (GAppInfo* app_info);
GType external_applications_customizer_dialog_get_type (void) G_GNUC_CONST;
ExternalApplicationsCustomizerDialog* external_applications_customizer_dialog_new (GAppInfo* app_info, GtkWidget* widget);
ExternalApplicationsCustomizerDialog* external_applications_customizer_dialog_construct (GType object_type, GAppInfo* app_info, GtkWidget* widget);
static void external_applications_customizer_dialog_finalize (GObject * obj);
GType external_applications_chooser_get_type (void) G_GNUC_CONST;
#define EXTERNAL_APPLICATIONS_CHOOSER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EXTERNAL_APPLICATIONS_TYPE_CHOOSER, ExternalApplicationsChooserPrivate))
ExternalApplicationsChooser* external_applications_chooser_new (const gchar* uri, const gchar* content_type);
ExternalApplicationsChooser* external_applications_chooser_construct (GType object_type, const gchar* uri, const gchar* content_type);
static gint external_applications_chooser_tree_sort_func (ExternalApplicationsChooser* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gint _external_applications_chooser_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void external_applications_chooser_on_render_icon (ExternalApplicationsChooser* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_chooser_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_chooser_on_render_text (ExternalApplicationsChooser* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_chooser_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_chooser_row_activated (ExternalApplicationsChooser* self, GtkTreePath* path, GtkTreeViewColumn* column);
static void _external_applications_chooser_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static gboolean external_applications_chooser_button_released (ExternalApplicationsChooser* self, GdkEventButton* event);
static gboolean _external_applications_chooser_button_released_gtk_widget_button_release_event (GtkWidget* _sender, GdkEventButton* event, gpointer self);
static void external_applications_chooser_launcher_added (ExternalApplicationsChooser* self, GAppInfo* app_info, const gchar* uri);
static gboolean external_applications_chooser_show_popup_menu (ExternalApplicationsChooser* self, GdkEventButton* event);
static Block1Data* block1_data_ref (Block1Data* _data1_);
static void block1_data_unref (void * _userdata_);
static void ___lambda4_ (Block1Data* _data1_);
static void external_applications_chooser_customize_app_info (ExternalApplicationsChooser* self, GAppInfo* app_info, const gchar* content_type, const gchar* uri);
static void ____lambda4__gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self);
GList* external_applications_chooser_get_available (ExternalApplicationsChooser* self);
GAppInfo* external_applications_chooser_get_app_info (ExternalApplicationsChooser* self);
static void g_cclosure_user_marshal_VOID__OBJECT_STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void external_applications_chooser_finalize (GObject * obj);
GType external_applications_chooser_dialog_get_type (void) G_GNUC_CONST;
#define EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialogPrivate))
ExternalApplicationsChooserDialog* external_applications_chooser_dialog_new (const gchar* uri, const gchar* content_type, GtkWidget* widget);
ExternalApplicationsChooserDialog* external_applications_chooser_dialog_construct (GType object_type, const gchar* uri, const gchar* content_type, GtkWidget* widget);
static void external_applications_chooser_dialog_set_chooser (ExternalApplicationsChooserDialog* self, ExternalApplicationsChooser* value);
ExternalApplicationsChooser* external_applications_chooser_dialog_get_chooser (ExternalApplicationsChooserDialog* self);
static void __lambda5_ (ExternalApplicationsChooserDialog* self, GAppInfo* app_info);
static void ___lambda5__external_applications_chooser_selected (ExternalApplicationsChooser* _sender, GAppInfo* app_info, gpointer self);
static void __lambda6_ (ExternalApplicationsChooserDialog* self, GAppInfo* app_info, const gchar* content_type, const gchar* uri);
static void ___lambda6__external_applications_chooser_customized (ExternalApplicationsChooser* _sender, GAppInfo* app_info, const gchar* content_type, const gchar* uri, gpointer self);
GAppInfo* external_applications_chooser_dialog_open_with (ExternalApplicationsChooserDialog* self);
static void external_applications_chooser_dialog_finalize (GObject * obj);
static void _vala_external_applications_chooser_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_external_applications_chooser_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType external_applications_chooser_button_get_type (void) G_GNUC_CONST;
#define EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButtonPrivate))
ExternalApplicationsChooserButton* external_applications_chooser_button_new (const gchar* mime_type, const gchar* commandline);
ExternalApplicationsChooserButton* external_applications_chooser_button_construct (GType object_type, const gchar* mime_type, const gchar* commandline);
static Block2Data* block2_data_ref (Block2Data* _data2_);
static void block2_data_unref (void * _userdata_);
void external_applications_chooser_button_set_app_info (ExternalApplicationsChooserButton* self, GAppInfo* value);
static void external_applications_chooser_button_update_label (ExternalApplicationsChooserButton* self);
static void __lambda7_ (Block2Data* _data2_);
GAppInfo* external_applications_chooser_button_get_app_info (ExternalApplicationsChooserButton* self);
static void ___lambda7__gtk_button_clicked (GtkButton* _sender, gpointer self);
const gchar* external_applications_chooser_button_get_commandline (ExternalApplicationsChooserButton* self);
void external_applications_chooser_button_set_commandline (ExternalApplicationsChooserButton* self, const gchar* value);
static void external_applications_chooser_button_finalize (GObject * obj);
static void _vala_external_applications_chooser_button_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_external_applications_chooser_button_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType external_applications_types_get_type (void) G_GNUC_CONST;
#define EXTERNAL_APPLICATIONS_TYPES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EXTERNAL_APPLICATIONS_TYPE_TYPES, ExternalApplicationsTypesPrivate))
ExternalApplicationsTypes* external_applications_types_new (void);
ExternalApplicationsTypes* external_applications_types_construct (GType object_type);
static gint external_applications_types_tree_sort_func (ExternalApplicationsTypes* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gint _external_applications_types_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void external_applications_types_on_render_type_icon (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_types_on_render_type_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_types_on_render_type_text (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_types_on_render_type_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_types_on_render_icon (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_types_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_types_on_render_text (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter);
static void _external_applications_types_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self);
static void external_applications_types_row_activated (ExternalApplicationsTypes* self, GtkTreePath* path, GtkTreeViewColumn* column);
static void _external_applications_types_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void external_applications_types_launcher_added (ExternalApplicationsTypes* self, const gchar* content_type);
static void _g_free0_ (gpointer var);
static inline void _g_list_free__g_free0_ (GList* self);
static void __lambda8_ (ExternalApplicationsTypes* self, GdkRectangle* allocation);
static void ___lambda8__gtk_widget_size_allocate (GtkWidget* _sender, GdkRectangle* allocation, gpointer self);
static void g_cclosure_user_marshal_VOID__STRING_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
static void external_applications_types_finalize (GObject * obj);
GType external_applications_manager_get_type (void) G_GNUC_CONST;
static GType external_applications_manager_next_step_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static gboolean external_applications_manager_open_uri (ExternalApplicationsManager* self, MidoriTab* tab, const gchar* uri);
static gchar* external_applications_manager_get_content_type (ExternalApplicationsManager* self, const gchar* uri, const gchar* mime_type);
static gboolean external_applications_manager_open_with_type (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget, ExternalApplicationsManagerNextStep next_step);
static gboolean external_applications_manager_navigation_requested (ExternalApplicationsManager* self, MidoriTab* tab, const gchar* uri);
static Block3Data* block3_data_ref (Block3Data* _data3_);
static void block3_data_unref (void * _userdata_);
static gboolean external_applications_manager_open_now (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget, ExternalApplicationsManagerNextStep next_step);
static void __lambda9_ (Block3Data* _data3_, GParamSpec* pspec);
static void ___lambda9__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static GAppInfo* external_applications_manager_open_with (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget);
static void external_applications_manager_context_menu (ExternalApplicationsManager* self, MidoriTab* tab, WebKitHitTestResult* hit_test_result, MidoriContextAction* menu);
static Block4Data* block4_data_ref (Block4Data* _data4_);
static void block4_data_unref (void * _userdata_);
static Block5Data* block5_data_ref (Block5Data* _data5_);
static void block5_data_unref (void * _userdata_);
static void ___lambda10_ (Block5Data* _data5_, GtkAction* action);
static void ____lambda10__gtk_action_activate (GtkAction* _sender, gpointer self);
static Block6Data* block6_data_ref (Block6Data* _data6_);
static void block6_data_unref (void * _userdata_);
static void ___lambda11_ (Block6Data* _data6_, GtkAction* action);
static void ____lambda11__gtk_action_activate (GtkAction* _sender, gpointer self);
static void external_applications_manager_show_preferences (ExternalApplicationsManager* self, KatzePreferences* preferences);
static Block7Data* block7_data_ref (Block7Data* _data7_);
static void block7_data_unref (void * _userdata_);
static void __lambda12_ (Block7Data* _data7_, const gchar* commandline);
static void ___lambda12__external_applications_chooser_button_selected (ExternalApplicationsChooserButton* _sender, const gchar* commandline, gpointer self);
static void __lambda13_ (Block7Data* _data7_, const gchar* commandline);
static void ___lambda13__external_applications_chooser_button_selected (ExternalApplicationsChooserButton* _sender, const gchar* commandline, gpointer self);
static void __lambda14_ (Block7Data* _data7_, const gchar* content_type, GtkTreeIter* iter);
static void ___lambda14__external_applications_types_selected (ExternalApplicationsTypes* _sender, const gchar* content_type, GtkTreeIter* iter, gpointer self);
void external_applications_manager_tab_added (ExternalApplicationsManager* self, MidoriBrowser* browser, MidoriView* view);
static gboolean _external_applications_manager_navigation_requested_midori_tab_navigation_requested (MidoriTab* _sender, const gchar* uri, gpointer self);
static gboolean _external_applications_manager_open_uri_midori_tab_open_uri (MidoriTab* _sender, const gchar* uri, gpointer self);
static void _external_applications_manager_context_menu_midori_tab_context_menu (MidoriTab* _sender, WebKitHitTestResult* hit_test_result, MidoriContextAction* menu, gpointer self);
void external_applications_manager_tab_removed (ExternalApplicationsManager* self, MidoriBrowser* browser, MidoriView* view);
static void external_applications_manager_browser_added (ExternalApplicationsManager* self, MidoriBrowser* browser);
static void _external_applications_manager_tab_added_midori_browser_add_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self);
static void _external_applications_manager_tab_removed_midori_browser_remove_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self);
static void _external_applications_manager_show_preferences_midori_browser_show_preferences (MidoriBrowser* _sender, KatzePreferences* preferences, gpointer self);
static void external_applications_manager_activated (ExternalApplicationsManager* self, MidoriApp* app);
static void _external_applications_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self);
static void external_applications_manager_browser_removed (ExternalApplicationsManager* self, MidoriBrowser* browser);
static void external_applications_manager_deactivated (ExternalApplicationsManager* self);
ExternalApplicationsManager* external_applications_manager_new (void);
ExternalApplicationsManager* external_applications_manager_construct (GType object_type);
static void _external_applications_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self);
static void _external_applications_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


static void _g_object_unref0_ (gpointer var) {
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 485 "open-with.c"
}


static inline void _g_list_free__g_object_unref0_ (GList* self) {
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_list_free_full (self, (GDestroyNotify) _g_object_unref0_);
#line 492 "open-with.c"
}


gboolean external_applications_open_app_info (GAppInfo* app_info, const gchar* uri, const gchar* content_type) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	GError * _inner_error_ = NULL;
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (app_info != NULL, FALSE);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 23 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, FALSE);
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = uri;
#line 24 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	midori_uri_recursive_fork_protection (_tmp0_, TRUE);
#line 510 "open-with.c"
	{
		GList* uris = NULL;
		const gchar* _tmp1_;
		GFile* _tmp2_;
		GAppInfo* _tmp3_;
		GList* _tmp4_;
#line 27 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		uris = NULL;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = uri;
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = g_file_new_for_uri (_tmp1_);
#line 28 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		uris = g_list_append (uris, _tmp2_);
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = app_info;
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = uris;
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_app_info_launch (_tmp3_, _tmp4_, NULL, &_inner_error_);
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 29 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			__g_list_free__g_object_unref0_0 (uris);
#line 535 "open-with.c"
			goto __catch0_g_error;
		}
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		__g_list_free__g_object_unref0_0 (uris);
#line 540 "open-with.c"
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError* _error_ = NULL;
		const gchar* _tmp5_;
		GError* _tmp6_;
		const gchar* _tmp7_;
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_error_ = _inner_error_;
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_inner_error_ = NULL;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = uri;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = _error_;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = _tmp6_->message;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_warning ("open-with.vala:31: Failed to open \"%s\": %s", _tmp5_, _tmp7_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = FALSE;
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_error_free0 (_error_);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 567 "open-with.c"
	}
	__finally0:
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 572 "open-with.c"
		gboolean _tmp8_ = FALSE;
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_clear_error (&_inner_error_);
#line 26 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return _tmp8_;
#line 580 "open-with.c"
	}
	{
		ExternalApplicationsAssociations* _tmp9_;
		ExternalApplicationsAssociations* _tmp10_;
		const gchar* _tmp11_;
		GAppInfo* _tmp12_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = external_applications_associations_new ();
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = _tmp9_;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = content_type;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = app_info;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_associations_remember (_tmp10_, _tmp11_, _tmp12_, &_inner_error_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp10_);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 601 "open-with.c"
			goto __catch1_g_error;
		}
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError* _error_ = NULL;
		const gchar* _tmp13_;
		GError* _tmp14_;
		const gchar* _tmp15_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_error_ = _inner_error_;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_inner_error_ = NULL;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = uri;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = _error_;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp15_ = _tmp14_->message;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_warning ("open-with.vala:38: Failed to save association for \"%s\": %s", _tmp13_, _tmp15_);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_error_free0 (_error_);
#line 626 "open-with.c"
	}
	__finally1:
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 631 "open-with.c"
		gboolean _tmp16_ = FALSE;
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_clear_error (&_inner_error_);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return _tmp16_;
#line 639 "open-with.c"
	}
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = TRUE;
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 645 "open-with.c"
}


ExternalApplicationsAssociations* external_applications_associations_construct (GType object_type) {
	ExternalApplicationsAssociations * self = NULL;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsAssociations*) g_object_new (object_type, NULL);
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 655 "open-with.c"
}


ExternalApplicationsAssociations* external_applications_associations_new (void) {
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_associations_construct (EXTERNAL_APPLICATIONS_TYPE_ASSOCIATIONS);
#line 662 "open-with.c"
}


gboolean external_applications_associations_open (ExternalApplicationsAssociations* self, const gchar* content_type, const gchar* uri) {
	gboolean result = FALSE;
	GAppInfo* app_info = NULL;
	const gchar* _tmp0_;
	GAppInfo* _tmp1_;
	GAppInfo* _tmp2_;
	GAppInfo* _tmp3_;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
	gboolean _tmp6_;
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, FALSE);
#line 106 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = content_type;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_app_info_get_default_for_type (_tmp0_, FALSE);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	app_info = _tmp1_;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = app_info;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp2_ == NULL) {
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = FALSE;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 698 "open-with.c"
	}
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = app_info;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = uri;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = content_type;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = external_applications_open_app_info (_tmp3_, _tmp4_, _tmp5_);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp6_;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 714 "open-with.c"
}


void external_applications_associations_remember (ExternalApplicationsAssociations* self, const gchar* content_type, GAppInfo* app_info, GError** error) {
	GAppInfo* _tmp0_;
	const gchar* _tmp1_;
	GAppInfo* _tmp2_;
	const gchar* _tmp3_;
	GError * _inner_error_ = NULL;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app_info != NULL);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = app_info;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = content_type;
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_app_info_set_as_last_used_for_type (_tmp0_, _tmp1_, &_inner_error_);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_propagate_error (error, _inner_error_);
#line 115 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 742 "open-with.c"
	}
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = app_info;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = content_type;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_app_info_set_as_default_for_type (_tmp2_, _tmp3_, &_inner_error_);
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_propagate_error (error, _inner_error_);
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 756 "open-with.c"
	}
}


static gboolean string_contains (const gchar* self, const gchar* needle) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	gchar* _tmp1_;
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, FALSE);
#line 1411 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (needle != NULL, FALSE);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = needle;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = strstr ((gchar*) self, (gchar*) _tmp0_);
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_ != NULL;
#line 1412 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 777 "open-with.c"
}


void external_applications_associations_remember_custom_commandline (ExternalApplicationsAssociations* self, const gchar* content_type, const gchar* commandline, const gchar* name, const gchar* uri) {
	GError * _inner_error_ = NULL;
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (commandline != NULL);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (name != NULL);
#line 120 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (uri != NULL);
#line 793 "open-with.c"
	{
		GAppInfoCreateFlags _tmp0_ = 0;
		const gchar* _tmp1_;
		gboolean _tmp2_;
		GAppInfo* app_info = NULL;
		const gchar* _tmp3_;
		const gchar* _tmp4_;
		GAppInfo* _tmp5_;
		const gchar* _tmp6_;
		const gchar* _tmp7_;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = commandline;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = string_contains (_tmp1_, "%u");
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (_tmp2_) {
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp0_ = G_APP_INFO_CREATE_SUPPORTS_URIS;
#line 812 "open-with.c"
		} else {
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp0_ = G_APP_INFO_CREATE_NONE;
#line 816 "open-with.c"
		}
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = commandline;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = name;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = g_app_info_create_from_commandline (_tmp3_, _tmp4_, _tmp0_, &_inner_error_);
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		app_info = _tmp5_;
#line 122 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 828 "open-with.c"
			goto __catch2_g_error;
		}
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = uri;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = content_type;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_open_app_info (app_info, _tmp6_, _tmp7_);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 839 "open-with.c"
	}
	goto __finally2;
	__catch2_g_error:
	{
		GError* _error_ = NULL;
		const gchar* _tmp8_;
		GError* _tmp9_;
		const gchar* _tmp10_;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_error_ = _inner_error_;
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_inner_error_ = NULL;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = uri;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = _error_;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = _tmp9_->message;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_warning ("open-with.vala:126: Failed to remember custom command line for \"%s\":" \
" %s", _tmp8_, _tmp10_);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_error_free0 (_error_);
#line 862 "open-with.c"
	}
	__finally2:
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_clear_error (&_inner_error_);
#line 121 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 873 "open-with.c"
	}
}


static void external_applications_associations_class_init (ExternalApplicationsAssociationsClass * klass) {
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_associations_parent_class = g_type_class_peek_parent (klass);
#line 881 "open-with.c"
}


static void external_applications_associations_instance_init (ExternalApplicationsAssociations * self) {
}


GType external_applications_associations_get_type (void) {
	static volatile gsize external_applications_associations_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_associations_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsAssociationsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_associations_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsAssociations), 0, (GInstanceInitFunc) external_applications_associations_instance_init, NULL };
		GType external_applications_associations_type_id;
		external_applications_associations_type_id = g_type_register_static (G_TYPE_OBJECT, "ExternalApplicationsAssociations", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_associations_type_id__volatile, external_applications_associations_type_id);
	}
	return external_applications_associations_type_id__volatile;
}


gchar* external_applications_get_commandline (GAppInfo* app_info) {
	gchar* result = NULL;
	const gchar* _tmp0_ = NULL;
	GAppInfo* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp5_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (app_info != NULL, NULL);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = app_info;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = g_app_info_get_commandline (_tmp1_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _tmp2_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp0_ == NULL) {
#line 917 "open-with.c"
		GAppInfo* _tmp3_;
		const gchar* _tmp4_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = app_info;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = g_app_info_get_executable (_tmp3_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp4_;
#line 926 "open-with.c"
	}
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = g_strdup (_tmp0_);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp5_;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 934 "open-with.c"
}


gchar* external_applications_describe_app_info (GAppInfo* app_info) {
	gchar* result = NULL;
	gchar* _tmp0_ = NULL;
	GAppInfo* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* name = NULL;
	gchar* _tmp7_;
	gchar* _tmp8_ = NULL;
	GAppInfo* _tmp9_;
	const gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* desc = NULL;
	gchar* _tmp14_;
	const gchar* _tmp15_;
	gchar* _tmp16_;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (app_info != NULL, NULL);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = app_info;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = g_app_info_get_display_name (_tmp1_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _tmp3_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp0_ == NULL) {
#line 966 "open-with.c"
		GAppInfo* _tmp4_;
		const gchar* _tmp5_;
		gchar* _tmp6_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = app_info;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = g_app_info_get_executable (_tmp4_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = g_path_get_basename (_tmp5_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp0_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp6_;
#line 980 "open-with.c"
	}
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = _tmp0_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = NULL;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	name = _tmp7_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = app_info;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = g_app_info_get_description (_tmp9_);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = g_strdup (_tmp10_);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = _tmp11_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp8_ == NULL) {
#line 998 "open-with.c"
		GAppInfo* _tmp12_;
		gchar* _tmp13_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = app_info;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = external_applications_get_commandline (_tmp12_);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp8_);
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _tmp13_;
#line 1009 "open-with.c"
	}
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = _tmp8_;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = NULL;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	desc = _tmp14_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = name;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = g_markup_printf_escaped ("<b>%s</b>\n%s", _tmp15_, desc);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp16_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (desc);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp8_);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (name);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp0_);
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1033 "open-with.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self ? g_object_ref (self) : NULL;
#line 1040 "open-with.c"
}


GIcon* external_applications_app_info_get_icon (GAppInfo* app_info) {
	GIcon* result = NULL;
	GAppInfo* _tmp0_;
	GIcon* _tmp1_;
	GIcon* _tmp2_;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (app_info != NULL, NULL);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = app_info;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_app_info_get_icon (_tmp0_);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp2_;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1061 "open-with.c"
}


ExternalApplicationsCustomizerDialog* external_applications_customizer_dialog_construct (GType object_type, GAppInfo* app_info, GtkWidget* widget) {
	ExternalApplicationsCustomizerDialog * self = NULL;
	MidoriBrowser* browser = NULL;
	GtkWidget* _tmp0_;
	MidoriBrowser* _tmp1_;
	MidoriBrowser* _tmp2_;
	GtkVBox* vbox = NULL;
	GtkVBox* _tmp3_;
	GtkWidget* _tmp4_;
	GtkSizeGroup* sizegroup = NULL;
	GtkSizeGroup* _tmp5_;
	GtkLabel* label = NULL;
	GtkLabel* _tmp6_;
	GtkLabel* _tmp7_;
	GtkLabel* _tmp8_;
	GtkLabel* _tmp9_;
	GtkEntry* _tmp10_;
	GtkEntry* _tmp11_;
	GtkEntry* _tmp12_;
	GtkEntry* _tmp13_;
	GtkLabel* _tmp14_;
	GtkLabel* _tmp15_;
	GtkLabel* _tmp16_;
	GtkLabel* _tmp17_;
	GtkEntry* _tmp18_;
	GtkEntry* _tmp19_;
	GtkEntry* _tmp20_;
	GtkEntry* _tmp21_;
	GtkEntry* _tmp22_;
	GtkWidget* _tmp23_;
	GtkEntry* _tmp24_;
	GAppInfo* _tmp25_;
	const gchar* _tmp26_;
	GtkEntry* _tmp27_;
	GAppInfo* _tmp28_;
	gchar* _tmp29_;
	gchar* _tmp30_;
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (app_info != NULL, NULL);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (widget != NULL, NULL);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsCustomizerDialog*) g_object_new (object_type, NULL);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = widget;
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = midori_browser_get_for_widget (_tmp0_);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 156 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	browser = _tmp2_;
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_transient_for ((GtkWindow*) self, (GtkWindow*) browser);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_title ((GtkWindow*) self, _ ("Custom…"));
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_set_has_separator ((GtkDialog*) self, FALSE);
#line 163 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_destroy_with_parent ((GtkWindow*) self, TRUE);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_icon_name ((GtkWindow*) self, GTK_STOCK_OPEN);
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_resizable ((GtkWindow*) self, FALSE);
#line 166 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_add_buttons ((GtkDialog*) self, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
#line 169 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = (GtkVBox*) gtk_vbox_new (FALSE, 8);
#line 169 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp3_);
#line 169 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	vbox = _tmp3_;
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_container_set_border_width ((GtkContainer*) vbox, (guint) 8);
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = gtk_dialog_get_content_area ((GtkDialog*) self);
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, GTK_TYPE_BOX) ? ((GtkBox*) _tmp4_) : NULL, (GtkWidget*) vbox, TRUE, TRUE, (guint) 8);
#line 173 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
#line 173 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	sizegroup = _tmp5_;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = (GtkLabel*) gtk_label_new (_ ("Name:"));
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp6_);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	label = _tmp6_;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = label;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp7_);
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = label;
#line 176 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_misc_set_alignment ((GtkMisc*) _tmp8_, 0.0f, 0.5f);
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = label;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) _tmp9_, FALSE, FALSE, (guint) 0);
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = (GtkEntry*) gtk_entry_new ();
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp10_);
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->name_entry);
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->name_entry = _tmp10_;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = self->name_entry;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_entry_set_activates_default (_tmp11_, TRUE);
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = self->name_entry;
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp12_);
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = self->name_entry;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) _tmp13_, TRUE, TRUE, (guint) 0);
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = (GtkLabel*) gtk_label_new (_ ("Command Line:"));
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp14_);
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (label);
#line 183 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	label = _tmp14_;
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = label;
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp15_);
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = label;
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_misc_set_alignment ((GtkMisc*) _tmp16_, 0.0f, 0.5f);
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = label;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) _tmp17_, FALSE, FALSE, (guint) 0);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = (GtkEntry*) gtk_entry_new ();
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp18_);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->commandline_entry);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->commandline_entry = _tmp18_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = self->commandline_entry;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_entry_set_activates_default (_tmp19_, TRUE);
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = self->name_entry;
#line 189 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp20_);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = self->commandline_entry;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp21_);
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = self->commandline_entry;
#line 191 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) _tmp22_, TRUE, TRUE, (guint) 0);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = gtk_dialog_get_content_area ((GtkDialog*) self);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show_all (_tmp23_);
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_set_default_response ((GtkDialog*) self, (gint) GTK_RESPONSE_ACCEPT);
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = self->name_entry;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = app_info;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = g_app_info_get_name (_tmp25_);
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_entry_set_text (_tmp24_, _tmp26_);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = self->commandline_entry;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = app_info;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = external_applications_get_commandline (_tmp28_);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = _tmp29_;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_entry_set_text (_tmp27_, _tmp30_);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp30_);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (label);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (sizegroup);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (vbox);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (browser);
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 1264 "open-with.c"
}


ExternalApplicationsCustomizerDialog* external_applications_customizer_dialog_new (GAppInfo* app_info, GtkWidget* widget) {
#line 155 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_customizer_dialog_construct (EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG, app_info, widget);
#line 1271 "open-with.c"
}


static void external_applications_customizer_dialog_class_init (ExternalApplicationsCustomizerDialogClass * klass) {
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_customizer_dialog_parent_class = g_type_class_peek_parent (klass);
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->finalize = external_applications_customizer_dialog_finalize;
#line 1280 "open-with.c"
}


static void external_applications_customizer_dialog_instance_init (ExternalApplicationsCustomizerDialog * self) {
}


static void external_applications_customizer_dialog_finalize (GObject * obj) {
	ExternalApplicationsCustomizerDialog * self;
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, EXTERNAL_APPLICATIONS_TYPE_CUSTOMIZER_DIALOG, ExternalApplicationsCustomizerDialog);
#line 152 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->name_entry);
#line 153 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->commandline_entry);
#line 151 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (external_applications_customizer_dialog_parent_class)->finalize (obj);
#line 1298 "open-with.c"
}


GType external_applications_customizer_dialog_get_type (void) {
	static volatile gsize external_applications_customizer_dialog_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_customizer_dialog_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsCustomizerDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_customizer_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsCustomizerDialog), 0, (GInstanceInitFunc) external_applications_customizer_dialog_instance_init, NULL };
		GType external_applications_customizer_dialog_type_id;
		external_applications_customizer_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "ExternalApplicationsCustomizerDialog", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_customizer_dialog_type_id__volatile, external_applications_customizer_dialog_type_id);
	}
	return external_applications_customizer_dialog_type_id__volatile;
}


static gint _external_applications_chooser_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	gint result;
	result = external_applications_chooser_tree_sort_func ((ExternalApplicationsChooser*) self, model, a, b);
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1319 "open-with.c"
}


static void _external_applications_chooser_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_on_render_icon ((ExternalApplicationsChooser*) self, cell_layout, cell, tree_model, iter);
#line 1326 "open-with.c"
}


static void _external_applications_chooser_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_on_render_text ((ExternalApplicationsChooser*) self, cell_layout, cell, tree_model, iter);
#line 1333 "open-with.c"
}


static void _external_applications_chooser_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_row_activated ((ExternalApplicationsChooser*) self, path, column);
#line 1340 "open-with.c"
}


static gboolean _external_applications_chooser_button_released_gtk_widget_button_release_event (GtkWidget* _sender, GdkEventButton* event, gpointer self) {
	gboolean result;
	result = external_applications_chooser_button_released ((ExternalApplicationsChooser*) self, event);
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1349 "open-with.c"
}


ExternalApplicationsChooser* external_applications_chooser_construct (GType object_type, const gchar* uri, const gchar* content_type) {
	ExternalApplicationsChooser * self = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	GtkTreeViewColumn* column = NULL;
	GtkListStore* _tmp4_;
	GtkTreeView* _tmp5_;
	GtkTreeView* _tmp6_;
	GtkListStore* _tmp7_;
	GtkListStore* _tmp8_;
	GtkTreeViewColumn* _tmp9_;
	GtkCellRendererPixbuf* renderer_icon = NULL;
	GtkCellRendererPixbuf* _tmp10_;
	GtkTreeViewColumn* _tmp11_;
	GtkCellRendererPixbuf* _tmp12_;
	GtkTreeViewColumn* _tmp13_;
	GtkCellRendererPixbuf* _tmp14_;
	GtkTreeView* _tmp15_;
	GtkTreeViewColumn* _tmp16_;
	GtkTreeViewColumn* _tmp17_;
	GtkTreeViewColumn* _tmp18_;
	GtkCellRendererText* renderer_text = NULL;
	GtkCellRendererText* _tmp19_;
	GtkTreeViewColumn* _tmp20_;
	GtkCellRendererText* _tmp21_;
	GtkTreeViewColumn* _tmp22_;
	GtkTreeViewColumn* _tmp23_;
	GtkCellRendererText* _tmp24_;
	GtkTreeView* _tmp25_;
	GtkTreeViewColumn* _tmp26_;
	GtkTreeView* _tmp27_;
	GtkTreeView* _tmp28_;
	GtkScrolledWindow* scrolled = NULL;
	GtkScrolledWindow* _tmp29_;
	GtkScrolledWindow* _tmp30_;
	GtkScrolledWindow* _tmp31_;
	GtkTreeView* _tmp32_;
	GtkScrolledWindow* _tmp33_;
	gint height = 0;
	GtkTreeView* _tmp34_;
	PangoLayout* _tmp35_;
	PangoLayout* _tmp36_;
	gint _tmp37_ = 0;
	GtkScrolledWindow* _tmp38_;
	gint _tmp39_;
	GtkTreeView* _tmp40_;
	GtkTreeView* _tmp41_;
	const gchar* _tmp42_;
	GList* _tmp43_;
	GtkListStore* _tmp47_;
	gint _tmp48_;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, NULL);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsChooser*) g_object_new (object_type, NULL);
#line 208 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = content_type;
#line 208 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 208 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (self->priv->content_type);
#line 208 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->content_type = _tmp1_;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = uri;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (self->priv->uri);
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->uri = _tmp3_;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = self->priv->store;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) _tmp4_);
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp5_);
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->treeview);
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->treeview = _tmp5_;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = self->priv->treeview;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_set_headers_visible (_tmp6_, FALSE);
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = self->priv->store;
#line 216 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp7_, 0, GTK_SORT_ASCENDING);
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = self->priv->store;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) _tmp8_, 0, _external_applications_chooser_tree_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = gtk_tree_view_column_new ();
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp9_);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp9_;
#line 220 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 220 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp10_);
#line 220 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_icon = _tmp10_;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = column;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = renderer_icon;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp11_, (GtkCellRenderer*) _tmp12_, FALSE);
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = column;
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = renderer_icon;
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp13_, (GtkCellRenderer*) _tmp14_, _external_applications_chooser_on_render_icon_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = self->priv->treeview;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = column;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp15_, _tmp16_);
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = gtk_tree_view_column_new ();
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp17_);
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp17_;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = column;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_sizing (_tmp18_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp19_);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_text = _tmp19_;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = column;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = renderer_text;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp20_, (GtkCellRenderer*) _tmp21_, TRUE);
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = column;
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_expand (_tmp22_, TRUE);
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = column;
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = renderer_text;
#line 230 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp23_, (GtkCellRenderer*) _tmp24_, _external_applications_chooser_on_render_text_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = self->priv->treeview;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = column;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp25_, _tmp26_);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = self->priv->treeview;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp27_, "row-activated", (GCallback) _external_applications_chooser_row_activated_gtk_tree_view_row_activated, self, 0);
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = self->priv->treeview;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show ((GtkWidget*) _tmp28_);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp29_);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	scrolled = _tmp29_;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = scrolled;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_scrolled_window_set_policy (_tmp30_, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp31_ = scrolled;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp32_ = self->priv->treeview;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_container_add ((GtkContainer*) _tmp31_, (GtkWidget*) _tmp32_);
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp33_ = scrolled;
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _tmp33_, TRUE, TRUE, (guint) 0);
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp34_ = self->priv->treeview;
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp35_ = gtk_widget_create_pango_layout ((GtkWidget*) _tmp34_, "a\nb");
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp36_ = _tmp35_;
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	pango_layout_get_pixel_size (_tmp36_, NULL, &_tmp37_);
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	height = _tmp37_;
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp36_);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp38_ = scrolled;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp39_ = height;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_set_size_request ((GtkWidget*) _tmp38_, -1, _tmp39_ * 5);
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp40_ = self->priv->treeview;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((GtkWidget*) _tmp40_, "button-release-event", (GCallback) _external_applications_chooser_button_released_gtk_widget_button_release_event, self, 0);
#line 243 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp41_ = self->priv->treeview;
#line 243 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_set_tooltip_text ((GtkWidget*) _tmp41_, _ ("Right-click a suggestion to customize it"));
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__g_list_free__g_object_unref0_0 (self->priv->available);
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->available = NULL;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp42_ = content_type;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp43_ = g_app_info_get_all_for_type (_tmp42_);
#line 1584 "open-with.c"
	{
		GList* app_info_collection = NULL;
		GList* app_info_it = NULL;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		app_info_collection = _tmp43_;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (app_info_it = app_info_collection; app_info_it != NULL; app_info_it = app_info_it->next) {
#line 1592 "open-with.c"
			GAppInfo* _tmp44_;
			GAppInfo* app_info = NULL;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp44_ = _g_object_ref0 ((GAppInfo*) app_info_it->data);
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			app_info = _tmp44_;
#line 1599 "open-with.c"
			{
				GAppInfo* _tmp45_;
				const gchar* _tmp46_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp45_ = app_info;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp46_ = uri;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_chooser_launcher_added (self, _tmp45_, _tmp46_);
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_object_unref0 (app_info);
#line 1611 "open-with.c"
			}
		}
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		__g_list_free__g_object_unref0_0 (app_info_collection);
#line 1616 "open-with.c"
	}
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp47_ = self->priv->store;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp48_ = gtk_tree_model_iter_n_children ((GtkTreeModel*) _tmp47_, NULL);
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp48_ < 1) {
#line 1624 "open-with.c"
		GList* _tmp49_;
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp49_ = g_app_info_get_all ();
#line 1628 "open-with.c"
		{
			GList* app_info_collection = NULL;
			GList* app_info_it = NULL;
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			app_info_collection = _tmp49_;
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			for (app_info_it = app_info_collection; app_info_it != NULL; app_info_it = app_info_it->next) {
#line 1636 "open-with.c"
				GAppInfo* _tmp50_;
				GAppInfo* app_info = NULL;
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp50_ = _g_object_ref0 ((GAppInfo*) app_info_it->data);
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				app_info = _tmp50_;
#line 1643 "open-with.c"
				{
					GAppInfo* _tmp51_;
					const gchar* _tmp52_;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					_tmp51_ = app_info;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					_tmp52_ = uri;
#line 251 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					external_applications_chooser_launcher_added (self, _tmp51_, _tmp52_);
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					_g_object_unref0 (app_info);
#line 1655 "open-with.c"
				}
			}
#line 250 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			__g_list_free__g_object_unref0_0 (app_info_collection);
#line 1660 "open-with.c"
		}
	}
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (scrolled);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_text);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_icon);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 1673 "open-with.c"
}


ExternalApplicationsChooser* external_applications_chooser_new (const gchar* uri, const gchar* content_type) {
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_chooser_construct (EXTERNAL_APPLICATIONS_TYPE_CHOOSER, uri, content_type);
#line 1680 "open-with.c"
}


static gboolean external_applications_chooser_button_released (ExternalApplicationsChooser* self, GdkEventButton* event) {
	gboolean result = FALSE;
	GdkEventButton _tmp0_;
	guint _tmp1_;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = *event;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _tmp0_.button;
#line 256 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp1_ == ((guint) 3)) {
#line 1698 "open-with.c"
		GdkEventButton _tmp2_;
		gboolean _tmp3_;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = *event;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = external_applications_chooser_show_popup_menu (self, &_tmp2_);
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = _tmp3_;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 1709 "open-with.c"
	}
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = FALSE;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1715 "open-with.c"
}


static Block1Data* block1_data_ref (Block1Data* _data1_) {
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data1_->_ref_count_);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data1_;
#line 1724 "open-with.c"
}


static void block1_data_unref (void * _userdata_) {
	Block1Data* _data1_;
	_data1_ = (Block1Data*) _userdata_;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data1_->_ref_count_)) {
#line 1733 "open-with.c"
		ExternalApplicationsChooser* self;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data1_->self;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data1_->app_info);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block1Data, _data1_);
#line 1743 "open-with.c"
	}
}


static void ___lambda4_ (Block1Data* _data1_) {
	ExternalApplicationsChooser* self;
	GAppInfo* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data1_->self;
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _data1_->app_info;
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = self->priv->content_type;
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = self->priv->uri;
#line 271 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_customize_app_info (self, _tmp0_, _tmp1_, _tmp2_);
#line 1763 "open-with.c"
}


static void ____lambda4__gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self) {
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	___lambda4_ (self);
#line 1770 "open-with.c"
}


static gboolean external_applications_chooser_show_popup_menu (ExternalApplicationsChooser* self, GdkEventButton* event) {
	gboolean result = FALSE;
	GtkTreeIter iter = {0};
	GtkTreeView* _tmp0_;
	GtkTreeSelection* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->treeview;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = gtk_tree_view_get_selection (_tmp0_);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = gtk_tree_selection_get_selected (_tmp1_, NULL, &_tmp2_);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp2_;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 1793 "open-with.c"
		Block1Data* _data1_;
		GtkListStore* _tmp4_;
		GtkTreeIter _tmp5_;
		GtkMenu* menu = NULL;
		GtkMenu* _tmp6_;
		GtkImageMenuItem* menuitem = NULL;
		GtkImageMenuItem* _tmp7_;
		GtkImageMenuItem* _tmp8_;
		GtkImage* _tmp9_;
		GtkImage* _tmp10_;
		GtkImageMenuItem* _tmp11_;
		GtkMenu* _tmp12_;
		GtkImageMenuItem* _tmp13_;
		GtkMenu* _tmp14_;
		GtkTreeView* _tmp15_;
		GtkMenu* _tmp16_;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data1_ = g_slice_new0 (Block1Data);
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data1_->_ref_count_ = 1;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data1_->self = g_object_ref (self);
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = self->priv->store;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = iter;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp4_, &_tmp5_, 0, &_data1_->app_info, -1);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = (GtkMenu*) gtk_menu_new ();
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_ref_sink (_tmp6_);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		menu = _tmp6_;
#line 268 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = (GtkImageMenuItem*) gtk_image_menu_item_new_with_mnemonic (_ ("Custom…"));
#line 268 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_ref_sink (_tmp7_);
#line 268 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		menuitem = _tmp7_;
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = menuitem;
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = (GtkImage*) gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_ref_sink (_tmp9_);
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = _tmp9_;
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_image_menu_item_set_image (_tmp8_, (GtkWidget*) _tmp10_);
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp10_);
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = menuitem;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_connect_data ((GtkMenuItem*) _tmp11_, "activate", (GCallback) ____lambda4__gtk_menu_item_activate, block1_data_ref (_data1_), (GClosureNotify) block1_data_unref, 0);
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = menu;
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = menuitem;
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_menu_shell_append ((GtkMenuShell*) _tmp12_, (GtkWidget*) ((GtkMenuItem*) _tmp13_));
#line 274 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = menu;
#line 274 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_widget_show_all ((GtkWidget*) _tmp14_);
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp15_ = self->priv->treeview;
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp16_ = menu;
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		katze_widget_popup ((GtkWidget*) _tmp15_, _tmp16_, NULL, KATZE_MENU_POSITION_CURSOR);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = TRUE;
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (menuitem);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (menu);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block1_data_unref (_data1_);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data1_ = NULL;
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 1878 "open-with.c"
	}
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = FALSE;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 1884 "open-with.c"
}


static void external_applications_chooser_customize_app_info (ExternalApplicationsChooser* self, GAppInfo* app_info, const gchar* content_type, const gchar* uri) {
	ExternalApplicationsCustomizerDialog* dialog = NULL;
	GAppInfo* _tmp0_;
	ExternalApplicationsCustomizerDialog* _tmp1_;
	gboolean accept = FALSE;
	ExternalApplicationsCustomizerDialog* _tmp2_;
	gint _tmp3_;
	gboolean _tmp4_;
	ExternalApplicationsCustomizerDialog* _tmp24_;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app_info != NULL);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (uri != NULL);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = app_info;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = external_applications_customizer_dialog_new (_tmp0_, (GtkWidget*) self);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp1_);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	dialog = _tmp1_;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = dialog;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = gtk_dialog_run ((GtkDialog*) _tmp2_);
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	accept = _tmp3_ == ((gint) GTK_RESPONSE_ACCEPT);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = accept;
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp4_) {
#line 1923 "open-with.c"
		gchar* name = NULL;
		ExternalApplicationsCustomizerDialog* _tmp5_;
		GtkEntry* _tmp6_;
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* commandline = NULL;
		ExternalApplicationsCustomizerDialog* _tmp10_;
		GtkEntry* _tmp11_;
		const gchar* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
		ExternalApplicationsAssociations* _tmp15_;
		ExternalApplicationsAssociations* _tmp16_;
		const gchar* _tmp17_;
		const gchar* _tmp18_;
		const gchar* _tmp19_;
		const gchar* _tmp20_;
		GAppInfo* _tmp21_;
		const gchar* _tmp22_;
		const gchar* _tmp23_;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = dialog;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = _tmp5_->name_entry;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = gtk_entry_get_text (_tmp6_);
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _tmp7_;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = g_strdup (_tmp8_);
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		name = _tmp9_;
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = dialog;
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = _tmp10_->commandline_entry;
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = gtk_entry_get_text (_tmp11_);
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = _tmp12_;
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = g_strdup (_tmp13_);
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		commandline = _tmp14_;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp15_ = external_applications_associations_new ();
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp16_ = _tmp15_;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp17_ = content_type;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp18_ = commandline;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp19_ = name;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp20_ = uri;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_associations_remember_custom_commandline (_tmp16_, _tmp17_, _tmp18_, _tmp19_, _tmp20_);
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp16_);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp21_ = app_info;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp22_ = content_type;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp23_ = uri;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_emit (self, external_applications_chooser_signals[EXTERNAL_APPLICATIONS_CHOOSER_CUSTOMIZED_SIGNAL], 0, _tmp21_, _tmp22_, _tmp23_);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (commandline);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (name);
#line 1997 "open-with.c"
	}
#line 291 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = dialog;
#line 291 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_object_destroy ((GtkObject*) _tmp24_);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (dialog);
#line 2005 "open-with.c"
}


GList* external_applications_chooser_get_available (ExternalApplicationsChooser* self) {
	GList* result = NULL;
	GList* _tmp0_;
	GList* _tmp1_;
#line 294 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->available;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_list_copy (_tmp0_);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp1_;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 2023 "open-with.c"
}


GAppInfo* external_applications_chooser_get_app_info (ExternalApplicationsChooser* self) {
	GAppInfo* result = NULL;
	GtkTreeIter iter = {0};
	GtkTreeView* _tmp0_;
	GtkTreeSelection* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
#line 298 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->treeview;
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = gtk_tree_view_get_selection (_tmp0_);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = gtk_tree_selection_get_selected (_tmp1_, NULL, &_tmp2_);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp2_;
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 2046 "open-with.c"
		GAppInfo* app_info = NULL;
		GtkListStore* _tmp4_;
		GtkTreeIter _tmp5_;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = self->priv->store;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = iter;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp4_, &_tmp5_, 0, &app_info, -1);
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = app_info;
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 2060 "open-with.c"
	}
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_assert_not_reached ();
#line 2064 "open-with.c"
}


static void external_applications_chooser_on_render_icon (ExternalApplicationsChooser* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	GAppInfo* app_info = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	GIcon* _tmp3_;
	GIcon* _tmp4_;
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &app_info, -1);
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = renderer;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_app_info_get_icon (app_info);
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp2_, "gicon", _tmp4_, "stock-size", GTK_ICON_SIZE_DIALOG, "xpad", 4, NULL);
#line 314 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp4_);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 2103 "open-with.c"
}


static void external_applications_chooser_on_render_text (ExternalApplicationsChooser* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	GAppInfo* app_info = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &app_info, -1);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = renderer;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_describe_app_info (app_info);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp2_, "markup", _tmp4_, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp4_);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 2142 "open-with.c"
}


static void external_applications_chooser_launcher_added (ExternalApplicationsChooser* self, GAppInfo* app_info, const gchar* uri) {
	GAppInfo* _tmp0_;
	gboolean _tmp1_;
	GtkTreeIter iter = {0};
	GtkListStore* _tmp2_;
	GtkTreeIter _tmp3_ = {0};
	GtkListStore* _tmp4_;
	GtkTreeIter _tmp5_;
	GAppInfo* _tmp6_;
	GAppInfo* _tmp7_;
	GAppInfo* _tmp8_;
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app_info != NULL);
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (uri != NULL);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = app_info;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_app_info_should_show (_tmp0_);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (!_tmp1_) {
#line 330 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 2171 "open-with.c"
	}
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = self->priv->store;
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_list_store_append (_tmp2_, &_tmp3_);
#line 333 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp3_;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = self->priv->store;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = iter;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = app_info;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_list_store_set (_tmp4_, &_tmp5_, 0, _tmp6_, -1);
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = app_info;
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = _g_object_ref0 (_tmp7_);
#line 336 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->available = g_list_append (self->priv->available, _tmp8_);
#line 2193 "open-with.c"
}


static gint external_applications_chooser_tree_sort_func (ExternalApplicationsChooser* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	gint result = 0;
	GAppInfo* app_info1 = NULL;
	GAppInfo* app_info2 = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkTreeModel* _tmp2_;
	GtkTreeIter _tmp3_;
	GCompareFunc _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gint _tmp7_;
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (a != NULL, 0);
#line 339 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (b != NULL, 0);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *a;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &app_info1, -1);
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = model;
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = *b;
#line 342 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp2_, &_tmp3_, 0, &app_info2, -1);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = g_strcmp0;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = g_app_info_get_display_name (app_info1);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = g_app_info_get_display_name (app_info2);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = _tmp4_ (_tmp5_, _tmp6_);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp7_;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info2);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info1);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 2245 "open-with.c"
}


static void external_applications_chooser_row_activated (ExternalApplicationsChooser* self, GtkTreePath* path, GtkTreeViewColumn* column) {
	GtkTreeIter iter = {0};
	GtkListStore* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (path != NULL);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->store;
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = path;
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp0_, &_tmp2_, _tmp1_);
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp2_;
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 2271 "open-with.c"
		GAppInfo* app_info = NULL;
		GtkListStore* _tmp4_;
		GtkTreeIter _tmp5_;
		GAppInfo* _tmp6_;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = self->priv->store;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = iter;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp4_, &_tmp5_, 0, &app_info, -1);
#line 351 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = app_info;
#line 351 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_emit (self, external_applications_chooser_signals[EXTERNAL_APPLICATIONS_CHOOSER_SELECTED_SIGNAL], 0, _tmp6_);
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 2288 "open-with.c"
	}
}


static void g_cclosure_user_marshal_VOID__OBJECT_STRING_STRING (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__OBJECT_STRING_STRING) (gpointer data1, gpointer arg_1, const char* arg_2, const char* arg_3, gpointer data2);
	register GMarshalFunc_VOID__OBJECT_STRING_STRING callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (n_param_values == 4);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data1 = closure->data;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data2 = param_values->data[0].v_pointer;
#line 2308 "open-with.c"
	} else {
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data1 = param_values->data[0].v_pointer;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data2 = closure->data;
#line 2314 "open-with.c"
	}
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	callback = (GMarshalFunc_VOID__OBJECT_STRING_STRING) (marshal_data ? marshal_data : cc->callback);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	callback (data1, g_value_get_object (param_values + 1), g_value_get_string (param_values + 2), g_value_get_string (param_values + 3), data2);
#line 2320 "open-with.c"
}


static void external_applications_chooser_class_init (ExternalApplicationsChooserClass * klass) {
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_parent_class = g_type_class_peek_parent (klass);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_type_class_add_private (klass, sizeof (ExternalApplicationsChooserPrivate));
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->finalize = external_applications_chooser_finalize;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_signals[EXTERNAL_APPLICATIONS_CHOOSER_SELECTED_SIGNAL] = g_signal_new ("selected", EXTERNAL_APPLICATIONS_TYPE_CHOOSER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, g_app_info_get_type ());
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_signals[EXTERNAL_APPLICATIONS_CHOOSER_CUSTOMIZED_SIGNAL] = g_signal_new ("customized", EXTERNAL_APPLICATIONS_TYPE_CHOOSER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__OBJECT_STRING_STRING, G_TYPE_NONE, 3, g_app_info_get_type (), G_TYPE_STRING, G_TYPE_STRING);
#line 2335 "open-with.c"
}


static void external_applications_chooser_instance_init (ExternalApplicationsChooser * self) {
	GtkListStore* _tmp0_;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv = EXTERNAL_APPLICATIONS_CHOOSER_GET_PRIVATE (self);
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = gtk_list_store_new (1, g_app_info_get_type ());
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->store = _tmp0_;
#line 2347 "open-with.c"
}


static void external_applications_chooser_finalize (GObject * obj) {
	ExternalApplicationsChooser * self;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, EXTERNAL_APPLICATIONS_TYPE_CHOOSER, ExternalApplicationsChooser);
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->store);
#line 202 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->treeview);
#line 203 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__g_list_free__g_object_unref0_0 (self->priv->available);
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (self->priv->content_type);
#line 205 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (self->priv->uri);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (external_applications_chooser_parent_class)->finalize (obj);
#line 2367 "open-with.c"
}


GType external_applications_chooser_get_type (void) {
	static volatile gsize external_applications_chooser_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_chooser_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsChooserClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_chooser_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsChooser), 0, (GInstanceInitFunc) external_applications_chooser_instance_init, NULL };
		GType external_applications_chooser_type_id;
		external_applications_chooser_type_id = g_type_register_static (GTK_TYPE_VBOX, "ExternalApplicationsChooser", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_chooser_type_id__volatile, external_applications_chooser_type_id);
	}
	return external_applications_chooser_type_id__volatile;
}


static void __lambda5_ (ExternalApplicationsChooserDialog* self, GAppInfo* app_info) {
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app_info != NULL);
#line 405 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_response ((GtkDialog*) self, (gint) GTK_RESPONSE_ACCEPT);
#line 2388 "open-with.c"
}


static void ___lambda5__external_applications_chooser_selected (ExternalApplicationsChooser* _sender, GAppInfo* app_info, gpointer self) {
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda5_ ((ExternalApplicationsChooserDialog*) self, app_info);
#line 2395 "open-with.c"
}


static void __lambda6_ (ExternalApplicationsChooserDialog* self, GAppInfo* app_info, const gchar* content_type, const gchar* uri) {
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app_info != NULL);
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (uri != NULL);
#line 408 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_response ((GtkDialog*) self, (gint) GTK_RESPONSE_CANCEL);
#line 2408 "open-with.c"
}


static void ___lambda6__external_applications_chooser_customized (ExternalApplicationsChooser* _sender, GAppInfo* app_info, const gchar* content_type, const gchar* uri, gpointer self) {
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda6_ ((ExternalApplicationsChooserDialog*) self, app_info, content_type, uri);
#line 2415 "open-with.c"
}


ExternalApplicationsChooserDialog* external_applications_chooser_dialog_construct (GType object_type, const gchar* uri, const gchar* content_type, GtkWidget* widget) {
	ExternalApplicationsChooserDialog * self = NULL;
	gchar* filename = NULL;
	const gchar* _tmp0_;
	gboolean _tmp1_;
	MidoriBrowser* browser = NULL;
	GtkWidget* _tmp6_;
	MidoriBrowser* _tmp7_;
	MidoriBrowser* _tmp8_;
	MidoriBrowser* _tmp9_;
	GtkVBox* vbox = NULL;
	GtkVBox* _tmp10_;
	GtkVBox* _tmp11_;
	GtkWidget* _tmp12_;
	GtkVBox* _tmp13_;
	GtkLabel* label = NULL;
	const gchar* _tmp14_;
	gchar* _tmp15_;
	gchar* _tmp16_;
	GtkLabel* _tmp17_;
	GtkLabel* _tmp18_;
	GtkLabel* _tmp19_;
	GtkVBox* _tmp20_;
	GtkLabel* _tmp21_;
	const gchar* _tmp22_;
	const gchar* _tmp24_;
	const gchar* _tmp25_;
	ExternalApplicationsChooser* _tmp26_;
	ExternalApplicationsChooser* _tmp27_;
	GtkVBox* _tmp28_;
	ExternalApplicationsChooser* _tmp29_;
	GtkWidget* _tmp30_;
	GtkRequisition req = {0};
	GtkWidget* _tmp31_;
	GtkRequisition _tmp32_ = {0};
	GtkRequisition _tmp33_;
	gint _tmp34_;
	GtkRequisition _tmp35_;
	gint _tmp36_;
	ExternalApplicationsChooser* _tmp37_;
	ExternalApplicationsChooser* _tmp38_;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, NULL);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (widget != NULL, NULL);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsChooserDialog*) g_object_new (object_type, NULL);
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = uri;
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_str_has_prefix (_tmp0_, "file://");
#line 364 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp1_) {
#line 2474 "open-with.c"
		const gchar* _tmp2_;
		gchar* _tmp3_;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = uri;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = midori_download_get_basename_for_display (_tmp2_);
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (filename);
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		filename = _tmp3_;
#line 2485 "open-with.c"
	} else {
		const gchar* _tmp4_;
		gchar* _tmp5_;
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = uri;
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = g_strdup (_tmp4_);
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (filename);
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		filename = _tmp5_;
#line 2497 "open-with.c"
	}
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = widget;
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = midori_browser_get_for_widget (_tmp6_);
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = _g_object_ref0 (_tmp7_);
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	browser = _tmp8_;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = browser;
#line 370 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_transient_for ((GtkWindow*) self, (GtkWindow*) _tmp9_);
#line 372 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_title ((GtkWindow*) self, _ ("Choose application"));
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_set_has_separator ((GtkDialog*) self, FALSE);
#line 376 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_destroy_with_parent ((GtkWindow*) self, TRUE);
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_icon_name ((GtkWindow*) self, GTK_STOCK_OPEN);
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_resizable ((GtkWindow*) self, TRUE);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_add_buttons ((GtkDialog*) self, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = (GtkVBox*) gtk_vbox_new (FALSE, 8);
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp10_);
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	vbox = _tmp10_;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = vbox;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_container_set_border_width ((GtkContainer*) _tmp11_, (guint) 8);
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = gtk_dialog_get_content_area ((GtkDialog*) self);
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = vbox;
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start (G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, GTK_TYPE_BOX) ? ((GtkBox*) _tmp12_) : NULL, (GtkWidget*) _tmp13_, TRUE, TRUE, (guint) 8);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = filename;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = g_strdup_printf ("Select an application to open \"%s\"", _tmp14_);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = _tmp15_;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = (GtkLabel*) gtk_label_new (_ (_tmp16_));
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp17_);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = _tmp17_;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp16_);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	label = _tmp18_;
#line 386 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = label;
#line 386 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_label_set_ellipsize (_tmp19_, PANGO_ELLIPSIZE_MIDDLE);
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = vbox;
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = label;
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) _tmp20_, (GtkWidget*) _tmp21_, FALSE, FALSE, (guint) 0);
#line 388 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = uri;
#line 388 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_strcmp0 (_tmp22_, "") == 0) {
#line 2569 "open-with.c"
		GtkLabel* _tmp23_;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp23_ = label;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_widget_set_no_show_all ((GtkWidget*) _tmp23_, TRUE);
#line 2575 "open-with.c"
	}
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = uri;
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = content_type;
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = external_applications_chooser_new (_tmp24_, _tmp25_);
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp26_);
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = _tmp26_;
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_dialog_set_chooser (self, _tmp27_);
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp27_);
#line 391 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = vbox;
#line 391 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = self->priv->_chooser;
#line 391 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) _tmp28_, (GtkWidget*) _tmp29_, TRUE, TRUE, (guint) 0);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = gtk_dialog_get_content_area ((GtkDialog*) self);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show_all (_tmp30_);
#line 399 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp31_ = gtk_dialog_get_content_area ((GtkDialog*) self);
#line 399 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_size_request (_tmp31_, &_tmp32_);
#line 399 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	req = _tmp32_;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp33_ = req;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp34_ = _tmp33_.width;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp35_ = req;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp36_ = _tmp35_.height;
#line 401 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_default_size (G_TYPE_CHECK_INSTANCE_TYPE (self, GTK_TYPE_WINDOW) ? ((GtkWindow*) self) : NULL, _tmp34_ * 2, (_tmp36_ * 3) / 2);
#line 403 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_dialog_set_default_response ((GtkDialog*) self, (gint) GTK_RESPONSE_ACCEPT);
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp37_ = self->priv->_chooser;
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp37_, "selected", (GCallback) ___lambda5__external_applications_chooser_selected, self, 0);
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp38_ = self->priv->_chooser;
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp38_, "customized", (GCallback) ___lambda6__external_applications_chooser_customized, self, 0);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (label);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (vbox);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (browser);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (filename);
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 2637 "open-with.c"
}


ExternalApplicationsChooserDialog* external_applications_chooser_dialog_new (const gchar* uri, const gchar* content_type, GtkWidget* widget) {
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_chooser_dialog_construct (EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, uri, content_type, widget);
#line 2644 "open-with.c"
}


GAppInfo* external_applications_chooser_dialog_open_with (ExternalApplicationsChooserDialog* self) {
	GAppInfo* result = NULL;
	gboolean accept = FALSE;
	gint _tmp0_;
	gboolean _tmp1_;
	ExternalApplicationsChooser* _tmp2_;
	GAppInfo* _tmp3_;
#line 412 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show ((GtkWidget*) self);
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = gtk_dialog_run ((GtkDialog*) self);
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	accept = _tmp0_ == ((gint) GTK_RESPONSE_ACCEPT);
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_hide ((GtkWidget*) self);
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = accept;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (!_tmp1_) {
#line 418 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = NULL;
#line 418 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 2673 "open-with.c"
	}
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = self->priv->_chooser;
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_chooser_get_app_info (_tmp2_);
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp3_;
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 2683 "open-with.c"
}


ExternalApplicationsChooser* external_applications_chooser_dialog_get_chooser (ExternalApplicationsChooserDialog* self) {
	ExternalApplicationsChooser* result;
	ExternalApplicationsChooser* _tmp0_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->_chooser;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp0_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 2698 "open-with.c"
}


static void external_applications_chooser_dialog_set_chooser (ExternalApplicationsChooserDialog* self, ExternalApplicationsChooser* value) {
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (external_applications_chooser_dialog_get_chooser (self) != value) {
#line 2707 "open-with.c"
		ExternalApplicationsChooser* _tmp0_;
		ExternalApplicationsChooser* _tmp1_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = value;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self->priv->_chooser);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self->priv->_chooser = _tmp1_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_notify_by_pspec ((GObject *) self, external_applications_chooser_dialog_properties[EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY]);
#line 2720 "open-with.c"
	}
}


static void external_applications_chooser_dialog_class_init (ExternalApplicationsChooserDialogClass * klass) {
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_dialog_parent_class = g_type_class_peek_parent (klass);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_type_class_add_private (klass, sizeof (ExternalApplicationsChooserDialogPrivate));
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_external_applications_chooser_dialog_get_property;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_external_applications_chooser_dialog_set_property;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->finalize = external_applications_chooser_dialog_finalize;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY, external_applications_chooser_dialog_properties[EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY] = g_param_spec_object ("chooser", "chooser", "chooser", EXTERNAL_APPLICATIONS_TYPE_CHOOSER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 2738 "open-with.c"
}


static void external_applications_chooser_dialog_instance_init (ExternalApplicationsChooserDialog * self) {
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv = EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_GET_PRIVATE (self);
#line 2745 "open-with.c"
}


static void external_applications_chooser_dialog_finalize (GObject * obj) {
	ExternalApplicationsChooserDialog * self;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialog);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->_chooser);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (external_applications_chooser_dialog_parent_class)->finalize (obj);
#line 2757 "open-with.c"
}


GType external_applications_chooser_dialog_get_type (void) {
	static volatile gsize external_applications_chooser_dialog_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_chooser_dialog_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsChooserDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_chooser_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsChooserDialog), 0, (GInstanceInitFunc) external_applications_chooser_dialog_instance_init, NULL };
		GType external_applications_chooser_dialog_type_id;
		external_applications_chooser_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "ExternalApplicationsChooserDialog", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_chooser_dialog_type_id__volatile, external_applications_chooser_dialog_type_id);
	}
	return external_applications_chooser_dialog_type_id__volatile;
}


static void _vala_external_applications_chooser_dialog_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ExternalApplicationsChooserDialog * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialog);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	switch (property_id) {
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY:
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_value_set_object (value, external_applications_chooser_dialog_get_chooser (self));
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 2784 "open-with.c"
		default:
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 2790 "open-with.c"
	}
}


static void _vala_external_applications_chooser_dialog_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ExternalApplicationsChooserDialog * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_DIALOG, ExternalApplicationsChooserDialog);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	switch (property_id) {
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_DIALOG_CHOOSER_PROPERTY:
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_chooser_dialog_set_chooser (self, g_value_get_object (value));
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 2806 "open-with.c"
		default:
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 2812 "open-with.c"
	}
}


static Block2Data* block2_data_ref (Block2Data* _data2_) {
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data2_->_ref_count_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data2_;
#line 2822 "open-with.c"
}


static void block2_data_unref (void * _userdata_) {
	Block2Data* _data2_;
	_data2_ = (Block2Data*) _userdata_;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data2_->_ref_count_)) {
#line 2831 "open-with.c"
		ExternalApplicationsChooserButton* self;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data2_->self;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_data2_->commandline);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block2Data, _data2_);
#line 2841 "open-with.c"
	}
}


static void __lambda7_ (Block2Data* _data2_) {
	ExternalApplicationsChooserButton* self;
	ExternalApplicationsChooserDialog* _tmp0_;
	GtkWidget* _tmp1_;
	ExternalApplicationsChooserDialog* _tmp2_;
	GAppInfo* _tmp3_;
	GAppInfo* _tmp4_;
	gchar* _tmp5_ = NULL;
	GAppInfo* _tmp6_;
	gchar* new_commandline = NULL;
	gchar* _tmp9_;
	gchar* _tmp10_;
#line 450 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data2_->self;
#line 451 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->dialog;
#line 451 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = gtk_widget_get_toplevel ((GtkWidget*) self);
#line 451 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_window_set_transient_for ((GtkWindow*) _tmp0_, G_TYPE_CHECK_INSTANCE_TYPE (_tmp1_, GTK_TYPE_WINDOW) ? ((GtkWindow*) _tmp1_) : NULL);
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = self->priv->dialog;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_chooser_dialog_open_with (_tmp2_);
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_set_app_info (self, _tmp4_);
#line 452 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp4_);
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = self->priv->_app_info;
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp6_ != NULL) {
#line 2880 "open-with.c"
		GAppInfo* _tmp7_;
		gchar* _tmp8_;
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = self->priv->_app_info;
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = external_applications_get_commandline (_tmp7_);
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp5_);
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = _tmp8_;
#line 2891 "open-with.c"
	} else {
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp5_);
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = NULL;
#line 2897 "open-with.c"
	}
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = g_strdup (_tmp5_);
#line 453 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	new_commandline = _tmp9_;
#line 454 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = g_strdup (new_commandline);
#line 454 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_data2_->commandline);
#line 454 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_->commandline = _tmp10_;
#line 455 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_emit (self, external_applications_chooser_button_signals[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_SELECTED_SIGNAL], 0, new_commandline);
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_update_label (self);
#line 450 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (new_commandline);
#line 450 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp5_);
#line 2917 "open-with.c"
}


static void ___lambda7__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 450 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda7_ (self);
#line 2924 "open-with.c"
}


ExternalApplicationsChooserButton* external_applications_chooser_button_construct (GType object_type, const gchar* mime_type, const gchar* commandline) {
	ExternalApplicationsChooserButton * self = NULL;
	Block2Data* _data2_;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* content_type = NULL;
	const gchar* _tmp2_;
	gchar* _tmp3_;
	const gchar* _tmp4_;
	ExternalApplicationsChooserDialog* _tmp5_;
	ExternalApplicationsChooserDialog* _tmp6_;
	ExternalApplicationsChooser* _tmp7_;
	ExternalApplicationsChooser* _tmp8_;
	GList* _tmp9_;
	GtkHBox* hbox = NULL;
	GtkHBox* _tmp17_;
	GtkImage* _tmp18_;
	GtkImage* _tmp19_;
	GtkLabel* _tmp20_;
	GtkLabel* _tmp21_;
	GtkLabel* _tmp22_;
	GtkLabel* _tmp23_;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (mime_type != NULL, NULL);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_ = g_slice_new0 (Block2Data);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_->_ref_count_ = 1;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = commandline;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_data2_->commandline);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_->commandline = _tmp1_;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsChooserButton*) g_object_new (object_type, NULL);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_->self = g_object_ref (self);
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = mime_type;
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_content_type_from_mime_type (_tmp2_);
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	content_type = _tmp3_;
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = content_type;
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = external_applications_chooser_dialog_new ("", _tmp4_, (GtkWidget*) self);
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp5_);
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->dialog);
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->dialog = _tmp5_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_set_app_info (self, NULL);
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = self->priv->dialog;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = external_applications_chooser_dialog_get_chooser (_tmp6_);
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = _tmp7_;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = external_applications_chooser_get_available (_tmp8_);
#line 2994 "open-with.c"
	{
		GList* candidate_collection = NULL;
		GList* candidate_it = NULL;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		candidate_collection = _tmp9_;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (candidate_it = candidate_collection; candidate_it != NULL; candidate_it = candidate_it->next) {
#line 3002 "open-with.c"
			GAppInfo* _tmp10_;
			GAppInfo* candidate = NULL;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp10_ = _g_object_ref0 ((GAppInfo*) candidate_it->data);
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			candidate = _tmp10_;
#line 3009 "open-with.c"
			{
				GAppInfo* _tmp11_;
				gchar* _tmp12_;
				gchar* _tmp13_;
				const gchar* _tmp14_;
				gboolean _tmp15_;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp11_ = candidate;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp12_ = external_applications_get_commandline (_tmp11_);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp13_ = _tmp12_;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp14_ = _data2_->commandline;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp15_ = g_strcmp0 (_tmp13_, _tmp14_) == 0;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (_tmp13_);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				if (_tmp15_) {
#line 3030 "open-with.c"
					GAppInfo* _tmp16_;
#line 436 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					_tmp16_ = candidate;
#line 436 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
					external_applications_chooser_button_set_app_info (self, _tmp16_);
#line 3036 "open-with.c"
				}
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_object_unref0 (candidate);
#line 3040 "open-with.c"
			}
		}
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		__g_list_free__g_object_unref0_0 (candidate_collection);
#line 3045 "open-with.c"
	}
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = (GtkHBox*) gtk_hbox_new (FALSE, 4);
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp17_);
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	hbox = _tmp17_;
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = (GtkImage*) gtk_image_new ();
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp18_);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->icon);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->icon = _tmp18_;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = self->priv->icon;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp19_, FALSE, FALSE, (guint) 0);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = (GtkLabel*) gtk_label_new (NULL);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp20_);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->app_name);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->app_name = _tmp20_;
#line 443 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = self->priv->app_name;
#line 443 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_label_set_use_markup (_tmp21_, TRUE);
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = self->priv->app_name;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_label_set_ellipsize (_tmp22_, PANGO_ELLIPSIZE_END);
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = self->priv->app_name;
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) _tmp23_, TRUE, TRUE, (guint) 0);
#line 446 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) hbox);
#line 447 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 448 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_update_label (self);
#line 450 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_data ((GtkButton*) self, "clicked", (GCallback) ___lambda7__gtk_button_clicked, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (hbox);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	block2_data_unref (_data2_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data2_ = NULL;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 3103 "open-with.c"
}


ExternalApplicationsChooserButton* external_applications_chooser_button_new (const gchar* mime_type, const gchar* commandline) {
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_chooser_button_construct (EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, mime_type, commandline);
#line 3110 "open-with.c"
}


static gchar* string_replace (const gchar* self, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	GError * _inner_error_ = NULL;
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if ((*((gchar*) self)) == '\0') {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = TRUE;
#line 3129 "open-with.c"
	} else {
		const gchar* _tmp2_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = (*((gchar*) _tmp2_)) == '\0';
#line 3136 "open-with.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_) {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = TRUE;
#line 3142 "open-with.c"
	} else {
		const gchar* _tmp3_;
		const gchar* _tmp4_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = replacement;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = g_strcmp0 (_tmp3_, _tmp4_) == 0;
#line 3152 "open-with.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 3156 "open-with.c"
		gchar* _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = g_strdup (self);
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 3164 "open-with.c"
	}
	{
		GRegex* regex = NULL;
		const gchar* _tmp6_;
		gchar* _tmp7_;
		gchar* _tmp8_;
		GRegex* _tmp9_;
		GRegex* _tmp10_;
		gchar* _tmp11_ = NULL;
		GRegex* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
		gchar* _tmp15_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = old;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = g_regex_escape_string (_tmp6_, -1);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp8_ = _tmp7_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = g_regex_new (_tmp8_, 0, 0, &_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = _tmp9_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp8_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		regex = _tmp10_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 3196 "open-with.c"
				goto __catch3_g_regex_error;
			}
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 3205 "open-with.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = regex;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = replacement;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp14_ = g_regex_replace_literal (_tmp12_, self, (gssize) -1, 0, _tmp13_, 0, &_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = _tmp14_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 3221 "open-with.c"
				goto __catch3_g_regex_error;
			}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 3232 "open-with.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = _tmp11_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = NULL;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp15_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp11_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 3246 "open-with.c"
	}
	goto __finally3;
	__catch3_g_regex_error:
	{
		GError* e = NULL;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		e = _inner_error_;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_inner_error_ = NULL;
#line 1423 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_assert_not_reached ();
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_error_free0 (e);
#line 3260 "open-with.c"
	}
	__finally3:
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_clear_error (&_inner_error_);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return NULL;
#line 3271 "open-with.c"
	}
}


static void external_applications_chooser_button_update_label (ExternalApplicationsChooserButton* self) {
	gchar* _tmp0_ = NULL;
	GAppInfo* _tmp1_;
	GtkLabel* _tmp7_;
	GIcon* _tmp8_ = NULL;
	GAppInfo* _tmp9_;
	GtkImage* _tmp12_;
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = self->priv->_app_info;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp1_ != NULL) {
#line 3289 "open-with.c"
		GAppInfo* _tmp2_;
		gchar* _tmp3_;
		gchar* _tmp4_;
		gchar* _tmp5_;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = self->priv->_app_info;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = external_applications_describe_app_info (_tmp2_);
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = _tmp3_;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = string_replace (_tmp4_, "\n", " ");
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp0_);
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp5_;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp4_);
#line 3308 "open-with.c"
	} else {
		gchar* _tmp6_;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = g_strdup (_ ("None"));
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_tmp0_);
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp6_;
#line 3317 "open-with.c"
	}
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = self->priv->app_name;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_label_set_label (_tmp7_, _tmp0_);
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = self->priv->_app_info;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp9_ != NULL) {
#line 3327 "open-with.c"
		GAppInfo* _tmp10_;
		GIcon* _tmp11_;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = self->priv->_app_info;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = external_applications_app_info_get_icon (_tmp10_);
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp8_);
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _tmp11_;
#line 3338 "open-with.c"
	} else {
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp8_);
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = NULL;
#line 3344 "open-with.c"
	}
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = self->priv->icon;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_image_set_from_gicon (_tmp12_, _tmp8_, GTK_ICON_SIZE_BUTTON);
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp8_);
#line 460 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp0_);
#line 3354 "open-with.c"
}


GAppInfo* external_applications_chooser_button_get_app_info (ExternalApplicationsChooserButton* self) {
	GAppInfo* result;
	GAppInfo* _tmp0_;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->_app_info;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp0_;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 3369 "open-with.c"
}


void external_applications_chooser_button_set_app_info (ExternalApplicationsChooserButton* self, GAppInfo* value) {
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (external_applications_chooser_button_get_app_info (self) != value) {
#line 3378 "open-with.c"
		GAppInfo* _tmp0_;
		GAppInfo* _tmp1_;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = value;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self->priv->_app_info);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self->priv->_app_info = _tmp1_;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_notify_by_pspec ((GObject *) self, external_applications_chooser_button_properties[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY]);
#line 3391 "open-with.c"
	}
}


const gchar* external_applications_chooser_button_get_commandline (ExternalApplicationsChooserButton* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->_commandline;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp0_;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 3407 "open-with.c"
}


void external_applications_chooser_button_set_commandline (ExternalApplicationsChooserButton* self, const gchar* value) {
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_strcmp0 (value, external_applications_chooser_button_get_commandline (self)) != 0) {
#line 3416 "open-with.c"
		const gchar* _tmp0_;
		gchar* _tmp1_;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = value;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = g_strdup (_tmp0_);
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (self->priv->_commandline);
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self->priv->_commandline = _tmp1_;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_notify_by_pspec ((GObject *) self, external_applications_chooser_button_properties[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY]);
#line 3429 "open-with.c"
	}
}


static void external_applications_chooser_button_class_init (ExternalApplicationsChooserButtonClass * klass) {
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_parent_class = g_type_class_peek_parent (klass);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_type_class_add_private (klass, sizeof (ExternalApplicationsChooserButtonPrivate));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_external_applications_chooser_button_get_property;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_external_applications_chooser_button_set_property;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->finalize = external_applications_chooser_button_finalize;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY, external_applications_chooser_button_properties[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY] = g_param_spec_object ("app-info", "app-info", "app-info", g_app_info_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY, external_applications_chooser_button_properties[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY] = g_param_spec_string ("commandline", "commandline", "commandline", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_chooser_button_signals[EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_SELECTED_SIGNAL] = g_signal_new ("selected", EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
#line 3451 "open-with.c"
}


static void external_applications_chooser_button_instance_init (ExternalApplicationsChooserButton * self) {
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv = EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_GET_PRIVATE (self);
#line 3458 "open-with.c"
}


static void external_applications_chooser_button_finalize (GObject * obj) {
	ExternalApplicationsChooserButton * self;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButton);
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->_app_info);
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (self->priv->_commandline);
#line 426 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->dialog);
#line 427 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->app_name);
#line 428 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->icon);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (external_applications_chooser_button_parent_class)->finalize (obj);
#line 3478 "open-with.c"
}


GType external_applications_chooser_button_get_type (void) {
	static volatile gsize external_applications_chooser_button_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_chooser_button_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsChooserButtonClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_chooser_button_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsChooserButton), 0, (GInstanceInitFunc) external_applications_chooser_button_instance_init, NULL };
		GType external_applications_chooser_button_type_id;
		external_applications_chooser_button_type_id = g_type_register_static (GTK_TYPE_BUTTON, "ExternalApplicationsChooserButton", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_chooser_button_type_id__volatile, external_applications_chooser_button_type_id);
	}
	return external_applications_chooser_button_type_id__volatile;
}


static void _vala_external_applications_chooser_button_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	ExternalApplicationsChooserButton * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButton);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	switch (property_id) {
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_value_set_object (value, external_applications_chooser_button_get_app_info (self));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_value_set_string (value, external_applications_chooser_button_get_commandline (self));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 3511 "open-with.c"
		default:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 3517 "open-with.c"
	}
}


static void _vala_external_applications_chooser_button_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	ExternalApplicationsChooserButton * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, EXTERNAL_APPLICATIONS_TYPE_CHOOSER_BUTTON, ExternalApplicationsChooserButton);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	switch (property_id) {
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_APP_INFO_PROPERTY:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_chooser_button_set_app_info (self, g_value_get_object (value));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		case EXTERNAL_APPLICATIONS_CHOOSER_BUTTON_COMMANDLINE_PROPERTY:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_chooser_button_set_commandline (self, g_value_get_string (value));
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 3539 "open-with.c"
		default:
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		break;
#line 3545 "open-with.c"
	}
}


static gint _external_applications_types_tree_sort_func_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	gint result;
	result = external_applications_types_tree_sort_func ((ExternalApplicationsTypes*) self, model, a, b);
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 3555 "open-with.c"
}


static void _external_applications_types_on_render_type_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 485 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_on_render_type_icon ((ExternalApplicationsTypes*) self, cell_layout, cell, tree_model, iter);
#line 3562 "open-with.c"
}


static void _external_applications_types_on_render_type_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_on_render_type_text ((ExternalApplicationsTypes*) self, cell_layout, cell, tree_model, iter);
#line 3569 "open-with.c"
}


static void _external_applications_types_on_render_icon_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_on_render_icon ((ExternalApplicationsTypes*) self, cell_layout, cell, tree_model, iter);
#line 3576 "open-with.c"
}


static void _external_applications_types_on_render_text_gtk_cell_layout_data_func (GtkCellLayout* cell_layout, GtkCellRenderer* cell, GtkTreeModel* tree_model, GtkTreeIter* iter, gpointer self) {
#line 507 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_on_render_text ((ExternalApplicationsTypes*) self, cell_layout, cell, tree_model, iter);
#line 3583 "open-with.c"
}


static void _external_applications_types_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
#line 510 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_row_activated ((ExternalApplicationsTypes*) self, path, column);
#line 3590 "open-with.c"
}


static void _g_free0_ (gpointer var) {
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	var = (g_free (var), NULL);
#line 3597 "open-with.c"
}


static inline void _g_list_free__g_free0_ (GList* self) {
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_list_free_full (self, (GDestroyNotify) _g_free0_);
#line 3604 "open-with.c"
}


static void __lambda8_ (ExternalApplicationsTypes* self, GdkRectangle* allocation) {
	GtkTreeView* _tmp0_;
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (allocation != NULL);
#line 526 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->priv->treeview;
#line 526 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_columns_autosize (_tmp0_);
#line 3616 "open-with.c"
}


static void ___lambda8__gtk_widget_size_allocate (GtkWidget* _sender, GdkRectangle* allocation, gpointer self) {
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda8_ ((ExternalApplicationsTypes*) self, allocation);
#line 3623 "open-with.c"
}


ExternalApplicationsTypes* external_applications_types_construct (GType object_type) {
	ExternalApplicationsTypes * self = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkListStore* _tmp0_;
	GtkTreeView* _tmp1_;
	GtkTreeView* _tmp2_;
	GtkListStore* _tmp3_;
	GtkListStore* _tmp4_;
	GtkTreeViewColumn* _tmp5_;
	GtkTreeViewColumn* _tmp6_;
	GtkCellRendererPixbuf* renderer_type_icon = NULL;
	GtkCellRendererPixbuf* _tmp7_;
	GtkTreeViewColumn* _tmp8_;
	GtkCellRendererPixbuf* _tmp9_;
	GtkTreeViewColumn* _tmp10_;
	GtkCellRendererPixbuf* _tmp11_;
	GtkTreeView* _tmp12_;
	GtkTreeViewColumn* _tmp13_;
	GtkTreeViewColumn* _tmp14_;
	GtkTreeViewColumn* _tmp15_;
	GtkCellRendererText* renderer_type_text = NULL;
	GtkCellRendererText* _tmp16_;
	GtkTreeViewColumn* _tmp17_;
	GtkCellRendererText* _tmp18_;
	GtkTreeViewColumn* _tmp19_;
	GtkCellRendererText* _tmp20_;
	GtkTreeView* _tmp21_;
	GtkTreeViewColumn* _tmp22_;
	GtkTreeViewColumn* _tmp23_;
	GtkTreeViewColumn* _tmp24_;
	GtkCellRendererPixbuf* renderer_icon = NULL;
	GtkCellRendererPixbuf* _tmp25_;
	GtkTreeViewColumn* _tmp26_;
	GtkCellRendererPixbuf* _tmp27_;
	GtkTreeViewColumn* _tmp28_;
	GtkCellRendererPixbuf* _tmp29_;
	GtkTreeView* _tmp30_;
	GtkTreeViewColumn* _tmp31_;
	GtkTreeViewColumn* _tmp32_;
	GtkTreeViewColumn* _tmp33_;
	GtkCellRendererText* renderer_text = NULL;
	GtkCellRendererText* _tmp34_;
	GtkTreeViewColumn* _tmp35_;
	GtkCellRendererText* _tmp36_;
	GtkTreeViewColumn* _tmp37_;
	GtkTreeViewColumn* _tmp38_;
	GtkCellRendererText* _tmp39_;
	GtkTreeView* _tmp40_;
	GtkTreeViewColumn* _tmp41_;
	GtkTreeView* _tmp42_;
	GtkTreeView* _tmp43_;
	GtkScrolledWindow* scrolled = NULL;
	GtkScrolledWindow* _tmp44_;
	GtkScrolledWindow* _tmp45_;
	GtkScrolledWindow* _tmp46_;
	GtkTreeView* _tmp47_;
	GtkScrolledWindow* _tmp48_;
	gint height = 0;
	GtkTreeView* _tmp49_;
	PangoLayout* _tmp50_;
	PangoLayout* _tmp51_;
	gint _tmp52_ = 0;
	GtkScrolledWindow* _tmp53_;
	gint _tmp54_;
	GList* _tmp55_;
	GVfs* _tmp57_;
	gchar** _tmp58_;
	gchar** _tmp59_;
	GtkTreeView* _tmp63_;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsTypes*) g_object_new (object_type, NULL);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->store;
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) _tmp0_);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp1_);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->treeview);
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv->treeview = _tmp1_;
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = self->priv->treeview;
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_set_headers_visible (_tmp2_, FALSE);
#line 478 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = self->store;
#line 478 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) _tmp3_, 0, GTK_SORT_ASCENDING);
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = self->store;
#line 479 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) _tmp4_, 0, _external_applications_types_tree_sort_func_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = gtk_tree_view_column_new ();
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp5_);
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp5_;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = column;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_sizing (_tmp6_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp7_);
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_type_icon = _tmp7_;
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = column;
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = renderer_type_icon;
#line 484 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp8_, (GtkCellRenderer*) _tmp9_, FALSE);
#line 485 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = column;
#line 485 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = renderer_type_icon;
#line 485 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp10_, (GtkCellRenderer*) _tmp11_, _external_applications_types_on_render_type_icon_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = self->priv->treeview;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = column;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp12_, _tmp13_);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = gtk_tree_view_column_new ();
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp14_);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp14_;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = column;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_sizing (_tmp15_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 490 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 490 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp16_);
#line 490 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_type_text = _tmp16_;
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = column;
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = renderer_type_text;
#line 491 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp17_, (GtkCellRenderer*) _tmp18_, TRUE);
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = column;
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = renderer_type_text;
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp19_, (GtkCellRenderer*) _tmp20_, _external_applications_types_on_render_type_text_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 493 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = self->priv->treeview;
#line 493 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = column;
#line 493 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp21_, _tmp22_);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = gtk_tree_view_column_new ();
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp23_);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp23_;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = column;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_sizing (_tmp24_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp25_);
#line 497 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_icon = _tmp25_;
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = column;
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = renderer_icon;
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp26_, (GtkCellRenderer*) _tmp27_, FALSE);
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = column;
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = renderer_icon;
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp28_, (GtkCellRenderer*) _tmp29_, _external_applications_types_on_render_icon_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = self->priv->treeview;
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp31_ = column;
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp30_, _tmp31_);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp32_ = gtk_tree_view_column_new ();
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp32_);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	column = _tmp32_;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp33_ = column;
#line 503 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_sizing (_tmp33_, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp34_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp34_);
#line 504 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	renderer_text = _tmp34_;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp35_ = column;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp36_ = renderer_text;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp35_, (GtkCellRenderer*) _tmp36_, TRUE);
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp37_ = column;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_column_set_expand (_tmp37_, TRUE);
#line 507 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp38_ = column;
#line 507 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp39_ = renderer_text;
#line 507 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_cell_layout_set_cell_data_func ((GtkCellLayout*) _tmp38_, (GtkCellRenderer*) _tmp39_, _external_applications_types_on_render_text_gtk_cell_layout_data_func, g_object_ref (self), g_object_unref);
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp40_ = self->priv->treeview;
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp41_ = column;
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_view_append_column (_tmp40_, _tmp41_);
#line 510 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp42_ = self->priv->treeview;
#line 510 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp42_, "row-activated", (GCallback) _external_applications_types_row_activated_gtk_tree_view_row_activated, self, 0);
#line 511 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp43_ = self->priv->treeview;
#line 511 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show ((GtkWidget*) _tmp43_);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp44_ = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp44_);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	scrolled = _tmp44_;
#line 513 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp45_ = scrolled;
#line 513 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_scrolled_window_set_policy (_tmp45_, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
#line 514 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp46_ = scrolled;
#line 514 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp47_ = self->priv->treeview;
#line 514 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_container_add ((GtkContainer*) _tmp46_, (GtkWidget*) _tmp47_);
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp48_ = scrolled;
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) _tmp48_, TRUE, TRUE, (guint) 0);
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp49_ = self->priv->treeview;
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp50_ = gtk_widget_create_pango_layout ((GtkWidget*) _tmp49_, "a\nb");
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp51_ = _tmp50_;
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	pango_layout_get_pixel_size (_tmp51_, NULL, &_tmp52_);
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	height = _tmp52_;
#line 517 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp51_);
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp53_ = scrolled;
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp54_ = height;
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_set_size_request ((GtkWidget*) _tmp53_, -1, _tmp54_ * 5);
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp55_ = g_content_types_get_registered ();
#line 3916 "open-with.c"
	{
		GList* content_type_collection = NULL;
		GList* content_type_it = NULL;
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		content_type_collection = _tmp55_;
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (content_type_it = content_type_collection; content_type_it != NULL; content_type_it = content_type_it->next) {
#line 3924 "open-with.c"
			const gchar* content_type = NULL;
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			content_type = (const gchar*) content_type_it->data;
#line 3928 "open-with.c"
			{
				const gchar* _tmp56_;
#line 521 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp56_ = content_type;
#line 521 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_types_launcher_added (self, _tmp56_);
#line 3935 "open-with.c"
			}
		}
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		__g_list_free__g_free0_0 (content_type_collection);
#line 3940 "open-with.c"
	}
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp57_ = g_vfs_get_default ();
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp59_ = _tmp58_ = g_vfs_get_supported_uri_schemes (_tmp57_);
#line 3946 "open-with.c"
	{
		gchar** scheme_collection = NULL;
		gint scheme_collection_length1 = 0;
		gint _scheme_collection_size_ = 0;
		gint scheme_it = 0;
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		scheme_collection = _tmp59_;
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		scheme_collection_length1 = _vala_array_length (_tmp58_);
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (scheme_it = 0; scheme_it < _vala_array_length (_tmp58_); scheme_it = scheme_it + 1) {
#line 3958 "open-with.c"
			const gchar* scheme = NULL;
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			scheme = scheme_collection[scheme_it];
#line 3962 "open-with.c"
			{
				const gchar* _tmp60_;
				gchar* _tmp61_;
				gchar* _tmp62_;
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp60_ = scheme;
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp61_ = g_strconcat ("x-scheme-handler/", _tmp60_, NULL);
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp62_ = _tmp61_;
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_types_launcher_added (self, _tmp62_);
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (_tmp62_);
#line 3977 "open-with.c"
			}
		}
	}
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp63_ = self->priv->treeview;
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((GtkWidget*) _tmp63_, "size-allocate", (GCallback) ___lambda8__gtk_widget_size_allocate, self, G_CONNECT_AFTER);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (scrolled);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_text);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_icon);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_type_text);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (renderer_type_icon);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (column);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 3999 "open-with.c"
}


ExternalApplicationsTypes* external_applications_types_new (void) {
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_types_construct (EXTERNAL_APPLICATIONS_TYPE_TYPES);
#line 4006 "open-with.c"
}


static void external_applications_types_on_render_type_icon (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	gchar* content_type = NULL;
	GtkListStore* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	GIcon* _tmp3_;
	GIcon* _tmp4_;
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->store;
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, 0, &content_type, -1);
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = renderer;
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_content_type_get_icon (content_type);
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp2_, "gicon", _tmp4_, "stock-size", GTK_ICON_SIZE_BUTTON, "xpad", 4, NULL);
#line 536 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp4_);
#line 530 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type);
#line 4045 "open-with.c"
}


static void external_applications_types_on_render_type_text (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	gchar* content_type = NULL;
	GAppInfo* app_info = NULL;
	GtkListStore* _tmp0_;
	GtkTreeIter _tmp1_;
	gchar* desc = NULL;
	gchar* mime_type = NULL;
	const gchar* _tmp2_;
	gboolean _tmp3_;
	GtkCellRenderer* _tmp15_;
	const gchar* _tmp16_;
	const gchar* _tmp17_;
	gchar* _tmp18_;
	gchar* _tmp19_;
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 546 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->store;
#line 546 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 546 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp0_, &_tmp1_, 0, &content_type, 1, &app_info, -1);
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = content_type;
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_str_has_prefix (_tmp2_, "x-scheme-handler/");
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 4085 "open-with.c"
		const gchar* _tmp4_;
		gchar** _tmp5_;
		gchar** _tmp6_;
		gchar** _tmp7_;
		gint _tmp7__length1;
		const gchar* _tmp8_;
		gchar* _tmp9_;
		gchar* _tmp10_;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = content_type;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = _tmp5_ = g_strsplit (_tmp4_, "/", 0);
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = _tmp6_;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7__length1 = _vala_array_length (_tmp5_);
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _tmp7_[1];
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = g_strconcat (_tmp8_, "://", NULL);
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (desc);
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		desc = _tmp9_;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = (_vala_array_free (_tmp7_, _tmp7__length1, (GDestroyNotify) g_free), NULL);
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = g_strdup ("");
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (mime_type);
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		mime_type = _tmp10_;
#line 4118 "open-with.c"
	} else {
		const gchar* _tmp11_;
		gchar* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = content_type;
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = g_content_type_get_description (_tmp11_);
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (desc);
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		desc = _tmp12_;
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = content_type;
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = g_content_type_get_mime_type (_tmp13_);
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (mime_type);
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		mime_type = _tmp14_;
#line 4140 "open-with.c"
	}
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = renderer;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = desc;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = mime_type;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = g_markup_printf_escaped ("<b>%s</b>\n%s", _tmp16_, _tmp17_);
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = _tmp18_;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp15_, "markup", _tmp19_, "width-chars", 30, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp19_);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (mime_type);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (desc);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 541 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type);
#line 4164 "open-with.c"
}


static void external_applications_types_on_render_icon (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	GAppInfo* app_info = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	GIcon* _tmp3_;
	GIcon* _tmp4_;
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 1, &app_info, -1);
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = renderer;
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_app_info_get_icon (app_info);
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp2_, "gicon", _tmp4_, "stock-size", GTK_ICON_SIZE_MENU, "xpad", 4, NULL);
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp4_);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 4203 "open-with.c"
}


static void external_applications_types_on_render_text (ExternalApplicationsTypes* self, GtkCellLayout* column, GtkCellRenderer* renderer, GtkTreeModel* model, GtkTreeIter* iter) {
	GAppInfo* app_info = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkCellRenderer* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (renderer != NULL);
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (model != NULL);
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *iter;
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 1, &app_info, -1);
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = renderer;
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_describe_app_info (app_info);
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_set ((GObject*) _tmp2_, "markup", _tmp4_, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp4_);
#line 579 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 4242 "open-with.c"
}


static void external_applications_types_launcher_added (ExternalApplicationsTypes* self, const gchar* content_type) {
	GAppInfo* app_info = NULL;
	const gchar* _tmp0_;
	GAppInfo* _tmp1_;
	GAppInfo* _tmp2_;
	GtkTreeIter iter = {0};
	GtkListStore* _tmp3_;
	GtkTreeIter _tmp4_ = {0};
	GtkListStore* _tmp5_;
	GtkTreeIter _tmp6_;
	const gchar* _tmp7_;
	GAppInfo* _tmp8_;
#line 588 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 588 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = content_type;
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_app_info_get_default_for_type (_tmp0_, FALSE);
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	app_info = _tmp1_;
#line 590 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = app_info;
#line 590 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp2_ == NULL) {
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 4276 "open-with.c"
	}
#line 594 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = self->store;
#line 594 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_list_store_append (_tmp3_, &_tmp4_);
#line 594 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp4_;
#line 595 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = self->store;
#line 595 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = iter;
#line 595 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = content_type;
#line 595 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = app_info;
#line 595 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_list_store_set (_tmp5_, &_tmp6_, 0, _tmp7_, 1, _tmp8_, -1);
#line 588 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 4296 "open-with.c"
}


static gint external_applications_types_tree_sort_func (ExternalApplicationsTypes* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	gint result = 0;
	gchar* content_type1 = NULL;
	gchar* content_type2 = NULL;
	GtkTreeModel* _tmp0_;
	GtkTreeIter _tmp1_;
	GtkTreeModel* _tmp2_;
	GtkTreeIter _tmp3_;
	GCompareFunc _tmp4_;
	gint _tmp5_;
#line 598 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 598 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (model != NULL, 0);
#line 598 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (a != NULL, 0);
#line 598 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (b != NULL, 0);
#line 600 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = model;
#line 600 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = *a;
#line 600 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp0_, &_tmp1_, 0, &content_type1, -1);
#line 601 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = model;
#line 601 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = *b;
#line 601 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_tree_model_get (_tmp2_, &_tmp3_, 0, &content_type2, -1);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = g_strcmp0;
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = _tmp4_ (content_type1, content_type2);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp5_;
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type2);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type1);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 4342 "open-with.c"
}


static void external_applications_types_row_activated (ExternalApplicationsTypes* self, GtkTreePath* path, GtkTreeViewColumn* column) {
	GtkTreeIter iter = {0};
	GtkListStore* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreeIter _tmp2_ = {0};
	gboolean _tmp3_;
#line 605 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 605 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (path != NULL);
#line 605 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (column != NULL);
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = self->store;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = path;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp0_, &_tmp2_, _tmp1_);
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	iter = _tmp2_;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 4368 "open-with.c"
		gchar* content_type = NULL;
		GtkListStore* _tmp4_;
		GtkTreeIter _tmp5_;
		const gchar* _tmp6_;
		GtkTreeIter _tmp7_;
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = self->store;
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = iter;
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp4_, &_tmp5_, 0, &content_type, -1);
#line 610 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = content_type;
#line 610 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = iter;
#line 610 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_emit (self, external_applications_types_signals[EXTERNAL_APPLICATIONS_TYPES_SELECTED_SIGNAL], 0, _tmp6_, &_tmp7_);
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (content_type);
#line 4388 "open-with.c"
	}
}


static void g_cclosure_user_marshal_VOID__STRING_BOXED (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data) {
	typedef void (*GMarshalFunc_VOID__STRING_BOXED) (gpointer data1, const char* arg_1, gpointer arg_2, gpointer data2);
	register GMarshalFunc_VOID__STRING_BOXED callback;
	register GCClosure * cc;
	register gpointer data1;
	register gpointer data2;
	cc = (GCClosure *) closure;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (n_param_values == 3);
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_CCLOSURE_SWAP_DATA (closure)) {
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data1 = closure->data;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data2 = param_values->data[0].v_pointer;
#line 4408 "open-with.c"
	} else {
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data1 = param_values->data[0].v_pointer;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		data2 = closure->data;
#line 4414 "open-with.c"
	}
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	callback = (GMarshalFunc_VOID__STRING_BOXED) (marshal_data ? marshal_data : cc->callback);
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	callback (data1, g_value_get_string (param_values + 1), g_value_get_boxed (param_values + 2), data2);
#line 4420 "open-with.c"
}


static void external_applications_types_class_init (ExternalApplicationsTypesClass * klass) {
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_parent_class = g_type_class_peek_parent (klass);
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_type_class_add_private (klass, sizeof (ExternalApplicationsTypesPrivate));
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (klass)->finalize = external_applications_types_finalize;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_types_signals[EXTERNAL_APPLICATIONS_TYPES_SELECTED_SIGNAL] = g_signal_new ("selected", EXTERNAL_APPLICATIONS_TYPE_TYPES, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_user_marshal_VOID__STRING_BOXED, G_TYPE_NONE, 2, G_TYPE_STRING, GTK_TYPE_TREE_ITER);
#line 4433 "open-with.c"
}


static void external_applications_types_instance_init (ExternalApplicationsTypes * self) {
	GtkListStore* _tmp0_;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->priv = EXTERNAL_APPLICATIONS_TYPES_GET_PRIVATE (self);
#line 469 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = gtk_list_store_new (2, G_TYPE_STRING, g_app_info_get_type ());
#line 469 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self->store = _tmp0_;
#line 4445 "open-with.c"
}


static void external_applications_types_finalize (GObject * obj) {
	ExternalApplicationsTypes * self;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, EXTERNAL_APPLICATIONS_TYPE_TYPES, ExternalApplicationsTypes);
#line 469 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->store);
#line 470 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (self->priv->treeview);
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	G_OBJECT_CLASS (external_applications_types_parent_class)->finalize (obj);
#line 4459 "open-with.c"
}


GType external_applications_types_get_type (void) {
	static volatile gsize external_applications_types_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_types_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsTypesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_types_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsTypes), 0, (GInstanceInitFunc) external_applications_types_instance_init, NULL };
		GType external_applications_types_type_id;
		external_applications_types_type_id = g_type_register_static (GTK_TYPE_VBOX, "ExternalApplicationsTypes", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_types_type_id__volatile, external_applications_types_type_id);
	}
	return external_applications_types_type_id__volatile;
}


static GType external_applications_manager_next_step_get_type (void) {
	static volatile gsize external_applications_manager_next_step_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_manager_next_step_type_id__volatile)) {
		static const GEnumValue values[] = {{EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN, "EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN", "try-open"}, {EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_OPEN_WITH, "EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_OPEN_WITH", "open-with"}, {0, NULL, NULL}};
		GType external_applications_manager_next_step_type_id;
		external_applications_manager_next_step_type_id = g_enum_register_static ("ExternalApplicationsManagerNextStep", values);
		g_once_init_leave (&external_applications_manager_next_step_type_id__volatile, external_applications_manager_next_step_type_id);
	}
	return external_applications_manager_next_step_type_id__volatile;
}


static gboolean external_applications_manager_open_uri (ExternalApplicationsManager* self, MidoriTab* tab, const gchar* uri) {
	gboolean result = FALSE;
	gchar* content_type = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	MidoriTab* _tmp3_;
	gboolean _tmp4_;
#line 624 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 624 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (tab != NULL, FALSE);
#line 624 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = uri;
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = external_applications_manager_get_content_type (self, _tmp0_, NULL);
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	content_type = _tmp1_;
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = uri;
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = tab;
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = external_applications_manager_open_with_type (self, _tmp2_, content_type, (GtkWidget*) _tmp3_, EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN);
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp4_;
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type);
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 4519 "open-with.c"
}


static gboolean external_applications_manager_navigation_requested (ExternalApplicationsManager* self, MidoriTab* tab, const gchar* uri) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	const gchar* _tmp2_;
	gboolean _tmp3_;
	const gchar* _tmp8_;
	gboolean _tmp9_;
	gchar* content_type = NULL;
	const gchar* _tmp10_;
	gchar* _tmp11_;
	const gchar* _tmp12_;
	const gchar* _tmp13_;
	MidoriTab* _tmp14_;
#line 629 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 629 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (tab != NULL, FALSE);
#line 629 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = uri;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = g_str_has_prefix (_tmp2_, "file://");
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_) {
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = TRUE;
#line 4551 "open-with.c"
	} else {
		const gchar* _tmp4_;
		gboolean _tmp5_;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = uri;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = midori_uri_is_http (_tmp4_);
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp1_ = _tmp5_;
#line 4561 "open-with.c"
	}
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp1_) {
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = TRUE;
#line 4567 "open-with.c"
	} else {
		const gchar* _tmp6_;
		gboolean _tmp7_;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = uri;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = midori_uri_is_blank (_tmp6_);
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp7_;
#line 4577 "open-with.c"
	}
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp0_) {
#line 632 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = FALSE;
#line 632 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 4585 "open-with.c"
	}
#line 635 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = uri;
#line 635 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = g_str_has_prefix (_tmp8_, "abp:");
#line 635 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp9_) {
#line 636 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = TRUE;
#line 636 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 4597 "open-with.c"
	}
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = uri;
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = external_applications_manager_get_content_type (self, _tmp10_, NULL);
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	content_type = _tmp11_;
#line 639 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = uri;
#line 639 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = content_type;
#line 639 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = tab;
#line 639 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_open_with_type (self, _tmp12_, _tmp13_, (GtkWidget*) _tmp14_, EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN);
#line 640 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = TRUE;
#line 640 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (content_type);
#line 640 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 4619 "open-with.c"
}


static gchar* external_applications_manager_get_content_type (ExternalApplicationsManager* self, const gchar* uri, const gchar* mime_type) {
	gchar* result = NULL;
	gboolean _tmp0_ = FALSE;
	const gchar* _tmp1_;
	gboolean _tmp2_;
	const gchar* _tmp24_;
	gchar* _tmp25_;
	GError * _inner_error_ = NULL;
#line 643 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 643 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = uri;
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = g_str_has_prefix (_tmp1_, "file://");
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (!_tmp2_) {
#line 4641 "open-with.c"
		const gchar* _tmp3_;
		gboolean _tmp4_;
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = uri;
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = midori_uri_is_http (_tmp3_);
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = !_tmp4_;
#line 4650 "open-with.c"
	} else {
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = FALSE;
#line 4654 "open-with.c"
	}
#line 644 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp0_) {
#line 4658 "open-with.c"
		gchar* protocol = NULL;
		const gchar* _tmp5_;
		gchar** _tmp6_;
		gchar** _tmp7_;
		gchar** _tmp8_;
		gint _tmp8__length1;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		const gchar* _tmp12_;
		gchar* _tmp13_;
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = uri;
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = _tmp6_ = g_strsplit (_tmp5_, ":", 2);
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _tmp7_;
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8__length1 = _vala_array_length (_tmp6_);
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = _tmp8_[0];
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = g_strdup (_tmp9_);
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = _tmp10_;
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = (_vala_array_free (_tmp8_, _tmp8__length1, (GDestroyNotify) g_free), NULL);
#line 645 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		protocol = _tmp11_;
#line 646 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp12_ = protocol;
#line 646 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = g_strconcat ("x-scheme-handler/", _tmp12_, NULL);
#line 646 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = _tmp13_;
#line 646 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (protocol);
#line 646 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 4698 "open-with.c"
	} else {
		const gchar* _tmp14_;
#line 647 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = mime_type;
#line 647 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (_tmp14_ == NULL) {
#line 4705 "open-with.c"
			gchar* filename = NULL;
			gboolean uncertain = FALSE;
			const gchar* _tmp21_;
			gboolean _tmp22_ = FALSE;
			gchar* _tmp23_;
			{
				gchar* _tmp15_ = NULL;
				const gchar* _tmp16_;
				gchar* _tmp17_;
				gchar* _tmp18_;
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp16_ = uri;
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp17_ = g_filename_from_uri (_tmp16_, NULL, &_inner_error_);
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp15_ = _tmp17_;
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 4724 "open-with.c"
					goto __catch4_g_error;
				}
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp18_ = _tmp15_;
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp15_ = NULL;
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (filename);
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				filename = _tmp18_;
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (_tmp15_);
#line 4737 "open-with.c"
			}
			goto __finally4;
			__catch4_g_error:
			{
				GError* _error_ = NULL;
				const gchar* _tmp19_;
				gchar* _tmp20_;
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_error_ = _inner_error_;
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_inner_error_ = NULL;
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp19_ = uri;
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp20_ = g_strdup (_tmp19_);
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (filename);
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				filename = _tmp20_;
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_error_free0 (_error_);
#line 4759 "open-with.c"
			}
			__finally4:
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_g_free0 (filename);
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				g_clear_error (&_inner_error_);
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				return NULL;
#line 4772 "open-with.c"
			}
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp21_ = filename;
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp23_ = g_content_type_guess (_tmp21_, NULL, (gsize) 0, &_tmp22_);
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			uncertain = _tmp22_;
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			result = _tmp23_;
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_g_free0 (filename);
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			return result;
#line 4786 "open-with.c"
		}
	}
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = mime_type;
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = g_content_type_from_mime_type (_tmp24_);
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp25_;
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 4797 "open-with.c"
}


static Block3Data* block3_data_ref (Block3Data* _data3_) {
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data3_->_ref_count_);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data3_;
#line 4806 "open-with.c"
}


static void block3_data_unref (void * _userdata_) {
	Block3Data* _data3_;
	_data3_ = (Block3Data*) _userdata_;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data3_->_ref_count_)) {
#line 4815 "open-with.c"
		ExternalApplicationsManager* self;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data3_->self;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data3_->download);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_data3_->content_type);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data3_->widget);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block3Data, _data3_);
#line 4829 "open-with.c"
	}
}


static void __lambda9_ (Block3Data* _data3_, GParamSpec* pspec) {
	ExternalApplicationsManager* self;
	WebKitDownload* _tmp0_;
	WebKitDownloadStatus _tmp1_;
	WebKitDownloadStatus _tmp2_;
#line 676 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data3_->self;
#line 676 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (pspec != NULL);
#line 677 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _data3_->download;
#line 677 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = webkit_download_get_status (_tmp0_);
#line 677 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = _tmp1_;
#line 677 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp2_ == WEBKIT_DOWNLOAD_STATUS_FINISHED) {
#line 4851 "open-with.c"
		WebKitDownload* _tmp3_;
		const gchar* _tmp4_;
		const gchar* _tmp5_;
		const gchar* _tmp6_;
		GtkWidget* _tmp7_;
		ExternalApplicationsManagerNextStep _tmp8_;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = _data3_->download;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = webkit_download_get_destination_uri (_tmp3_);
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = _tmp4_;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = _data3_->content_type;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = _data3_->widget;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _data3_->next_step;
#line 678 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		external_applications_manager_open_now (self, _tmp5_, _tmp6_, _tmp7_, _tmp8_);
#line 4872 "open-with.c"
	} else {
		WebKitDownload* _tmp9_;
		WebKitDownloadStatus _tmp10_;
		WebKitDownloadStatus _tmp11_;
#line 680 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = _data3_->download;
#line 680 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = webkit_download_get_status (_tmp9_);
#line 680 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = _tmp10_;
#line 680 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (_tmp11_ == WEBKIT_DOWNLOAD_STATUS_ERROR) {
#line 4885 "open-with.c"
			WebKitDownload* _tmp12_;
			const gchar* _tmp13_;
			const gchar* _tmp14_;
			gchar* _tmp15_;
			gchar* _tmp16_;
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp12_ = _data3_->download;
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp13_ = webkit_download_get_destination_uri (_tmp12_);
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp14_ = _tmp13_;
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp15_ = g_strdup_printf (_ ("Cannot open '%s' because the download failed."), _tmp14_);
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_tmp16_ = _tmp15_;
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			midori_show_message_dialog (GTK_MESSAGE_ERROR, _ ("Download error"), _tmp16_, FALSE);
#line 681 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			_g_free0 (_tmp16_);
#line 4905 "open-with.c"
		}
	}
}


static void ___lambda9__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 676 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda9_ (self, pspec);
#line 4914 "open-with.c"
}


static gboolean external_applications_manager_open_with_type (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget, ExternalApplicationsManagerNextStep next_step) {
	gboolean result = FALSE;
	Block3Data* _data3_;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	GtkWidget* _tmp2_;
	GtkWidget* _tmp3_;
	ExternalApplicationsManagerNextStep _tmp4_;
	const gchar* _tmp5_;
	gboolean _tmp6_;
	const gchar* _tmp12_;
	const gchar* _tmp18_;
	WebKitNetworkRequest* _tmp19_;
	WebKitNetworkRequest* _tmp20_;
	WebKitDownload* _tmp21_;
	WebKitDownload* _tmp22_;
	WebKitDownload* _tmp23_;
	WebKitDownload* _tmp24_;
	gchar* _tmp25_;
	gchar* _tmp26_;
	WebKitDownload* _tmp27_;
	WebKitDownload* _tmp28_;
	const gchar* _tmp29_;
	const gchar* _tmp30_;
	gboolean _tmp31_;
	WebKitDownload* _tmp32_;
	WebKitDownload* _tmp33_;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, FALSE);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_ = g_slice_new0 (Block3Data);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->_ref_count_ = 1;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->self = g_object_ref (self);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = content_type;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_data3_->content_type);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->content_type = _tmp1_;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = widget;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_data3_->widget);
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->widget = _tmp3_;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = next_step;
#line 661 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->next_step = _tmp4_;
#line 665 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = uri;
#line 665 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = midori_uri_is_http (_tmp5_);
#line 665 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (!_tmp6_) {
#line 4985 "open-with.c"
		const gchar* _tmp7_;
		const gchar* _tmp8_;
		GtkWidget* _tmp9_;
		ExternalApplicationsManagerNextStep _tmp10_;
		gboolean _tmp11_;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = uri;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = _data3_->content_type;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = _data3_->widget;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = _data3_->next_step;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = external_applications_manager_open_now (self, _tmp7_, _tmp8_, _tmp9_, _tmp10_);
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = _tmp11_;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block3_data_unref (_data3_);
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data3_ = NULL;
#line 666 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5009 "open-with.c"
	}
#line 668 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = _data3_->content_type;
#line 668 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_strcmp0 (_tmp12_, "application/octet-stream") == 0) {
#line 5015 "open-with.c"
		const gchar* _tmp13_;
		const gchar* _tmp14_;
		GtkWidget* _tmp15_;
		ExternalApplicationsManagerNextStep _tmp16_;
		gboolean _tmp17_;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp13_ = uri;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = _data3_->content_type;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp15_ = _data3_->widget;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp16_ = _data3_->next_step;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp17_ = external_applications_manager_open_now (self, _tmp13_, _tmp14_, _tmp15_, _tmp16_);
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = _tmp17_;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block3_data_unref (_data3_);
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data3_ = NULL;
#line 669 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5039 "open-with.c"
	}
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = uri;
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = webkit_network_request_new (_tmp18_);
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = _tmp19_;
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = webkit_download_new (_tmp20_);
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = _tmp21_;
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp20_);
#line 671 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_->download = _tmp22_;
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = _data3_->download;
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = _data3_->download;
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = midori_download_prepare_destination_uri (_tmp24_, NULL);
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = _tmp25_;
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	webkit_download_set_destination_uri (_tmp23_, _tmp26_);
#line 672 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp26_);
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = _data3_->download;
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = _data3_->download;
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = webkit_download_get_destination_uri (_tmp28_);
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = _tmp29_;
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp31_ = midori_download_has_enough_space (_tmp27_, _tmp30_, FALSE);
#line 673 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (!_tmp31_) {
#line 674 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = FALSE;
#line 674 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block3_data_unref (_data3_);
#line 674 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data3_ = NULL;
#line 674 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5087 "open-with.c"
	}
#line 676 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp32_ = _data3_->download;
#line 676 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_data ((GObject*) _tmp32_, "notify::status", (GCallback) ___lambda9__g_object_notify, block3_data_ref (_data3_), (GClosureNotify) block3_data_unref, 0);
#line 686 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp33_ = _data3_->download;
#line 686 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	webkit_download_start (_tmp33_);
#line 687 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = TRUE;
#line 687 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	block3_data_unref (_data3_);
#line 687 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data3_ = NULL;
#line 687 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 5105 "open-with.c"
}


static gboolean external_applications_manager_open_now (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget, ExternalApplicationsManagerNextStep next_step) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	ExternalApplicationsManagerNextStep _tmp1_;
	const gchar* _tmp7_;
	const gchar* _tmp8_;
	GtkWidget* _tmp9_;
	GAppInfo* _tmp10_;
	GAppInfo* _tmp11_;
	gboolean _tmp12_;
#line 694 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 694 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, FALSE);
#line 694 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, FALSE);
#line 694 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = next_step;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp1_ == EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN) {
#line 5131 "open-with.c"
		ExternalApplicationsAssociations* _tmp2_;
		ExternalApplicationsAssociations* _tmp3_;
		const gchar* _tmp4_;
		const gchar* _tmp5_;
		gboolean _tmp6_;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp2_ = external_applications_associations_new ();
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp3_ = _tmp2_;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = content_type;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = uri;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = external_applications_associations_open (_tmp3_, _tmp4_, _tmp5_);
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = _tmp6_;
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_tmp3_);
#line 5151 "open-with.c"
	} else {
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp0_ = FALSE;
#line 5155 "open-with.c"
	}
#line 695 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp0_) {
#line 696 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = TRUE;
#line 696 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5163 "open-with.c"
	}
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = uri;
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = content_type;
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = widget;
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = external_applications_manager_open_with (self, _tmp7_, _tmp8_, _tmp9_);
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = _tmp10_;
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = _tmp11_ != NULL;
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_tmp11_);
#line 698 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp12_) {
#line 699 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = TRUE;
#line 699 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5185 "open-with.c"
	}
#line 700 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = FALSE;
#line 700 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 5191 "open-with.c"
}


static GAppInfo* external_applications_manager_open_with (ExternalApplicationsManager* self, const gchar* uri, const gchar* content_type, GtkWidget* widget) {
	GAppInfo* result = NULL;
	ExternalApplicationsChooserDialog* dialog = NULL;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	GtkWidget* _tmp2_;
	ExternalApplicationsChooserDialog* _tmp3_;
	GAppInfo* app_info = NULL;
	ExternalApplicationsChooserDialog* _tmp4_;
	GAppInfo* _tmp5_;
	ExternalApplicationsChooserDialog* _tmp6_;
	const gchar* _tmp7_;
	GAppInfo* _tmp8_;
	GAppInfo* _tmp9_ = NULL;
	GAppInfo* _tmp10_;
	const gchar* _tmp11_;
	const gchar* _tmp12_;
	gboolean _tmp13_;
	GAppInfo* _tmp15_;
#line 705 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 705 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 705 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (content_type != NULL, NULL);
#line 705 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_val_if_fail (widget != NULL, NULL);
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = uri;
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = content_type;
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = widget;
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = external_applications_chooser_dialog_new (_tmp0_, _tmp1_, _tmp2_);
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp3_);
#line 706 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	dialog = _tmp3_;
#line 708 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = dialog;
#line 708 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = external_applications_chooser_dialog_open_with (_tmp4_);
#line 708 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	app_info = _tmp5_;
#line 709 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = dialog;
#line 709 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_object_destroy ((GtkObject*) _tmp6_);
#line 711 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = uri;
#line 711 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_strcmp0 (_tmp7_, "") == 0) {
#line 712 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = app_info;
#line 712 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (dialog);
#line 712 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5254 "open-with.c"
	}
#line 714 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = app_info;
#line 714 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp8_ == NULL) {
#line 715 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		result = app_info;
#line 715 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (dialog);
#line 715 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return result;
#line 5266 "open-with.c"
	}
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = app_info;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = uri;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = content_type;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = external_applications_open_app_info (_tmp10_, _tmp11_, _tmp12_);
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp13_) {
#line 5278 "open-with.c"
		GAppInfo* _tmp14_;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp14_ = app_info;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = _tmp14_;
#line 5284 "open-with.c"
	} else {
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = NULL;
#line 5288 "open-with.c"
	}
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = _g_object_ref0 (_tmp9_);
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = _tmp15_;
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (dialog);
#line 717 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 5300 "open-with.c"
}


static Block4Data* block4_data_ref (Block4Data* _data4_) {
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data4_->_ref_count_);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data4_;
#line 5309 "open-with.c"
}


static void block4_data_unref (void * _userdata_) {
	Block4Data* _data4_;
	_data4_ = (Block4Data*) _userdata_;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data4_->_ref_count_)) {
#line 5318 "open-with.c"
		ExternalApplicationsManager* self;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data4_->self;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data4_->tab);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block4Data, _data4_);
#line 5328 "open-with.c"
	}
}


static Block5Data* block5_data_ref (Block5Data* _data5_) {
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data5_->_ref_count_);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data5_;
#line 5338 "open-with.c"
}


static void block5_data_unref (void * _userdata_) {
	Block5Data* _data5_;
	_data5_ = (Block5Data*) _userdata_;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data5_->_ref_count_)) {
#line 5347 "open-with.c"
		ExternalApplicationsManager* self;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data5_->_data4_->self;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_data5_->uri);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block4_data_unref (_data5_->_data4_);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_->_data4_ = NULL;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block5Data, _data5_);
#line 5359 "open-with.c"
	}
}


static void ___lambda10_ (Block5Data* _data5_, GtkAction* action) {
	Block4Data* _data4_;
	ExternalApplicationsManager* self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	MidoriTab* _tmp4_;
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_ = _data5_->_data4_;
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data4_->self;
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (action != NULL);
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _data5_->uri;
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _data5_->uri;
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = external_applications_manager_get_content_type (self, _tmp1_, NULL);
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = _tmp2_;
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _data4_->tab;
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_open_with_type (self, _tmp0_, _tmp3_, (GtkWidget*) _tmp4_, EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_OPEN_WITH);
#line 725 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp3_);
#line 5392 "open-with.c"
}


static void ____lambda10__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	___lambda10_ (self, _sender);
#line 5399 "open-with.c"
}


static Block6Data* block6_data_ref (Block6Data* _data6_) {
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data6_->_ref_count_);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data6_;
#line 5408 "open-with.c"
}


static void block6_data_unref (void * _userdata_) {
	Block6Data* _data6_;
	_data6_ = (Block6Data*) _userdata_;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data6_->_ref_count_)) {
#line 5417 "open-with.c"
		ExternalApplicationsManager* self;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data6_->_data4_->self;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_free0 (_data6_->uri);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block4_data_unref (_data6_->_data4_);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_->_data4_ = NULL;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block6Data, _data6_);
#line 5429 "open-with.c"
	}
}


static void ___lambda11_ (Block6Data* _data6_, GtkAction* action) {
	Block4Data* _data4_;
	ExternalApplicationsManager* self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	MidoriTab* _tmp4_;
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_ = _data6_->_data4_;
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data4_->self;
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (action != NULL);
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = _data6_->uri;
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _data6_->uri;
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = external_applications_manager_get_content_type (self, _tmp1_, NULL);
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = _tmp2_;
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _data4_->tab;
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_open_with_type (self, _tmp0_, _tmp3_, (GtkWidget*) _tmp4_, EXTERNAL_APPLICATIONS_MANAGER_NEXT_STEP_TRY_OPEN);
#line 734 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_free0 (_tmp3_);
#line 5462 "open-with.c"
}


static void ____lambda11__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	___lambda11_ (self, _sender);
#line 5469 "open-with.c"
}


static void external_applications_manager_context_menu (ExternalApplicationsManager* self, MidoriTab* tab, WebKitHitTestResult* hit_test_result, MidoriContextAction* menu) {
	Block4Data* _data4_;
	MidoriTab* _tmp0_;
	MidoriTab* _tmp1_;
	WebKitHitTestResult* _tmp2_;
	WebKitHitTestResultContext _tmp3_;
	WebKitHitTestResultContext _tmp4_;
	WebKitHitTestResult* _tmp12_;
	WebKitHitTestResultContext _tmp13_;
	WebKitHitTestResultContext _tmp14_;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (tab != NULL);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (hit_test_result != NULL);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (menu != NULL);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_ = g_slice_new0 (Block4Data);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_->_ref_count_ = 1;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_->self = g_object_ref (self);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = tab;
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_data4_->tab);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_->tab = _tmp1_;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = hit_test_result;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_get (_tmp2_, "context", &_tmp3_, NULL);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if ((_tmp4_ & WEBKIT_HIT_TEST_RESULT_CONTEXT_LINK) != 0) {
#line 5513 "open-with.c"
		Block5Data* _data5_;
		WebKitHitTestResult* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
		GtkAction* action = NULL;
		GtkAction* _tmp8_;
		GtkAction* _tmp9_;
		MidoriContextAction* _tmp10_;
		GtkAction* _tmp11_;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_ = g_slice_new0 (Block5Data);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_->_ref_count_ = 1;
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_->_data4_ = block4_data_ref (_data4_);
#line 722 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = hit_test_result;
#line 722 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_get (_tmp5_, "link-uri", &_tmp6_, NULL);
#line 722 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = _tmp6_;
#line 722 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_->uri = _tmp7_;
#line 723 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = gtk_action_new ("OpenWith", _ ("Open _with…"), NULL, NULL);
#line 723 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		action = _tmp8_;
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = action;
#line 724 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_connect_data (_tmp9_, "activate", (GCallback) ____lambda10__gtk_action_activate, block5_data_ref (_data5_), (GClosureNotify) block5_data_unref, 0);
#line 727 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = menu;
#line 727 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = action;
#line 727 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		midori_context_action_add (_tmp10_, _tmp11_);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (action);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block5_data_unref (_data5_);
#line 721 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data5_ = NULL;
#line 5557 "open-with.c"
	}
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = hit_test_result;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_get (_tmp12_, "context", &_tmp13_, NULL);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = _tmp13_;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if ((_tmp14_ & WEBKIT_HIT_TEST_RESULT_CONTEXT_IMAGE) != 0) {
#line 5567 "open-with.c"
		Block6Data* _data6_;
		WebKitHitTestResult* _tmp15_;
		gchar* _tmp16_;
		gchar* _tmp17_;
		GtkAction* action = NULL;
		GtkAction* _tmp18_;
		GtkAction* _tmp19_;
		MidoriContextAction* _tmp20_;
		GtkAction* _tmp21_;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_ = g_slice_new0 (Block6Data);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_->_ref_count_ = 1;
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_->_data4_ = block4_data_ref (_data4_);
#line 731 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp15_ = hit_test_result;
#line 731 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_object_get (_tmp15_, "image-uri", &_tmp16_, NULL);
#line 731 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp17_ = _tmp16_;
#line 731 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_->uri = _tmp17_;
#line 732 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp18_ = gtk_action_new ("OpenImageInViewer", _ ("Open in Image _Viewer"), NULL, NULL);
#line 732 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		action = _tmp18_;
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp19_ = action;
#line 733 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_signal_connect_data (_tmp19_, "activate", (GCallback) ____lambda11__gtk_action_activate, block6_data_ref (_data6_), (GClosureNotify) block6_data_unref, 0);
#line 736 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp20_ = menu;
#line 736 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp21_ = action;
#line 736 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		midori_context_action_add (_tmp20_, _tmp21_);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (action);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		block6_data_unref (_data6_);
#line 730 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_data6_ = NULL;
#line 5611 "open-with.c"
	}
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	block4_data_unref (_data4_);
#line 720 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data4_ = NULL;
#line 5617 "open-with.c"
}


static Block7Data* block7_data_ref (Block7Data* _data7_) {
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_atomic_int_inc (&_data7_->_ref_count_);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return _data7_;
#line 5626 "open-with.c"
}


static void block7_data_unref (void * _userdata_) {
	Block7Data* _data7_;
	_data7_ = (Block7Data*) _userdata_;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (g_atomic_int_dec_and_test (&_data7_->_ref_count_)) {
#line 5635 "open-with.c"
		ExternalApplicationsManager* self;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		self = _data7_->self;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data7_->types);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data7_->settings);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (_data7_->preferences);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (self);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_slice_free (Block7Data, _data7_);
#line 5649 "open-with.c"
	}
}


static void __lambda12_ (Block7Data* _data7_, const gchar* commandline) {
	ExternalApplicationsManager* self;
	const gchar* _tmp0_;
#line 753 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data7_->self;
#line 754 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = commandline;
#line 754 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	midori_settings_set_text_editor ((MidoriSettings*) _data7_->settings, _tmp0_);
#line 5663 "open-with.c"
}


static void ___lambda12__external_applications_chooser_button_selected (ExternalApplicationsChooserButton* _sender, const gchar* commandline, gpointer self) {
#line 753 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda12_ (self, commandline);
#line 5670 "open-with.c"
}


static void __lambda13_ (Block7Data* _data7_, const gchar* commandline) {
	ExternalApplicationsManager* self;
	const gchar* _tmp0_;
#line 764 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data7_->self;
#line 765 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = commandline;
#line 765 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	midori_settings_set_news_aggregator ((MidoriSettings*) _data7_->settings, _tmp0_);
#line 5683 "open-with.c"
}


static void ___lambda13__external_applications_chooser_button_selected (ExternalApplicationsChooserButton* _sender, const gchar* commandline, gpointer self) {
#line 764 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda13_ (self, commandline);
#line 5690 "open-with.c"
}


static void __lambda14_ (Block7Data* _data7_, const gchar* content_type, GtkTreeIter* iter) {
	ExternalApplicationsManager* self;
	GAppInfo* app_info = NULL;
	const gchar* _tmp0_;
	KatzePreferences* _tmp1_;
	GAppInfo* _tmp2_;
	GAppInfo* _tmp3_;
	GError * _inner_error_ = NULL;
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = _data7_->self;
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (content_type != NULL);
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (iter != NULL);
#line 771 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = content_type;
#line 771 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _data7_->preferences;
#line 771 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = external_applications_manager_open_with (self, "", _tmp0_, (GtkWidget*) _tmp1_);
#line 771 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	app_info = _tmp2_;
#line 772 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = app_info;
#line 772 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (_tmp3_ == NULL) {
#line 773 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 773 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 5724 "open-with.c"
	}
	{
		GAppInfo* _tmp4_;
		const gchar* _tmp5_;
		GtkListStore* _tmp6_;
		GtkTreeIter _tmp7_;
		GAppInfo* _tmp8_;
#line 775 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp4_ = app_info;
#line 775 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp5_ = content_type;
#line 775 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_app_info_set_as_default_for_type (_tmp4_, _tmp5_, &_inner_error_);
#line 775 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 5740 "open-with.c"
			goto __catch5_g_error;
		}
#line 776 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp6_ = _data7_->types->store;
#line 776 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp7_ = *iter;
#line 776 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp8_ = app_info;
#line 776 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		gtk_list_store_set (_tmp6_, &_tmp7_, 1, _tmp8_, -1);
#line 5751 "open-with.c"
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError* _error_ = NULL;
		const gchar* _tmp9_;
		GError* _tmp10_;
		const gchar* _tmp11_;
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_error_ = _inner_error_;
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_inner_error_ = NULL;
#line 778 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp9_ = content_type;
#line 778 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp10_ = _error_;
#line 778 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_tmp11_ = _tmp10_->message;
#line 778 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_warning ("open-with.vala:778: Failed to select default for \"%s\": %s", _tmp9_, _tmp11_);
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_error_free0 (_error_);
#line 5774 "open-with.c"
	}
	__finally5:
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_object_unref0 (app_info);
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		g_clear_error (&_inner_error_);
#line 774 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		return;
#line 5787 "open-with.c"
	}
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app_info);
#line 5791 "open-with.c"
}


static void ___lambda14__external_applications_types_selected (ExternalApplicationsTypes* _sender, const gchar* content_type, GtkTreeIter* iter, gpointer self) {
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	__lambda14_ (self, content_type, iter);
#line 5798 "open-with.c"
}


static void external_applications_manager_show_preferences (ExternalApplicationsManager* self, KatzePreferences* preferences) {
	Block7Data* _data7_;
	KatzePreferences* _tmp0_;
	KatzePreferences* _tmp1_;
	MidoriApp* _tmp2_;
	MidoriWebSettings* _tmp3_;
	MidoriWebSettings* _tmp4_;
	GtkBox* category = NULL;
	KatzePreferences* _tmp5_;
	GtkBox* _tmp6_;
	GtkBox* _tmp7_;
	KatzePreferences* _tmp8_;
	GtkSizeGroup* sizegroup = NULL;
	GtkSizeGroup* _tmp9_;
	GtkLabel* label = NULL;
	GtkLabel* _tmp10_;
	GtkLabel* _tmp11_;
	GtkLabel* _tmp12_;
	KatzePreferences* _tmp13_;
	GtkLabel* _tmp14_;
	ExternalApplicationsChooserButton* entry = NULL;
	const gchar* _tmp15_;
	const gchar* _tmp16_;
	ExternalApplicationsChooserButton* _tmp17_;
	ExternalApplicationsChooserButton* _tmp18_;
	ExternalApplicationsChooserButton* _tmp19_;
	KatzePreferences* _tmp20_;
	ExternalApplicationsChooserButton* _tmp21_;
	GtkLabel* _tmp22_;
	GtkLabel* _tmp23_;
	GtkLabel* _tmp24_;
	KatzePreferences* _tmp25_;
	GtkLabel* _tmp26_;
	const gchar* _tmp27_;
	const gchar* _tmp28_;
	ExternalApplicationsChooserButton* _tmp29_;
	ExternalApplicationsChooserButton* _tmp30_;
	ExternalApplicationsChooserButton* _tmp31_;
	KatzePreferences* _tmp32_;
	ExternalApplicationsChooserButton* _tmp33_;
	ExternalApplicationsTypes* _tmp34_;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (preferences != NULL);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_ = g_slice_new0 (Block7Data);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_->_ref_count_ = 1;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_->self = g_object_ref (self);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = preferences;
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (_data7_->preferences);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_->preferences = _tmp1_;
#line 742 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = midori_extension_get_app ((MidoriExtension*) self);
#line 742 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_get (_tmp2_, "settings", &_tmp3_, NULL);
#line 742 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = _tmp3_;
#line 742 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_->settings = _tmp4_;
#line 743 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = _data7_->preferences;
#line 743 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = katze_preferences_add_category (_tmp5_, _ ("File Types"), GTK_STOCK_FILE);
#line 743 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp7_ = _g_object_ref0 (_tmp6_);
#line 743 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	category = _tmp7_;
#line 744 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = _data7_->preferences;
#line 744 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	katze_preferences_add_group (_tmp8_, NULL);
#line 746 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp9_ = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
#line 746 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	sizegroup = _tmp9_;
#line 747 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp10_ = (GtkLabel*) gtk_label_new (_ ("Text Editor"));
#line 747 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp10_);
#line 747 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	label = _tmp10_;
#line 748 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp11_ = label;
#line 748 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp11_);
#line 749 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp12_ = label;
#line 749 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_misc_set_alignment ((GtkMisc*) _tmp12_, 0.0f, 0.5f);
#line 750 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp13_ = _data7_->preferences;
#line 750 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp14_ = label;
#line 750 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	katze_preferences_add_widget (_tmp13_, (GtkWidget*) _tmp14_, "indented");
#line 751 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp15_ = midori_settings_get_text_editor ((MidoriSettings*) _data7_->settings);
#line 751 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp16_ = _tmp15_;
#line 751 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp17_ = external_applications_chooser_button_new ("text/plain", _tmp16_);
#line 751 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp17_);
#line 751 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	entry = _tmp17_;
#line 752 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp18_ = entry;
#line 752 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp18_);
#line 753 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp19_ = entry;
#line 753 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_data (_tmp19_, "selected", (GCallback) ___lambda12__external_applications_chooser_button_selected, block7_data_ref (_data7_), (GClosureNotify) block7_data_unref, 0);
#line 756 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp20_ = _data7_->preferences;
#line 756 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp21_ = entry;
#line 756 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	katze_preferences_add_widget (_tmp20_, (GtkWidget*) _tmp21_, "spanned");
#line 758 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp22_ = (GtkLabel*) gtk_label_new (_ ("News Aggregator"));
#line 758 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp22_);
#line 758 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (label);
#line 758 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	label = _tmp22_;
#line 759 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp23_ = label;
#line 759 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp23_);
#line 760 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp24_ = label;
#line 760 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_misc_set_alignment ((GtkMisc*) _tmp24_, 0.0f, 0.5f);
#line 761 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp25_ = _data7_->preferences;
#line 761 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp26_ = label;
#line 761 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	katze_preferences_add_widget (_tmp25_, (GtkWidget*) _tmp26_, "indented");
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp27_ = midori_settings_get_news_aggregator ((MidoriSettings*) _data7_->settings);
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp28_ = _tmp27_;
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp29_ = external_applications_chooser_button_new ("application/rss+xml", _tmp28_);
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp29_);
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (entry);
#line 762 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	entry = _tmp29_;
#line 763 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp30_ = entry;
#line 763 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_size_group_add_widget (sizegroup, (GtkWidget*) _tmp30_);
#line 764 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp31_ = entry;
#line 764 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_data (_tmp31_, "selected", (GCallback) ___lambda13__external_applications_chooser_button_selected, block7_data_ref (_data7_), (GClosureNotify) block7_data_unref, 0);
#line 767 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp32_ = _data7_->preferences;
#line 767 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp33_ = entry;
#line 767 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	katze_preferences_add_widget (_tmp32_, (GtkWidget*) _tmp33_, "spanned");
#line 769 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp34_ = external_applications_types_new ();
#line 769 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_object_ref_sink (_tmp34_);
#line 769 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_->types = _tmp34_;
#line 770 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_data (_data7_->types, "selected", (GCallback) ___lambda14__external_applications_types_selected, block7_data_ref (_data7_), (GClosureNotify) block7_data_unref, 0);
#line 781 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_box_pack_start (category, (GtkWidget*) _data7_->types, TRUE, TRUE, (guint) 0);
#line 782 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	gtk_widget_show_all ((GtkWidget*) _data7_->types);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (entry);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (label);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (sizegroup);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (category);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	block7_data_unref (_data7_);
#line 741 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_data7_ = NULL;
#line 6001 "open-with.c"
}


static gboolean _external_applications_manager_navigation_requested_midori_tab_navigation_requested (MidoriTab* _sender, const gchar* uri, gpointer self) {
	gboolean result;
	result = external_applications_manager_navigation_requested ((ExternalApplicationsManager*) self, _sender, uri);
#line 786 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 6010 "open-with.c"
}


static gboolean _external_applications_manager_open_uri_midori_tab_open_uri (MidoriTab* _sender, const gchar* uri, gpointer self) {
	gboolean result;
	result = external_applications_manager_open_uri ((ExternalApplicationsManager*) self, _sender, uri);
#line 787 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 6019 "open-with.c"
}


static void _external_applications_manager_context_menu_midori_tab_context_menu (MidoriTab* _sender, WebKitHitTestResult* hit_test_result, MidoriContextAction* menu, gpointer self) {
#line 788 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_context_menu ((ExternalApplicationsManager*) self, _sender, hit_test_result, menu);
#line 6026 "open-with.c"
}


void external_applications_manager_tab_added (ExternalApplicationsManager* self, MidoriBrowser* browser, MidoriView* view) {
	MidoriView* _tmp0_;
	MidoriView* _tmp1_;
	MidoriView* _tmp2_;
#line 785 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 785 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (browser != NULL);
#line 785 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (view != NULL);
#line 786 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = view;
#line 786 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((MidoriTab*) _tmp0_, "navigation-requested", (GCallback) _external_applications_manager_navigation_requested_midori_tab_navigation_requested, self, G_CONNECT_AFTER);
#line 787 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = view;
#line 787 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((MidoriTab*) _tmp1_, "open-uri", (GCallback) _external_applications_manager_open_uri_midori_tab_open_uri, self, 0);
#line 788 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = view;
#line 788 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((MidoriTab*) _tmp2_, "context-menu", (GCallback) _external_applications_manager_context_menu_midori_tab_context_menu, self, 0);
#line 6052 "open-with.c"
}


void external_applications_manager_tab_removed (ExternalApplicationsManager* self, MidoriBrowser* browser, MidoriView* view) {
	MidoriView* _tmp0_;
	guint _tmp1_;
	MidoriView* _tmp2_;
	guint _tmp3_;
	MidoriView* _tmp4_;
	guint _tmp5_;
#line 791 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 791 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (browser != NULL);
#line 791 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (view != NULL);
#line 792 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = view;
#line 792 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("navigation-requested", MIDORI_TYPE_TAB, &_tmp1_, NULL, FALSE);
#line 792 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched ((MidoriTab*) _tmp0_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp1_, 0, NULL, (GCallback) _external_applications_manager_navigation_requested_midori_tab_navigation_requested, self);
#line 793 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = view;
#line 793 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("open-uri", MIDORI_TYPE_TAB, &_tmp3_, NULL, FALSE);
#line 793 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched ((MidoriTab*) _tmp2_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp3_, 0, NULL, (GCallback) _external_applications_manager_open_uri_midori_tab_open_uri, self);
#line 794 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = view;
#line 794 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("context-menu", MIDORI_TYPE_TAB, &_tmp5_, NULL, FALSE);
#line 794 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched ((MidoriTab*) _tmp4_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp5_, 0, NULL, (GCallback) _external_applications_manager_context_menu_midori_tab_context_menu, self);
#line 6087 "open-with.c"
}


static void _external_applications_manager_tab_added_midori_browser_add_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self) {
#line 800 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_tab_added ((ExternalApplicationsManager*) self, _sender, tab);
#line 6094 "open-with.c"
}


static void _external_applications_manager_tab_removed_midori_browser_remove_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self) {
#line 801 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_tab_removed ((ExternalApplicationsManager*) self, _sender, tab);
#line 6101 "open-with.c"
}


static void _external_applications_manager_show_preferences_midori_browser_show_preferences (MidoriBrowser* _sender, KatzePreferences* preferences, gpointer self) {
#line 802 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_show_preferences ((ExternalApplicationsManager*) self, preferences);
#line 6108 "open-with.c"
}


static void external_applications_manager_browser_added (ExternalApplicationsManager* self, MidoriBrowser* browser) {
	MidoriBrowser* _tmp0_;
	GList* _tmp1_;
	MidoriBrowser* _tmp4_;
	MidoriBrowser* _tmp5_;
	MidoriBrowser* _tmp6_;
#line 797 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 797 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (browser != NULL);
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = browser;
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = midori_browser_get_tabs (_tmp0_);
#line 6126 "open-with.c"
	{
		GList* tab_collection = NULL;
		GList* tab_it = NULL;
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		tab_collection = _tmp1_;
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (tab_it = tab_collection; tab_it != NULL; tab_it = tab_it->next) {
#line 6134 "open-with.c"
			MidoriView* tab = NULL;
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			tab = (MidoriView*) tab_it->data;
#line 6138 "open-with.c"
			{
				MidoriBrowser* _tmp2_;
				MidoriView* _tmp3_;
#line 799 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp2_ = browser;
#line 799 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp3_ = tab;
#line 799 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_manager_tab_added (self, _tmp2_, _tmp3_);
#line 6148 "open-with.c"
			}
		}
#line 798 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_list_free0 (tab_collection);
#line 6153 "open-with.c"
	}
#line 800 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = browser;
#line 800 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp4_, "add-tab", (GCallback) _external_applications_manager_tab_added_midori_browser_add_tab, self, 0);
#line 801 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = browser;
#line 801 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp5_, "remove-tab", (GCallback) _external_applications_manager_tab_removed_midori_browser_remove_tab, self, 0);
#line 802 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = browser;
#line 802 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp6_, "show-preferences", (GCallback) _external_applications_manager_show_preferences_midori_browser_show_preferences, self, 0);
#line 6167 "open-with.c"
}


static void _external_applications_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self) {
#line 808 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_browser_added ((ExternalApplicationsManager*) self, browser);
#line 6174 "open-with.c"
}


static void external_applications_manager_activated (ExternalApplicationsManager* self, MidoriApp* app) {
	MidoriApp* _tmp0_;
	GList* _tmp1_;
	MidoriApp* _tmp3_;
#line 805 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 805 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (app != NULL);
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = app;
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = midori_app_get_browsers (_tmp0_);
#line 6190 "open-with.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		browser_collection = _tmp1_;
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 6198 "open-with.c"
			MidoriBrowser* browser = NULL;
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 6202 "open-with.c"
			{
				MidoriBrowser* _tmp2_;
#line 807 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp2_ = browser;
#line 807 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_manager_browser_added (self, _tmp2_);
#line 6209 "open-with.c"
			}
		}
#line 806 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_list_free0 (browser_collection);
#line 6214 "open-with.c"
	}
#line 808 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = app;
#line 808 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object (_tmp3_, "add-browser", (GCallback) _external_applications_manager_browser_added_midori_app_add_browser, self, 0);
#line 6220 "open-with.c"
}


static void external_applications_manager_browser_removed (ExternalApplicationsManager* self, MidoriBrowser* browser) {
	MidoriBrowser* _tmp0_;
	GList* _tmp1_;
	MidoriBrowser* _tmp4_;
	guint _tmp5_;
	MidoriBrowser* _tmp6_;
	guint _tmp7_;
	MidoriBrowser* _tmp8_;
	guint _tmp9_;
#line 811 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 811 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (browser != NULL);
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = browser;
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = midori_browser_get_tabs (_tmp0_);
#line 6241 "open-with.c"
	{
		GList* tab_collection = NULL;
		GList* tab_it = NULL;
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		tab_collection = _tmp1_;
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (tab_it = tab_collection; tab_it != NULL; tab_it = tab_it->next) {
#line 6249 "open-with.c"
			MidoriView* tab = NULL;
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			tab = (MidoriView*) tab_it->data;
#line 6253 "open-with.c"
			{
				MidoriBrowser* _tmp2_;
				MidoriView* _tmp3_;
#line 813 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp2_ = browser;
#line 813 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp3_ = tab;
#line 813 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_manager_tab_removed (self, _tmp2_, _tmp3_);
#line 6263 "open-with.c"
			}
		}
#line 812 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_list_free0 (tab_collection);
#line 6268 "open-with.c"
	}
#line 814 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp4_ = browser;
#line 814 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("add-tab", MIDORI_TYPE_BROWSER, &_tmp5_, NULL, FALSE);
#line 814 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched (_tmp4_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp5_, 0, NULL, (GCallback) _external_applications_manager_tab_added_midori_browser_add_tab, self);
#line 815 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp6_ = browser;
#line 815 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("remove-tab", MIDORI_TYPE_BROWSER, &_tmp7_, NULL, FALSE);
#line 815 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched (_tmp6_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp7_, 0, NULL, (GCallback) _external_applications_manager_tab_removed_midori_browser_remove_tab, self);
#line 816 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp8_ = browser;
#line 816 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("show-preferences", MIDORI_TYPE_BROWSER, &_tmp9_, NULL, FALSE);
#line 816 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched (_tmp8_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp9_, 0, NULL, (GCallback) _external_applications_manager_show_preferences_midori_browser_show_preferences, self);
#line 6288 "open-with.c"
}


static void external_applications_manager_deactivated (ExternalApplicationsManager* self) {
	MidoriApp* app = NULL;
	MidoriApp* _tmp0_;
	MidoriApp* _tmp1_;
	MidoriApp* _tmp2_;
	GList* _tmp3_;
	MidoriApp* _tmp5_;
	guint _tmp6_;
#line 819 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_return_if_fail (self != NULL);
#line 820 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = midori_extension_get_app ((MidoriExtension*) self);
#line 820 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 820 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	app = _tmp1_;
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp2_ = app;
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp3_ = midori_app_get_browsers (_tmp2_);
#line 6312 "open-with.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		browser_collection = _tmp3_;
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 6320 "open-with.c"
			MidoriBrowser* browser = NULL;
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 6324 "open-with.c"
			{
				MidoriBrowser* _tmp4_;
#line 822 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				_tmp4_ = browser;
#line 822 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
				external_applications_manager_browser_removed (self, _tmp4_);
#line 6331 "open-with.c"
			}
		}
#line 821 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
		_g_list_free0 (browser_collection);
#line 6336 "open-with.c"
	}
#line 823 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp5_ = app;
#line 823 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_parse_name ("add-browser", MIDORI_TYPE_APP, &_tmp6_, NULL, FALSE);
#line 823 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_handlers_disconnect_matched (_tmp5_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp6_, 0, NULL, (GCallback) _external_applications_manager_browser_added_midori_app_add_browser, self);
#line 819 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_g_object_unref0 (app);
#line 6346 "open-with.c"
}


static void _external_applications_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self) {
#line 833 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_activated ((ExternalApplicationsManager*) self, app);
#line 6353 "open-with.c"
}


static void _external_applications_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self) {
#line 834 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_deactivated ((ExternalApplicationsManager*) self);
#line 6360 "open-with.c"
}


ExternalApplicationsManager* external_applications_manager_construct (GType object_type) {
	ExternalApplicationsManager * self = NULL;
#line 828 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	self = (ExternalApplicationsManager*) g_object_new (object_type, "name", "External Applications", "description", "Choose what to open unknown file types with", "version", "0.1" MIDORI_VERSION_SUFFIX, "authors", "Christian Dywan <christian@twotoasts.de>", NULL);
#line 833 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((MidoriExtension*) self, "activate", (GCallback) _external_applications_manager_activated_midori_extension_activate, self, 0);
#line 834 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	g_signal_connect_object ((MidoriExtension*) self, "deactivate", (GCallback) _external_applications_manager_deactivated_midori_extension_deactivate, self, 0);
#line 827 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return self;
#line 6374 "open-with.c"
}


ExternalApplicationsManager* external_applications_manager_new (void) {
#line 827 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return external_applications_manager_construct (EXTERNAL_APPLICATIONS_TYPE_MANAGER);
#line 6381 "open-with.c"
}


static void external_applications_manager_class_init (ExternalApplicationsManagerClass * klass) {
#line 618 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	external_applications_manager_parent_class = g_type_class_peek_parent (klass);
#line 6388 "open-with.c"
}


static void external_applications_manager_instance_init (ExternalApplicationsManager * self) {
}


GType external_applications_manager_get_type (void) {
	static volatile gsize external_applications_manager_type_id__volatile = 0;
	if (g_once_init_enter (&external_applications_manager_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ExternalApplicationsManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) external_applications_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ExternalApplicationsManager), 0, (GInstanceInitFunc) external_applications_manager_instance_init, NULL };
		GType external_applications_manager_type_id;
		external_applications_manager_type_id = g_type_register_static (MIDORI_TYPE_EXTENSION, "ExternalApplicationsManager", &g_define_type_info, 0);
		g_once_init_leave (&external_applications_manager_type_id__volatile, external_applications_manager_type_id);
	}
	return external_applications_manager_type_id__volatile;
}


MidoriExtension* extension_init (void) {
	MidoriExtension* result = NULL;
	ExternalApplicationsManager* _tmp0_;
#line 840 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	_tmp0_ = external_applications_manager_new ();
#line 840 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	result = (MidoriExtension*) _tmp0_;
#line 840 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/open-with.vala"
	return result;
#line 6417 "open-with.c"
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



