/* history-list.c generated by valac 0.38.8, the Vala compiler
 * generated from history-list.vala, do not modify */

/*
   Copyright (C) 2010-2013 André Stösel <andre@stoesel.de>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include "midori/midori-browser.h"
#include "midori/midori-view.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdlib.h>
#include <string.h>
#include "midori-core.h"
#include <gdk/gdk.h>
#include <glib/gi18n-lib.h>
#include "midori/midori-extension.h"
#include "midori/midori-app.h"
#include "midori/sokoke.h"
#include "history-list.h"


#define HISTORY_LIST_TYPE_TAB_TREE_CELLS (history_list_tab_tree_cells_get_type ())

#define HISTORY_LIST_TYPE_TAB_CLOSING_BEHAVIOR (history_list_tab_closing_behavior_get_type ())

#define HISTORY_LIST_TYPE_TAB_CLOSING_BEHAVIOR_MODEL (history_list_tab_closing_behavior_model_get_type ())

#define HISTORY_LIST_TYPE_HISTORY_WINDOW (history_list_history_window_get_type ())
#define HISTORY_LIST_HISTORY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindow))
#define HISTORY_LIST_HISTORY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindowClass))
#define HISTORY_LIST_IS_HISTORY_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HISTORY_LIST_TYPE_HISTORY_WINDOW))
#define HISTORY_LIST_IS_HISTORY_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HISTORY_LIST_TYPE_HISTORY_WINDOW))
#define HISTORY_LIST_HISTORY_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindowClass))

typedef struct _HistoryListHistoryWindow HistoryListHistoryWindow;
typedef struct _HistoryListHistoryWindowClass HistoryListHistoryWindowClass;
typedef struct _HistoryListHistoryWindowPrivate HistoryListHistoryWindowPrivate;
enum  {
	HISTORY_LIST_HISTORY_WINDOW_0_PROPERTY,
	HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY,
	HISTORY_LIST_HISTORY_WINDOW_NUM_PROPERTIES
};
static GParamSpec* history_list_history_window_properties[HISTORY_LIST_HISTORY_WINDOW_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))

#define HISTORY_LIST_TYPE_TAB_WINDOW (history_list_tab_window_get_type ())
#define HISTORY_LIST_TAB_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindow))
#define HISTORY_LIST_TAB_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindowClass))
#define HISTORY_LIST_IS_TAB_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HISTORY_LIST_TYPE_TAB_WINDOW))
#define HISTORY_LIST_IS_TAB_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HISTORY_LIST_TYPE_TAB_WINDOW))
#define HISTORY_LIST_TAB_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindowClass))

typedef struct _HistoryListTabWindow HistoryListTabWindow;
typedef struct _HistoryListTabWindowClass HistoryListTabWindowClass;
typedef struct _HistoryListTabWindowPrivate HistoryListTabWindowPrivate;
enum  {
	HISTORY_LIST_TAB_WINDOW_0_PROPERTY,
	HISTORY_LIST_TAB_WINDOW_NUM_PROPERTIES
};
static GParamSpec* history_list_tab_window_properties[HISTORY_LIST_TAB_WINDOW_NUM_PROPERTIES];

#define HISTORY_LIST_TYPE_NEW_TAB_WINDOW (history_list_new_tab_window_get_type ())
#define HISTORY_LIST_NEW_TAB_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HISTORY_LIST_TYPE_NEW_TAB_WINDOW, HistoryListNewTabWindow))
#define HISTORY_LIST_NEW_TAB_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HISTORY_LIST_TYPE_NEW_TAB_WINDOW, HistoryListNewTabWindowClass))
#define HISTORY_LIST_IS_NEW_TAB_WINDOW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HISTORY_LIST_TYPE_NEW_TAB_WINDOW))
#define HISTORY_LIST_IS_NEW_TAB_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HISTORY_LIST_TYPE_NEW_TAB_WINDOW))
#define HISTORY_LIST_NEW_TAB_WINDOW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HISTORY_LIST_TYPE_NEW_TAB_WINDOW, HistoryListNewTabWindowClass))

typedef struct _HistoryListNewTabWindow HistoryListNewTabWindow;
typedef struct _HistoryListNewTabWindowClass HistoryListNewTabWindowClass;
typedef struct _HistoryListNewTabWindowPrivate HistoryListNewTabWindowPrivate;
enum  {
	HISTORY_LIST_NEW_TAB_WINDOW_0_PROPERTY,
	HISTORY_LIST_NEW_TAB_WINDOW_NUM_PROPERTIES
};
static GParamSpec* history_list_new_tab_window_properties[HISTORY_LIST_NEW_TAB_WINDOW_NUM_PROPERTIES];

#define HISTORY_LIST_TYPE_PREFERENCES_DIALOG (history_list_preferences_dialog_get_type ())
#define HISTORY_LIST_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HISTORY_LIST_TYPE_PREFERENCES_DIALOG, HistoryListPreferencesDialog))
#define HISTORY_LIST_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HISTORY_LIST_TYPE_PREFERENCES_DIALOG, HistoryListPreferencesDialogClass))
#define HISTORY_LIST_IS_PREFERENCES_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HISTORY_LIST_TYPE_PREFERENCES_DIALOG))
#define HISTORY_LIST_IS_PREFERENCES_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HISTORY_LIST_TYPE_PREFERENCES_DIALOG))
#define HISTORY_LIST_PREFERENCES_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HISTORY_LIST_TYPE_PREFERENCES_DIALOG, HistoryListPreferencesDialogClass))

typedef struct _HistoryListPreferencesDialog HistoryListPreferencesDialog;
typedef struct _HistoryListPreferencesDialogClass HistoryListPreferencesDialogClass;
typedef struct _HistoryListPreferencesDialogPrivate HistoryListPreferencesDialogPrivate;

#define HISTORY_LIST_TYPE_MANAGER (history_list_manager_get_type ())
#define HISTORY_LIST_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HISTORY_LIST_TYPE_MANAGER, HistoryListManager))
#define HISTORY_LIST_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), HISTORY_LIST_TYPE_MANAGER, HistoryListManagerClass))
#define HISTORY_LIST_IS_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HISTORY_LIST_TYPE_MANAGER))
#define HISTORY_LIST_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HISTORY_LIST_TYPE_MANAGER))
#define HISTORY_LIST_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), HISTORY_LIST_TYPE_MANAGER, HistoryListManagerClass))

typedef struct _HistoryListManager HistoryListManager;
typedef struct _HistoryListManagerClass HistoryListManagerClass;
enum  {
	HISTORY_LIST_PREFERENCES_DIALOG_0_PROPERTY,
	HISTORY_LIST_PREFERENCES_DIALOG_NUM_PROPERTIES
};
static GParamSpec* history_list_preferences_dialog_properties[HISTORY_LIST_PREFERENCES_DIALOG_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _HistoryListManagerPrivate HistoryListManagerPrivate;
enum  {
	HISTORY_LIST_MANAGER_0_PROPERTY,
	HISTORY_LIST_MANAGER_NUM_PROPERTIES
};
static GParamSpec* history_list_manager_properties[HISTORY_LIST_MANAGER_NUM_PROPERTIES];
typedef struct _Block1Data Block1Data;
typedef struct _Block2Data Block2Data;
#define _g_list_free0(var) ((var == NULL) ? NULL : (var = (g_list_free (var), NULL)))
enum  {
	HISTORY_LIST_MANAGER_PREFERENCES_CHANGED_SIGNAL,
	HISTORY_LIST_MANAGER_NUM_SIGNALS
};
static guint history_list_manager_signals[HISTORY_LIST_MANAGER_NUM_SIGNALS] = {0};

typedef enum  {
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_PIXBUF,
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_STRING,
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_FG,
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG,
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER,
	HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_COUNT
} HistoryListTabTreeCells;

typedef enum  {
	HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NONE,
	HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST,
	HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW
} HistoryListTabClosingBehavior;

typedef enum  {
	HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT,
	HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE
} HistoryListTabClosingBehaviorModel;

struct _HistoryListHistoryWindow {
	GtkWindow parent_instance;
	HistoryListHistoryWindowPrivate * priv;
	GtkTreeView* treeview;
};

struct _HistoryListHistoryWindowClass {
	GtkWindowClass parent_class;
	void (*walk) (HistoryListHistoryWindow* self, gint step);
	void (*make_update) (HistoryListHistoryWindow* self);
	void (*clean_up) (HistoryListHistoryWindow* self);
	void (*close_tab) (HistoryListHistoryWindow* self);
};

struct _HistoryListHistoryWindowPrivate {
	MidoriBrowser* _browser;
};

struct _HistoryListTabWindow {
	HistoryListHistoryWindow parent_instance;
	HistoryListTabWindowPrivate * priv;
	GtkHBox* hbox;
	GtkVBox* vbox;
	gboolean is_dirty;
	GtkScrolledWindow* scroll_windows;
};

struct _HistoryListTabWindowClass {
	HistoryListHistoryWindowClass parent_class;
	void (*insert_rows) (HistoryListTabWindow* self, GtkListStore* store);
};

struct _HistoryListNewTabWindow {
	HistoryListTabWindow parent_instance;
	HistoryListNewTabWindowPrivate * priv;
	gboolean old_tabs;
	gboolean first_step;
};

struct _HistoryListNewTabWindowClass {
	HistoryListTabWindowClass parent_class;
};

struct _HistoryListPreferencesDialog {
	GtkDialog parent_instance;
	HistoryListPreferencesDialogPrivate * priv;
	HistoryListManager* hl_manager;
	GtkComboBox* closing_behavior;
};

struct _HistoryListPreferencesDialogClass {
	GtkDialogClass parent_class;
};

struct _HistoryListManager {
	MidoriExtension parent_instance;
	HistoryListManagerPrivate * priv;
	guint escKeyval;
	guint delKeyval;
	guint modifier_count;
	gint closing_behavior;
	HistoryListHistoryWindow* history_window;
	gulong* tmp_sig_ids;
	gint tmp_sig_ids_length1;
	gint _tmp_sig_ids_size_;
	gboolean ignoreNextChange;
};

struct _HistoryListManagerClass {
	MidoriExtensionClass parent_class;
};

struct _Block1Data {
	int _ref_count_;
	HistoryListManager* self;
	MidoriBrowser* browser;
};

struct _Block2Data {
	int _ref_count_;
	HistoryListManager* self;
	MidoriBrowser* browser;
};


static gpointer history_list_history_window_parent_class = NULL;
static gpointer history_list_tab_window_parent_class = NULL;
static gpointer history_list_new_tab_window_parent_class = NULL;
static gpointer history_list_preferences_dialog_parent_class = NULL;
static gpointer history_list_manager_parent_class = NULL;

GType history_list_tab_tree_cells_get_type (void) G_GNUC_CONST;
GType history_list_tab_closing_behavior_get_type (void) G_GNUC_CONST;
GType history_list_tab_closing_behavior_model_get_type (void) G_GNUC_CONST;
GType history_list_history_window_get_type (void) G_GNUC_CONST;
#define HISTORY_LIST_HISTORY_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindowPrivate))
HistoryListHistoryWindow* history_list_history_window_construct (GType object_type, MidoriBrowser* browser);
void history_list_history_window_walk (HistoryListHistoryWindow* self, gint step);
static void history_list_history_window_real_walk (HistoryListHistoryWindow* self, gint step);
void history_list_history_window_make_update (HistoryListHistoryWindow* self);
static void history_list_history_window_real_make_update (HistoryListHistoryWindow* self);
void history_list_history_window_clean_up (HistoryListHistoryWindow* self);
static void history_list_history_window_real_clean_up (HistoryListHistoryWindow* self);
void history_list_history_window_close_tab (HistoryListHistoryWindow* self);
static void history_list_history_window_real_close_tab (HistoryListHistoryWindow* self);
MidoriBrowser* history_list_history_window_get_browser (HistoryListHistoryWindow* self);
void history_list_history_window_set_browser (HistoryListHistoryWindow* self, MidoriBrowser* value);
static void history_list_history_window_finalize (GObject * obj);
static void _vala_history_list_history_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_history_list_history_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
GType history_list_tab_window_get_type (void) G_GNUC_CONST;
void history_list_tab_window_store_append_row (HistoryListTabWindow* self, GPtrArray* list, GtkListStore* store);
void history_list_tab_window_insert_rows (HistoryListTabWindow* self, GtkListStore* store);
static void history_list_tab_window_real_insert_rows (HistoryListTabWindow* self, GtkListStore* store);
void history_list_tab_window_resize_treeview (HistoryListTabWindow* self);
HistoryListTabWindow* history_list_tab_window_new (MidoriBrowser* browser);
HistoryListTabWindow* history_list_tab_window_construct (GType object_type, MidoriBrowser* browser);
static void history_list_tab_window_real_make_update (HistoryListHistoryWindow* base);
static void history_list_tab_window_real_clean_up (HistoryListHistoryWindow* base);
static void history_list_tab_window_real_close_tab (HistoryListHistoryWindow* base);
static void history_list_tab_window_finalize (GObject * obj);
GType history_list_new_tab_window_get_type (void) G_GNUC_CONST;
static void history_list_new_tab_window_real_insert_rows (HistoryListTabWindow* base, GtkListStore* store);
static void history_list_new_tab_window_real_walk (HistoryListHistoryWindow* base, gint step);
static void history_list_new_tab_window_real_clean_up (HistoryListHistoryWindow* base);
HistoryListNewTabWindow* history_list_new_tab_window_new (MidoriBrowser* browser);
HistoryListNewTabWindow* history_list_new_tab_window_construct (GType object_type, MidoriBrowser* browser);
static void history_list_new_tab_window_finalize (GObject * obj);
GType history_list_preferences_dialog_get_type (void) G_GNUC_CONST;
GType history_list_manager_get_type (void) G_GNUC_CONST;
HistoryListPreferencesDialog* history_list_preferences_dialog_new (HistoryListManager* manager);
HistoryListPreferencesDialog* history_list_preferences_dialog_construct (GType object_type, HistoryListManager* manager);
static void history_list_preferences_dialog_create_widgets (HistoryListPreferencesDialog* self);
static void history_list_preferences_dialog_response_cb (HistoryListPreferencesDialog* self, GtkDialog* source, gint response_id);
static void _history_list_preferences_dialog_response_cb_gtk_dialog_response (GtkDialog* _sender, gint response_id, gpointer self);
static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self);
static void history_list_preferences_dialog_finalize (GObject * obj);
void history_list_manager_preferences_changed_cb (HistoryListManager* self);
gboolean history_list_manager_is_key_a_modifier (HistoryListManager* self, GdkEventKey* event_key);
gboolean history_list_manager_key_press (HistoryListManager* self, GdkEventKey* event_key);
gboolean history_list_manager_key_release (HistoryListManager* self, GdkEventKey* event_key, MidoriBrowser* browser);
void history_list_manager_walk (HistoryListManager* self, GtkAction* action, MidoriBrowser* browser, GType type, gint step);
static Block1Data* block1_data_ref (Block1Data* _data1_);
static void block1_data_unref (void * _userdata_);
static void history_list_manager_tab_list_resort (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view);
static gboolean ____lambda4_ (HistoryListManager* self, GdkEventKey* ek);
static gboolean _____lambda4__gtk_widget_key_press_event (GtkWidget* _sender, GdkEventKey* event, gpointer self);
static gboolean ____lambda5_ (Block1Data* _data1_, GdkEventKey* ek);
static gboolean _____lambda5__gtk_widget_key_release_event (GtkWidget* _sender, GdkEventKey* event, gpointer self);
void history_list_manager_special_function (HistoryListManager* self, GtkAction* action, MidoriBrowser* browser);
static void history_list_manager_browser_added (HistoryListManager* self, MidoriBrowser* browser);
static Block2Data* block2_data_ref (Block2Data* _data2_);
static void block2_data_unref (void * _userdata_);
static void __lambda6_ (Block2Data* _data2_, GtkAction* a);
static void ___lambda6__gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda7_ (Block2Data* _data2_, GtkAction* a);
static void ___lambda7__gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda8_ (Block2Data* _data2_, GtkAction* a);
static void ___lambda8__gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda9_ (Block2Data* _data2_, GtkAction* a);
static void ___lambda9__gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda10_ (Block2Data* _data2_, GtkAction* a);
static void ___lambda10__gtk_action_activate (GtkAction* _sender, gpointer self);
static void history_list_manager_tab_added (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view);
static void _history_list_manager_tab_added_midori_browser_add_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self);
static void history_list_manager_tab_removed (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view);
static void _history_list_manager_tab_removed_midori_browser_remove_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self);
static void history_list_manager_tab_changed (HistoryListManager* self, MidoriView* old_view, MidoriView* new_view);
static void _history_list_manager_tab_changed_midori_browser_switch_tab (MidoriBrowser* _sender, MidoriView* old_view, MidoriView* new_view, gpointer self);
static void history_list_manager_browser_removed (HistoryListManager* self, MidoriBrowser* browser);
static void history_list_manager_activated (HistoryListManager* self, MidoriApp* app);
static void _history_list_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self);
static void _history_list_manager_browser_removed_midori_app_remove_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self);
static void history_list_manager_deactivated (HistoryListManager* self);
static void history_list_manager_show_preferences (HistoryListManager* self);
HistoryListManager* history_list_manager_new (void);
HistoryListManager* history_list_manager_construct (GType object_type);
static void _history_list_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self);
static void _history_list_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self);
static void _history_list_manager_show_preferences_midori_extension_open_preferences (MidoriExtension* _sender, gpointer self);
static void _history_list_manager_preferences_changed_cb_history_list_manager_preferences_changed (HistoryListManager* _sender, gpointer self);
static GObject * history_list_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void history_list_manager_finalize (GObject * obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


GType history_list_tab_tree_cells_get_type (void) {
	static volatile gsize history_list_tab_tree_cells_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_tab_tree_cells_type_id__volatile)) {
		static const GEnumValue values[] = {{HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_PIXBUF, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_PIXBUF", "tree-cell-pixbuf"}, {HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_STRING, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_STRING", "tree-cell-string"}, {HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_FG, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_FG", "tree-cell-fg"}, {HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG", "tree-cell-bg"}, {HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER", "tree-cell-pointer"}, {HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_COUNT, "HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_COUNT", "tree-cell-count"}, {0, NULL, NULL}};
		GType history_list_tab_tree_cells_type_id;
		history_list_tab_tree_cells_type_id = g_enum_register_static ("HistoryListTabTreeCells", values);
		g_once_init_leave (&history_list_tab_tree_cells_type_id__volatile, history_list_tab_tree_cells_type_id);
	}
	return history_list_tab_tree_cells_type_id__volatile;
}


GType history_list_tab_closing_behavior_get_type (void) {
	static volatile gsize history_list_tab_closing_behavior_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_tab_closing_behavior_type_id__volatile)) {
		static const GEnumValue values[] = {{HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NONE, "HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NONE", "none"}, {HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST, "HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST", "last"}, {HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW, "HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW", "new"}, {0, NULL, NULL}};
		GType history_list_tab_closing_behavior_type_id;
		history_list_tab_closing_behavior_type_id = g_enum_register_static ("HistoryListTabClosingBehavior", values);
		g_once_init_leave (&history_list_tab_closing_behavior_type_id__volatile, history_list_tab_closing_behavior_type_id);
	}
	return history_list_tab_closing_behavior_type_id__volatile;
}


GType history_list_tab_closing_behavior_model_get_type (void) {
	static volatile gsize history_list_tab_closing_behavior_model_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_tab_closing_behavior_model_type_id__volatile)) {
		static const GEnumValue values[] = {{HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT, "HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT", "text"}, {HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE, "HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE", "value"}, {0, NULL, NULL}};
		GType history_list_tab_closing_behavior_model_type_id;
		history_list_tab_closing_behavior_model_type_id = g_enum_register_static ("HistoryListTabClosingBehaviorModel", values);
		g_once_init_leave (&history_list_tab_closing_behavior_model_type_id__volatile, history_list_tab_closing_behavior_model_type_id);
	}
	return history_list_tab_closing_behavior_model_type_id__volatile;
}


HistoryListHistoryWindow* history_list_history_window_construct (GType object_type, MidoriBrowser* browser) {
	HistoryListHistoryWindow * self = NULL;
	MidoriBrowser* _tmp0_;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (browser != NULL, NULL);
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 38 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListHistoryWindow*) g_object_new (object_type, "type", GTK_WINDOW_POPUP, "window-position", GTK_WIN_POS_CENTER, "browser", _tmp0_, NULL);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self;
#line 385 "history-list.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self ? g_object_ref (self) : NULL;
#line 392 "history-list.c"
}


static void history_list_history_window_real_walk (HistoryListHistoryWindow* self, gint step) {
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkTreeView* _tmp0_;
	GtkTreePath* _tmp1_ = NULL;
	GtkTreeViewColumn* _tmp2_ = NULL;
	GtkTreeViewColumn* _tmp3_;
	GtkTreePath* _tmp4_;
	gint* indices = NULL;
	GtkTreePath* _tmp5_;
	gint* _tmp6_;
	gint indices_length1;
	gint _indices_size_;
	gint new_index = 0;
	gint* _tmp7_;
	gint _tmp7__length1;
	gint _tmp8_;
	gint _tmp9_;
	GtkListStore* model = NULL;
	GtkTreeView* _tmp10_;
	GtkTreeModel* _tmp11_;
	GtkListStore* _tmp12_;
	gint length = 0;
	GtkListStore* _tmp13_;
	gint _tmp14_;
	gint _tmp25_;
	GtkTreePath* _tmp26_;
	GtkTreeView* _tmp27_;
	GtkTreePath* _tmp28_;
	GtkTreeViewColumn* _tmp29_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = self->treeview;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_get_cursor (_tmp0_, &_tmp1_, &_tmp2_);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	path = _tmp1_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	column = _tmp3_;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = path;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp4_ == NULL) {
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (column);
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		return;
#line 450 "history-list.c"
	}
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = path;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = gtk_tree_path_get_indices (_tmp5_);
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	indices = _tmp6_;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	indices_length1 = -1;
#line 51 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_indices_size_ = indices_length1;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = indices;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7__length1 = indices_length1;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = _tmp7_[0];
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = step;
#line 52 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	new_index = _tmp8_ + _tmp9_;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = self->treeview;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = gtk_tree_view_get_model (_tmp10_);
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp11_, GTK_TYPE_LIST_STORE) ? ((GtkListStore*) _tmp11_) : NULL);
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	model = _tmp12_;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = model;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp14_ = gtk_tree_model_iter_n_children ((GtkTreeModel*) _tmp13_, NULL);
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	length = _tmp14_;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	while (TRUE) {
#line 488 "history-list.c"
		gboolean _tmp15_ = FALSE;
		gint _tmp16_;
		gint _tmp19_ = 0;
		gint _tmp20_;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp16_ = new_index;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp16_ < 0) {
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp15_ = TRUE;
#line 499 "history-list.c"
		} else {
			gint _tmp17_;
			gint _tmp18_;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp17_ = new_index;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp18_ = length;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp15_ = _tmp17_ >= _tmp18_;
#line 509 "history-list.c"
		}
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (!_tmp15_) {
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			break;
#line 515 "history-list.c"
		}
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp20_ = new_index;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp20_ < 0) {
#line 521 "history-list.c"
			gint _tmp21_;
			gint _tmp22_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21_ = length;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp22_ = new_index;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp19_ = _tmp21_ + _tmp22_;
#line 530 "history-list.c"
		} else {
			gint _tmp23_;
			gint _tmp24_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp23_ = new_index;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp24_ = length;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp19_ = _tmp23_ - _tmp24_;
#line 540 "history-list.c"
		}
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		new_index = _tmp19_;
#line 544 "history-list.c"
	}
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp25_ = new_index;
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp26_ = gtk_tree_path_new_from_indices (_tmp25_, -1);
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 60 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	path = _tmp26_;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp27_ = self->treeview;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp28_ = path;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp29_ = column;
#line 61 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_set_cursor (_tmp27_, _tmp28_, _tmp29_, FALSE);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 568 "history-list.c"
}


void history_list_history_window_walk (HistoryListHistoryWindow* self, gint step) {
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 43 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	HISTORY_LIST_HISTORY_WINDOW_GET_CLASS (self)->walk (self, step);
#line 577 "history-list.c"
}


static void history_list_history_window_real_make_update (HistoryListHistoryWindow* self) {
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_critical ("Type `%s' does not implement abstract method `history_list_history_window_make_update'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return;
#line 586 "history-list.c"
}


void history_list_history_window_make_update (HistoryListHistoryWindow* self) {
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	HISTORY_LIST_HISTORY_WINDOW_GET_CLASS (self)->make_update (self);
#line 595 "history-list.c"
}


static void history_list_history_window_real_clean_up (HistoryListHistoryWindow* self) {
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_critical ("Type `%s' does not implement abstract method `history_list_history_window_clean_up'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return;
#line 604 "history-list.c"
}


void history_list_history_window_clean_up (HistoryListHistoryWindow* self) {
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 65 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	HISTORY_LIST_HISTORY_WINDOW_GET_CLASS (self)->clean_up (self);
#line 613 "history-list.c"
}


static void history_list_history_window_real_close_tab (HistoryListHistoryWindow* self) {
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_critical ("Type `%s' does not implement abstract method `history_list_history_window_close_tab'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return;
#line 622 "history-list.c"
}


void history_list_history_window_close_tab (HistoryListHistoryWindow* self) {
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 66 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	HISTORY_LIST_HISTORY_WINDOW_GET_CLASS (self)->close_tab (self);
#line 631 "history-list.c"
}


MidoriBrowser* history_list_history_window_get_browser (HistoryListHistoryWindow* self) {
	MidoriBrowser* result;
	MidoriBrowser* _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = self->priv->_browser;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = _tmp0_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 646 "history-list.c"
}


void history_list_history_window_set_browser (HistoryListHistoryWindow* self, MidoriBrowser* value) {
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (history_list_history_window_get_browser (self) != value) {
#line 655 "history-list.c"
		MidoriBrowser* _tmp0_;
		MidoriBrowser* _tmp1_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp0_ = value;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = _g_object_ref0 (_tmp0_);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (self->priv->_browser);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->priv->_browser = _tmp1_;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_notify_by_pspec ((GObject *) self, history_list_history_window_properties[HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY]);
#line 668 "history-list.c"
	}
}


static void history_list_history_window_class_init (HistoryListHistoryWindowClass * klass) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_history_window_parent_class = g_type_class_peek_parent (klass);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_type_class_add_private (klass, sizeof (HistoryListHistoryWindowPrivate));
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->walk = (void (*) (HistoryListHistoryWindow *, gint)) history_list_history_window_real_walk;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->make_update = (void (*) (HistoryListHistoryWindow *)) history_list_history_window_real_make_update;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->clean_up = (void (*) (HistoryListHistoryWindow *)) history_list_history_window_real_clean_up;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->close_tab = (void (*) (HistoryListHistoryWindow *)) history_list_history_window_real_close_tab;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_history_list_history_window_get_property;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_history_list_history_window_set_property;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->finalize = history_list_history_window_finalize;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY, history_list_history_window_properties[HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY] = g_param_spec_object ("browser", "browser", "browser", MIDORI_TYPE_BROWSER, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
#line 694 "history-list.c"
}


static void history_list_history_window_instance_init (HistoryListHistoryWindow * self) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->priv = HISTORY_LIST_HISTORY_WINDOW_GET_PRIVATE (self);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->treeview = NULL;
#line 703 "history-list.c"
}


static void history_list_history_window_finalize (GObject * obj) {
	HistoryListHistoryWindow * self;
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindow);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->priv->_browser);
#line 35 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->treeview);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (history_list_history_window_parent_class)->finalize (obj);
#line 717 "history-list.c"
}


GType history_list_history_window_get_type (void) {
	static volatile gsize history_list_history_window_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_history_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HistoryListHistoryWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) history_list_history_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HistoryListHistoryWindow), 0, (GInstanceInitFunc) history_list_history_window_instance_init, NULL };
		GType history_list_history_window_type_id;
		history_list_history_window_type_id = g_type_register_static (GTK_TYPE_WINDOW, "HistoryListHistoryWindow", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&history_list_history_window_type_id__volatile, history_list_history_window_type_id);
	}
	return history_list_history_window_type_id__volatile;
}


static void _vala_history_list_history_window_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	HistoryListHistoryWindow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindow);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	switch (property_id) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		case HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY:
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_value_set_object (value, history_list_history_window_get_browser (self));
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		break;
#line 744 "history-list.c"
		default:
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		break;
#line 750 "history-list.c"
	}
}


static void _vala_history_list_history_window_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	HistoryListHistoryWindow * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, HISTORY_LIST_TYPE_HISTORY_WINDOW, HistoryListHistoryWindow);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	switch (property_id) {
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		case HISTORY_LIST_HISTORY_WINDOW_BROWSER_PROPERTY:
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		history_list_history_window_set_browser (self, g_value_get_object (value));
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		break;
#line 766 "history-list.c"
		default:
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		break;
#line 772 "history-list.c"
	}
}


void history_list_tab_window_store_append_row (HistoryListTabWindow* self, GPtrArray* list, GtkListStore* store) {
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (list != NULL);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (store != NULL);
#line 784 "history-list.c"
	{
		guint i = 0U;
		GPtrArray* _tmp0_;
		guint _tmp1_;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp0_ = list;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = _tmp0_->len;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		i = _tmp1_;
#line 795 "history-list.c"
		{
			gboolean _tmp2_ = FALSE;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp2_ = TRUE;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			while (TRUE) {
#line 802 "history-list.c"
				guint _tmp4_;
				MidoriView* view = NULL;
				GPtrArray* _tmp5_;
				guint _tmp6_;
				void* _tmp7_;
				MidoriView* _tmp8_;
				GdkPixbuf* icon = NULL;
				MidoriView* _tmp9_;
				const gchar* title = NULL;
				MidoriView* _tmp10_;
				const gchar* _tmp11_;
				GtkTreeIter iter = {0};
				GtkListStore* _tmp12_;
				GtkTreeIter _tmp13_ = {0};
				GtkListStore* _tmp14_;
				GtkTreeIter _tmp15_;
				GdkPixbuf* _tmp16_;
				const gchar* _tmp17_;
				MidoriView* _tmp18_;
				GdkColor* _tmp19_;
				GdkColor* _tmp20_;
				MidoriView* _tmp21_;
				GdkColor* _tmp22_;
				GdkColor* _tmp23_;
				MidoriView* _tmp24_;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				if (!_tmp2_) {
#line 830 "history-list.c"
					guint _tmp3_;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					_tmp3_ = i;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					i = _tmp3_ - 1;
#line 836 "history-list.c"
				}
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp2_ = FALSE;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp4_ = i;
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				if (!(_tmp4_ > ((guint) 0))) {
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					break;
#line 846 "history-list.c"
				}
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp5_ = list;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp6_ = i;
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp7_ = g_ptr_array_index (_tmp5_, _tmp6_ - 1);
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp8_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, MIDORI_TYPE_VIEW) ? ((MidoriView*) _tmp7_) : NULL);
#line 77 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				view = _tmp8_;
#line 79 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				icon = NULL;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp9_ = view;
#line 80 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				g_object_get ((GObject*) _tmp9_, "icon", &icon, NULL);
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp10_ = view;
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp11_ = midori_view_get_display_title (_tmp10_);
#line 82 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				title = _tmp11_;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp12_ = store;
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				gtk_list_store_append (_tmp12_, &_tmp13_);
#line 85 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				iter = _tmp13_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp14_ = store;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp15_ = iter;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp16_ = icon;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp17_ = title;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp18_ = view;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp19_ = midori_tab_get_fg_color ((MidoriTab*) _tmp18_);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp20_ = _tmp19_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp21_ = view;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp22_ = midori_tab_get_bg_color ((MidoriTab*) _tmp21_);
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp23_ = _tmp22_;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp24_ = view;
#line 86 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				gtk_list_store_set (_tmp14_, &_tmp15_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_PIXBUF, _tmp16_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_STRING, _tmp17_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_FG, _tmp20_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG, _tmp23_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER, _tmp24_, -1);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (icon);
#line 76 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (view);
#line 904 "history-list.c"
			}
		}
	}
}


static void history_list_tab_window_real_insert_rows (HistoryListTabWindow* self, GtkListStore* store) {
	GPtrArray* list = NULL;
	MidoriBrowser* _tmp0_;
	MidoriBrowser* _tmp1_;
	gconstpointer _tmp2_;
	GPtrArray* list_new = NULL;
	MidoriBrowser* _tmp3_;
	MidoriBrowser* _tmp4_;
	gconstpointer _tmp5_;
	GtkListStore* _tmp6_;
	GtkListStore* _tmp7_;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (store != NULL);
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _tmp0_;
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = g_object_get_data ((GObject*) _tmp1_, "history-list-tab-history");
#line 95 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list = (GPtrArray*) _tmp2_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = _tmp3_;
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = g_object_get_data ((GObject*) _tmp4_, "history-list-tab-history-new");
#line 96 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list_new = (GPtrArray*) _tmp5_;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = store;
#line 97 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_store_append_row (self, list, _tmp6_);
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = store;
#line 98 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_store_append_row (self, list_new, _tmp7_);
#line 948 "history-list.c"
}


void history_list_tab_window_insert_rows (HistoryListTabWindow* self, GtkListStore* store) {
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	HISTORY_LIST_TAB_WINDOW_GET_CLASS (self)->insert_rows (self, store);
#line 957 "history-list.c"
}


void history_list_tab_window_resize_treeview (HistoryListTabWindow* self) {
	GtkRequisition requisition = {0};
	gint height = 0;
	gint max_lines = 0;
	GtkTreeView* _tmp0_;
	GtkRequisition _tmp1_ = {0};
	GtkListStore* model = NULL;
	GtkTreeView* _tmp2_;
	GtkTreeModel* _tmp3_;
	GtkListStore* _tmp4_;
	gint length = 0;
	GtkListStore* _tmp5_;
	gint _tmp6_;
	gint _tmp7_;
	gint _tmp8_;
	GtkScrolledWindow* _tmp15_;
	gint _tmp16_;
	gint _tmp17_;
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 104 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	max_lines = 10;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_widget_size_request ((GtkWidget*) _tmp0_, &_tmp1_);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	requisition = _tmp1_;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = gtk_tree_view_get_model (_tmp2_);
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp3_, GTK_TYPE_LIST_STORE) ? ((GtkListStore*) _tmp3_) : NULL);
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	model = _tmp4_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = model;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = gtk_tree_model_iter_n_children ((GtkTreeModel*) _tmp5_, NULL);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	length = _tmp6_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = length;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = max_lines;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp7_ > _tmp8_) {
#line 1009 "history-list.c"
		GtkRequisition _tmp9_;
		gint _tmp10_;
		gint _tmp11_;
		gint _tmp12_;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9_ = requisition;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp10_ = _tmp9_.height;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = length;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12_ = max_lines;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		height = ((_tmp10_ / _tmp11_) * _tmp12_) + 2;
#line 1024 "history-list.c"
	} else {
		GtkRequisition _tmp13_;
		gint _tmp14_;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = requisition;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = _tmp13_.height;
#line 116 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		height = _tmp14_ + 2;
#line 1034 "history-list.c"
	}
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = self->scroll_windows;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp16_ = height;
#line 118 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_widget_set_size_request ((GtkWidget*) _tmp15_, 320, _tmp16_);
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp17_ = height;
#line 119 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_window_resize ((GtkWindow*) self, 320, _tmp17_);
#line 101 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 1048 "history-list.c"
}


HistoryListTabWindow* history_list_tab_window_construct (GType object_type, MidoriBrowser* browser) {
	HistoryListTabWindow * self = NULL;
	MidoriBrowser* _tmp0_;
	GtkVBox* _tmp1_;
	GtkVBox* _tmp2_;
	GtkHBox* _tmp3_;
	GtkScrolledWindow* _tmp4_;
	GtkScrolledWindow* _tmp5_;
	GtkScrolledWindow* _tmp6_;
	GtkHBox* _tmp7_;
	GtkScrolledWindow* _tmp8_;
	GtkListStore* store = NULL;
	GtkListStore* _tmp9_;
	GtkVBox* _tmp10_;
	GtkHBox* _tmp11_;
	GtkTreeView* _tmp12_;
	GtkScrolledWindow* _tmp13_;
	GtkTreeView* _tmp14_;
	GtkTreeView* _tmp15_;
	GtkTreeView* _tmp16_;
	GtkTreeView* _tmp17_;
	GtkCellRendererPixbuf* _tmp18_;
	GtkCellRendererPixbuf* _tmp19_;
	GtkTreeView* _tmp20_;
	GtkCellRendererText* _tmp21_;
	GtkCellRendererText* _tmp22_;
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (browser != NULL, NULL);
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListTabWindow*) history_list_history_window_construct (object_type, _tmp0_);
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = (GtkVBox*) gtk_vbox_new (FALSE, 1);
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp1_);
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->vbox);
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->vbox = _tmp1_;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = self->vbox;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) _tmp2_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = (GtkHBox*) gtk_hbox_new (FALSE, 1);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp3_);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->hbox);
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->hbox = _tmp3_;
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = (GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp4_);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->scroll_windows);
#line 131 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->scroll_windows = _tmp4_;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = self->scroll_windows;
#line 132 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_scrolled_window_set_policy (_tmp5_, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = self->scroll_windows;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_scrolled_window_set_shadow_type (_tmp6_, GTK_SHADOW_ETCHED_IN);
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = self->hbox;
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = self->scroll_windows;
#line 134 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_box_pack_start ((GtkBox*) _tmp7_, (GtkWidget*) _tmp8_, TRUE, TRUE, (guint) 0);
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = gtk_list_store_new ((gint) HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_COUNT, gdk_pixbuf_get_type (), G_TYPE_STRING, GDK_TYPE_COLOR, GDK_TYPE_COLOR, G_TYPE_POINTER);
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	store = _tmp9_;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_insert_rows (self, store);
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = self->vbox;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = self->hbox;
#line 142 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_box_pack_start ((GtkBox*) _tmp10_, (GtkWidget*) _tmp11_, TRUE, TRUE, (guint) 0);
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) store);
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp12_);
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (((HistoryListHistoryWindow*) self)->treeview);
#line 144 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindow*) self)->treeview = _tmp12_;
#line 145 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = self->scroll_windows;
#line 145 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp14_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 145 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_container_add ((GtkContainer*) _tmp13_, (GtkWidget*) _tmp14_);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_set_model (_tmp15_, (GtkTreeModel*) store);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp16_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set ((GObject*) _tmp16_, "headers-visible", FALSE, NULL);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp17_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp18_ = (GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ();
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp18_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp19_ = _tmp18_;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_insert_column_with_attributes (_tmp17_, -1, "Icon", (GtkCellRenderer*) _tmp19_, "pixbuf", HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_PIXBUF, "cell-background-gdk", HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG, NULL);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (_tmp19_);
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp20_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp21_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp21_);
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp22_ = _tmp21_;
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_insert_column_with_attributes (_tmp20_, -1, "Title", (GtkCellRenderer*) _tmp22_, "text", HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_STRING, "foreground-gdk", HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_FG, "cell-background-gdk", HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_BG, NULL);
#line 154 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (_tmp22_);
#line 160 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_resize_treeview (self);
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (store);
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self;
#line 1192 "history-list.c"
}


HistoryListTabWindow* history_list_tab_window_new (MidoriBrowser* browser) {
#line 123 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return history_list_tab_window_construct (HISTORY_LIST_TYPE_TAB_WINDOW, browser);
#line 1199 "history-list.c"
}


static void history_list_tab_window_real_make_update (HistoryListHistoryWindow* base) {
	HistoryListTabWindow * self;
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkTreeView* _tmp0_;
	GtkTreePath* _tmp1_ = NULL;
	GtkTreeViewColumn* _tmp2_ = NULL;
	GtkTreeViewColumn* _tmp3_;
	GtkTreePath* _tmp4_;
	GtkListStore* model = NULL;
	GtkTreeView* _tmp5_;
	GtkTreeModel* _tmp6_;
	GtkListStore* _tmp7_;
	GtkTreeIter iter = {0};
	MidoriView* view = NULL;
	GtkListStore* _tmp8_;
	GtkTreePath* _tmp9_;
	GtkTreeIter _tmp10_ = {0};
	gboolean _tmp11_;
	GtkListStore* _tmp12_;
	GtkTreeIter _tmp13_;
	MidoriBrowser* _tmp14_;
	MidoriBrowser* _tmp15_;
	MidoriView* _tmp16_;
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListTabWindow*) base;
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->is_dirty = TRUE;
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_get_cursor (_tmp0_, &_tmp1_, &_tmp2_);
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	path = _tmp1_;
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	column = _tmp3_;
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = path;
#line 171 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp4_ == NULL) {
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (column);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 172 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		return;
#line 1255 "history-list.c"
	}
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = gtk_tree_view_get_model (_tmp5_);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, GTK_TYPE_LIST_STORE) ? ((GtkListStore*) _tmp6_) : NULL);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	model = _tmp7_;
#line 177 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	view = NULL;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = model;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = path;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = gtk_tree_model_get_iter ((GtkTreeModel*) _tmp8_, &_tmp10_, _tmp9_);
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	iter = _tmp10_;
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (!_tmp11_) {
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (model);
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (column);
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 180 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		return;
#line 1285 "history-list.c"
	}
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = model;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = iter;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_model_get ((GtkTreeModel*) _tmp12_, &_tmp13_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER, &view, -1);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp14_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = _tmp14_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp16_ = view;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set ((GObject*) _tmp15_, "tab", _tmp16_, NULL);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 1307 "history-list.c"
}


static void history_list_tab_window_real_clean_up (HistoryListHistoryWindow* base) {
	HistoryListTabWindow * self;
	gboolean _tmp0_;
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListTabWindow*) base;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = self->is_dirty;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp0_) {
#line 1320 "history-list.c"
		GtkTreePath* path = NULL;
		GtkTreeViewColumn* column = NULL;
		GtkTreeView* _tmp1_;
		GtkTreePath* _tmp2_ = NULL;
		GtkTreeViewColumn* _tmp3_ = NULL;
		GtkTreeViewColumn* _tmp4_;
		GtkTreePath* _tmp5_;
		GtkTreeView* _tmp6_;
		GtkTreePath* _tmp7_;
		GtkTreeViewColumn* _tmp8_;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_tree_view_get_cursor (_tmp1_, &_tmp2_, &_tmp3_);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		path = _tmp2_;
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (column);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp4_ = _g_object_ref0 (_tmp3_);
#line 190 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		column = _tmp4_;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp5_ = gtk_tree_path_new_from_indices (0, -1);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		path = _tmp5_;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp6_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp7_ = path;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp8_ = column;
#line 193 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_tree_view_set_cursor (_tmp6_, _tmp7_, _tmp8_, FALSE);
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		history_list_history_window_make_update ((HistoryListHistoryWindow*) self);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->is_dirty = FALSE;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (column);
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_gtk_tree_path_free0 (path);
#line 1367 "history-list.c"
	}
}


static void history_list_tab_window_real_close_tab (HistoryListHistoryWindow* base) {
	HistoryListTabWindow * self;
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* column = NULL;
	GtkTreeView* _tmp0_;
	GtkTreePath* _tmp1_ = NULL;
	GtkTreeViewColumn* _tmp2_ = NULL;
	GtkTreeViewColumn* _tmp3_;
	GtkListStore* model = NULL;
	GtkTreeView* _tmp4_;
	GtkTreeModel* _tmp5_;
	GtkListStore* _tmp6_;
	gint length = 0;
	GtkListStore* _tmp7_;
	gint _tmp8_;
	gint _tmp9_;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListTabWindow*) base;
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_tree_view_get_cursor (_tmp0_, &_tmp1_, &_tmp2_);
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	path = _tmp1_;
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = _g_object_ref0 (_tmp2_);
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	column = _tmp3_;
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = gtk_tree_view_get_model (_tmp4_);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GTK_TYPE_LIST_STORE) ? ((GtkListStore*) _tmp5_) : NULL);
#line 206 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	model = _tmp6_;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = model;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = gtk_tree_model_iter_n_children ((GtkTreeModel*) _tmp7_, NULL);
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	length = _tmp8_;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = length;
#line 209 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp9_ > 1) {
#line 1422 "history-list.c"
		GtkTreeIter iter = {0};
		MidoriView* view = NULL;
		GtkListStore* _tmp10_;
		GtkTreePath* _tmp11_;
		GtkTreeIter _tmp12_ = {0};
		GtkListStore* _tmp13_;
		GtkTreeIter _tmp14_;
		GtkTreePath* _tmp15_;
		gboolean _tmp16_;
		GtkTreeView* _tmp18_;
		GtkTreePath* _tmp19_;
		GtkTreeViewColumn* _tmp20_;
		GtkListStore* _tmp21_;
		MidoriBrowser* _tmp22_;
		MidoriBrowser* _tmp23_;
		MidoriView* _tmp24_;
		gint _tmp25_;
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		view = NULL;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp10_ = model;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = path;
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_tree_model_get_iter ((GtkTreeModel*) _tmp10_, &_tmp12_, _tmp11_);
#line 213 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		iter = _tmp12_;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = model;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = iter;
#line 214 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_tree_model_get ((GtkTreeModel*) _tmp13_, &_tmp14_, HISTORY_LIST_TAB_TREE_CELLS_TREE_CELL_POINTER, &view, -1);
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp15_ = path;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp16_ = gtk_tree_path_prev (_tmp15_);
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp16_ == FALSE) {
#line 1462 "history-list.c"
			GtkTreePath* _tmp17_;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp17_ = path;
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_tree_path_next (_tmp17_);
#line 1468 "history-list.c"
		}
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp18_ = ((HistoryListHistoryWindow*) self)->treeview;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp19_ = path;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp20_ = column;
#line 219 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_tree_view_set_cursor (_tmp18_, _tmp19_, _tmp20_, FALSE);
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp21_ = model;
#line 226 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_list_store_remove (_tmp21_, &iter);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp22_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp23_ = _tmp22_;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp24_ = view;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		midori_browser_close_tab (_tmp23_, _tmp24_);
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp25_ = length;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp25_ > 2) {
#line 229 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_tab_window_resize_treeview (self);
#line 1496 "history-list.c"
		} else {
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_widget_hide ((GtkWidget*) self);
#line 1500 "history-list.c"
		}
	}
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (column);
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_gtk_tree_path_free0 (path);
#line 1509 "history-list.c"
}


static void history_list_tab_window_class_init (HistoryListTabWindowClass * klass) {
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_parent_class = g_type_class_peek_parent (klass);
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListTabWindowClass *) klass)->insert_rows = (void (*) (HistoryListTabWindow *, GtkListStore*)) history_list_tab_window_real_insert_rows;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->make_update = (void (*) (HistoryListHistoryWindow *)) history_list_tab_window_real_make_update;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->clean_up = (void (*) (HistoryListHistoryWindow *)) history_list_tab_window_real_clean_up;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->close_tab = (void (*) (HistoryListHistoryWindow *)) history_list_tab_window_real_close_tab;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->finalize = history_list_tab_window_finalize;
#line 1526 "history-list.c"
}


static void history_list_tab_window_instance_init (HistoryListTabWindow * self) {
#line 72 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->is_dirty = FALSE;
#line 1533 "history-list.c"
}


static void history_list_tab_window_finalize (GObject * obj) {
	HistoryListTabWindow * self;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindow);
#line 70 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->hbox);
#line 71 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->vbox);
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->scroll_windows);
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (history_list_tab_window_parent_class)->finalize (obj);
#line 1549 "history-list.c"
}


GType history_list_tab_window_get_type (void) {
	static volatile gsize history_list_tab_window_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_tab_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HistoryListTabWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) history_list_tab_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HistoryListTabWindow), 0, (GInstanceInitFunc) history_list_tab_window_instance_init, NULL };
		GType history_list_tab_window_type_id;
		history_list_tab_window_type_id = g_type_register_static (HISTORY_LIST_TYPE_HISTORY_WINDOW, "HistoryListTabWindow", &g_define_type_info, 0);
		g_once_init_leave (&history_list_tab_window_type_id__volatile, history_list_tab_window_type_id);
	}
	return history_list_tab_window_type_id__volatile;
}


static void history_list_new_tab_window_real_insert_rows (HistoryListTabWindow* base, GtkListStore* store) {
	HistoryListNewTabWindow * self;
	GPtrArray* list = NULL;
	MidoriBrowser* _tmp0_;
	MidoriBrowser* _tmp1_;
	gconstpointer _tmp2_;
	GPtrArray* _tmp3_;
	GtkListStore* _tmp4_;
	GPtrArray* _tmp5_;
	guint _tmp6_;
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListNewTabWindow*) base;
#line 240 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (store != NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _tmp0_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = g_object_get_data ((GObject*) _tmp1_, "history-list-tab-history-new");
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list = (GPtrArray*) _tmp2_;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = list;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = store;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_tab_window_store_append_row ((HistoryListTabWindow*) self, _tmp3_, _tmp4_);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = list;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = _tmp5_->len;
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (((gint) _tmp6_) == 0) {
#line 1599 "history-list.c"
		GtkLabel* label = NULL;
		GtkLabel* _tmp7_;
		GtkVBox* _tmp8_;
		GtkLabel* _tmp9_;
		GPtrArray* list_old = NULL;
		MidoriBrowser* _tmp10_;
		MidoriBrowser* _tmp11_;
		gconstpointer _tmp12_;
		GPtrArray* _tmp13_;
		GtkListStore* _tmp14_;
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->old_tabs = TRUE;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp7_ = (GtkLabel*) gtk_label_new (_ ("There are no unvisited tabs"));
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_ref_sink (_tmp7_);
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		label = _tmp7_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp8_ = ((HistoryListTabWindow*) self)->vbox;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9_ = label;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_box_pack_start ((GtkBox*) _tmp8_, (GtkWidget*) _tmp9_, TRUE, TRUE, (guint) 0);
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp10_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = _tmp10_;
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12_ = g_object_get_data ((GObject*) _tmp11_, "history-list-tab-history");
#line 248 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		list_old = (GPtrArray*) _tmp12_;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = list_old;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = store;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		history_list_tab_window_store_append_row ((HistoryListTabWindow*) self, _tmp13_, _tmp14_);
#line 244 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (label);
#line 1640 "history-list.c"
	}
}


static void history_list_new_tab_window_real_walk (HistoryListHistoryWindow* base, gint step) {
	HistoryListNewTabWindow * self;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_;
#line 253 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListNewTabWindow*) base;
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = self->first_step;
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp1_ == FALSE) {
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp0_ = TRUE;
#line 1657 "history-list.c"
	} else {
		gint _tmp2_;
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp2_ = step;
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp0_ = _tmp2_ != 1;
#line 1664 "history-list.c"
	}
#line 254 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp0_) {
#line 1668 "history-list.c"
		gint _tmp3_;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp3_ = step;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		HISTORY_LIST_HISTORY_WINDOW_CLASS (history_list_new_tab_window_parent_class)->walk ((HistoryListHistoryWindow*) G_TYPE_CHECK_INSTANCE_CAST (self, HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindow), _tmp3_);
#line 1674 "history-list.c"
	}
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->first_step = FALSE;
#line 1678 "history-list.c"
}


static void history_list_new_tab_window_real_clean_up (HistoryListHistoryWindow* base) {
	HistoryListNewTabWindow * self;
	gboolean _tmp0_;
#line 260 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListNewTabWindow*) base;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = ((HistoryListTabWindow*) self)->is_dirty;
#line 261 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp0_) {
#line 1691 "history-list.c"
		gboolean _tmp1_;
#line 262 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = self->old_tabs;
#line 262 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp1_) {
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			HISTORY_LIST_HISTORY_WINDOW_CLASS (history_list_new_tab_window_parent_class)->clean_up ((HistoryListHistoryWindow*) G_TYPE_CHECK_INSTANCE_CAST (self, HISTORY_LIST_TYPE_TAB_WINDOW, HistoryListTabWindow));
#line 1699 "history-list.c"
		} else {
			GPtrArray* list = NULL;
			MidoriBrowser* _tmp2_;
			MidoriBrowser* _tmp3_;
			gconstpointer _tmp4_;
			MidoriView* view = NULL;
			GPtrArray* _tmp5_;
			GPtrArray* _tmp6_;
			guint _tmp7_;
			void* _tmp8_;
			MidoriView* _tmp9_;
			MidoriBrowser* _tmp10_;
			MidoriBrowser* _tmp11_;
			MidoriView* _tmp12_;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp2_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp3_ = _tmp2_;
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp4_ = g_object_get_data ((GObject*) _tmp3_, "history-list-tab-history");
#line 265 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			list = (GPtrArray*) _tmp4_;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp5_ = list;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp6_ = list;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp7_ = _tmp6_->len;
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp8_ = g_ptr_array_index (_tmp5_, _tmp7_ - 1);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp9_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, MIDORI_TYPE_VIEW) ? ((MidoriView*) _tmp8_) : NULL);
#line 266 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			view = _tmp9_;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp10_ = history_list_history_window_get_browser ((HistoryListHistoryWindow*) self);
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp11_ = _tmp10_;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp12_ = view;
#line 267 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			g_object_set ((GObject*) _tmp11_, "tab", _tmp12_, NULL);
#line 262 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_g_object_unref0 (view);
#line 1744 "history-list.c"
		}
	}
}


HistoryListNewTabWindow* history_list_new_tab_window_construct (GType object_type, MidoriBrowser* browser) {
	HistoryListNewTabWindow * self = NULL;
	MidoriBrowser* _tmp0_;
#line 272 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (browser != NULL, NULL);
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListNewTabWindow*) history_list_tab_window_construct (object_type, _tmp0_);
#line 272 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self;
#line 1761 "history-list.c"
}


HistoryListNewTabWindow* history_list_new_tab_window_new (MidoriBrowser* browser) {
#line 272 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return history_list_new_tab_window_construct (HISTORY_LIST_TYPE_NEW_TAB_WINDOW, browser);
#line 1768 "history-list.c"
}


static void history_list_new_tab_window_class_init (HistoryListNewTabWindowClass * klass) {
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_new_tab_window_parent_class = g_type_class_peek_parent (klass);
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListTabWindowClass *) klass)->insert_rows = (void (*) (HistoryListTabWindow *, GtkListStore*)) history_list_new_tab_window_real_insert_rows;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->walk = (void (*) (HistoryListHistoryWindow *, gint)) history_list_new_tab_window_real_walk;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	((HistoryListHistoryWindowClass *) klass)->clean_up = (void (*) (HistoryListHistoryWindow *)) history_list_new_tab_window_real_clean_up;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->finalize = history_list_new_tab_window_finalize;
#line 1783 "history-list.c"
}


static void history_list_new_tab_window_instance_init (HistoryListNewTabWindow * self) {
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->old_tabs = FALSE;
#line 238 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->first_step = TRUE;
#line 1792 "history-list.c"
}


static void history_list_new_tab_window_finalize (GObject * obj) {
	HistoryListNewTabWindow * self;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_NEW_TAB_WINDOW, HistoryListNewTabWindow);
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (history_list_new_tab_window_parent_class)->finalize (obj);
#line 1802 "history-list.c"
}


GType history_list_new_tab_window_get_type (void) {
	static volatile gsize history_list_new_tab_window_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_new_tab_window_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HistoryListNewTabWindowClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) history_list_new_tab_window_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HistoryListNewTabWindow), 0, (GInstanceInitFunc) history_list_new_tab_window_instance_init, NULL };
		GType history_list_new_tab_window_type_id;
		history_list_new_tab_window_type_id = g_type_register_static (HISTORY_LIST_TYPE_TAB_WINDOW, "HistoryListNewTabWindow", &g_define_type_info, 0);
		g_once_init_leave (&history_list_new_tab_window_type_id__volatile, history_list_new_tab_window_type_id);
	}
	return history_list_new_tab_window_type_id__volatile;
}


static void _history_list_preferences_dialog_response_cb_gtk_dialog_response (GtkDialog* _sender, gint response_id, gpointer self) {
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_preferences_dialog_response_cb ((HistoryListPreferencesDialog*) self, _sender, response_id);
#line 1821 "history-list.c"
}


HistoryListPreferencesDialog* history_list_preferences_dialog_construct (GType object_type, HistoryListManager* manager) {
	HistoryListPreferencesDialog * self = NULL;
	HistoryListManager* _tmp0_;
	HistoryListManager* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	GObjectClass* _tmp4_;
	GParamSpec* _tmp5_;
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (manager != NULL, NULL);
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListPreferencesDialog*) g_object_new (object_type, NULL);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = manager;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->hl_manager);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->hl_manager = _tmp1_;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = g_strdup_printf (_ ("Preferences for %s"), _ ("History-List"));
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = _tmp2_;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_window_set_title ((GtkWindow*) self, _tmp3_);
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_free0 (_tmp3_);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = G_OBJECT_GET_CLASS ((GObject*) self);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = g_object_class_find_property (_tmp4_, "has-separator");
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp5_ != NULL) {
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_set ((GObject*) self, "has-separator", FALSE, NULL);
#line 1861 "history-list.c"
	}
#line 287 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_window_set_modal ((GtkWindow*) self, TRUE);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_window_set_default_size ((GtkWindow*) self, 350, 100);
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_preferences_dialog_create_widgets (self);
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object ((GtkDialog*) self, "response", (GCallback) _history_list_preferences_dialog_response_cb_gtk_dialog_response, self, 0);
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self;
#line 1875 "history-list.c"
}


HistoryListPreferencesDialog* history_list_preferences_dialog_new (HistoryListManager* manager) {
#line 281 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return history_list_preferences_dialog_construct (HISTORY_LIST_TYPE_PREFERENCES_DIALOG, manager);
#line 1882 "history-list.c"
}


static void history_list_preferences_dialog_response_cb (HistoryListPreferencesDialog* self, GtkDialog* source, gint response_id) {
	gint _tmp0_;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (source != NULL);
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = response_id;
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	switch (_tmp0_) {
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		case GTK_RESPONSE_APPLY:
#line 1898 "history-list.c"
		{
			gint value = 0;
			GtkTreeIter iter = {0};
			GtkComboBox* _tmp1_;
			GtkTreeIter _tmp2_ = {0};
			GtkTreeModel* model = NULL;
			GtkComboBox* _tmp3_;
			GtkTreeModel* _tmp4_;
			GtkTreeModel* _tmp5_;
			GtkTreeModel* _tmp6_;
			GtkTreeIter _tmp7_;
			HistoryListManager* _tmp8_;
			gint _tmp9_;
			HistoryListManager* _tmp10_;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp1_ = self->closing_behavior;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_combo_box_get_active_iter (_tmp1_, &_tmp2_);
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			iter = _tmp2_;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp3_ = self->closing_behavior;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp4_ = gtk_combo_box_get_model (_tmp3_);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp5_ = _g_object_ref0 (_tmp4_);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			model = _tmp5_;
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp6_ = model;
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp7_ = iter;
#line 303 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_tree_model_get (_tmp6_, &_tmp7_, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE, &value, -1);
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp8_ = self->hl_manager;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp9_ = value;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			midori_extension_set_integer ((MidoriExtension*) _tmp8_, "TabClosingBehavior", _tmp9_);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp10_ = self->hl_manager;
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			g_signal_emit (_tmp10_, history_list_manager_signals[HISTORY_LIST_MANAGER_PREFERENCES_CHANGED_SIGNAL], 0);
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_object_destroy ((GtkObject*) self);
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_g_object_unref0 (model);
#line 309 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			break;
#line 1949 "history-list.c"
		}
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		case GTK_RESPONSE_CANCEL:
#line 1953 "history-list.c"
		{
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_object_destroy ((GtkObject*) self);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			break;
#line 1959 "history-list.c"
		}
		default:
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		break;
#line 1964 "history-list.c"
	}
}


static GtkTreeIter* _gtk_tree_iter_dup (GtkTreeIter* self) {
	GtkTreeIter* dup;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	dup = g_new0 (GtkTreeIter, 1);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	memcpy (dup, self, sizeof (GtkTreeIter));
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return dup;
#line 1977 "history-list.c"
}


static gpointer __gtk_tree_iter_dup0 (gpointer self) {
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self ? _gtk_tree_iter_dup (self) : NULL;
#line 1984 "history-list.c"
}


static void history_list_preferences_dialog_create_widgets (HistoryListPreferencesDialog* self) {
	GtkListStore* model = NULL;
	GtkTreeIter iter = {0};
	GtkTreeIter* active_iter = NULL;
	GtkTable* table = NULL;
	GtkTable* _tmp0_;
	GtkCellRendererText* renderer = NULL;
	GtkCellRendererText* _tmp1_;
	GtkLabel* label = NULL;
	GtkLabel* _tmp2_;
	GtkTable* _tmp3_;
	GtkLabel* _tmp4_;
	gint tab_closing_behavior = 0;
	HistoryListManager* _tmp5_;
	gint _tmp6_;
	GtkListStore* _tmp7_;
	GtkListStore* _tmp8_;
	GtkTreeIter _tmp9_ = {0};
	GtkListStore* _tmp10_;
	GtkTreeIter _tmp11_;
	gint _tmp12_;
	GtkListStore* _tmp15_;
	GtkTreeIter _tmp16_ = {0};
	GtkListStore* _tmp17_;
	GtkTreeIter _tmp18_;
	gint _tmp19_;
	GtkListStore* _tmp22_;
	GtkTreeIter _tmp23_ = {0};
	GtkListStore* _tmp24_;
	GtkTreeIter _tmp25_;
	gint _tmp26_;
	GtkListStore* _tmp29_;
	GtkComboBox* _tmp30_;
	GtkComboBox* _tmp31_;
	GtkTreeIter* _tmp32_;
	GtkComboBox* _tmp33_;
	GtkCellRendererText* _tmp34_;
	GtkComboBox* _tmp35_;
	GtkCellRendererText* _tmp36_;
	GtkTable* _tmp37_;
	GtkComboBox* _tmp38_;
	GtkWidget* proxy = NULL;
	HistoryListManager* _tmp39_;
	MidoriApp* _tmp40_;
	MidoriWebSettings* _tmp41_;
	MidoriWebSettings* _tmp42_;
	GtkWidget* _tmp43_;
	GtkWidget* _tmp44_;
	GtkTable* _tmp45_;
	GtkVBox* _tmp46_;
	GtkTable* _tmp47_;
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 319 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	active_iter = NULL;
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = (GtkTable*) gtk_table_new ((guint) 1, (guint) 2, TRUE);
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp0_);
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	table = _tmp0_;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = (GtkCellRendererText*) gtk_cell_renderer_text_new ();
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp1_);
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	renderer = _tmp1_;
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = (GtkLabel*) gtk_label_new (_ ("Tab closing behavior"));
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp2_);
#line 324 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	label = _tmp2_;
#line 325 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = table;
#line 325 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = label;
#line 325 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_table_attach_defaults (_tmp3_, (GtkWidget*) _tmp4_, (guint) 0, (guint) 1, (guint) 0, (guint) 1);
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = self->hl_manager;
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = midori_extension_get_integer ((MidoriExtension*) _tmp5_, "TabClosingBehavior");
#line 327 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	tab_closing_behavior = _tmp6_;
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 329 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	model = _tmp7_;
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = model;
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_append (_tmp8_, &_tmp9_);
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	iter = _tmp9_;
#line 332 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = model;
#line 332 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = iter;
#line 332 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_set (_tmp10_, &_tmp11_, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT, _ ("Do nothing"), HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NONE, -1);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = tab_closing_behavior;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NONE == _tmp12_) {
#line 2095 "history-list.c"
		GtkTreeIter _tmp13_;
		GtkTreeIter* _tmp14_;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = iter;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = __gtk_tree_iter_dup0 (&_tmp13_);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_free0 (active_iter);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		active_iter = _tmp14_;
#line 2106 "history-list.c"
	}
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = model;
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_append (_tmp15_, &_tmp16_);
#line 337 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	iter = _tmp16_;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp17_ = model;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp18_ = iter;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_set (_tmp17_, &_tmp18_, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT, _ ("Switch to last viewed tab"), HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST, -1);
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp19_ = tab_closing_behavior;
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST == _tmp19_) {
#line 2124 "history-list.c"
		GtkTreeIter _tmp20_;
		GtkTreeIter* _tmp21_;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp20_ = iter;
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp21_ = __gtk_tree_iter_dup0 (&_tmp20_);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_free0 (active_iter);
#line 341 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		active_iter = _tmp21_;
#line 2135 "history-list.c"
	}
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp22_ = model;
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_append (_tmp22_, &_tmp23_);
#line 343 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	iter = _tmp23_;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp24_ = model;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp25_ = iter;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_list_store_set (_tmp24_, &_tmp25_, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_TEXT, _ ("Switch to newest tab"), HISTORY_LIST_TAB_CLOSING_BEHAVIOR_MODEL_VALUE, HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW, -1);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp26_ = tab_closing_behavior;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW == _tmp26_) {
#line 2153 "history-list.c"
		GtkTreeIter _tmp27_;
		GtkTreeIter* _tmp28_;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp27_ = iter;
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp28_ = __gtk_tree_iter_dup0 (&_tmp27_);
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_free0 (active_iter);
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		active_iter = _tmp28_;
#line 2164 "history-list.c"
	}
#line 349 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp29_ = model;
#line 349 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp30_ = (GtkComboBox*) gtk_combo_box_new_with_model ((GtkTreeModel*) _tmp29_);
#line 349 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp30_);
#line 349 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->closing_behavior);
#line 349 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->closing_behavior = _tmp30_;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp31_ = self->closing_behavior;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp32_ = active_iter;
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_combo_box_set_active_iter (_tmp31_, _tmp32_);
#line 351 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp33_ = self->closing_behavior;
#line 351 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp34_ = renderer;
#line 351 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_cell_layout_pack_start ((GtkCellLayout*) _tmp33_, (GtkCellRenderer*) _tmp34_, TRUE);
#line 352 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp35_ = self->closing_behavior;
#line 352 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp36_ = renderer;
#line 352 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_cell_layout_set_attributes ((GtkCellLayout*) _tmp35_, (GtkCellRenderer*) _tmp36_, "text", 0, NULL);
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp37_ = table;
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp38_ = self->closing_behavior;
#line 354 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_table_attach_defaults (_tmp37_, (GtkWidget*) _tmp38_, (guint) 1, (guint) 2, (guint) 0, (guint) 1);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp39_ = self->hl_manager;
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp40_ = midori_extension_get_app ((MidoriExtension*) _tmp39_);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_get (_tmp40_, "settings", &_tmp41_, NULL);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp42_ = _tmp41_;
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp43_ = katze_property_proxy (_tmp42_, "flash-window-on-new-bg-tabs", NULL);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp44_ = _g_object_ref0 (_tmp43_);
#line 357 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	proxy = _tmp44_;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_button_set_label (G_TYPE_CHECK_INSTANCE_TYPE (proxy, GTK_TYPE_BUTTON) ? ((GtkButton*) proxy) : NULL, _ ("Flash window on background tabs"));
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp45_ = table;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_table_attach_defaults (_tmp45_, proxy, (guint) 0, (guint) 2, (guint) 1, (guint) 2);
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp46_ = ((GtkDialog*) self)->vbox;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp47_ = table;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_box_pack_start ((GtkBox*) _tmp46_, (GtkWidget*) _tmp47_, FALSE, TRUE, (guint) 0);
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_APPLY, (gint) GTK_RESPONSE_APPLY);
#line 371 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_widget_show_all ((GtkWidget*) self);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (proxy);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (label);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (renderer);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (table);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_free0 (active_iter);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (model);
#line 2244 "history-list.c"
}


static void history_list_preferences_dialog_class_init (HistoryListPreferencesDialogClass * klass) {
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_preferences_dialog_parent_class = g_type_class_peek_parent (klass);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->finalize = history_list_preferences_dialog_finalize;
#line 2253 "history-list.c"
}


static void history_list_preferences_dialog_instance_init (HistoryListPreferencesDialog * self) {
}


static void history_list_preferences_dialog_finalize (GObject * obj) {
	HistoryListPreferencesDialog * self;
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_PREFERENCES_DIALOG, HistoryListPreferencesDialog);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->hl_manager);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->closing_behavior);
#line 277 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (history_list_preferences_dialog_parent_class)->finalize (obj);
#line 2271 "history-list.c"
}


GType history_list_preferences_dialog_get_type (void) {
	static volatile gsize history_list_preferences_dialog_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_preferences_dialog_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HistoryListPreferencesDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) history_list_preferences_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HistoryListPreferencesDialog), 0, (GInstanceInitFunc) history_list_preferences_dialog_instance_init, NULL };
		GType history_list_preferences_dialog_type_id;
		history_list_preferences_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "HistoryListPreferencesDialog", &g_define_type_info, 0);
		g_once_init_leave (&history_list_preferences_dialog_type_id__volatile, history_list_preferences_dialog_type_id);
	}
	return history_list_preferences_dialog_type_id__volatile;
}


void history_list_manager_preferences_changed_cb (HistoryListManager* self) {
	gint _tmp0_;
#line 386 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = midori_extension_get_integer ((MidoriExtension*) self, "TabClosingBehavior");
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->closing_behavior = _tmp0_;
#line 2295 "history-list.c"
}


gboolean history_list_manager_is_key_a_modifier (HistoryListManager* self, GdkEventKey* event_key) {
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	guint _tmp1_;
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 390 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (event_key != NULL, FALSE);
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = *event_key;
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _tmp0_.is_modifier;
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp1_ > ((guint) 0)) {
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		result = TRUE;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		return result;
#line 2317 "history-list.c"
	}
#line 399 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = FALSE;
#line 399 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2323 "history-list.c"
}


gboolean history_list_manager_key_press (HistoryListManager* self, GdkEventKey* event_key) {
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	gboolean _tmp1_;
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 402 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (event_key != NULL, FALSE);
#line 403 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = *event_key;
#line 403 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = history_list_manager_is_key_a_modifier (self, &_tmp0_);
#line 403 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp1_) {
#line 2341 "history-list.c"
		guint _tmp2_;
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp2_ = self->modifier_count;
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->modifier_count = _tmp2_ + 1;
#line 2347 "history-list.c"
	}
#line 406 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = FALSE;
#line 406 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2353 "history-list.c"
}


gboolean history_list_manager_key_release (HistoryListManager* self, GdkEventKey* event_key, MidoriBrowser* browser) {
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp3_ = FALSE;
	guint _tmp4_;
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (event_key != NULL, FALSE);
#line 409 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (browser != NULL, FALSE);
#line 410 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = *event_key;
#line 410 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = history_list_manager_is_key_a_modifier (self, &_tmp0_);
#line 410 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp1_) {
#line 2375 "history-list.c"
		guint _tmp2_;
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp2_ = self->modifier_count;
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->modifier_count = _tmp2_ - 1;
#line 2381 "history-list.c"
	}
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = self->modifier_count;
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp4_ == ((guint) 0)) {
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp3_ = TRUE;
#line 2389 "history-list.c"
	} else {
		GdkEventKey _tmp5_;
		guint _tmp6_;
		guint _tmp7_;
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp5_ = *event_key;
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp6_ = _tmp5_.keyval;
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp7_ = self->escKeyval;
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp3_ = _tmp6_ == _tmp7_;
#line 2402 "history-list.c"
	}
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp3_) {
#line 2406 "history-list.c"
		MidoriBrowser* _tmp8_;
		gulong* _tmp9_;
		gint _tmp9__length1;
		gulong _tmp10_;
		MidoriBrowser* _tmp11_;
		gulong* _tmp12_;
		gint _tmp12__length1;
		gulong _tmp13_;
		guint _tmp14_;
		HistoryListHistoryWindow* _tmp17_;
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp8_ = browser;
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9_ = self->tmp_sig_ids;
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9__length1 = self->tmp_sig_ids_length1;
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp10_ = _tmp9_[0];
#line 414 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_signal_handler_disconnect ((GObject*) _tmp8_, _tmp10_);
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = browser;
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12_ = self->tmp_sig_ids;
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12__length1 = self->tmp_sig_ids_length1;
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = _tmp12_[1];
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_signal_handler_disconnect ((GObject*) _tmp11_, _tmp13_);
#line 416 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = self->modifier_count;
#line 416 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp14_ == ((guint) 0)) {
#line 2441 "history-list.c"
			HistoryListHistoryWindow* _tmp15_;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp15_ = self->history_window;
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_history_window_make_update (_tmp15_);
#line 2447 "history-list.c"
		} else {
			HistoryListHistoryWindow* _tmp16_;
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			self->modifier_count = (guint) 0;
#line 420 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp16_ = self->history_window;
#line 420 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_history_window_clean_up (_tmp16_);
#line 2456 "history-list.c"
		}
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp17_ = self->history_window;
#line 422 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		gtk_object_destroy ((GtkObject*) _tmp17_);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (self->history_window);
#line 423 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->history_window = NULL;
#line 2466 "history-list.c"
	} else {
		GdkEventKey _tmp18_;
		guint _tmp19_;
		guint _tmp20_;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp18_ = *event_key;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp19_ = _tmp18_.keyval;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp20_ = self->delKeyval;
#line 424 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp19_ == _tmp20_) {
#line 2479 "history-list.c"
			HistoryListHistoryWindow* _tmp21_;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21_ = self->history_window;
#line 425 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_history_window_close_tab (_tmp21_);
#line 2485 "history-list.c"
		}
	}
#line 427 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = FALSE;
#line 427 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2492 "history-list.c"
}


static Block1Data* block1_data_ref (Block1Data* _data1_) {
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_atomic_int_inc (&_data1_->_ref_count_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return _data1_;
#line 2501 "history-list.c"
}


static void block1_data_unref (void * _userdata_) {
	Block1Data* _data1_;
	_data1_ = (Block1Data*) _userdata_;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (g_atomic_int_dec_and_test (&_data1_->_ref_count_)) {
#line 2510 "history-list.c"
		HistoryListManager* self;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self = _data1_->self;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (_data1_->browser);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (self);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_slice_free (Block1Data, _data1_);
#line 2520 "history-list.c"
	}
}


static gboolean ____lambda4_ (HistoryListManager* self, GdkEventKey* ek) {
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	gboolean _tmp1_;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (ek != NULL, FALSE);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = *ek;
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = history_list_manager_key_press (self, &_tmp0_);
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = _tmp1_;
#line 442 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2539 "history-list.c"
}


static gboolean _____lambda4__gtk_widget_key_press_event (GtkWidget* _sender, GdkEventKey* event, gpointer self) {
	gboolean result;
	result = ____lambda4_ ((HistoryListManager*) self, event);
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2548 "history-list.c"
}


static gboolean ____lambda5_ (Block1Data* _data1_, GdkEventKey* ek) {
	HistoryListManager* self;
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	MidoriBrowser* _tmp1_;
	gboolean _tmp2_;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data1_->self;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_val_if_fail (ek != NULL, FALSE);
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = *ek;
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data1_->browser;
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = history_list_manager_key_release (self, &_tmp0_, _tmp1_);
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = _tmp2_;
#line 445 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2572 "history-list.c"
}


static gboolean _____lambda5__gtk_widget_key_release_event (GtkWidget* _sender, GdkEventKey* event, gpointer self) {
	gboolean result;
	result = ____lambda5_ (self, event);
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 2581 "history-list.c"
}


void history_list_manager_walk (HistoryListManager* self, GtkAction* action, MidoriBrowser* browser, GType type, gint step) {
	Block1Data* _data1_;
	MidoriBrowser* _tmp0_;
	MidoriBrowser* _tmp1_;
	MidoriView* view = NULL;
	MidoriBrowser* _tmp2_;
	gconstpointer _tmp3_;
	MidoriView* _tmp4_;
	MidoriView* _tmp5_;
	gboolean _tmp9_ = FALSE;
	HistoryListHistoryWindow* _tmp10_;
	HistoryListHistoryWindow* hw = NULL;
	HistoryListHistoryWindow* _tmp32_;
	HistoryListHistoryWindow* _tmp33_;
	gint _tmp34_;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (action != NULL);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data1_ = g_slice_new0 (Block1Data);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data1_->_ref_count_ = 1;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data1_->self = g_object_ref (self);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (_data1_->browser);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data1_->browser = _tmp1_;
#line 431 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	view = NULL;
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = _data1_->browser;
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = g_object_get_data ((GObject*) _tmp2_, "history-list-last-change");
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = _g_object_ref0 ((MidoriView*) _tmp3_);
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (view);
#line 432 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	view = _tmp4_;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = view;
#line 433 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp5_ != NULL) {
#line 2636 "history-list.c"
		MidoriBrowser* _tmp6_;
		MidoriView* _tmp7_;
		MidoriBrowser* _tmp8_;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp6_ = _data1_->browser;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp7_ = view;
#line 434 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		history_list_manager_tab_list_resort (self, _tmp6_, _tmp7_);
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp8_ = _data1_->browser;
#line 435 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_set_data_full ((GObject*) _tmp8_, "history-list-last-change", NULL, g_object_unref);
#line 2650 "history-list.c"
	}
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = self->history_window;
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp10_ == NULL) {
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9_ = TRUE;
#line 2658 "history-list.c"
	} else {
		HistoryListHistoryWindow* _tmp11_;
		GType _tmp12_;
		GType _tmp13_;
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = self->history_window;
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12_ = G_TYPE_FROM_INSTANCE ((GObject*) _tmp11_);
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp13_ = type;
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp9_ = _tmp12_ != _tmp13_;
#line 2671 "history-list.c"
	}
#line 438 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp9_) {
#line 2675 "history-list.c"
		HistoryListHistoryWindow* _tmp14_;
		GType _tmp26_;
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = self->history_window;
#line 439 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp14_ == NULL) {
#line 2682 "history-list.c"
			GtkAction* _tmp15_;
			guint _tmp16_;
			gulong* _tmp17_;
			gint _tmp17__length1;
			MidoriBrowser* _tmp18_;
			gulong _tmp19_;
			gulong _tmp20_;
			gulong* _tmp21_;
			gint _tmp21__length1;
			MidoriBrowser* _tmp22_;
			gulong _tmp23_;
			gulong _tmp24_;
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp15_ = action;
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp16_ = sokoke_gtk_action_count_modifiers (_tmp15_);
#line 440 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			self->modifier_count = _tmp16_;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp17_ = self->tmp_sig_ids;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp17__length1 = self->tmp_sig_ids_length1;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp18_ = _data1_->browser;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp19_ = g_signal_connect_object ((GtkWidget*) _tmp18_, "key-press-event", (GCallback) _____lambda4__gtk_widget_key_press_event, self, 0);
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp17_[0] = _tmp19_;
#line 441 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp20_ = _tmp17_[0];
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21_ = self->tmp_sig_ids;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21__length1 = self->tmp_sig_ids_length1;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp22_ = _data1_->browser;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp23_ = g_signal_connect_data ((GtkWidget*) _tmp22_, "key-release-event", (GCallback) _____lambda5__gtk_widget_key_release_event, block1_data_ref (_data1_), (GClosureNotify) block1_data_unref, 0);
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21_[1] = _tmp23_;
#line 444 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp24_ = _tmp21_[1];
#line 2725 "history-list.c"
		} else {
			HistoryListHistoryWindow* _tmp25_;
#line 448 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp25_ = self->history_window;
#line 448 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_object_destroy ((GtkObject*) _tmp25_);
#line 449 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_g_object_unref0 (self->history_window);
#line 449 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			self->history_window = NULL;
#line 2736 "history-list.c"
		}
#line 455 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp26_ = type;
#line 455 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp26_ == HISTORY_LIST_TYPE_TAB_WINDOW) {
#line 2742 "history-list.c"
			MidoriBrowser* _tmp27_;
			HistoryListTabWindow* _tmp28_;
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp27_ = _data1_->browser;
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp28_ = history_list_tab_window_new (_tmp27_);
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			g_object_ref_sink (_tmp28_);
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_g_object_unref0 (self->history_window);
#line 456 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			self->history_window = (HistoryListHistoryWindow*) _tmp28_;
#line 2755 "history-list.c"
		} else {
			GType _tmp29_;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp29_ = type;
#line 457 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			if (_tmp29_ == HISTORY_LIST_TYPE_NEW_TAB_WINDOW) {
#line 2762 "history-list.c"
				MidoriBrowser* _tmp30_;
				HistoryListNewTabWindow* _tmp31_;
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp30_ = _data1_->browser;
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp31_ = history_list_new_tab_window_new (_tmp30_);
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				g_object_ref_sink (_tmp31_);
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (self->history_window);
#line 458 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				self->history_window = (HistoryListHistoryWindow*) _tmp31_;
#line 2775 "history-list.c"
			}
		}
	}
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp32_ = self->history_window;
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp33_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp32_, HISTORY_LIST_TYPE_HISTORY_WINDOW) ? ((HistoryListHistoryWindow*) _tmp32_) : NULL);
#line 461 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	hw = _tmp33_;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp34_ = step;
#line 462 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_history_window_walk (hw, _tmp34_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (hw);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (view);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	block1_data_unref (_data1_);
#line 430 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data1_ = NULL;
#line 2797 "history-list.c"
}


void history_list_manager_special_function (HistoryListManager* self, GtkAction* action, MidoriBrowser* browser) {
	HistoryListHistoryWindow* _tmp0_;
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (action != NULL);
#line 465 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 466 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = self->history_window;
#line 466 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp0_ != NULL) {
#line 2813 "history-list.c"
		HistoryListHistoryWindow* _tmp1_;
#line 467 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->ignoreNextChange = TRUE;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = self->history_window;
#line 468 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		history_list_history_window_make_update (_tmp1_);
#line 2821 "history-list.c"
	}
}


static Block2Data* block2_data_ref (Block2Data* _data2_) {
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_atomic_int_inc (&_data2_->_ref_count_);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return _data2_;
#line 2831 "history-list.c"
}


static void block2_data_unref (void * _userdata_) {
	Block2Data* _data2_;
	_data2_ = (Block2Data*) _userdata_;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (g_atomic_int_dec_and_test (&_data2_->_ref_count_)) {
#line 2840 "history-list.c"
		HistoryListManager* self;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self = _data2_->self;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (_data2_->browser);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (self);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_slice_free (Block2Data, _data2_);
#line 2850 "history-list.c"
	}
}


static void __lambda6_ (Block2Data* _data2_, GtkAction* a) {
	HistoryListManager* self;
	GtkAction* _tmp0_;
	MidoriBrowser* _tmp1_;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data2_->self;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (a != NULL);
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = a;
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data2_->browser;
#line 483 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_walk (self, _tmp0_, _tmp1_, HISTORY_LIST_TYPE_TAB_WINDOW, 1);
#line 2869 "history-list.c"
}


static void ___lambda6__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	__lambda6_ (self, _sender);
#line 2876 "history-list.c"
}


static void __lambda7_ (Block2Data* _data2_, GtkAction* a) {
	HistoryListManager* self;
	GtkAction* _tmp0_;
	MidoriBrowser* _tmp1_;
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data2_->self;
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (a != NULL);
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = a;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data2_->browser;
#line 489 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_walk (self, _tmp0_, _tmp1_, HISTORY_LIST_TYPE_TAB_WINDOW, -1);
#line 2894 "history-list.c"
}


static void ___lambda7__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	__lambda7_ (self, _sender);
#line 2901 "history-list.c"
}


static void __lambda8_ (Block2Data* _data2_, GtkAction* a) {
	HistoryListManager* self;
	GtkAction* _tmp0_;
	MidoriBrowser* _tmp1_;
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data2_->self;
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (a != NULL);
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = a;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data2_->browser;
#line 496 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_walk (self, _tmp0_, _tmp1_, HISTORY_LIST_TYPE_NEW_TAB_WINDOW, 1);
#line 2919 "history-list.c"
}


static void ___lambda8__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	__lambda8_ (self, _sender);
#line 2926 "history-list.c"
}


static void __lambda9_ (Block2Data* _data2_, GtkAction* a) {
	HistoryListManager* self;
	GtkAction* _tmp0_;
	MidoriBrowser* _tmp1_;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data2_->self;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (a != NULL);
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = a;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data2_->browser;
#line 506 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_walk (self, _tmp0_, _tmp1_, HISTORY_LIST_TYPE_NEW_TAB_WINDOW, -1);
#line 2944 "history-list.c"
}


static void ___lambda9__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	__lambda9_ (self, _sender);
#line 2951 "history-list.c"
}


static void __lambda10_ (Block2Data* _data2_, GtkAction* a) {
	HistoryListManager* self;
	GtkAction* _tmp0_;
	MidoriBrowser* _tmp1_;
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = _data2_->self;
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (a != NULL);
#line 516 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = a;
#line 516 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _data2_->browser;
#line 516 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_special_function (self, _tmp0_, _tmp1_);
#line 2969 "history-list.c"
}


static void ___lambda10__gtk_action_activate (GtkAction* _sender, gpointer self) {
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	__lambda10_ (self, _sender);
#line 2976 "history-list.c"
}


static void _history_list_manager_tab_added_midori_browser_add_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self) {
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_tab_added ((HistoryListManager*) self, _sender, tab);
#line 2983 "history-list.c"
}


static void _history_list_manager_tab_removed_midori_browser_remove_tab (MidoriBrowser* _sender, MidoriView* tab, gpointer self) {
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_tab_removed ((HistoryListManager*) self, _sender, tab);
#line 2990 "history-list.c"
}


static void _history_list_manager_tab_changed_midori_browser_switch_tab (MidoriBrowser* _sender, MidoriView* old_view, MidoriView* new_view, gpointer self) {
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_tab_changed ((HistoryListManager*) self, old_view, new_view);
#line 2997 "history-list.c"
}


static void history_list_manager_browser_added (HistoryListManager* self, MidoriBrowser* browser) {
	Block2Data* _data2_;
	MidoriBrowser* _tmp0_;
	MidoriBrowser* _tmp1_;
	gulong sidTabNext = 0UL;
	gulong sidTabPrevious = 0UL;
	GtkAccelGroup* acg = NULL;
	GtkAccelGroup* _tmp2_;
	MidoriBrowser* _tmp3_;
	GtkAccelGroup* _tmp4_;
	GtkActionGroup* action_group = NULL;
	MidoriBrowser* _tmp5_;
	GtkActionGroup* _tmp6_;
	GtkActionGroup* _tmp7_;
	GtkAction* action = NULL;
	GtkActionGroup* _tmp8_;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	MidoriBrowser* _tmp11_;
	GtkAction* _tmp12_;
	GtkAction* _tmp13_;
	gulong _tmp14_;
	GtkActionGroup* _tmp15_;
	GtkAction* _tmp16_;
	GtkAction* _tmp17_;
	MidoriBrowser* _tmp18_;
	GtkAction* _tmp19_;
	GtkAction* _tmp20_;
	gulong _tmp21_;
	GtkAction* _tmp22_;
	GtkAction* _tmp23_;
	GtkActionGroup* _tmp24_;
	GtkAction* _tmp25_;
	GtkAction* _tmp26_;
	GtkAccelGroup* _tmp27_;
	GtkAction* _tmp28_;
	GtkAction* _tmp29_;
	GtkAction* _tmp30_;
	GtkActionGroup* _tmp31_;
	GtkAction* _tmp32_;
	GtkAction* _tmp33_;
	GtkAccelGroup* _tmp34_;
	GtkAction* _tmp35_;
	GtkAction* _tmp36_;
	GtkAction* _tmp37_;
	GtkActionGroup* _tmp38_;
	GtkAction* _tmp39_;
	GtkAction* _tmp40_;
	GtkAccelGroup* _tmp41_;
	GtkAction* _tmp42_;
	MidoriBrowser* _tmp43_;
	gulong _tmp44_;
	MidoriBrowser* _tmp45_;
	gulong _tmp46_;
	MidoriBrowser* _tmp47_;
	GPtrArray* _tmp48_;
	MidoriBrowser* _tmp49_;
	GPtrArray* _tmp50_;
	MidoriBrowser* _tmp51_;
	MidoriBrowser* _tmp52_;
	GList* _tmp53_;
	MidoriBrowser* _tmp56_;
	MidoriBrowser* _tmp57_;
	MidoriBrowser* _tmp58_;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data2_ = g_slice_new0 (Block2Data);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data2_->_ref_count_ = 1;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data2_->self = g_object_ref (self);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (_data2_->browser);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data2_->browser = _tmp1_;
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = gtk_accel_group_new ();
#line 474 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	acg = _tmp2_;
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = _data2_->browser;
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = acg;
#line 475 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_window_add_accel_group ((GtkWindow*) _tmp3_, _tmp4_);
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = _data2_->browser;
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = midori_browser_get_action_group (_tmp5_);
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = _g_object_ref0 (_tmp6_);
#line 476 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action_group = _tmp7_;
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = action_group;
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = gtk_action_group_get_action (_tmp8_, "TabNext");
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = _g_object_ref0 (_tmp9_);
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 480 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp10_;
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = _data2_->browser;
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = action;
#line 481 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	midori_browser_block_action (_tmp11_, _tmp12_);
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = action;
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp14_ = g_signal_connect_data (_tmp13_, "activate", (GCallback) ___lambda6__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 482 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	sidTabNext = _tmp14_;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = action_group;
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp16_ = gtk_action_group_get_action (_tmp15_, "TabPrevious");
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp17_ = _g_object_ref0 (_tmp16_);
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 486 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp17_;
#line 487 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp18_ = _data2_->browser;
#line 487 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp19_ = action;
#line 487 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	midori_browser_block_action (_tmp18_, _tmp19_);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp20_ = action;
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp21_ = g_signal_connect_data (_tmp20_, "activate", (GCallback) ___lambda7__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 488 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	sidTabPrevious = _tmp21_;
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp22_ = gtk_action_new ("HistoryListNextNewTab", _ ("Next new Tab (History List)"), _ ("Next new tab from history"), NULL);
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 492 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp22_;
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp23_ = action;
#line 495 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_data (_tmp23_, "activate", (GCallback) ___lambda8__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp24_ = action_group;
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp25_ = action;
#line 498 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_group_add_action_with_accel (_tmp24_, _tmp25_, "<Ctrl>1");
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp26_ = action;
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp27_ = acg;
#line 499 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_set_accel_group (_tmp26_, _tmp27_);
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp28_ = action;
#line 500 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_connect_accelerator (_tmp28_);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp29_ = gtk_action_new ("HistoryListPreviousNewTab", _ ("Previous new Tab (History List)"), _ ("Previous new tab from history"), NULL);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 502 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp29_;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp30_ = action;
#line 505 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_data (_tmp30_, "activate", (GCallback) ___lambda9__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp31_ = action_group;
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp32_ = action;
#line 508 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_group_add_action_with_accel (_tmp31_, _tmp32_, "<Ctrl>2");
#line 509 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp33_ = action;
#line 509 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp34_ = acg;
#line 509 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_set_accel_group (_tmp33_, _tmp34_);
#line 510 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp35_ = action;
#line 510 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_connect_accelerator (_tmp35_);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp36_ = gtk_action_new ("HistoryListSpecialFunction", _ ("Display tab in background (History List)"), _ ("Display the current selected tab in background"), NULL);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 512 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp36_;
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp37_ = action;
#line 515 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_data (_tmp37_, "activate", (GCallback) ___lambda10__gtk_action_activate, block2_data_ref (_data2_), (GClosureNotify) block2_data_unref, 0);
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp38_ = action_group;
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp39_ = action;
#line 518 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_group_add_action_with_accel (_tmp38_, _tmp39_, "<Ctrl>3");
#line 519 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp40_ = action;
#line 519 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp41_ = acg;
#line 519 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_set_accel_group (_tmp40_, _tmp41_);
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp42_ = action;
#line 520 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_action_connect_accelerator (_tmp42_);
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp43_ = _data2_->browser;
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp44_ = sidTabNext;
#line 522 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set_data_full ((GObject*) _tmp43_, "history-list-sid-tab-next", (gpointer) ((guintptr) _tmp44_), NULL);
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp45_ = _data2_->browser;
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp46_ = sidTabPrevious;
#line 523 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set_data_full ((GObject*) _tmp45_, "history-list-sid-tab-previous", (gpointer) ((guintptr) _tmp46_), NULL);
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp47_ = _data2_->browser;
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp48_ = g_ptr_array_new ();
#line 525 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set_data_full ((GObject*) _tmp47_, "history-list-tab-history", _tmp48_, NULL);
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp49_ = _data2_->browser;
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp50_ = g_ptr_array_new ();
#line 527 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set_data_full ((GObject*) _tmp49_, "history-list-tab-history-new", _tmp50_, NULL);
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp51_ = _data2_->browser;
#line 529 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_set_data_full ((GObject*) _tmp51_, "history-list-last-change", NULL, g_object_unref);
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp52_ = _data2_->browser;
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp53_ = midori_browser_get_tabs (_tmp52_);
#line 3255 "history-list.c"
	{
		GList* tab_collection = NULL;
		GList* tab_it = NULL;
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		tab_collection = _tmp53_;
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		for (tab_it = tab_collection; tab_it != NULL; tab_it = tab_it->next) {
#line 3263 "history-list.c"
			MidoriView* tab = NULL;
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			tab = (MidoriView*) tab_it->data;
#line 3267 "history-list.c"
			{
				MidoriBrowser* _tmp54_;
				MidoriView* _tmp55_;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp54_ = _data2_->browser;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp55_ = tab;
#line 532 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				history_list_manager_tab_added (self, _tmp54_, _tmp55_);
#line 3277 "history-list.c"
			}
		}
#line 531 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_list_free0 (tab_collection);
#line 3282 "history-list.c"
	}
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp56_ = _data2_->browser;
#line 533 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (_tmp56_, "add-tab", (GCallback) _history_list_manager_tab_added_midori_browser_add_tab, self, 0);
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp57_ = _data2_->browser;
#line 534 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (_tmp57_, "remove-tab", (GCallback) _history_list_manager_tab_removed_midori_browser_remove_tab, self, 0);
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp58_ = _data2_->browser;
#line 535 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (_tmp58_, "switch-tab", (GCallback) _history_list_manager_tab_changed_midori_browser_switch_tab, self, 0);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action_group);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (acg);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	block2_data_unref (_data2_);
#line 472 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_data2_ = NULL;
#line 3306 "history-list.c"
}


static void history_list_manager_browser_removed (HistoryListManager* self, MidoriBrowser* browser) {
	gchar** callbacks = NULL;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar** _tmp3_;
	gint callbacks_length1;
	gint _callbacks_size_;
	gulong sidTabNext = 0UL;
	gulong sidTabPrevious = 0UL;
	MidoriBrowser* _tmp4_;
	gconstpointer _tmp5_;
	MidoriBrowser* _tmp6_;
	gconstpointer _tmp7_;
	GtkAction* action = NULL;
	GtkActionGroup* action_group = NULL;
	MidoriBrowser* _tmp8_;
	GtkActionGroup* _tmp9_;
	GtkActionGroup* _tmp10_;
	GtkActionGroup* _tmp11_;
	GtkAction* _tmp12_;
	GtkAction* _tmp13_;
	GtkAction* _tmp14_;
	gulong _tmp15_;
	MidoriBrowser* _tmp16_;
	GtkAction* _tmp17_;
	GtkActionGroup* _tmp18_;
	GtkAction* _tmp19_;
	GtkAction* _tmp20_;
	GtkAction* _tmp21_;
	gulong _tmp22_;
	MidoriBrowser* _tmp23_;
	GtkAction* _tmp24_;
	MidoriBrowser* _tmp38_;
	guint _tmp39_;
	MidoriBrowser* _tmp40_;
	guint _tmp41_;
	MidoriBrowser* _tmp42_;
	guint _tmp43_;
#line 538 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 538 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = g_strdup ("HistoryListNextNewTab");
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = g_strdup ("HistoryListPreviousNewTab");
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = g_strdup ("HistoryListSpecialFunction");
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = g_new0 (gchar*, 3 + 1);
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_[0] = _tmp0_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_[1] = _tmp1_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_[2] = _tmp2_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	callbacks = _tmp3_;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	callbacks_length1 = 3;
#line 539 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_callbacks_size_ = callbacks_length1;
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = browser;
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = g_object_get_data ((GObject*) _tmp4_, "history-list-sid-tab-next");
#line 542 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	sidTabNext = (gulong) ((guintptr) _tmp5_);
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = browser;
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = g_object_get_data ((GObject*) _tmp6_, "history-list-sid-tab-previous");
#line 543 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	sidTabPrevious = (gulong) ((guintptr) _tmp7_);
#line 547 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = browser;
#line 547 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = midori_browser_get_action_group (_tmp8_);
#line 547 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = _g_object_ref0 (_tmp9_);
#line 547 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action_group);
#line 547 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action_group = _tmp10_;
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = action_group;
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = gtk_action_group_get_action (_tmp11_, "TabNext");
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = _g_object_ref0 (_tmp12_);
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 549 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp13_;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp14_ = action;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = sidTabNext;
#line 550 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handler_disconnect ((GObject*) _tmp14_, _tmp15_);
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp16_ = browser;
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp17_ = action;
#line 551 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	midori_browser_unblock_action (_tmp16_, _tmp17_);
#line 552 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp18_ = action_group;
#line 552 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp19_ = gtk_action_group_get_action (_tmp18_, "TabPrevious");
#line 552 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp20_ = _g_object_ref0 (_tmp19_);
#line 552 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 552 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	action = _tmp20_;
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp21_ = action;
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp22_ = sidTabPrevious;
#line 553 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handler_disconnect ((GObject*) _tmp21_, _tmp22_);
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp23_ = browser;
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp24_ = action;
#line 554 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	midori_browser_unblock_action (_tmp23_, _tmp24_);
#line 3439 "history-list.c"
	{
		gint i = 0;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		i = 0;
#line 3444 "history-list.c"
		{
			gboolean _tmp25_ = FALSE;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp25_ = TRUE;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			while (TRUE) {
#line 3451 "history-list.c"
				gint _tmp27_;
				gchar** _tmp28_;
				gint _tmp28__length1;
				GtkActionGroup* _tmp29_;
				gchar** _tmp30_;
				gint _tmp30__length1;
				gint _tmp31_;
				const gchar* _tmp32_;
				GtkAction* _tmp33_;
				GtkAction* _tmp34_;
				GtkAction* _tmp35_;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				if (!_tmp25_) {
#line 3465 "history-list.c"
					gint _tmp26_;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					_tmp26_ = i;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					i = _tmp26_ + 1;
#line 3471 "history-list.c"
				}
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp25_ = FALSE;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp27_ = i;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp28_ = callbacks;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp28__length1 = callbacks_length1;
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				if (!(_tmp27_ < _tmp28__length1)) {
#line 556 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					break;
#line 3485 "history-list.c"
				}
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp29_ = action_group;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp30_ = callbacks;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp30__length1 = callbacks_length1;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp31_ = i;
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp32_ = _tmp30_[_tmp31_];
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp33_ = gtk_action_group_get_action (_tmp29_, _tmp32_);
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp34_ = _g_object_ref0 (_tmp33_);
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (action);
#line 557 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				action = _tmp34_;
#line 558 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp35_ = action;
#line 558 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				if (_tmp35_ != NULL) {
#line 3509 "history-list.c"
					GtkActionGroup* _tmp36_;
					GtkAction* _tmp37_;
#line 559 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					_tmp36_ = action_group;
#line 559 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					_tmp37_ = action;
#line 559 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
					gtk_action_group_remove_action (_tmp36_, _tmp37_);
#line 3518 "history-list.c"
				}
			}
		}
	}
#line 562 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp38_ = browser;
#line 562 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_parse_name ("add-tab", MIDORI_TYPE_BROWSER, &_tmp39_, NULL, FALSE);
#line 562 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handlers_disconnect_matched (_tmp38_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp39_, 0, NULL, (GCallback) _history_list_manager_tab_added_midori_browser_add_tab, self);
#line 563 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp40_ = browser;
#line 563 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_parse_name ("remove-tab", MIDORI_TYPE_BROWSER, &_tmp41_, NULL, FALSE);
#line 563 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handlers_disconnect_matched (_tmp40_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp41_, 0, NULL, (GCallback) _history_list_manager_tab_removed_midori_browser_remove_tab, self);
#line 564 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp42_ = browser;
#line 564 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_parse_name ("switch-tab", MIDORI_TYPE_BROWSER, &_tmp43_, NULL, FALSE);
#line 564 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handlers_disconnect_matched (_tmp42_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp43_, 0, NULL, (GCallback) _history_list_manager_tab_changed_midori_browser_switch_tab, self);
#line 538 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action_group);
#line 538 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (action);
#line 538 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	callbacks = (_vala_array_free (callbacks, callbacks_length1, (GDestroyNotify) g_free), NULL);
#line 3547 "history-list.c"
}


static void history_list_manager_tab_added (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view) {
	GPtrArray* list = NULL;
	MidoriBrowser* _tmp0_;
	gconstpointer _tmp1_;
	MidoriView* _tmp2_;
#line 567 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 567 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 567 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (view != NULL);
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = g_object_get_data ((GObject*) _tmp0_, "history-list-tab-history-new");
#line 568 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list = (GPtrArray*) _tmp1_;
#line 569 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = view;
#line 569 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_add (list, _tmp2_);
#line 3572 "history-list.c"
}


static void history_list_manager_tab_removed (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view) {
	GPtrArray* list = NULL;
	MidoriBrowser* _tmp0_;
	gconstpointer _tmp1_;
	GPtrArray* list_new = NULL;
	MidoriBrowser* _tmp2_;
	gconstpointer _tmp3_;
	GPtrArray* _tmp4_;
	MidoriView* _tmp5_;
	GPtrArray* _tmp6_;
	MidoriView* _tmp7_;
	MidoriView* current_view = NULL;
	MidoriBrowser* _tmp8_;
	GtkWidget* _tmp9_;
	GtkWidget* _tmp10_;
	MidoriView* _tmp11_;
	MidoriView* _tmp12_;
	MidoriView* _tmp13_;
	gboolean _tmp14_ = FALSE;
	gint _tmp15_;
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (view != NULL);
#line 573 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 573 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = g_object_get_data ((GObject*) _tmp0_, "history-list-tab-history");
#line 573 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list = (GPtrArray*) _tmp1_;
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = browser;
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = g_object_get_data ((GObject*) _tmp2_, "history-list-tab-history-new");
#line 574 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list_new = (GPtrArray*) _tmp3_;
#line 575 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = list;
#line 575 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = view;
#line 575 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_remove (_tmp4_, _tmp5_);
#line 576 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = list_new;
#line 576 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = view;
#line 576 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_remove (_tmp6_, _tmp7_);
#line 578 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp8_ = browser;
#line 578 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp9_ = midori_browser_get_tab (_tmp8_);
#line 578 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp10_ = _tmp9_;
#line 578 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp11_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp10_, MIDORI_TYPE_VIEW) ? ((MidoriView*) _tmp10_) : NULL);
#line 578 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	current_view = _tmp11_;
#line 580 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp12_ = current_view;
#line 580 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp13_ = view;
#line 580 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp12_ != _tmp13_) {
#line 581 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (current_view);
#line 581 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		return;
#line 3646 "history-list.c"
	}
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp15_ = self->closing_behavior;
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp15_ == ((gint) HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST)) {
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = TRUE;
#line 3654 "history-list.c"
	} else {
		gint _tmp16_;
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp16_ = self->closing_behavior;
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp14_ = _tmp16_ == ((gint) HISTORY_LIST_TAB_CLOSING_BEHAVIOR_NEW);
#line 3661 "history-list.c"
	}
#line 583 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp14_) {
#line 3665 "history-list.c"
		MidoriBrowser* _tmp17_;
		gboolean _tmp18_ = FALSE;
		GPtrArray* _tmp19_;
		guint _tmp20_;
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp17_ = browser;
#line 584 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_set_data_full ((GObject*) _tmp17_, "history-list-last-change", NULL, g_object_unref);
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp19_ = list;
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp20_ = _tmp19_->len;
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (((gint) _tmp20_) > 0) {
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp18_ = TRUE;
#line 3682 "history-list.c"
		} else {
			GPtrArray* _tmp21_;
			guint _tmp22_;
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp21_ = list_new;
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp22_ = _tmp21_->len;
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp18_ = ((gint) _tmp22_) > 0;
#line 3692 "history-list.c"
		}
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp18_) {
#line 3696 "history-list.c"
			HistoryListTabWindow* hw = NULL;
			gint _tmp23_;
			HistoryListTabWindow* _tmp28_;
			HistoryListTabWindow* _tmp29_;
#line 588 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp23_ = self->closing_behavior;
#line 588 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			if (_tmp23_ == ((gint) HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST)) {
#line 3705 "history-list.c"
				MidoriBrowser* _tmp24_;
				HistoryListTabWindow* _tmp25_;
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp24_ = browser;
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp25_ = history_list_tab_window_new (_tmp24_);
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				g_object_ref_sink (_tmp25_);
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (hw);
#line 589 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				hw = _tmp25_;
#line 3718 "history-list.c"
			} else {
				MidoriBrowser* _tmp26_;
				HistoryListNewTabWindow* _tmp27_;
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp26_ = browser;
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp27_ = history_list_new_tab_window_new (_tmp26_);
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				g_object_ref_sink (_tmp27_);
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_g_object_unref0 (hw);
#line 591 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				hw = (HistoryListTabWindow*) _tmp27_;
#line 3732 "history-list.c"
			}
#line 592 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp28_ = hw;
#line 592 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_history_window_make_update ((HistoryListHistoryWindow*) _tmp28_);
#line 593 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp29_ = hw;
#line 593 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			gtk_object_destroy ((GtkObject*) _tmp29_);
#line 586 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_g_object_unref0 (hw);
#line 3744 "history-list.c"
		}
	}
#line 572 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (current_view);
#line 3749 "history-list.c"
}


static void history_list_manager_tab_changed (HistoryListManager* self, MidoriView* old_view, MidoriView* new_view) {
	gboolean _tmp0_;
#line 598 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 599 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = self->ignoreNextChange;
#line 599 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	if (_tmp0_) {
#line 600 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		self->ignoreNextChange = FALSE;
#line 3763 "history-list.c"
	} else {
		MidoriBrowser* browser = NULL;
		MidoriView* _tmp1_;
		MidoriBrowser* _tmp2_;
		MidoriBrowser* _tmp3_;
		MidoriView* last_view = NULL;
		MidoriBrowser* _tmp4_;
		gconstpointer _tmp5_;
		MidoriView* _tmp6_;
		MidoriView* _tmp7_;
		MidoriBrowser* _tmp10_;
		MidoriView* _tmp11_;
		MidoriView* _tmp12_;
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp1_ = new_view;
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp2_ = midori_browser_get_for_widget ((GtkWidget*) _tmp1_);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp3_ = _g_object_ref0 (_tmp2_);
#line 602 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		browser = _tmp3_;
#line 603 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp4_ = browser;
#line 603 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp5_ = g_object_get_data ((GObject*) _tmp4_, "history-list-last-change");
#line 603 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp6_ = _g_object_ref0 ((MidoriView*) _tmp5_);
#line 603 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		last_view = _tmp6_;
#line 606 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp7_ = last_view;
#line 606 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		if (_tmp7_ != NULL) {
#line 3797 "history-list.c"
			MidoriBrowser* _tmp8_;
			MidoriView* _tmp9_;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp8_ = browser;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			_tmp9_ = last_view;
#line 607 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			history_list_manager_tab_list_resort (self, _tmp8_, _tmp9_);
#line 3806 "history-list.c"
		}
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp10_ = browser;
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp11_ = new_view;
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_tmp12_ = _g_object_ref0 (_tmp11_);
#line 609 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		g_object_set_data_full ((GObject*) _tmp10_, "history-list-last-change", _tmp12_, g_object_unref);
#line 599 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (last_view);
#line 599 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_object_unref0 (browser);
#line 3820 "history-list.c"
	}
}


static void history_list_manager_tab_list_resort (HistoryListManager* self, MidoriBrowser* browser, MidoriView* view) {
	GPtrArray* list = NULL;
	MidoriBrowser* _tmp0_;
	gconstpointer _tmp1_;
	GPtrArray* list_new = NULL;
	MidoriBrowser* _tmp2_;
	gconstpointer _tmp3_;
	MidoriView* _tmp4_;
	MidoriView* _tmp5_;
	MidoriView* _tmp6_;
#line 613 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 613 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (browser != NULL);
#line 613 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (view != NULL);
#line 614 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = browser;
#line 614 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = g_object_get_data ((GObject*) _tmp0_, "history-list-tab-history");
#line 614 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list = (GPtrArray*) _tmp1_;
#line 615 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = browser;
#line 615 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = g_object_get_data ((GObject*) _tmp2_, "history-list-tab-history-new");
#line 615 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	list_new = (GPtrArray*) _tmp3_;
#line 616 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = view;
#line 616 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_remove (list, _tmp4_);
#line 617 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = view;
#line 617 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_remove (list_new, _tmp5_);
#line 618 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp6_ = view;
#line 618 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_ptr_array_add (list, _tmp6_);
#line 3865 "history-list.c"
}


static void _history_list_manager_browser_added_midori_app_add_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self) {
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_browser_added ((HistoryListManager*) self, browser);
#line 3872 "history-list.c"
}


static void _history_list_manager_browser_removed_midori_app_remove_browser (MidoriApp* _sender, MidoriBrowser* browser, gpointer self) {
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_browser_removed ((HistoryListManager*) self, browser);
#line 3879 "history-list.c"
}


static void history_list_manager_activated (HistoryListManager* self, MidoriApp* app) {
	MidoriApp* _tmp0_;
	GList* _tmp1_;
	MidoriApp* _tmp3_;
	MidoriApp* _tmp4_;
#line 621 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 621 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (app != NULL);
#line 622 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_emit (self, history_list_manager_signals[HISTORY_LIST_MANAGER_PREFERENCES_CHANGED_SIGNAL], 0);
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = app;
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = midori_app_get_browsers (_tmp0_);
#line 3898 "history-list.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		browser_collection = _tmp1_;
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 3906 "history-list.c"
			MidoriBrowser* browser = NULL;
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 3910 "history-list.c"
			{
				MidoriBrowser* _tmp2_;
#line 624 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp2_ = browser;
#line 624 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				history_list_manager_browser_added (self, _tmp2_);
#line 3917 "history-list.c"
			}
		}
#line 623 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_list_free0 (browser_collection);
#line 3922 "history-list.c"
	}
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = app;
#line 625 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (_tmp3_, "add-browser", (GCallback) _history_list_manager_browser_added_midori_app_add_browser, self, 0);
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp4_ = app;
#line 626 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (_tmp4_, "remove-browser", (GCallback) _history_list_manager_browser_removed_midori_app_remove_browser, self, 0);
#line 3932 "history-list.c"
}


static void history_list_manager_deactivated (HistoryListManager* self) {
	MidoriApp* app = NULL;
	MidoriApp* _tmp0_;
	MidoriApp* _tmp1_;
	MidoriApp* _tmp2_;
	GList* _tmp3_;
	MidoriApp* _tmp5_;
	guint _tmp6_;
	MidoriApp* _tmp7_;
	guint _tmp8_;
#line 629 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 630 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = midori_extension_get_app ((MidoriExtension*) self);
#line 630 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 630 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	app = _tmp1_;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp2_ = app;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp3_ = midori_app_get_browsers (_tmp2_);
#line 3958 "history-list.c"
	{
		GList* browser_collection = NULL;
		GList* browser_it = NULL;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		browser_collection = _tmp3_;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		for (browser_it = browser_collection; browser_it != NULL; browser_it = browser_it->next) {
#line 3966 "history-list.c"
			MidoriBrowser* browser = NULL;
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
			browser = (MidoriBrowser*) browser_it->data;
#line 3970 "history-list.c"
			{
				MidoriBrowser* _tmp4_;
#line 632 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				_tmp4_ = browser;
#line 632 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
				history_list_manager_browser_removed (self, _tmp4_);
#line 3977 "history-list.c"
			}
		}
#line 631 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
		_g_list_free0 (browser_collection);
#line 3982 "history-list.c"
	}
#line 633 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp5_ = app;
#line 633 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_parse_name ("add-browser", MIDORI_TYPE_APP, &_tmp6_, NULL, FALSE);
#line 633 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handlers_disconnect_matched (_tmp5_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp6_, 0, NULL, (GCallback) _history_list_manager_browser_added_midori_app_add_browser, self);
#line 634 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp7_ = app;
#line 634 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_parse_name ("remove-browser", MIDORI_TYPE_APP, &_tmp8_, NULL, FALSE);
#line 634 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_handlers_disconnect_matched (_tmp7_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp8_, 0, NULL, (GCallback) _history_list_manager_browser_removed_midori_app_remove_browser, self);
#line 629 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (app);
#line 3998 "history-list.c"
}


static void history_list_manager_show_preferences (HistoryListManager* self) {
	HistoryListPreferencesDialog* dialog = NULL;
	HistoryListPreferencesDialog* _tmp0_;
#line 637 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_return_if_fail (self != NULL);
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = history_list_preferences_dialog_new (self);
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_object_ref_sink (_tmp0_);
#line 638 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	dialog = _tmp0_;
#line 639 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	gtk_widget_show ((GtkWidget*) dialog);
#line 637 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (dialog);
#line 4017 "history-list.c"
}


static void _history_list_manager_activated_midori_extension_activate (MidoriExtension* _sender, MidoriApp* app, gpointer self) {
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_activated ((HistoryListManager*) self, app);
#line 4024 "history-list.c"
}


static void _history_list_manager_deactivated_midori_extension_deactivate (MidoriExtension* _sender, gpointer self) {
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_deactivated ((HistoryListManager*) self);
#line 4031 "history-list.c"
}


static void _history_list_manager_show_preferences_midori_extension_open_preferences (MidoriExtension* _sender, gpointer self) {
#line 652 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_show_preferences ((HistoryListManager*) self);
#line 4038 "history-list.c"
}


static void _history_list_manager_preferences_changed_cb_history_list_manager_preferences_changed (HistoryListManager* _sender, gpointer self) {
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_preferences_changed_cb ((HistoryListManager*) self);
#line 4045 "history-list.c"
}


HistoryListManager* history_list_manager_construct (GType object_type) {
	HistoryListManager * self = NULL;
#line 643 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = (HistoryListManager*) g_object_new (object_type, "name", _ ("History List"), "description", _ ("Move to the last used tab when switching or closing tabs"), "version", "0.4" MIDORI_VERSION_SUFFIX, "authors", "André Stösel <andre@stoesel.de>", NULL);
#line 648 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	midori_extension_install_integer ((MidoriExtension*) self, "TabClosingBehavior", (gint) HISTORY_LIST_TAB_CLOSING_BEHAVIOR_LAST);
#line 650 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object ((MidoriExtension*) self, "activate", (GCallback) _history_list_manager_activated_midori_extension_activate, self, 0);
#line 651 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object ((MidoriExtension*) self, "deactivate", (GCallback) _history_list_manager_deactivated_midori_extension_deactivate, self, 0);
#line 652 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object ((MidoriExtension*) self, "open-preferences", (GCallback) _history_list_manager_show_preferences_midori_extension_open_preferences, self, 0);
#line 653 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	g_signal_connect_object (self, "preferences-changed", (GCallback) _history_list_manager_preferences_changed_cb_history_list_manager_preferences_changed, self, 0);
#line 642 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return self;
#line 4065 "history-list.c"
}


HistoryListManager* history_list_manager_new (void) {
#line 642 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return history_list_manager_construct (HISTORY_LIST_TYPE_MANAGER);
#line 4072 "history-list.c"
}


static GObject * history_list_manager_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	HistoryListManager * self;
	guint _tmp0_;
	guint _tmp1_;
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	parent_class = G_OBJECT_CLASS (history_list_manager_parent_class);
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_MANAGER, HistoryListManager);
#line 656 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = gdk_keyval_from_name ("Escape");
#line 656 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->escKeyval = _tmp0_;
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp1_ = gdk_keyval_from_name ("Delete");
#line 657 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->delKeyval = _tmp1_;
#line 655 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return obj;
#line 4098 "history-list.c"
}


static void history_list_manager_class_init (HistoryListManagerClass * klass) {
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_parent_class = g_type_class_peek_parent (klass);
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->constructor = history_list_manager_constructor;
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (klass)->finalize = history_list_manager_finalize;
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	history_list_manager_signals[HISTORY_LIST_MANAGER_PREFERENCES_CHANGED_SIGNAL] = g_signal_new ("preferences-changed", HISTORY_LIST_TYPE_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 4111 "history-list.c"
}


static void history_list_manager_instance_init (HistoryListManager * self) {
	gulong* _tmp0_;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = g_new0 (gulong, 2);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->tmp_sig_ids = _tmp0_;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->tmp_sig_ids_length1 = 2;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->_tmp_sig_ids_size_ = self->tmp_sig_ids_length1;
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->ignoreNextChange = FALSE;
#line 4127 "history-list.c"
}


static void history_list_manager_finalize (GObject * obj) {
	HistoryListManager * self;
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, HISTORY_LIST_TYPE_MANAGER, HistoryListManager);
#line 382 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_g_object_unref0 (self->history_window);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	self->tmp_sig_ids = (g_free (self->tmp_sig_ids), NULL);
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	G_OBJECT_CLASS (history_list_manager_parent_class)->finalize (obj);
#line 4141 "history-list.c"
}


GType history_list_manager_get_type (void) {
	static volatile gsize history_list_manager_type_id__volatile = 0;
	if (g_once_init_enter (&history_list_manager_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (HistoryListManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) history_list_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HistoryListManager), 0, (GInstanceInitFunc) history_list_manager_instance_init, NULL };
		GType history_list_manager_type_id;
		history_list_manager_type_id = g_type_register_static (MIDORI_TYPE_EXTENSION, "HistoryListManager", &g_define_type_info, 0);
		g_once_init_leave (&history_list_manager_type_id__volatile, history_list_manager_type_id);
	}
	return history_list_manager_type_id__volatile;
}


MidoriExtension* extension_init (void) {
	MidoriExtension* result = NULL;
	HistoryListManager* _tmp0_;
#line 663 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	_tmp0_ = history_list_manager_new ();
#line 663 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	result = (MidoriExtension*) _tmp0_;
#line 663 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/extensions/history-list.vala"
	return result;
#line 4166 "history-list.c"
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



