/* midori-speeddial.c generated by valac 0.38.8, the Vala compiler
 * generated from midori-speeddial.vala, do not modify */

/*
 Copyright (C) 2011-2013 Christian Dywan <christian@twotoats.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "webkit/webkit.h"
#include <glib/gstdio.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <glib/gi18n-lib.h>
#include <gtk/gtk.h>
#include <float.h>
#include <math.h>
#include <gobject/gvaluecollector.h>


#define MIDORI_TYPE_SPEED_DIAL (midori_speed_dial_get_type ())
#define MIDORI_SPEED_DIAL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDial))
#define MIDORI_SPEED_DIAL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDialClass))
#define MIDORI_IS_SPEED_DIAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_TYPE_SPEED_DIAL))
#define MIDORI_IS_SPEED_DIAL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_TYPE_SPEED_DIAL))
#define MIDORI_SPEED_DIAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDialClass))

typedef struct _MidoriSpeedDial MidoriSpeedDial;
typedef struct _MidoriSpeedDialClass MidoriSpeedDialClass;
typedef struct _MidoriSpeedDialPrivate MidoriSpeedDialPrivate;

#define MIDORI_SPEED_DIAL_TYPE_SPEC (midori_speed_dial_spec_get_type ())
#define MIDORI_SPEED_DIAL_SPEC(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MIDORI_SPEED_DIAL_TYPE_SPEC, MidoriSpeedDialSpec))
#define MIDORI_SPEED_DIAL_SPEC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MIDORI_SPEED_DIAL_TYPE_SPEC, MidoriSpeedDialSpecClass))
#define MIDORI_SPEED_DIAL_IS_SPEC(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MIDORI_SPEED_DIAL_TYPE_SPEC))
#define MIDORI_SPEED_DIAL_IS_SPEC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MIDORI_SPEED_DIAL_TYPE_SPEC))
#define MIDORI_SPEED_DIAL_SPEC_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), MIDORI_SPEED_DIAL_TYPE_SPEC, MidoriSpeedDialSpecClass))

typedef struct _MidoriSpeedDialSpec MidoriSpeedDialSpec;
typedef struct _MidoriSpeedDialSpecClass MidoriSpeedDialSpecClass;
enum  {
	MIDORI_SPEED_DIAL_0_PROPERTY,
	MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY,
	MIDORI_SPEED_DIAL_NUM_PROPERTIES
};
static GParamSpec* midori_speed_dial_properties[MIDORI_SPEED_DIAL_NUM_PROPERTIES];
#define _g_free0(var) (var = (g_free (var), NULL))
#define __g_list_free__midori_speed_dial_spec_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__midori_speed_dial_spec_unref0_ (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _midori_speed_dial_spec_unref0(var) ((var == NULL) ? NULL : (var = (midori_speed_dial_spec_unref (var), NULL)))
#define _g_key_file_unref0(var) ((var == NULL) ? NULL : (var = (g_key_file_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
typedef struct _MidoriSpeedDialSpecPrivate MidoriSpeedDialSpecPrivate;
typedef struct _MidoriSpeedDialParamSpecSpec MidoriSpeedDialParamSpecSpec;
enum  {
	MIDORI_SPEED_DIAL_REFRESH_SIGNAL,
	MIDORI_SPEED_DIAL_NUM_SIGNALS
};
static guint midori_speed_dial_signals[MIDORI_SPEED_DIAL_NUM_SIGNALS] = {0};

typedef enum  {
	MIDORI_SPEED_DIAL_ERROR_INVALID_MESSAGE,
	MIDORI_SPEED_DIAL_ERROR_NO_ACTION,
	MIDORI_SPEED_DIAL_ERROR_NO_ID,
	MIDORI_SPEED_DIAL_ERROR_NO_URL,
	MIDORI_SPEED_DIAL_ERROR_NO_TITLE,
	MIDORI_SPEED_DIAL_ERROR_NO_ID2,
	MIDORI_SPEED_DIAL_ERROR_INVALID_ACTION
} MidoriSpeedDialError;
#define MIDORI_SPEED_DIAL_ERROR midori_speed_dial_error_quark ()
struct _MidoriSpeedDial {
	GObject parent_instance;
	MidoriSpeedDialPrivate * priv;
	GKeyFile* keyfile;
};

struct _MidoriSpeedDialClass {
	GObjectClass parent_class;
};

struct _MidoriSpeedDialPrivate {
	gchar* filename;
	gchar* html;
	GList* thumb_queue;
	WebKitWebView* thumb_view;
	MidoriSpeedDialSpec* spec;
	gboolean _close_buttons_left;
};

struct _MidoriSpeedDialSpec {
	GTypeInstance parent_instance;
	volatile int ref_count;
	MidoriSpeedDialSpecPrivate * priv;
	gchar* dial_id;
	gchar* uri;
};

struct _MidoriSpeedDialSpecClass {
	GTypeClass parent_class;
	void (*finalize) (MidoriSpeedDialSpec *self);
};

struct _MidoriSpeedDialParamSpecSpec {
	GParamSpec parent_instance;
};


static gpointer midori_speed_dial_parent_class = NULL;
static gpointer midori_speed_dial_spec_parent_class = NULL;

gchar* katze_mkdir_with_parents (const gchar* pathname, gint mode);
gchar* sokoke_js_script_eval (void* ctx, const gchar* script, void* _error_);
GQuark midori_speed_dial_error_quark (void);
GType midori_speed_dial_get_type (void) G_GNUC_CONST;
gpointer midori_speed_dial_spec_ref (gpointer instance);
void midori_speed_dial_spec_unref (gpointer instance);
GParamSpec* midori_speed_dial_param_spec_spec (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void midori_speed_dial_value_set_spec (GValue* value, gpointer v_object);
void midori_speed_dial_value_take_spec (GValue* value, gpointer v_object);
gpointer midori_speed_dial_value_get_spec (const GValue* value);
GType midori_speed_dial_spec_get_type (void) G_GNUC_CONST;
#define MIDORI_SPEED_DIAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDialPrivate))
static void _midori_speed_dial_spec_unref0_ (gpointer var);
static inline void _g_list_free__midori_speed_dial_spec_unref0_ (GList* self);
MidoriSpeedDial* midori_speed_dial_new (const gchar* new_filename, const gchar* fallback);
MidoriSpeedDial* midori_speed_dial_construct (GType object_type, const gchar* new_filename, const gchar* fallback);
static gchar* midori_speed_dial_build_thumbnail_path (MidoriSpeedDial* self, const gchar* filename);
gchar* midori_speed_dial_get_next_free_slot (MidoriSpeedDial* self, guint* count);
void midori_speed_dial_add (MidoriSpeedDial* self, const gchar* uri, const gchar* title, GdkPixbuf* img);
void midori_speed_dial_save_message (MidoriSpeedDial* self, const gchar* message, GError** error);
void midori_speed_dial_add_with_id (MidoriSpeedDial* self, const gchar* id, const gchar* uri, const gchar* title, GdkPixbuf* img);
const gchar* midori_paths_get_cache_dir (void);
static void midori_speed_dial_save (MidoriSpeedDial* self);
const gchar* midori_speed_dial_get_html (MidoriSpeedDial* self, GError** error);
gchar* midori_paths_get_res_filename (const gchar* filename);
gboolean midori_paths_is_readonly (void);
gboolean midori_speed_dial_get_close_buttons_left (MidoriSpeedDial* self);
static void midori_speed_dial_get_thumb (MidoriSpeedDial* self, const gchar* dial_id, const gchar* uri);
static void midori_speed_dial_load_status (MidoriSpeedDial* self, GObject* thumb_view_, GParamSpec* pspec);
static void _midori_speed_dial_load_status_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static gboolean midori_speed_dial_save_thumbnail (MidoriSpeedDial* self);
static gboolean _midori_speed_dial_save_thumbnail_gsource_func (gpointer self);
MidoriSpeedDialSpec* midori_speed_dial_spec_new (const gchar* dial_id, const gchar* uri);
MidoriSpeedDialSpec* midori_speed_dial_spec_construct (GType object_type, const gchar* dial_id, const gchar* uri);
void midori_speed_dial_set_close_buttons_left (MidoriSpeedDial* self, gboolean value);
static void midori_speed_dial_spec_finalize (MidoriSpeedDialSpec * obj);
static void midori_speed_dial_finalize (GObject * obj);
static void _vala_midori_speed_dial_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_midori_speed_dial_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);


GQuark midori_speed_dial_error_quark (void) {
	return g_quark_from_static_string ("midori_speed_dial_error-quark");
}


static void _midori_speed_dial_spec_unref0_ (gpointer var) {
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	(var == NULL) ? NULL : (var = (midori_speed_dial_spec_unref (var), NULL));
#line 174 "midori-speeddial.c"
}


static inline void _g_list_free__midori_speed_dial_spec_unref0_ (GList* self) {
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_list_free_full (self, (GDestroyNotify) _midori_speed_dial_spec_unref0_);
#line 181 "midori-speeddial.c"
}


static gchar string_get (const gchar* self, glong index) {
	gchar result = '\0';
	glong _tmp0_;
	gchar _tmp1_;
#line 1118 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, '\0');
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = index;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = ((gchar*) self)[_tmp0_];
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp1_;
#line 1119 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 199 "midori-speeddial.c"
}


MidoriSpeedDial* midori_speed_dial_construct (GType object_type, const gchar* new_filename, const gchar* fallback) {
	MidoriSpeedDial * self = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	GKeyFile* _tmp2_;
	GError * _inner_error_ = NULL;
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (new_filename != NULL, NULL);
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self = (MidoriSpeedDial*) g_object_new (object_type, NULL);
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = new_filename;
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->priv->filename);
#line 54 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->filename = _tmp1_;
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	__g_list_free__midori_speed_dial_spec_unref0_0 (self->priv->thumb_queue);
#line 55 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->thumb_queue = NULL;
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = g_key_file_new ();
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_key_file_unref0 (self->keyfile);
#line 56 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->keyfile = _tmp2_;
#line 231 "midori-speeddial.c"
	{
		GKeyFile* _tmp3_;
		const gchar* _tmp4_;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = self->keyfile;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp4_ = self->priv->filename;
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_key_file_load_from_file (_tmp3_, _tmp4_, G_KEY_FILE_NONE, &_inner_error_);
#line 58 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 243 "midori-speeddial.c"
			goto __catch26_g_error;
		}
	}
	goto __finally26;
	__catch26_g_error:
	{
		GError* io_error = NULL;
		gchar* json = NULL;
		gsize len = 0UL;
		GString* script = NULL;
		gsize _tmp13_;
		GString* _tmp14_;
		GString* _tmp15_;
		GString* _tmp16_;
		const gchar* _tmp17_;
		gsize _tmp18_;
		GString* _tmp19_;
		const gchar* _tmp32_;
		gchar* _tmp33_;
		gchar* _tmp34_;
		gchar* _tmp35_;
		gchar* _tmp36_;
		GKeyFile* _tmp37_;
		gsize _tmp38_;
		gchar** _tmp39_;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		io_error = _inner_error_;
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = NULL;
#line 273 "midori-speeddial.c"
		{
			gchar* _tmp5_ = NULL;
			const gchar* _tmp6_;
			gchar* _tmp7_;
			gchar* _tmp10_ = NULL;
			gsize _tmp11_ = 0UL;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp6_ = fallback;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp7_ = g_strdup (_tmp6_);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp5_ = _tmp7_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (_tmp5_ == NULL) {
#line 288 "midori-speeddial.c"
				const gchar* _tmp8_;
				gchar* _tmp9_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp8_ = self->priv->filename;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp9_ = g_strconcat (_tmp8_, ".json", NULL);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (_tmp5_);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp5_ = _tmp9_;
#line 299 "midori-speeddial.c"
			}
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_file_get_contents (_tmp5_, &_tmp10_, &_tmp11_, &_inner_error_);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			json = _tmp10_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			len = _tmp11_;
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 64 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (_tmp5_);
#line 313 "midori-speeddial.c"
				goto __catch27_g_error;
			}
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (_tmp5_);
#line 318 "midori-speeddial.c"
		}
		goto __finally27;
		__catch27_g_error:
		{
			GError* fallback_error = NULL;
			gchar* _tmp12_;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			fallback_error = _inner_error_;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_inner_error_ = NULL;
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp12_ = g_strdup ("'{}'");
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 68 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			json = _tmp12_;
#line 69 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			len = (gsize) 4;
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (fallback_error);
#line 339 "midori-speeddial.c"
		}
		__finally27:
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (io_error);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (io_error);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_clear_error (&_inner_error_);
#line 63 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return NULL;
#line 358 "midori-speeddial.c"
		}
#line 72 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp13_ = len;
#line 72 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp14_ = g_string_sized_new (_tmp13_);
#line 72 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		script = _tmp14_;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp15_ = script;
#line 73 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append (_tmp15_, "var json = JSON.parse (");
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp16_ = script;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp17_ = json;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp18_ = len;
#line 74 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append_len (_tmp16_, _tmp17_, (gssize) _tmp18_);
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp19_ = script;
#line 75 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append (_tmp19_, "\n" \
"        );\n" \
"        var keyfile = '';\n" \
"        for (var i in json['shortcuts']) {\n" \
"        var tile = json['shortcuts'][i];\n" \
"        keyfile += '[Dial ' + tile['id'].substring (1) + ']\\n'\n" \
"                +  'uri=' + tile['href'] + '\\n'\n" \
"                +  'img=' + tile['img'] + '\\n'\n" \
"                +  'title=' + tile['title'] + '\\n\\n';\n" \
"        }\n" \
"        var columns = json['width'] ? json['width'] : 3;\n" \
"        var rows = json['shortcuts'] ? json['shortcuts'].length / colu" \
"mns : 0;\n" \
"        keyfile += '[settings]\\n'\n" \
"                +  'columns=' + columns + '\\n'\n" \
"                +  'rows=' + (rows > 3 ? rows : 3) + '\\n\\n';\n" \
"        keyfile;\n" \
"                    ");
#line 382 "midori-speeddial.c"
		{
			GKeyFile* _tmp20_;
			GString* _tmp21_;
			const gchar* _tmp22_;
			gchar* _tmp23_;
			gchar* _tmp24_;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp20_ = self->keyfile;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp21_ = script;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp22_ = _tmp21_->str;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp23_ = sokoke_js_script_eval (NULL, _tmp22_, NULL);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp24_ = _tmp23_;
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_key_file_load_from_data (_tmp20_, _tmp24_, (gsize) -1, 0, &_inner_error_);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (_tmp24_);
#line 94 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 405 "midori-speeddial.c"
				goto __catch28_g_error;
			}
		}
		goto __finally28;
		__catch28_g_error:
		{
			GError* eval_error = NULL;
			gchar* _tmp25_ = NULL;
			const gchar* _tmp26_;
			gchar* _tmp27_;
			GError* _tmp30_;
			const gchar* _tmp31_;
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			eval_error = _inner_error_;
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_inner_error_ = NULL;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp26_ = fallback;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp27_ = g_strdup (_tmp26_);
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp25_ = _tmp27_;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (_tmp25_ == NULL) {
#line 430 "midori-speeddial.c"
				const gchar* _tmp28_;
				gchar* _tmp29_;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp28_ = self->priv->filename;
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp29_ = g_strconcat (_tmp28_, ".json", NULL);
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (_tmp25_);
#line 100 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp25_ = _tmp29_;
#line 441 "midori-speeddial.c"
			}
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp30_ = eval_error;
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp31_ = _tmp30_->message;
#line 99 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_critical ("midori-speeddial.vala:99: Failed to parse %s as speed dial JSON: %s", _tmp25_, _tmp31_);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (_tmp25_);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (eval_error);
#line 453 "midori-speeddial.c"
		}
		__finally28:
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_string_free0 (script);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (io_error);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_string_free0 (script);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (json);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_error_free0 (io_error);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_clear_error (&_inner_error_);
#line 93 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return NULL;
#line 476 "midori-speeddial.c"
		}
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp32_ = g_get_user_cache_dir ();
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp33_ = g_build_path (G_DIR_SEPARATOR_S, _tmp32_, PACKAGE_NAME, "thumbnails", NULL);
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp34_ = _tmp33_;
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp35_ = katze_mkdir_with_parents (_tmp34_, 0700);
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp36_ = _tmp35_;
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp36_);
#line 102 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp34_);
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp37_ = self->keyfile;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp39_ = g_key_file_get_groups (_tmp37_, &_tmp38_);
#line 496 "midori-speeddial.c"
		{
			gchar** tile_collection = NULL;
			gint tile_collection_length1 = 0;
			gint _tile_collection_size_ = 0;
			gint tile_it = 0;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection = _tmp39_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection_length1 = _tmp38_;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			for (tile_it = 0; tile_it < _tmp38_; tile_it = tile_it + 1) {
#line 508 "midori-speeddial.c"
				const gchar* tile = NULL;
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				tile = tile_collection[tile_it];
#line 512 "midori-speeddial.c"
				{
					{
						gchar* img = NULL;
						GKeyFile* _tmp40_;
						const gchar* _tmp41_;
						gchar* _tmp42_;
						GKeyFile* _tmp43_;
						const gchar* _tmp44_;
						gchar* uri = NULL;
						GKeyFile* _tmp45_;
						const gchar* _tmp46_;
						gchar* _tmp47_;
						gboolean _tmp48_ = FALSE;
						gboolean _tmp49_ = FALSE;
						const gchar* _tmp50_;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp40_ = self->keyfile;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp41_ = tile;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp42_ = g_key_file_get_string (_tmp40_, _tmp41_, "img", &_inner_error_);
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						img = _tmp42_;
#line 109 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 538 "midori-speeddial.c"
							goto __catch29_g_error;
						}
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp43_ = self->keyfile;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp44_ = tile;
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_key_file_remove_key (_tmp43_, _tmp44_, "img", &_inner_error_);
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 110 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (img);
#line 551 "midori-speeddial.c"
							goto __catch29_g_error;
						}
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp45_ = self->keyfile;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp46_ = tile;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp47_ = g_key_file_get_string (_tmp45_, _tmp46_, "uri", &_inner_error_);
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						uri = _tmp47_;
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 111 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (img);
#line 566 "midori-speeddial.c"
							goto __catch29_g_error;
						}
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp50_ = img;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp50_ != NULL) {
#line 573 "midori-speeddial.c"
							const gchar* _tmp51_;
							gchar _tmp52_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp51_ = uri;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp52_ = string_get (_tmp51_, (glong) 0);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp49_ = _tmp52_ != '\0';
#line 582 "midori-speeddial.c"
						} else {
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp49_ = FALSE;
#line 586 "midori-speeddial.c"
						}
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp49_) {
#line 590 "midori-speeddial.c"
							const gchar* _tmp53_;
							gchar _tmp54_;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp53_ = uri;
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp54_ = string_get (_tmp53_, (glong) 0);
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp48_ = _tmp54_ != '#';
#line 599 "midori-speeddial.c"
						} else {
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp48_ = FALSE;
#line 603 "midori-speeddial.c"
						}
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp48_) {
#line 607 "midori-speeddial.c"
							guchar* decoded = NULL;
							const gchar* _tmp55_;
							size_t _tmp56_;
							guchar* _tmp57_;
							gint decoded_length1;
							gint _decoded_size_;
							const gchar* _tmp58_;
							gchar* _tmp59_;
							gchar* _tmp60_;
							guchar* _tmp61_;
							gint _tmp61__length1;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp55_ = img;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp57_ = g_base64_decode (_tmp55_, &_tmp56_);
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							decoded = _tmp57_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							decoded_length1 = _tmp56_;
#line 113 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_decoded_size_ = decoded_length1;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp58_ = uri;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp59_ = midori_speed_dial_build_thumbnail_path (self, _tmp58_);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp60_ = _tmp59_;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp61_ = decoded;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp61__length1 = decoded_length1;
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							g_file_set_contents (_tmp60_, (const char*) _tmp61_, (size_t) _tmp61__length1, &_inner_error_);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (_tmp60_);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								decoded = (g_free (decoded), NULL);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (uri);
#line 114 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (img);
#line 651 "midori-speeddial.c"
								goto __catch29_g_error;
							}
#line 112 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							decoded = (g_free (decoded), NULL);
#line 656 "midori-speeddial.c"
						}
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (img);
#line 662 "midori-speeddial.c"
					}
					goto __finally29;
					__catch29_g_error:
					{
						GError* img_error = NULL;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						img_error = _inner_error_;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_inner_error_ = NULL;
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_error_free0 (img_error);
#line 674 "midori-speeddial.c"
					}
					__finally29:
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_string_free0 (script);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (json);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_error_free0 (io_error);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_string_free0 (script);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (json);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_error_free0 (io_error);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_clear_error (&_inner_error_);
#line 108 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return NULL;
#line 701 "midori-speeddial.c"
					}
				}
			}
#line 107 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 707 "midori-speeddial.c"
		}
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_string_free0 (script);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (json);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_error_free0 (io_error);
#line 715 "midori-speeddial.c"
	}
	__finally26:
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_clear_error (&_inner_error_);
#line 57 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return NULL;
#line 726 "midori-speeddial.c"
	}
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return self;
#line 730 "midori-speeddial.c"
}


MidoriSpeedDial* midori_speed_dial_new (const gchar* new_filename, const gchar* fallback) {
#line 53 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return midori_speed_dial_construct (MIDORI_TYPE_SPEED_DIAL, new_filename, fallback);
#line 737 "midori-speeddial.c"
}


gchar* midori_speed_dial_get_next_free_slot (MidoriSpeedDial* self, guint* count) {
	guint _vala_count = 0U;
	gchar* result = NULL;
	guint slot_count = 0U;
	GKeyFile* _tmp0_;
	gsize _tmp1_;
	gchar** _tmp2_;
	guint _tmp8_;
	guint slot = 0U;
	guint _tmp17_;
	gchar* _tmp18_;
	GError * _inner_error_ = NULL;
#line 124 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 125 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	slot_count = (guint) 0;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->keyfile;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = g_key_file_get_groups (_tmp0_, &_tmp1_);
#line 761 "midori-speeddial.c"
	{
		gchar** tile_collection = NULL;
		gint tile_collection_length1 = 0;
		gint _tile_collection_size_ = 0;
		gint tile_it = 0;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		tile_collection = _tmp2_;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		tile_collection_length1 = _tmp1_;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		for (tile_it = 0; tile_it < _tmp1_; tile_it = tile_it + 1) {
#line 773 "midori-speeddial.c"
			const gchar* tile = NULL;
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile = tile_collection[tile_it];
#line 777 "midori-speeddial.c"
			{
				{
					gboolean _tmp3_ = FALSE;
					GKeyFile* _tmp4_;
					const gchar* _tmp5_;
					gboolean _tmp6_;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp4_ = self->keyfile;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp5_ = tile;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp6_ = g_key_file_has_key (_tmp4_, _tmp5_, "uri", &_inner_error_);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp3_ = _tmp6_;
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_inner_error_->domain == G_KEY_FILE_ERROR) {
#line 796 "midori-speeddial.c"
							goto __catch30_g_key_file_error;
						}
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_clear_error (&_inner_error_);
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return NULL;
#line 807 "midori-speeddial.c"
					}
#line 128 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (_tmp3_) {
#line 811 "midori-speeddial.c"
						guint _tmp7_;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp7_ = slot_count;
#line 129 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						slot_count = _tmp7_ + 1;
#line 817 "midori-speeddial.c"
					}
				}
				goto __finally30;
				__catch30_g_key_file_error:
				{
					GError* _error_ = NULL;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_error_ = _inner_error_;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_inner_error_ = NULL;
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_error_free0 (_error_);
#line 830 "midori-speeddial.c"
				}
				__finally30:
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_clear_error (&_inner_error_);
#line 127 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					return NULL;
#line 843 "midori-speeddial.c"
				}
			}
		}
#line 126 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 849 "midori-speeddial.c"
	}
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp8_ = slot_count;
#line 133 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_vala_count = _tmp8_;
#line 135 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	slot = (guint) 1;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	while (TRUE) {
#line 859 "midori-speeddial.c"
		guint _tmp9_;
		guint _tmp10_;
		gchar* tile = NULL;
		guint _tmp11_;
		gchar* _tmp12_;
		GKeyFile* _tmp13_;
		const gchar* _tmp14_;
		gboolean _tmp15_;
		guint _tmp16_;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp9_ = slot;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp10_ = slot_count;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (!(_tmp9_ <= _tmp10_)) {
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			break;
#line 877 "midori-speeddial.c"
		}
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp11_ = slot;
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp12_ = g_strdup_printf ("Dial %u", _tmp11_);
#line 137 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		tile = _tmp12_;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp13_ = self->keyfile;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp14_ = tile;
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp15_ = g_key_file_has_group (_tmp13_, _tmp14_);
#line 138 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (!_tmp15_) {
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			result = tile;
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (count) {
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				*count = _vala_count;
#line 899 "midori-speeddial.c"
			}
#line 139 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return result;
#line 903 "midori-speeddial.c"
		}
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp16_ = slot;
#line 140 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		slot = _tmp16_ + 1;
#line 136 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (tile);
#line 911 "midori-speeddial.c"
	}
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp17_ = slot_count;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp18_ = g_strdup_printf ("Dial %u", _tmp17_ + 1);
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	result = _tmp18_;
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (count) {
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		*count = _vala_count;
#line 923 "midori-speeddial.c"
	}
#line 143 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 927 "midori-speeddial.c"
}


static glong string_strnlen (gchar* str, glong maxlen) {
	glong result = 0L;
	gchar* end = NULL;
	gchar* _tmp0_;
	glong _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp0_ = str;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = maxlen;
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp2_ = memchr (_tmp0_, 0, (gsize) _tmp1_);
#line 1330 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	end = _tmp2_;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp3_ = end;
#line 1331 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp3_ == NULL) {
#line 950 "midori-speeddial.c"
		glong _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = maxlen;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp4_;
#line 1332 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 958 "midori-speeddial.c"
	} else {
		gchar* _tmp5_;
		gchar* _tmp6_;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = end;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = str;
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = (glong) (_tmp5_ - _tmp6_);
#line 1334 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 970 "midori-speeddial.c"
	}
}


static gchar* string_substring (const gchar* self, glong offset, glong len) {
	gchar* result = NULL;
	glong string_length = 0L;
	gboolean _tmp0_ = FALSE;
	glong _tmp1_;
	glong _tmp8_;
	glong _tmp14_;
	glong _tmp17_;
	glong _tmp18_;
	glong _tmp19_;
	glong _tmp20_;
	glong _tmp21_;
	gchar* _tmp22_;
#line 1341 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp1_ = offset;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_ >= ((glong) 0)) {
#line 994 "midori-speeddial.c"
		glong _tmp2_;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = len;
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = _tmp2_ >= ((glong) 0);
#line 1000 "midori-speeddial.c"
	} else {
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = FALSE;
#line 1004 "midori-speeddial.c"
	}
#line 1343 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 1008 "midori-speeddial.c"
		glong _tmp3_;
		glong _tmp4_;
		glong _tmp5_;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = offset;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = len;
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = string_strnlen ((gchar*) self, _tmp3_ + _tmp4_);
#line 1345 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = _tmp5_;
#line 1020 "midori-speeddial.c"
	} else {
		gint _tmp6_;
		gint _tmp7_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = strlen (self);
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = _tmp6_;
#line 1347 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		string_length = (glong) _tmp7_;
#line 1030 "midori-speeddial.c"
	}
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp8_ = offset;
#line 1350 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp8_ < ((glong) 0)) {
#line 1036 "midori-speeddial.c"
		glong _tmp9_;
		glong _tmp10_;
		glong _tmp11_;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = string_length;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = offset;
#line 1351 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		offset = _tmp9_ + _tmp10_;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = offset;
#line 1352 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp11_ >= ((glong) 0), NULL);
#line 1050 "midori-speeddial.c"
	} else {
		glong _tmp12_;
		glong _tmp13_;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = offset;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = string_length;
#line 1354 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_return_val_if_fail (_tmp12_ <= _tmp13_, NULL);
#line 1060 "midori-speeddial.c"
	}
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp14_ = len;
#line 1356 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp14_ < ((glong) 0)) {
#line 1066 "midori-speeddial.c"
		glong _tmp15_;
		glong _tmp16_;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = string_length;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp16_ = offset;
#line 1357 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		len = _tmp15_ - _tmp16_;
#line 1075 "midori-speeddial.c"
	}
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp17_ = offset;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp18_ = len;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp19_ = string_length;
#line 1359 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail ((_tmp17_ + _tmp18_) <= _tmp19_, NULL);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp20_ = offset;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp21_ = len;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	_tmp22_ = g_strndup (((gchar*) self) + _tmp20_, (gsize) _tmp21_);
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	result = _tmp22_;
#line 1360 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	return result;
#line 1095 "midori-speeddial.c"
}


void midori_speed_dial_add (MidoriSpeedDial* self, const gchar* uri, const gchar* title, GdkPixbuf* img) {
	gchar* id = NULL;
	gchar* _tmp0_;
	guint slot = 0U;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gint _tmp3_;
	guint _tmp4_;
	GError * _inner_error_ = NULL;
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (uri != NULL);
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (title != NULL);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = midori_speed_dial_get_next_free_slot (self, NULL);
#line 147 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	id = _tmp0_;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = string_substring (id, (glong) 5, (glong) -1);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = _tmp1_;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = atoi (_tmp2_);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = (guint) _tmp3_;
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (_tmp2_);
#line 148 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	slot = _tmp4_;
#line 1130 "midori-speeddial.c"
	{
		const gchar* _tmp5_;
		gchar* _tmp6_;
		gchar* _tmp7_;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp5_ = uri;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp6_ = g_strdup_printf ("speed_dial-save-add %u %s", slot, _tmp5_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp7_ = _tmp6_;
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_save_message (self, _tmp7_, &_inner_error_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp7_);
#line 150 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1147 "midori-speeddial.c"
			goto __catch31_g_error;
		}
	}
	goto __finally31;
	__catch31_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp8_;
		const gchar* _tmp9_;
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_error_ = _inner_error_;
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = NULL;
#line 153 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp8_ = _error_;
#line 153 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp9_ = _tmp8_->message;
#line 153 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("midori-speeddial.vala:153: Failed to add speed dial thumbnail: %s", _tmp9_);
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_error_free0 (_error_);
#line 1169 "midori-speeddial.c"
	}
	__finally31:
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (id);
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_clear_error (&_inner_error_);
#line 149 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 1182 "midori-speeddial.c"
	}
#line 146 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (id);
#line 1186 "midori-speeddial.c"
}


void midori_speed_dial_add_with_id (MidoriSpeedDial* self, const gchar* id, const gchar* uri, const gchar* title, GdkPixbuf* img) {
	GKeyFile* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	GKeyFile* _tmp3_;
	const gchar* _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	gchar* _tmp9_;
	gchar* _tmp10_;
	gchar* filename = NULL;
	const gchar* _tmp11_;
	gchar* _tmp12_;
	GError * _inner_error_ = NULL;
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (id != NULL);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (uri != NULL);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (title != NULL);
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->keyfile;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = id;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = uri;
#line 158 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_key_file_set_string (_tmp0_, _tmp1_, "uri", _tmp2_);
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = self->keyfile;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = id;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp5_ = title;
#line 159 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_key_file_set_string (_tmp3_, _tmp4_, "title", _tmp5_);
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp6_ = midori_paths_get_cache_dir ();
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp7_ = g_build_path (G_DIR_SEPARATOR_S, _tmp6_, "thumbnails", NULL);
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp8_ = _tmp7_;
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp9_ = katze_mkdir_with_parents (_tmp8_, 0700);
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp10_ = _tmp9_;
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (_tmp10_);
#line 161 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (_tmp8_);
#line 163 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp11_ = uri;
#line 163 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp12_ = midori_speed_dial_build_thumbnail_path (self, _tmp11_);
#line 163 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	filename = _tmp12_;
#line 1250 "midori-speeddial.c"
	{
		GdkPixbuf* _tmp13_;
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp13_ = img;
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		gdk_pixbuf_save (_tmp13_, filename, "png", &_inner_error_, NULL, "compression", "7", NULL, NULL);
#line 165 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1259 "midori-speeddial.c"
			goto __catch32_g_error;
		}
	}
	goto __finally32;
	__catch32_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp14_;
		const gchar* _tmp15_;
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_error_ = _inner_error_;
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = NULL;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp14_ = _error_;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp15_ = _tmp14_->message;
#line 168 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("midori-speeddial.vala:168: Failed to save speed dial thumbnail: %s", _tmp15_);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_error_free0 (_error_);
#line 1281 "midori-speeddial.c"
	}
	__finally32:
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (filename);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_clear_error (&_inner_error_);
#line 164 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 1294 "midori-speeddial.c"
	}
#line 170 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_save (self);
#line 157 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (filename);
#line 1300 "midori-speeddial.c"
}


static gchar* midori_speed_dial_build_thumbnail_path (MidoriSpeedDial* self, const gchar* filename) {
	gchar* result = NULL;
	gchar* thumbnail = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar* _tmp4_;
	const gchar* _tmp5_;
	gchar* _tmp6_;
#line 173 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 173 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (filename != NULL, NULL);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = filename;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = g_compute_checksum_for_string (G_CHECKSUM_MD5, _tmp0_, (gsize) -1);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = _tmp1_;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = g_strconcat (_tmp2_, ".png", NULL);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = _tmp3_;
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (_tmp2_);
#line 174 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	thumbnail = _tmp4_;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp5_ = midori_paths_get_cache_dir ();
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp6_ = g_build_filename (_tmp5_, "thumbnails", thumbnail, NULL);
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	result = _tmp6_;
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (thumbnail);
#line 175 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 1342 "midori-speeddial.c"
}


static gchar* string_replace (const gchar* self, const gchar* old, const gchar* replacement) {
	gchar* result = NULL;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	GError * _inner_error_ = NULL;
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (old != NULL, NULL);
#line 1415 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	g_return_val_if_fail (replacement != NULL, NULL);
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if ((*((gchar*) self)) == '\0') {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = TRUE;
#line 1361 "midori-speeddial.c"
	} else {
		const gchar* _tmp2_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp2_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp1_ = (*((gchar*) _tmp2_)) == '\0';
#line 1368 "midori-speeddial.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp1_) {
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = TRUE;
#line 1374 "midori-speeddial.c"
	} else {
		const gchar* _tmp3_;
		const gchar* _tmp4_;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp3_ = old;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp4_ = replacement;
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp0_ = g_strcmp0 (_tmp3_, _tmp4_) == 0;
#line 1384 "midori-speeddial.c"
	}
#line 1416 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (_tmp0_) {
#line 1388 "midori-speeddial.c"
		gchar* _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp5_ = g_strdup (self);
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp5_;
#line 1417 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 1396 "midori-speeddial.c"
	}
	{
		GRegex* regex = NULL;
		const gchar* _tmp6_;
		gchar* _tmp7_;
		gchar* _tmp8_;
		GRegex* _tmp9_;
		GRegex* _tmp10_;
		gchar* _tmp11_ = NULL;
		GRegex* _tmp12_;
		const gchar* _tmp13_;
		gchar* _tmp14_;
		gchar* _tmp15_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp6_ = old;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp7_ = g_regex_escape_string (_tmp6_, -1);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp8_ = _tmp7_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp9_ = g_regex_new (_tmp8_, 0, 0, &_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp10_ = _tmp9_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp8_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		regex = _tmp10_;
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 1428 "midori-speeddial.c"
				goto __catch33_g_regex_error;
			}
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1420 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 1437 "midori-speeddial.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp12_ = regex;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp13_ = replacement;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp14_ = g_regex_replace_literal (_tmp12_, self, (gssize) -1, 0, _tmp13_, 0, &_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = _tmp14_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			if (_inner_error_->domain == G_REGEX_ERROR) {
#line 1453 "midori-speeddial.c"
				goto __catch33_g_regex_error;
			}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			g_clear_error (&_inner_error_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
			return NULL;
#line 1464 "midori-speeddial.c"
		}
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp15_ = _tmp11_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_tmp11_ = NULL;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		result = _tmp15_;
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_free0 (_tmp11_);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_regex_unref0 (regex);
#line 1421 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return result;
#line 1478 "midori-speeddial.c"
	}
	goto __finally33;
	__catch33_g_regex_error:
	{
		GError* e = NULL;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		e = _inner_error_;
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_inner_error_ = NULL;
#line 1423 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_assert_not_reached ();
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		_g_error_free0 (e);
#line 1492 "midori-speeddial.c"
	}
	__finally33:
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		g_clear_error (&_inner_error_);
#line 1419 "/usr/share/vala-0.38/vapi/glib-2.0.vapi"
		return NULL;
#line 1503 "midori-speeddial.c"
	}
}


const gchar* midori_speed_dial_get_html (MidoriSpeedDial* self, GError** error) {
	const gchar* result = NULL;
	gboolean load_missing = FALSE;
	const gchar* _tmp0_;
	gchar* head = NULL;
	gchar* filename = NULL;
	gchar* _tmp2_;
	gboolean _tmp3_ = FALSE;
	GKeyFile* _tmp4_;
	const gchar* _tmp101_;
	GError * _inner_error_ = NULL;
#line 178 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 179 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	load_missing = TRUE;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->priv->html;
#line 181 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp0_ != NULL) {
#line 1527 "midori-speeddial.c"
		const gchar* _tmp1_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp1_ = self->priv->html;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		result = _tmp1_;
#line 182 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return result;
#line 1535 "midori-speeddial.c"
	}
#line 184 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	head = NULL;
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = midori_paths_get_res_filename ("speeddial-head.html");
#line 185 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	filename = _tmp2_;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = self->keyfile;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp4_ != NULL) {
#line 1547 "midori-speeddial.c"
		gboolean _tmp5_ = FALSE;
		const gchar* _tmp6_;
		gchar* _tmp7_ = NULL;
		gboolean _tmp8_;
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp6_ = filename;
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp8_ = g_file_get_contents (_tmp6_, &_tmp7_, NULL, &_inner_error_);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (head);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		head = _tmp7_;
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp5_ = _tmp8_;
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_propagate_error (error, _inner_error_);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (filename);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (head);
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return NULL;
#line 1572 "midori-speeddial.c"
		}
#line 187 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = _tmp5_;
#line 1576 "midori-speeddial.c"
	} else {
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = FALSE;
#line 1580 "midori-speeddial.c"
	}
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp3_) {
#line 1584 "midori-speeddial.c"
		gchar* header = NULL;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		gchar* _tmp12_;
		gchar* _tmp13_;
		gchar* _tmp14_;
		gchar* _tmp15_;
		gchar* _tmp16_;
		gchar* _tmp17_;
		GString* markup = NULL;
		const gchar* _tmp18_;
		GString* _tmp19_;
		guint slot_count = 0U;
		gchar* dial_id = NULL;
		guint _tmp20_ = 0U;
		gchar* _tmp21_;
		guint next_slot = 0U;
		const gchar* _tmp22_;
		gchar* _tmp23_;
		gchar* _tmp24_;
		gint _tmp25_;
		guint _tmp26_;
		guint grid_index = 0U;
		guint slot_size = 0U;
		guint _tmp31_;
		const gchar* _tmp32_ = NULL;
		gboolean _tmp33_;
		const gchar* _tmp34_ = NULL;
		gboolean _tmp35_;
		GString* _tmp36_;
		guint _tmp37_;
		guint _tmp38_;
		guint div_factor = 0U;
		gboolean _tmp39_ = FALSE;
		guint _tmp40_;
		guint _tmp41_;
		guint margin = 0U;
		guint _tmp43_;
		guint _tmp44_;
		guint _tmp45_;
		guint _tmp46_;
		GString* _tmp48_;
		guint _tmp49_;
		gboolean _tmp50_;
		GKeyFile* _tmp52_;
		gsize _tmp53_;
		gchar** _tmp54_;
		GString* _tmp94_;
		guint _tmp95_;
		GString* _tmp96_;
		GString* _tmp97_;
		const gchar* _tmp98_;
		gchar* _tmp99_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp9_ = head;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp10_ = string_replace (_tmp9_, "{title}", _ ("Speed Dial"));
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp11_ = _tmp10_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp12_ = string_replace (_tmp11_, "{click_to_add}", _ ("Click to add a shortcut"));
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp13_ = _tmp12_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp14_ = string_replace (_tmp13_, "{enter_shortcut_address}", _ ("Enter shortcut address"));
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp15_ = _tmp14_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp16_ = string_replace (_tmp15_, "{are_you_sure}", _ ("Are you sure you want to delete this shortcut?"));
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp17_ = _tmp16_;
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp15_);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp13_);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp11_);
#line 188 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		header = _tmp17_;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp18_ = header;
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp19_ = g_string_new (_tmp18_);
#line 192 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		markup = _tmp19_;
#line 194 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		slot_count = (guint) 1;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp21_ = midori_speed_dial_get_next_free_slot (self, &_tmp20_);
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		slot_count = _tmp20_;
#line 195 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		dial_id = _tmp21_;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp22_ = dial_id;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp23_ = string_substring (_tmp22_, (glong) 5, (glong) -1);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp24_ = _tmp23_;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp25_ = atoi (_tmp24_);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp26_ = (guint) _tmp25_;
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp24_);
#line 196 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		next_slot = _tmp26_;
#line 199 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		grid_index = (guint) 3;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		while (TRUE) {
#line 1697 "midori-speeddial.c"
			guint _tmp27_;
			guint _tmp28_;
			guint _tmp29_;
			guint _tmp30_;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp27_ = grid_index;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp28_ = grid_index;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp29_ = slot_count;
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (!((_tmp27_ * _tmp28_) < _tmp29_)) {
#line 200 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				break;
#line 1712 "midori-speeddial.c"
			}
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp30_ = grid_index;
#line 201 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			grid_index = _tmp30_ + 1;
#line 1718 "midori-speeddial.c"
		}
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp31_ = grid_index;
#line 204 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		slot_size = 100 / _tmp31_;
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp33_ = midori_paths_is_readonly ();
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (_tmp33_) {
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp32_ = "";
#line 1730 "midori-speeddial.c"
		} else {
#line 210 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp32_ = "<noscript>";
#line 1734 "midori-speeddial.c"
		}
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp35_ = midori_paths_is_readonly ();
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (_tmp35_) {
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp34_ = "";
#line 1742 "midori-speeddial.c"
		} else {
#line 211 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp34_ = "</noscript>";
#line 1746 "midori-speeddial.c"
		}
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp36_ = markup;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp37_ = slot_size;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp38_ = slot_size;
#line 207 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append_printf (_tmp36_, "%s<style>.cross { display:none }</style>%s" "<style> div.shortcut { height: %d%%; width: %d%%; }</style>\n", _tmp32_, _tmp34_, _tmp37_ + 1, _tmp38_ - 4);
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp40_ = slot_size;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp41_ = grid_index;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if ((_tmp40_ * _tmp41_) >= ((guint) 100)) {
#line 1762 "midori-speeddial.c"
			guint _tmp42_;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp42_ = grid_index;
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp39_ = _tmp42_ > ((guint) 4);
#line 1768 "midori-speeddial.c"
		} else {
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp39_ = FALSE;
#line 1772 "midori-speeddial.c"
		}
#line 217 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (_tmp39_) {
#line 218 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			div_factor = (guint) 8;
#line 1778 "midori-speeddial.c"
		} else {
#line 220 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			div_factor = (guint) 2;
#line 1782 "midori-speeddial.c"
		}
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp43_ = slot_size;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp44_ = grid_index;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp45_ = div_factor;
#line 221 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		margin = (100 - ((_tmp43_ - 4) * _tmp44_)) / _tmp45_;
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp46_ = margin;
#line 222 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (_tmp46_ > ((guint) 9)) {
#line 1796 "midori-speeddial.c"
			guint _tmp47_;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp47_ = margin;
#line 223 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			margin = _tmp47_ % 10;
#line 1802 "midori-speeddial.c"
		}
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp48_ = markup;
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp49_ = margin;
#line 225 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append_printf (_tmp48_, "<style> body { overflow:hidden } #content { margin-left: %u%%; }</styl" \
"e>", _tmp49_);
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp50_ = self->priv->_close_buttons_left;
#line 227 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (_tmp50_) {
#line 1814 "midori-speeddial.c"
			GString* _tmp51_;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp51_ = markup;
#line 228 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_string_append_printf (_tmp51_, "<style>.cross { left: -14px }</style>");
#line 1820 "midori-speeddial.c"
		}
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp52_ = self->keyfile;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp54_ = g_key_file_get_groups (_tmp52_, &_tmp53_);
#line 1826 "midori-speeddial.c"
		{
			gchar** tile_collection = NULL;
			gint tile_collection_length1 = 0;
			gint _tile_collection_size_ = 0;
			gint tile_it = 0;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection = _tmp54_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection_length1 = _tmp53_;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			for (tile_it = 0; tile_it < _tmp53_; tile_it = tile_it + 1) {
#line 1838 "midori-speeddial.c"
				const gchar* tile = NULL;
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				tile = tile_collection[tile_it];
#line 1842 "midori-speeddial.c"
				{
					{
						gchar* uri = NULL;
						GKeyFile* _tmp55_;
						const gchar* _tmp56_;
						gchar* _tmp57_;
						gboolean _tmp58_ = FALSE;
						gboolean _tmp59_ = FALSE;
						const gchar* _tmp60_;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp55_ = self->keyfile;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp56_ = tile;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp57_ = g_key_file_get_string (_tmp55_, _tmp56_, "uri", &_inner_error_);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						uri = _tmp57_;
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (_inner_error_->domain == G_KEY_FILE_ERROR) {
#line 1864 "midori-speeddial.c"
								goto __catch34_g_key_file_error;
							}
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (dial_id);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_string_free0 (markup);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (header);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (filename);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (head);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							g_clear_error (&_inner_error_);
#line 233 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							return NULL;
#line 1885 "midori-speeddial.c"
						}
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp60_ = uri;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp60_ != NULL) {
#line 1891 "midori-speeddial.c"
							const gchar* _tmp61_;
							const gchar* _tmp62_;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp61_ = uri;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp62_ = strstr (_tmp61_, "://");
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp59_ = _tmp62_ != NULL;
#line 1900 "midori-speeddial.c"
						} else {
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp59_ = FALSE;
#line 1904 "midori-speeddial.c"
						}
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp59_) {
#line 1908 "midori-speeddial.c"
							const gchar* _tmp63_;
							gboolean _tmp64_;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp63_ = tile;
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp64_ = g_str_has_prefix (_tmp63_, "Dial ");
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp58_ = _tmp64_;
#line 1917 "midori-speeddial.c"
						} else {
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp58_ = FALSE;
#line 1921 "midori-speeddial.c"
						}
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						if (_tmp58_) {
#line 1925 "midori-speeddial.c"
							gchar* title = NULL;
							GKeyFile* _tmp65_;
							const gchar* _tmp66_;
							gchar* _tmp67_;
							gchar* thumb_filename = NULL;
							const gchar* _tmp68_;
							gchar* _tmp69_;
							guint slot = 0U;
							const gchar* _tmp70_;
							gchar* _tmp71_;
							gchar* _tmp72_;
							gint _tmp73_;
							guint _tmp74_;
							gchar* encoded = NULL;
							const gchar* _tmp83_ = NULL;
							const gchar* _tmp84_;
							const gchar* _tmp85_ = NULL;
							const gchar* _tmp86_;
							GString* _tmp87_;
							guint _tmp88_;
							const gchar* _tmp89_;
							const gchar* _tmp90_;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp65_ = self->keyfile;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp66_ = tile;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp67_ = g_key_file_get_string (_tmp65_, _tmp66_, "title", &_inner_error_);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							title = _tmp67_;
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (uri);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								if (_inner_error_->domain == G_KEY_FILE_ERROR) {
#line 1962 "midori-speeddial.c"
									goto __catch34_g_key_file_error;
								}
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (uri);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (dial_id);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_string_free0 (markup);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (header);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (filename);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (head);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_clear_error (&_inner_error_);
#line 235 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								return NULL;
#line 1985 "midori-speeddial.c"
							}
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp68_ = uri;
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp69_ = midori_speed_dial_build_thumbnail_path (self, _tmp68_);
#line 236 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							thumb_filename = _tmp69_;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp70_ = tile;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp71_ = string_substring (_tmp70_, (glong) 5, (glong) -1);
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp72_ = _tmp71_;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp73_ = atoi (_tmp72_);
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp74_ = (guint) _tmp73_;
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (_tmp72_);
#line 237 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							slot = _tmp74_;
#line 2007 "midori-speeddial.c"
							{
								guint8* thumb = NULL;
								gint thumb_length1 = 0;
								gint _thumb_size_ = 0;
								const gchar* _tmp75_;
								guint8* _tmp76_ = NULL;
								size_t _tmp77_;
								guint8* _tmp78_;
								gint _tmp78__length1;
								gchar* _tmp79_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp75_ = thumb_filename;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_file_get_contents (_tmp75_, (gchar**) (&_tmp76_), &_tmp77_, &_inner_error_);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								thumb = (g_free (thumb), NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								thumb = _tmp76_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								thumb_length1 = _tmp77_;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_thumb_size_ = thumb_length1;
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									thumb = (g_free (thumb), NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									if (_inner_error_->domain == G_FILE_ERROR) {
#line 2036 "midori-speeddial.c"
										goto __catch35_g_file_error;
									}
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									thumb = (g_free (thumb), NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (encoded);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (thumb_filename);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (title);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (uri);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (dial_id);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_string_free0 (markup);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (header);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (filename);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (head);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									g_clear_error (&_inner_error_);
#line 241 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									return NULL;
#line 2067 "midori-speeddial.c"
								}
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp78_ = thumb;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp78__length1 = thumb_length1;
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp79_ = g_base64_encode (_tmp78_, _tmp78__length1);
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (encoded);
#line 242 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								encoded = _tmp79_;
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								thumb = (g_free (thumb), NULL);
#line 2081 "midori-speeddial.c"
							}
							goto __finally35;
							__catch35_g_file_error:
							{
								GError* _error_ = NULL;
								gboolean _tmp80_;
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_error_ = _inner_error_;
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_inner_error_ = NULL;
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (encoded);
#line 245 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								encoded = NULL;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp80_ = load_missing;
#line 246 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								if (_tmp80_) {
#line 2100 "midori-speeddial.c"
									const gchar* _tmp81_;
									const gchar* _tmp82_;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_tmp81_ = tile;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_tmp82_ = uri;
#line 247 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									midori_speed_dial_get_thumb (self, _tmp81_, _tmp82_);
#line 2109 "midori-speeddial.c"
								}
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_error_free0 (_error_);
#line 2113 "midori-speeddial.c"
							}
							__finally35:
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (encoded);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (thumb_filename);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (title);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (uri);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								if (_inner_error_->domain == G_KEY_FILE_ERROR) {
#line 2128 "midori-speeddial.c"
									goto __catch34_g_key_file_error;
								}
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (encoded);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (thumb_filename);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (title);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (uri);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (dial_id);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_string_free0 (markup);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (header);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (filename);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_g_free0 (head);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_clear_error (&_inner_error_);
#line 239 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								return NULL;
#line 2157 "midori-speeddial.c"
							}
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp84_ = encoded;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp83_ = _tmp84_;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (_tmp83_ == NULL) {
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp83_ = "";
#line 2167 "midori-speeddial.c"
							}
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp86_ = title;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp85_ = _tmp86_;
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (_tmp85_ == NULL) {
#line 255 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp85_ = "";
#line 2177 "midori-speeddial.c"
							}
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp87_ = markup;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp88_ = slot;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp89_ = uri;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp90_ = title;
#line 249 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							g_string_append_printf (_tmp87_, "\n" \
"                                <div class=\"shortcut\" id=\"%u\"><div" \
" class=\"preview\">\n" \
"                                <a class=\"cross\" href=\"#\"></a>\n" \
"                                <a href=\"%s\"><img src=\"data:image/p" \
"ng;base64,%s\" title='%s'></a>\n" \
"                                </div><input type=\"text\" class=\"tit" \
"le selectable\" value=\"%s\"></div>\n" \
"                                ", _tmp88_, _tmp89_, _tmp83_, _tmp90_, _tmp85_);
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (encoded);
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (thumb_filename);
#line 234 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_g_free0 (title);
#line 2195 "midori-speeddial.c"
						} else {
							const gchar* _tmp91_;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							_tmp91_ = tile;
#line 257 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
							if (g_strcmp0 (_tmp91_, "settings") != 0) {
#line 2202 "midori-speeddial.c"
								GKeyFile* _tmp92_;
								const gchar* _tmp93_;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp92_ = self->keyfile;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								_tmp93_ = tile;
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								g_key_file_remove_group (_tmp92_, _tmp93_, &_inner_error_);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
								if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (uri);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									if (_inner_error_->domain == G_KEY_FILE_ERROR) {
#line 2217 "midori-speeddial.c"
										goto __catch34_g_key_file_error;
									}
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (uri);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (dial_id);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_string_free0 (markup);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (header);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (filename);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									_g_free0 (head);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									g_clear_error (&_inner_error_);
#line 258 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
									return NULL;
#line 2240 "midori-speeddial.c"
								}
							}
						}
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri);
#line 2246 "midori-speeddial.c"
					}
					goto __finally34;
					__catch34_g_key_file_error:
					{
						GError* _error_ = NULL;
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_error_ = _inner_error_;
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_inner_error_ = NULL;
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_error_free0 (_error_);
#line 2258 "midori-speeddial.c"
					}
					__finally34:
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_string_free0 (markup);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (header);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (filename);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (head);
#line 232 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return NULL;
#line 2279 "midori-speeddial.c"
					}
				}
			}
#line 231 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			tile_collection = (_vala_array_free (tile_collection, tile_collection_length1, (GDestroyNotify) g_free), NULL);
#line 2285 "midori-speeddial.c"
		}
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp94_ = markup;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp95_ = next_slot;
#line 263 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append_printf (_tmp94_, "\n" \
"                    <div class=\"shortcut\" id=\"%u\"><div class=\"pre" \
"view new\">\n" \
"                    <a class=\"add\" href=\"#\"></a>\n" \
"                    </div><div class=\"title\">%s</div></div>\n" \
"                    ", _tmp95_, _ ("Click to add a shortcut"));
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp96_ = markup;
#line 269 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_string_append_printf (_tmp96_, "</div>\n</body>\n</html>\n");
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp97_ = markup;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp98_ = _tmp97_->str;
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp99_ = g_strdup (_tmp98_);
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (self->priv->html);
#line 270 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		self->priv->html = _tmp99_;
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (dial_id);
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_string_free0 (markup);
#line 186 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (header);
#line 2313 "midori-speeddial.c"
	} else {
		gchar* _tmp100_;
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp100_ = g_strdup ("");
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (self->priv->html);
#line 273 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		self->priv->html = _tmp100_;
#line 2322 "midori-speeddial.c"
	}
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp101_ = self->priv->html;
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	result = _tmp101_;
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (filename);
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (head);
#line 275 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 2334 "midori-speeddial.c"
}


void midori_speed_dial_save_message (MidoriSpeedDial* self, const gchar* message, GError** error) {
	const gchar* _tmp0_;
	gboolean _tmp1_;
	gchar* msg = NULL;
	const gchar* _tmp4_;
	gchar* _tmp5_;
	gchar** parts = NULL;
	const gchar* _tmp6_;
	gchar** _tmp7_;
	gchar** _tmp8_;
	gint parts_length1;
	gint _parts_size_;
	gchar** _tmp9_;
	gint _tmp9__length1;
	const gchar* _tmp10_;
	gchar* action = NULL;
	gchar** _tmp12_;
	gint _tmp12__length1;
	const gchar* _tmp13_;
	gchar* _tmp14_;
	gchar** _tmp15_;
	gint _tmp15__length1;
	const gchar* _tmp16_;
	gchar* dial_id = NULL;
	gchar** _tmp18_;
	gint _tmp18__length1;
	const gchar* _tmp19_;
	gchar* _tmp20_;
	const gchar* _tmp21_;
	GError * _inner_error_ = NULL;
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (message != NULL);
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = message;
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = g_str_has_prefix (_tmp0_, "speed_dial-save-");
#line 279 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (!_tmp1_) {
#line 2378 "midori-speeddial.c"
		const gchar* _tmp2_;
		GError* _tmp3_;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp2_ = message;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = g_error_new (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_INVALID_MESSAGE, "Invalid message '%s'", _tmp2_);
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = _tmp3_;
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_propagate_error (error, _inner_error_);
#line 280 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 2391 "midori-speeddial.c"
	}
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = message;
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp5_ = string_substring (_tmp4_, (glong) 16, (glong) -1);
#line 282 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	msg = _tmp5_;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp6_ = msg;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp8_ = _tmp7_ = g_strsplit (_tmp6_, " ", 3);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	parts = _tmp8_;
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	parts_length1 = _vala_array_length (_tmp7_);
#line 283 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_parts_size_ = parts_length1;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp9_ = parts;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp9__length1 = parts_length1;
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp10_ = _tmp9_[0];
#line 284 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp10_ == NULL) {
#line 2417 "midori-speeddial.c"
		GError* _tmp11_;
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp11_ = g_error_new_literal (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_NO_ACTION, "No action.");
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = _tmp11_;
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_propagate_error (error, _inner_error_);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (msg);
#line 285 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 2431 "midori-speeddial.c"
	}
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp12_ = parts;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp12__length1 = parts_length1;
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp13_ = _tmp12_[0];
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp14_ = g_strdup (_tmp13_);
#line 286 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	action = _tmp14_;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp15_ = parts;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp15__length1 = parts_length1;
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp16_ = _tmp15_[1];
#line 288 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp16_ == NULL) {
#line 2451 "midori-speeddial.c"
		GError* _tmp17_;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp17_ = g_error_new_literal (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_NO_ID, "No ID argument.");
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = _tmp17_;
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_propagate_error (error, _inner_error_);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (action);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (msg);
#line 289 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 2467 "midori-speeddial.c"
	}
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp18_ = parts;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp18__length1 = parts_length1;
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp19_ = _tmp18_[1];
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp20_ = g_strconcat ("Dial ", _tmp19_, NULL);
#line 290 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	dial_id = _tmp20_;
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp21_ = action;
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (g_strcmp0 (_tmp21_, "delete") == 0) {
#line 2483 "midori-speeddial.c"
		gchar* uri = NULL;
		GKeyFile* _tmp22_;
		const gchar* _tmp23_;
		gchar* _tmp24_;
		gchar* file_path = NULL;
		const gchar* _tmp25_;
		gchar* _tmp26_;
		GKeyFile* _tmp27_;
		const gchar* _tmp28_;
		const gchar* _tmp29_;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp22_ = self->keyfile;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp23_ = dial_id;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp24_ = g_key_file_get_string (_tmp22_, _tmp23_, "uri", &_inner_error_);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		uri = _tmp24_;
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_propagate_error (error, _inner_error_);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (dial_id);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (action);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (msg);
#line 293 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return;
#line 2516 "midori-speeddial.c"
		}
#line 294 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp25_ = uri;
#line 294 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp26_ = midori_speed_dial_build_thumbnail_path (self, _tmp25_);
#line 294 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		file_path = _tmp26_;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp27_ = self->keyfile;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp28_ = dial_id;
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_key_file_remove_group (_tmp27_, _tmp28_, &_inner_error_);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_propagate_error (error, _inner_error_);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (file_path);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (uri);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (dial_id);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (action);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_g_free0 (msg);
#line 295 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return;
#line 2548 "midori-speeddial.c"
		}
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp29_ = file_path;
#line 296 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_unlink (_tmp29_);
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (file_path);
#line 292 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (uri);
#line 2558 "midori-speeddial.c"
	} else {
		const gchar* _tmp30_;
#line 298 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp30_ = action;
#line 298 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (g_strcmp0 (_tmp30_, "add") == 0) {
#line 2565 "midori-speeddial.c"
			gchar** _tmp31_;
			gint _tmp31__length1;
			const gchar* _tmp32_;
			GKeyFile* _tmp34_;
			const gchar* _tmp35_;
			gchar** _tmp36_;
			gint _tmp36__length1;
			const gchar* _tmp37_;
			const gchar* _tmp38_;
			gchar** _tmp39_;
			gint _tmp39__length1;
			const gchar* _tmp40_;
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp31_ = parts;
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp31__length1 = parts_length1;
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp32_ = _tmp31_[2];
#line 299 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (_tmp32_ == NULL) {
#line 2586 "midori-speeddial.c"
				GError* _tmp33_;
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp33_ = g_error_new_literal (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_NO_URL, "No URL argument.");
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_inner_error_ = _tmp33_;
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				g_propagate_error (error, _inner_error_);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (dial_id);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (action);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (msg);
#line 300 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				return;
#line 2604 "midori-speeddial.c"
			}
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp34_ = self->keyfile;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp35_ = dial_id;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp36_ = parts;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp36__length1 = parts_length1;
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp37_ = _tmp36_[2];
#line 301 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			g_key_file_set_string (_tmp34_, _tmp35_, "uri", _tmp37_);
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp38_ = dial_id;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp39_ = parts;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp39__length1 = parts_length1;
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp40_ = _tmp39_[2];
#line 302 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			midori_speed_dial_get_thumb (self, _tmp38_, _tmp40_);
#line 2628 "midori-speeddial.c"
		} else {
			const gchar* _tmp41_;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp41_ = action;
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			if (g_strcmp0 (_tmp41_, "rename") == 0) {
#line 2635 "midori-speeddial.c"
				gchar** _tmp42_;
				gint _tmp42__length1;
				const gchar* _tmp43_;
				gchar* title = NULL;
				gchar** _tmp45_;
				gint _tmp45__length1;
				const gchar* _tmp46_;
				gchar* _tmp47_;
				GKeyFile* _tmp48_;
				const gchar* _tmp49_;
				const gchar* _tmp50_;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp42_ = parts;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp42__length1 = parts_length1;
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp43_ = _tmp42_[2];
#line 305 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				if (_tmp43_ == NULL) {
#line 2655 "midori-speeddial.c"
					GError* _tmp44_;
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp44_ = g_error_new_literal (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_NO_TITLE, "No title argument.");
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_inner_error_ = _tmp44_;
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_propagate_error (error, _inner_error_);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (dial_id);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (action);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (msg);
#line 306 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					return;
#line 2673 "midori-speeddial.c"
				}
#line 307 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp45_ = parts;
#line 307 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp45__length1 = parts_length1;
#line 307 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp46_ = _tmp45_[2];
#line 307 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp47_ = g_strdup (_tmp46_);
#line 307 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				title = _tmp47_;
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp48_ = self->keyfile;
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp49_ = dial_id;
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp50_ = title;
#line 308 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				g_key_file_set_string (_tmp48_, _tmp49_, "title", _tmp50_);
#line 304 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_g_free0 (title);
#line 2695 "midori-speeddial.c"
			} else {
				const gchar* _tmp51_;
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp51_ = action;
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				if (g_strcmp0 (_tmp51_, "swap") == 0) {
#line 2702 "midori-speeddial.c"
					gchar** _tmp52_;
					gint _tmp52__length1;
					const gchar* _tmp53_;
					gchar* dial2_id = NULL;
					gchar** _tmp55_;
					gint _tmp55__length1;
					const gchar* _tmp56_;
					gchar* _tmp57_;
					gchar* uri = NULL;
					GKeyFile* _tmp58_;
					const gchar* _tmp59_;
					gchar* _tmp60_;
					gchar* title = NULL;
					GKeyFile* _tmp61_;
					const gchar* _tmp62_;
					gchar* _tmp63_;
					gchar* uri2 = NULL;
					GKeyFile* _tmp64_;
					const gchar* _tmp65_;
					gchar* _tmp66_;
					gchar* title2 = NULL;
					GKeyFile* _tmp67_;
					const gchar* _tmp68_;
					gchar* _tmp69_;
					GKeyFile* _tmp70_;
					const gchar* _tmp71_;
					const gchar* _tmp72_;
					GKeyFile* _tmp73_;
					const gchar* _tmp74_;
					const gchar* _tmp75_;
					GKeyFile* _tmp76_;
					const gchar* _tmp77_;
					const gchar* _tmp78_;
					GKeyFile* _tmp79_;
					const gchar* _tmp80_;
					const gchar* _tmp81_;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp52_ = parts;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp52__length1 = parts_length1;
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp53_ = _tmp52_[2];
#line 311 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (_tmp53_ == NULL) {
#line 2747 "midori-speeddial.c"
						GError* _tmp54_;
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_tmp54_ = g_error_new_literal (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_NO_ID2, "No ID2 argument.");
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_inner_error_ = _tmp54_;
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (action);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (msg);
#line 312 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return;
#line 2765 "midori-speeddial.c"
					}
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp55_ = parts;
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp55__length1 = parts_length1;
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp56_ = _tmp55_[2];
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp57_ = g_strconcat ("Dial ", _tmp56_, NULL);
#line 313 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					dial2_id = _tmp57_;
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp58_ = self->keyfile;
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp59_ = dial_id;
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp60_ = g_key_file_get_string (_tmp58_, _tmp59_, "uri", &_inner_error_);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					uri = _tmp60_;
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial2_id);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (action);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (msg);
#line 315 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return;
#line 2801 "midori-speeddial.c"
					}
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp61_ = self->keyfile;
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp62_ = dial_id;
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp63_ = g_key_file_get_string (_tmp61_, _tmp62_, "title", &_inner_error_);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					title = _tmp63_;
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial2_id);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (action);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (msg);
#line 316 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return;
#line 2829 "midori-speeddial.c"
					}
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp64_ = self->keyfile;
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp65_ = dial2_id;
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp66_ = g_key_file_get_string (_tmp64_, _tmp65_, "uri", &_inner_error_);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					uri2 = _tmp66_;
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (title);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial2_id);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (action);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (msg);
#line 317 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return;
#line 2859 "midori-speeddial.c"
					}
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp67_ = self->keyfile;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp68_ = dial2_id;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp69_ = g_key_file_get_string (_tmp67_, _tmp68_, "title", &_inner_error_);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					title2 = _tmp69_;
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						g_propagate_error (error, _inner_error_);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri2);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (title);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (uri);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial2_id);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (dial_id);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (action);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						_g_free0 (msg);
#line 318 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
						return;
#line 2891 "midori-speeddial.c"
					}
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp70_ = self->keyfile;
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp71_ = dial_id;
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp72_ = uri2;
#line 320 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_key_file_set_string (_tmp70_, _tmp71_, "uri", _tmp72_);
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp73_ = self->keyfile;
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp74_ = dial2_id;
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp75_ = uri;
#line 321 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_key_file_set_string (_tmp73_, _tmp74_, "uri", _tmp75_);
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp76_ = self->keyfile;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp77_ = dial_id;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp78_ = title2;
#line 322 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_key_file_set_string (_tmp76_, _tmp77_, "title", _tmp78_);
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp79_ = self->keyfile;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp80_ = dial2_id;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp81_ = title;
#line 323 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_key_file_set_string (_tmp79_, _tmp80_, "title", _tmp81_);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (title2);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (uri2);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (title);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (uri);
#line 310 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (dial2_id);
#line 2935 "midori-speeddial.c"
				} else {
					const gchar* _tmp82_;
					GError* _tmp83_;
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp82_ = action;
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_tmp83_ = g_error_new (MIDORI_SPEED_DIAL_ERROR, MIDORI_SPEED_DIAL_ERROR_INVALID_ACTION, "Invalid action '%s'", _tmp82_);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_inner_error_ = _tmp83_;
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					g_propagate_error (error, _inner_error_);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (dial_id);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (action);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_g_free0 (msg);
#line 326 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					return;
#line 2957 "midori-speeddial.c"
				}
			}
		}
	}
#line 328 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_save (self);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (dial_id);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (action);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
#line 278 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (msg);
#line 2972 "midori-speeddial.c"
}


static void midori_speed_dial_save (MidoriSpeedDial* self) {
	GError * _inner_error_ = NULL;
#line 331 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 332 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->priv->html);
#line 332 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->html = NULL;
#line 2984 "midori-speeddial.c"
	{
		const gchar* _tmp0_;
		GKeyFile* _tmp1_;
		gchar* _tmp2_;
		gchar* _tmp3_;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp0_ = self->priv->filename;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp1_ = self->keyfile;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp2_ = g_key_file_to_data (_tmp1_, NULL, NULL);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = _tmp2_;
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_file_set_contents (_tmp0_, _tmp3_, (gssize) -1, &_inner_error_);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_free0 (_tmp3_);
#line 335 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 3004 "midori-speeddial.c"
			goto __catch36_g_error;
		}
	}
	goto __finally36;
	__catch36_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp4_;
		const gchar* _tmp5_;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_error_ = _inner_error_;
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_inner_error_ = NULL;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp4_ = _error_;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp5_ = _tmp4_->message;
#line 338 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("midori-speeddial.vala:338: Failed to update speed dial: %s", _tmp5_);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_error_free0 (_error_);
#line 3026 "midori-speeddial.c"
	}
	__finally36:
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_clear_error (&_inner_error_);
#line 334 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 3037 "midori-speeddial.c"
	}
#line 340 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_signal_emit (self, midori_speed_dial_signals[MIDORI_SPEED_DIAL_REFRESH_SIGNAL], 0);
#line 3041 "midori-speeddial.c"
}


static void _midori_speed_dial_load_status_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_load_status ((MidoriSpeedDial*) self, _sender, pspec);
#line 3048 "midori-speeddial.c"
}


static gboolean _midori_speed_dial_save_thumbnail_gsource_func (gpointer self) {
	gboolean result;
	result = midori_speed_dial_save_thumbnail ((MidoriSpeedDial*) self);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 3057 "midori-speeddial.c"
}


static void midori_speed_dial_load_status (MidoriSpeedDial* self, GObject* thumb_view_, GParamSpec* pspec) {
	gboolean _tmp0_ = FALSE;
	WebKitWebView* _tmp1_;
	WebKitLoadStatus _tmp2_;
	WebKitLoadStatus _tmp3_;
	WebKitWebView* _tmp7_;
	guint _tmp8_;
	GQuark _tmp9_;
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (thumb_view_ != NULL);
#line 344 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (pspec != NULL);
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = self->priv->thumb_view;
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = webkit_web_view_get_load_status (_tmp1_);
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = _tmp2_;
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp3_ != WEBKIT_LOAD_FINISHED) {
#line 3083 "midori-speeddial.c"
		WebKitWebView* _tmp4_;
		WebKitLoadStatus _tmp5_;
		WebKitLoadStatus _tmp6_;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp4_ = self->priv->thumb_view;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp5_ = webkit_web_view_get_load_status (_tmp4_);
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp6_ = _tmp5_;
#line 346 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp0_ = _tmp6_ != WEBKIT_LOAD_FAILED;
#line 3095 "midori-speeddial.c"
	} else {
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp0_ = FALSE;
#line 3099 "midori-speeddial.c"
	}
#line 345 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp0_) {
#line 347 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 3105 "midori-speeddial.c"
	}
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp7_ = self->priv->thumb_view;
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_signal_parse_name ("notify::load-status", G_TYPE_OBJECT, &_tmp8_, &_tmp9_, TRUE);
#line 348 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_signal_handlers_disconnect_matched ((GObject*) _tmp7_, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_DETAIL | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, _tmp8_, _tmp9_, NULL, (GCallback) _midori_speed_dial_load_status_g_object_notify, self);
#line 350 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, _midori_speed_dial_save_thumbnail_gsource_func, g_object_ref (self), g_object_unref);
#line 3115 "midori-speeddial.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return self ? g_object_ref (self) : NULL;
#line 3122 "midori-speeddial.c"
}


static gpointer _midori_speed_dial_spec_ref0 (gpointer self) {
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return self ? midori_speed_dial_spec_ref (self) : NULL;
#line 3129 "midori-speeddial.c"
}


static gboolean midori_speed_dial_save_thumbnail (MidoriSpeedDial* self) {
	gboolean result = FALSE;
	MidoriSpeedDialSpec* _tmp0_;
	GtkOffscreenWindow* offscreen = NULL;
	WebKitWebView* _tmp1_;
	GtkContainer* _tmp2_;
	GtkContainer* _tmp3_;
	GtkOffscreenWindow* _tmp4_;
	GdkPixbuf* pixbuf = NULL;
	GtkOffscreenWindow* _tmp5_;
	GdkPixbuf* _tmp6_;
	gint image_width = 0;
	GdkPixbuf* _tmp7_;
	gint _tmp8_;
	gint image_height = 0;
	GdkPixbuf* _tmp9_;
	gint _tmp10_;
	gint thumb_width = 0;
	gint thumb_height = 0;
	gfloat image_ratio = 0.0F;
	gint _tmp11_;
	gint _tmp12_;
	gfloat thumb_ratio = 0.0F;
	gint _tmp13_;
	gint _tmp14_;
	gint x_offset = 0;
	gint y_offset = 0;
	gint computed_width = 0;
	gint computed_height = 0;
	gfloat _tmp15_;
	gfloat _tmp16_;
	GdkPixbuf* sub = NULL;
	GdkPixbuf* _tmp25_;
	GdkPixbuf* _tmp26_;
	gint _tmp27_;
	gint _tmp28_;
	gint _tmp29_;
	GdkPixbuf* scaled = NULL;
	GdkPixbuf* _tmp36_;
	gint _tmp37_;
	gint _tmp38_;
	GdkPixbuf* _tmp39_;
	const gchar* _tmp40_ = NULL;
	WebKitWebView* _tmp41_;
	const gchar* _tmp42_;
	MidoriSpeedDialSpec* _tmp45_;
	const gchar* _tmp46_;
	MidoriSpeedDialSpec* _tmp47_;
	const gchar* _tmp48_;
	GdkPixbuf* _tmp49_;
	MidoriSpeedDialSpec* _tmp50_;
	GList* _tmp51_;
	guint _tmp52_;
#line 355 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 356 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->priv->spec;
#line 356 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (_tmp0_ != NULL, FALSE);
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = self->priv->thumb_view;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = gtk_widget_get_parent ((GtkWidget*) _tmp1_);
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = _tmp2_;
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp4_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp3_, GTK_TYPE_OFFSCREEN_WINDOW) ? ((GtkOffscreenWindow*) _tmp3_) : NULL);
#line 358 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	offscreen = _tmp4_;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp5_ = offscreen;
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp6_ = gtk_offscreen_window_get_pixbuf (_tmp5_);
#line 359 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	pixbuf = _tmp6_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp7_ = pixbuf;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp8_ = gdk_pixbuf_get_width (_tmp7_);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	image_width = _tmp8_;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp9_ = pixbuf;
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp10_ = gdk_pixbuf_get_height (_tmp9_);
#line 360 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	image_height = _tmp10_;
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	thumb_width = 240;
#line 361 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	thumb_height = 160;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp11_ = image_width;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp12_ = image_height;
#line 362 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	image_ratio = (gfloat) (_tmp11_ / _tmp12_);
#line 363 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp13_ = thumb_width;
#line 363 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp14_ = thumb_height;
#line 363 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	thumb_ratio = (gfloat) (_tmp13_ / _tmp14_);
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp15_ = image_ratio;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp16_ = thumb_ratio;
#line 365 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp15_ > _tmp16_) {
#line 3242 "midori-speeddial.c"
		gint _tmp17_;
		gfloat _tmp18_;
		gint _tmp19_;
		gint _tmp20_;
		gint _tmp21_;
#line 366 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp17_ = image_height;
#line 366 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp18_ = thumb_ratio;
#line 366 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		computed_width = (gint) (_tmp17_ * _tmp18_);
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp19_ = image_height;
#line 367 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		computed_height = _tmp19_;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp20_ = image_width;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp21_ = computed_width;
#line 368 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		x_offset = (_tmp20_ - _tmp21_) / 2;
#line 369 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		y_offset = 0;
#line 3266 "midori-speeddial.c"
	} else {
		gint _tmp22_;
		gint _tmp23_;
		gfloat _tmp24_;
#line 372 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp22_ = image_width;
#line 372 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		computed_width = _tmp22_;
#line 373 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp23_ = image_width;
#line 373 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp24_ = thumb_ratio;
#line 373 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		computed_height = (gint) (_tmp23_ / _tmp24_);
#line 374 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		x_offset = 0;
#line 375 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		y_offset = 0;
#line 3285 "midori-speeddial.c"
	}
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp25_ = pixbuf;
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp26_ = _g_object_ref0 (_tmp25_);
#line 377 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	sub = _tmp26_;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp27_ = y_offset;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp28_ = computed_height;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp29_ = image_height;
#line 378 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if ((_tmp27_ + _tmp28_) <= _tmp29_) {
#line 3301 "midori-speeddial.c"
		GdkPixbuf* _tmp30_;
		gint _tmp31_;
		gint _tmp32_;
		gint _tmp33_;
		gint _tmp34_;
		GdkPixbuf* _tmp35_;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp30_ = pixbuf;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp31_ = x_offset;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp32_ = y_offset;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp33_ = computed_width;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp34_ = computed_height;
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp35_ = gdk_pixbuf_new_subpixbuf (_tmp30_, _tmp31_, _tmp32_, _tmp33_, _tmp34_);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_object_unref0 (sub);
#line 379 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		sub = _tmp35_;
#line 3324 "midori-speeddial.c"
	}
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp36_ = sub;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp37_ = thumb_width;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp38_ = thumb_height;
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp39_ = gdk_pixbuf_scale_simple (_tmp36_, _tmp37_, _tmp38_, GDK_INTERP_TILES);
#line 380 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	scaled = _tmp39_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp41_ = self->priv->thumb_view;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp42_ = webkit_web_view_get_title (_tmp41_);
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp40_ = _tmp42_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp40_ == NULL) {
#line 3344 "midori-speeddial.c"
		MidoriSpeedDialSpec* _tmp43_;
		const gchar* _tmp44_;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp43_ = self->priv->spec;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp44_ = _tmp43_->uri;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp40_ = _tmp44_;
#line 3353 "midori-speeddial.c"
	}
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp45_ = self->priv->spec;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp46_ = _tmp45_->dial_id;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp47_ = self->priv->spec;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp48_ = _tmp47_->uri;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp49_ = scaled;
#line 381 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_add_with_id (self, _tmp46_, _tmp48_, _tmp40_, _tmp49_);
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp50_ = self->priv->spec;
#line 383 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->thumb_queue = g_list_remove (self->priv->thumb_queue, _tmp50_);
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp51_ = self->priv->thumb_queue;
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp52_ = g_list_length (_tmp51_);
#line 384 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp52_ > ((guint) 0)) {
#line 3377 "midori-speeddial.c"
		GList* _tmp53_;
		gconstpointer _tmp54_;
		MidoriSpeedDialSpec* _tmp55_;
		WebKitWebView* _tmp56_;
		WebKitWebView* _tmp57_;
		MidoriSpeedDialSpec* _tmp58_;
		const gchar* _tmp59_;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp53_ = self->priv->thumb_queue;
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp54_ = g_list_nth_data (_tmp53_, (guint) 0);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp55_ = _midori_speed_dial_spec_ref0 ((MidoriSpeedDialSpec*) _tmp54_);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_midori_speed_dial_spec_unref0 (self->priv->spec);
#line 385 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		self->priv->spec = _tmp55_;
#line 386 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp56_ = self->priv->thumb_view;
#line 386 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_signal_connect_object ((GObject*) _tmp56_, "notify::load-status", (GCallback) _midori_speed_dial_load_status_g_object_notify, self, 0);
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp57_ = self->priv->thumb_view;
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp58_ = self->priv->spec;
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp59_ = _tmp58_->uri;
#line 387 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		webkit_web_view_load_uri (_tmp57_, _tmp59_);
#line 3407 "midori-speeddial.c"
	}
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	result = FALSE;
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_object_unref0 (scaled);
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_object_unref0 (sub);
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_object_unref0 (pixbuf);
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_object_unref0 (offscreen);
#line 389 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 3421 "midori-speeddial.c"
}


static void midori_speed_dial_get_thumb (MidoriSpeedDial* self, const gchar* dial_id, const gchar* uri) {
	WebKitWebView* _tmp0_;
	GList* _tmp10_;
	const gchar* _tmp15_;
	const gchar* _tmp16_;
	MidoriSpeedDialSpec* _tmp17_;
	GList* _tmp18_;
	guint _tmp19_;
	GList* _tmp20_;
	gconstpointer _tmp21_;
	MidoriSpeedDialSpec* _tmp22_;
	WebKitWebView* _tmp23_;
	WebKitWebView* _tmp24_;
	MidoriSpeedDialSpec* _tmp25_;
	const gchar* _tmp26_;
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (dial_id != NULL);
#line 393 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (uri != NULL);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->priv->thumb_view;
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp0_ == NULL) {
#line 3450 "midori-speeddial.c"
		WebKitWebView* _tmp1_;
		WebKitWebView* _tmp2_;
		WebKitWebSettings* _tmp3_;
		WebKitWebSettings* _tmp4_;
		GtkOffscreenWindow* offscreen = NULL;
		GtkOffscreenWindow* _tmp5_;
		GtkOffscreenWindow* _tmp6_;
		WebKitWebView* _tmp7_;
		WebKitWebView* _tmp8_;
		GtkOffscreenWindow* _tmp9_;
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp1_ = (WebKitWebView*) webkit_web_view_new ();
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_object_ref_sink (_tmp1_);
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_object_unref0 (self->priv->thumb_view);
#line 396 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		self->priv->thumb_view = _tmp1_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp2_ = self->priv->thumb_view;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp3_ = webkit_web_view_get_settings (_tmp2_);
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp4_ = _tmp3_;
#line 397 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_object_set ((GObject*) _tmp4_, "enable-scripts", FALSE, "enable-plugins", FALSE, "auto-load-images", TRUE, "enable-html5-database", FALSE, "enable-html5-local-storage", FALSE, "enable-java-applet", FALSE, NULL);
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp5_ = (GtkOffscreenWindow*) gtk_offscreen_window_new ();
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_object_ref_sink (_tmp5_);
#line 404 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		offscreen = _tmp5_;
#line 405 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp6_ = offscreen;
#line 405 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp7_ = self->priv->thumb_view;
#line 405 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		gtk_container_add ((GtkContainer*) _tmp6_, (GtkWidget*) _tmp7_);
#line 406 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp8_ = self->priv->thumb_view;
#line 406 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		gtk_widget_set_size_request ((GtkWidget*) _tmp8_, 800, 600);
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp9_ = offscreen;
#line 407 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		gtk_widget_show_all ((GtkWidget*) _tmp9_);
#line 395 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_g_object_unref0 (offscreen);
#line 3499 "midori-speeddial.c"
	}
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp10_ = self->priv->thumb_queue;
#line 3503 "midori-speeddial.c"
	{
		GList* spec__collection = NULL;
		GList* spec__it = NULL;
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		spec__collection = _tmp10_;
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		for (spec__it = spec__collection; spec__it != NULL; spec__it = spec__it->next) {
#line 3511 "midori-speeddial.c"
			MidoriSpeedDialSpec* _tmp11_;
			MidoriSpeedDialSpec* spec_ = NULL;
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			_tmp11_ = _midori_speed_dial_spec_ref0 ((MidoriSpeedDialSpec*) spec__it->data);
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			spec_ = _tmp11_;
#line 3518 "midori-speeddial.c"
			{
				MidoriSpeedDialSpec* _tmp12_;
				const gchar* _tmp13_;
				const gchar* _tmp14_;
#line 412 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp12_ = spec_;
#line 412 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp13_ = _tmp12_->dial_id;
#line 412 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_tmp14_ = dial_id;
#line 412 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				if (g_strcmp0 (_tmp13_, _tmp14_) == 0) {
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					_midori_speed_dial_spec_unref0 (spec_);
#line 413 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
					return;
#line 3535 "midori-speeddial.c"
				}
#line 411 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
				_midori_speed_dial_spec_unref0 (spec_);
#line 3539 "midori-speeddial.c"
			}
		}
	}
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp15_ = dial_id;
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp16_ = uri;
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp17_ = midori_speed_dial_spec_new (_tmp15_, _tmp16_);
#line 415 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->thumb_queue = g_list_append (self->priv->thumb_queue, _tmp17_);
#line 416 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp18_ = self->priv->thumb_queue;
#line 416 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp19_ = g_list_length (_tmp18_);
#line 416 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (_tmp19_ > ((guint) 1)) {
#line 417 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return;
#line 3559 "midori-speeddial.c"
	}
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp20_ = self->priv->thumb_queue;
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp21_ = g_list_nth_data (_tmp20_, (guint) 0);
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp22_ = _midori_speed_dial_spec_ref0 ((MidoriSpeedDialSpec*) _tmp21_);
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_midori_speed_dial_spec_unref0 (self->priv->spec);
#line 419 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->spec = _tmp22_;
#line 420 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp23_ = self->priv->thumb_view;
#line 420 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_signal_connect_object ((GObject*) _tmp23_, "notify::load-status", (GCallback) _midori_speed_dial_load_status_g_object_notify, self, 0);
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp24_ = self->priv->thumb_view;
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp25_ = self->priv->spec;
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp26_ = _tmp25_->uri;
#line 421 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	webkit_web_view_load_uri (_tmp24_, _tmp26_);
#line 3583 "midori-speeddial.c"
}


gboolean midori_speed_dial_get_close_buttons_left (MidoriSpeedDial* self) {
	gboolean result;
	gboolean _tmp0_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = self->priv->_close_buttons_left;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	result = _tmp0_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return result;
#line 3598 "midori-speeddial.c"
}


void midori_speed_dial_set_close_buttons_left (MidoriSpeedDial* self, gboolean value) {
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (self != NULL);
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (midori_speed_dial_get_close_buttons_left (self) != value) {
#line 3607 "midori-speeddial.c"
		gboolean _tmp0_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		_tmp0_ = value;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		self->priv->_close_buttons_left = _tmp0_;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_object_notify_by_pspec ((GObject *) self, midori_speed_dial_properties[MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY]);
#line 3615 "midori-speeddial.c"
	}
}


MidoriSpeedDialSpec* midori_speed_dial_spec_construct (GType object_type, const gchar* dial_id, const gchar* uri) {
	MidoriSpeedDialSpec* self = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	const gchar* _tmp2_;
	gchar* _tmp3_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (dial_id != NULL, NULL);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (uri != NULL, NULL);
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self = (MidoriSpeedDialSpec*) g_type_create_instance (object_type);
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp0_ = dial_id;
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->dial_id);
#line 48 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->dial_id = _tmp1_;
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp2_ = uri;
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_tmp3_ = g_strdup (_tmp2_);
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->uri);
#line 49 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->uri = _tmp3_;
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return self;
#line 3650 "midori-speeddial.c"
}


MidoriSpeedDialSpec* midori_speed_dial_spec_new (const gchar* dial_id, const gchar* uri) {
#line 47 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return midori_speed_dial_spec_construct (MIDORI_SPEED_DIAL_TYPE_SPEC, dial_id, uri);
#line 3657 "midori-speeddial.c"
}


static void midori_speed_dial_value_spec_init (GValue* value) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	value->data[0].v_pointer = NULL;
#line 3664 "midori-speeddial.c"
}


static void midori_speed_dial_value_spec_free_value (GValue* value) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (value->data[0].v_pointer) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_spec_unref (value->data[0].v_pointer);
#line 3673 "midori-speeddial.c"
	}
}


static void midori_speed_dial_value_spec_copy_value (const GValue* src_value, GValue* dest_value) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (src_value->data[0].v_pointer) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		dest_value->data[0].v_pointer = midori_speed_dial_spec_ref (src_value->data[0].v_pointer);
#line 3683 "midori-speeddial.c"
	} else {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		dest_value->data[0].v_pointer = NULL;
#line 3687 "midori-speeddial.c"
	}
}


static gpointer midori_speed_dial_value_spec_peek_pointer (const GValue* value) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return value->data[0].v_pointer;
#line 3695 "midori-speeddial.c"
}


static gchar* midori_speed_dial_value_spec_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (collect_values[0].v_pointer) {
#line 3702 "midori-speeddial.c"
		MidoriSpeedDialSpec * object;
		object = collect_values[0].v_pointer;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		if (object->parent_instance.g_class == NULL) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 3709 "midori-speeddial.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 3713 "midori-speeddial.c"
		}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = midori_speed_dial_spec_ref (object);
#line 3717 "midori-speeddial.c"
	} else {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = NULL;
#line 3721 "midori-speeddial.c"
	}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return NULL;
#line 3725 "midori-speeddial.c"
}


static gchar* midori_speed_dial_value_spec_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	MidoriSpeedDialSpec ** object_p;
	object_p = collect_values[0].v_pointer;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (!object_p) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 3736 "midori-speeddial.c"
	}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (!value->data[0].v_pointer) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		*object_p = NULL;
#line 3742 "midori-speeddial.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		*object_p = value->data[0].v_pointer;
#line 3746 "midori-speeddial.c"
	} else {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		*object_p = midori_speed_dial_spec_ref (value->data[0].v_pointer);
#line 3750 "midori-speeddial.c"
	}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return NULL;
#line 3754 "midori-speeddial.c"
}


GParamSpec* midori_speed_dial_param_spec_spec (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	MidoriSpeedDialParamSpecSpec* spec;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (g_type_is_a (object_type, MIDORI_SPEED_DIAL_TYPE_SPEC), NULL);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return G_PARAM_SPEC (spec);
#line 3768 "midori-speeddial.c"
}


gpointer midori_speed_dial_value_get_spec (const GValue* value) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MIDORI_SPEED_DIAL_TYPE_SPEC), NULL);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return value->data[0].v_pointer;
#line 3777 "midori-speeddial.c"
}


void midori_speed_dial_value_set_spec (GValue* value, gpointer v_object) {
	MidoriSpeedDialSpec * old;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MIDORI_SPEED_DIAL_TYPE_SPEC));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	old = value->data[0].v_pointer;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (v_object) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MIDORI_SPEED_DIAL_TYPE_SPEC));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = v_object;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_spec_ref (value->data[0].v_pointer);
#line 3797 "midori-speeddial.c"
	} else {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = NULL;
#line 3801 "midori-speeddial.c"
	}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (old) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_spec_unref (old);
#line 3807 "midori-speeddial.c"
	}
}


void midori_speed_dial_value_take_spec (GValue* value, gpointer v_object) {
	MidoriSpeedDialSpec * old;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, MIDORI_SPEED_DIAL_TYPE_SPEC));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	old = value->data[0].v_pointer;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (v_object) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, MIDORI_SPEED_DIAL_TYPE_SPEC));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = v_object;
#line 3826 "midori-speeddial.c"
	} else {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		value->data[0].v_pointer = NULL;
#line 3830 "midori-speeddial.c"
	}
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (old) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_spec_unref (old);
#line 3836 "midori-speeddial.c"
	}
}


static void midori_speed_dial_spec_class_init (MidoriSpeedDialSpecClass * klass) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_spec_parent_class = g_type_class_peek_parent (klass);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	((MidoriSpeedDialSpecClass *) klass)->finalize = midori_speed_dial_spec_finalize;
#line 3846 "midori-speeddial.c"
}


static void midori_speed_dial_spec_instance_init (MidoriSpeedDialSpec * self) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->ref_count = 1;
#line 3853 "midori-speeddial.c"
}


static void midori_speed_dial_spec_finalize (MidoriSpeedDialSpec * obj) {
	MidoriSpeedDialSpec * self;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MIDORI_SPEED_DIAL_TYPE_SPEC, MidoriSpeedDialSpec);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_signal_handlers_destroy (self);
#line 45 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->dial_id);
#line 46 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->uri);
#line 3867 "midori-speeddial.c"
}


GType midori_speed_dial_spec_get_type (void) {
	static volatile gsize midori_speed_dial_spec_type_id__volatile = 0;
	if (g_once_init_enter (&midori_speed_dial_spec_type_id__volatile)) {
		static const GTypeValueTable g_define_type_value_table = { midori_speed_dial_value_spec_init, midori_speed_dial_value_spec_free_value, midori_speed_dial_value_spec_copy_value, midori_speed_dial_value_spec_peek_pointer, "p", midori_speed_dial_value_spec_collect_value, "p", midori_speed_dial_value_spec_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (MidoriSpeedDialSpecClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_speed_dial_spec_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriSpeedDialSpec), 0, (GInstanceInitFunc) midori_speed_dial_spec_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		GType midori_speed_dial_spec_type_id;
		midori_speed_dial_spec_type_id = g_type_register_fundamental (g_type_fundamental_next (), "MidoriSpeedDialSpec", &g_define_type_info, &g_define_type_fundamental_info, 0);
		g_once_init_leave (&midori_speed_dial_spec_type_id__volatile, midori_speed_dial_spec_type_id);
	}
	return midori_speed_dial_spec_type_id__volatile;
}


gpointer midori_speed_dial_spec_ref (gpointer instance) {
	MidoriSpeedDialSpec * self;
	self = instance;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_atomic_int_inc (&self->ref_count);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	return instance;
#line 3892 "midori-speeddial.c"
}


void midori_speed_dial_spec_unref (gpointer instance) {
	MidoriSpeedDialSpec * self;
	self = instance;
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		MIDORI_SPEED_DIAL_SPEC_GET_CLASS (self)->finalize (self);
#line 44 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 3905 "midori-speeddial.c"
	}
}


static void midori_speed_dial_class_init (MidoriSpeedDialClass * klass) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_parent_class = g_type_class_peek_parent (klass);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_type_class_add_private (klass, sizeof (MidoriSpeedDialPrivate));
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_midori_speed_dial_get_property;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_midori_speed_dial_set_property;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	G_OBJECT_CLASS (klass)->finalize = midori_speed_dial_finalize;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY, midori_speed_dial_properties[MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY] = g_param_spec_boolean ("close-buttons-left", "close-buttons-left", "close-buttons-left", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	midori_speed_dial_signals[MIDORI_SPEED_DIAL_REFRESH_SIGNAL] = g_signal_new ("refresh", MIDORI_TYPE_SPEED_DIAL, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 3925 "midori-speeddial.c"
}


static void midori_speed_dial_instance_init (MidoriSpeedDial * self) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv = MIDORI_SPEED_DIAL_GET_PRIVATE (self);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->html = NULL;
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->thumb_queue = NULL;
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->thumb_view = NULL;
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->spec = NULL;
#line 41 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self->priv->_close_buttons_left = FALSE;
#line 3942 "midori-speeddial.c"
}


static void midori_speed_dial_finalize (GObject * obj) {
	MidoriSpeedDial * self;
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDial);
#line 32 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->priv->filename);
#line 33 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_free0 (self->priv->html);
#line 34 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	__g_list_free__midori_speed_dial_spec_unref0_0 (self->priv->thumb_queue);
#line 36 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_object_unref0 (self->priv->thumb_view);
#line 37 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_midori_speed_dial_spec_unref0 (self->priv->spec);
#line 40 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	_g_key_file_unref0 (self->keyfile);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	G_OBJECT_CLASS (midori_speed_dial_parent_class)->finalize (obj);
#line 3964 "midori-speeddial.c"
}


GType midori_speed_dial_get_type (void) {
	static volatile gsize midori_speed_dial_type_id__volatile = 0;
	if (g_once_init_enter (&midori_speed_dial_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (MidoriSpeedDialClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) midori_speed_dial_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MidoriSpeedDial), 0, (GInstanceInitFunc) midori_speed_dial_instance_init, NULL };
		GType midori_speed_dial_type_id;
		midori_speed_dial_type_id = g_type_register_static (G_TYPE_OBJECT, "MidoriSpeedDial", &g_define_type_info, 0);
		g_once_init_leave (&midori_speed_dial_type_id__volatile, midori_speed_dial_type_id);
	}
	return midori_speed_dial_type_id__volatile;
}


static void _vala_midori_speed_dial_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	MidoriSpeedDial * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDial);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	switch (property_id) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		case MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY:
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		g_value_set_boolean (value, midori_speed_dial_get_close_buttons_left (self));
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		break;
#line 3991 "midori-speeddial.c"
		default:
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		break;
#line 3997 "midori-speeddial.c"
	}
}


static void _vala_midori_speed_dial_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	MidoriSpeedDial * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, MIDORI_TYPE_SPEED_DIAL, MidoriSpeedDial);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
	switch (property_id) {
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		case MIDORI_SPEED_DIAL_CLOSE_BUTTONS_LEFT_PROPERTY:
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		midori_speed_dial_set_close_buttons_left (self, g_value_get_boolean (value));
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		break;
#line 4013 "midori-speeddial.c"
		default:
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 31 "/usr/src/ports/midori/midori-0.5.11-2.x86_64/src/midori-0.5.11/midori/midori-speeddial.vala"
		break;
#line 4019 "midori-speeddial.c"
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}



